import _defineProperty from '@babel/runtime/helpers/defineProperty';
import React, { useState, useEffect, createContext, useMemo, useCallback, Suspense } from 'react';
import DynamicMultiRender, { DynamicMultiRenderProvider } from 'react-dynamic-multi-render';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';
import { getLogger, DivaUtils, useTranslation, Trans, useDivaCore } from '@crystaldesign/diva-core';
import { observer } from 'mobx-react-lite';
import _objectWithoutProperties from '@babel/runtime/helpers/objectWithoutProperties';
import _asyncToGenerator from '@babel/runtime/helpers/asyncToGenerator';
import _slicedToArray from '@babel/runtime/helpers/slicedToArray';
import _regeneratorRuntime from '@babel/runtime/regenerator';
import _classCallCheck from '@babel/runtime/helpers/classCallCheck';
import _createClass from '@babel/runtime/helpers/createClass';
import _toConsumableArray from '@babel/runtime/helpers/toConsumableArray';
import DivaUtils$1, { DivaError, UTMUtils, UtmSource, UtmMedium, UtmCampaign } from '@crystaldesign/diva-utils';
import { makeAutoObservable, makeObservable, observable, action, computed } from 'mobx';
import DivaWebPlanner, { Vector2 } from '@crystaldesign/web-planner-logic';
import classnames from 'classnames';
import { QRCodeSVG } from 'qrcode.react';
import _typeof from '@babel/runtime/helpers/typeof';
import { flatten } from 'flat';
import { ContactFormComponent } from '@crystaldesign/widget-contact-form';

(function() {
    const env = {"STAGE":"production"};
    try {
        if (process) {
            process.env = Object.assign({}, process.env);
            Object.assign(process.env, env);
            return;
        }
    } catch (e) {} // avoid ReferenceError: process is not defined
    globalThis.process = { env:env };
})();

if (typeof __webpack_public_path__ !== 'undefined') {
  var _document;
  //if __webpack_public_path__ is defiend, we are within a umd build, otherwise this workarround is not needed
  if (!__webpack_public_path__.startsWith('http') && (_document = document) !== null && _document !== void 0 && (_document = _document.currentScript) !== null && _document !== void 0 && _document.src.includes('.umd.')) {
    //if public_path is a relativ url script tag is present, load the chunks from same location as the main script
    //this is only present for backwards compatibility and can be removed if we are sure it is not needed any more
    var url$1 = new URL(document.currentScript.src);
    var urlBase = url$1.href.slice(0, url$1.href.lastIndexOf('/') + 1);
    __webpack_public_path__ = urlBase;
    //if global var diva_source_location is set, load the chunks from that location
  } else if (window.diva_source_location) {
    __webpack_public_path__ = window.diva_source_location;
  }
}

function ownKeys$h(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$h(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$h(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$h(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var Layout = function Layout(props) {
  return /*#__PURE__*/jsx(DynamicMultiRender, _objectSpread$h(_objectSpread$h({}, props), {}, {
    componentName: "Layout"
  }));
};

var index$e = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': Layout
});

var LOG$e = getLogger('Planner', 'ConfigurationStructureHandler');
var ConfiguratorStructureHandler = /*#__PURE__*/function () {
  function ConfiguratorStructureHandler() {
    var _this = this;
    _classCallCheck(this, ConfiguratorStructureHandler);
    _defineProperty(this, "configuratorSturucture", void 0);
    _defineProperty(this, "curTabGroup", void 0);
    _defineProperty(this, "curOption", void 0);
    _defineProperty(this, "curConfigStep", void 0);
    _defineProperty(this, "actGroupedConfigSteps", void 0);
    _defineProperty(this, "_loadedConfigStepsOfActTabGroup", void 0);
    _defineProperty(this, "tabGrousVisitedWithTargetArticles", {});
    _defineProperty(this, "hideSingleOPVOnlyAlways", false);
    _defineProperty(this, "onEnterPlannerMode", void 0);
    _defineProperty(this, "tabGroupState", void 0);
    _defineProperty(this, "navigateStepByStep", void 0);
    _defineProperty(this, "navigateToTabgroupByPosNr", function (newTabGroupPosNr) {
      var newTabGroup = _this.getTabByPosNr(newTabGroupPosNr);
      if (newTabGroup != null) {
        var firstKeyOfOptionStep = Object.keys(newTabGroup.options)[0];
        var firstLeyOfConifgStep = Object.keys(newTabGroup.options[firstKeyOfOptionStep].configSteps)[0];
        if (Object.entries(newTabGroup.options) && Object.entries(newTabGroup.options[firstKeyOfOptionStep].configSteps).length > 0) {
          _this.setActStepByPosNr(newTabGroup.options[firstKeyOfOptionStep].configSteps[firstLeyOfConifgStep].posNr);
          return true;
        }
      }
      return false;
    });
    _defineProperty(this, "tabGroupAlreadyVisited", function (tabgroup) {
      var ret = false;
      if (!tabgroup) {
        tabgroup = _this.curTabGroup;
      }
      var actArticles = [];
      for (var iKey in tabgroup.options) {
        var option = tabgroup.options[iKey];
        for (var jKey in option.configSteps) {
          var configStep = option.configSteps[jKey];
          if (configStep.notDisplayedInMainConfigTree) continue;
          actArticles = actArticles.concat(configStep.targetArticles);
        }
      }
      var optionsOfTabGroup = Object.keys(tabgroup.options);
      if (_this.tabGrousVisitedWithTargetArticles[tabgroup.key] && actArticles) {
        var visitedArticles = _this.tabGrousVisitedWithTargetArticles[tabgroup.key];
        ret = true;
        for (var i = 0; i < actArticles.length; i++) {
          for (var j = 0; j < optionsOfTabGroup.length; j++) {
            if (visitedArticles[actArticles[i]] == undefined || !visitedArticles[actArticles[i]].includes(optionsOfTabGroup[j])) {
              ret = false;
              break;
            }
          }
          if (!ret) break;
        }
        //da der Planner keine TargetArtikels haben kann, wird das targetArticleArray mit ["ALL"] gefüllt falls der Planner bereits besucht wurde
        if (Object.keys(visitedArticles).length == 1 && visitedArticles['ALL'] && visitedArticles['ALL'][0] == tabgroup.type) {
          ret = true;
        }
      }
      return ret;
    });
    _defineProperty(this, "configStepAlreadyVisited", function (configStep) {
      var actArticles = _this.getTargetArticleOfConfigStepArr([configStep]);
      var step = _this.getConfigStepByKey(configStep.key);
      var ret = false;
      if (!step) return ret;
      var tabGroup = step.tabGroup;
      var visitedArticles = _this.tabGrousVisitedWithTargetArticles[tabGroup.key];
      if (visitedArticles) {
        ret = true;
        for (var i = 0; i < actArticles.length; i++) {
          var article = actArticles[i];
          if (visitedArticles[article] == undefined || !visitedArticles[article].includes(configStep.option)) {
            ret = false;
          }
        }
        //da der Planner keine TargetArtikels haben kann, wird das targetArticleArray mit ["ALL"] gefüllt falls der Planner bereits besucht wurde
        if (Object.keys(visitedArticles).length == 1 && visitedArticles['ALL'] && visitedArticles['ALL'][0] == tabGroup.type) {
          ret = true;
        }
      }
      return ret;
    });
  }
  return _createClass(ConfiguratorStructureHandler, [{
    key: "initialize",
    value: function initialize(tabGroupState, configuratorStructure, hideSingleOPVOnlyAlways, onEnterPlannerMode, navigateStepByStep) {
      this.tabGroupState = tabGroupState;
      this.onEnterPlannerMode = onEnterPlannerMode;
      this.configuratorSturucture = configuratorStructure;
      this.hideSingleOPVOnlyAlways = hideSingleOPVOnlyAlways;
      this.curTabGroup = this.configuratorSturucture[Object.keys(this.configuratorSturucture)[0]];
      this.curOption = this.curTabGroup.options[Object.keys(this.curTabGroup.options)[0]];
      this.curConfigStep = this.curOption.configSteps[Object.keys(this.curOption.configSteps)[0]];
      this.callEnterFunctions(undefined);
      this.actGroupedConfigSteps = [this.curConfigStep];
      this._loadedConfigStepsOfActTabGroup = [this.curConfigStep];
      this.navigateStepByStep = navigateStepByStep;
      this.tabGrousVisitedWithTargetArticles = {};
      this.updateView();
    }
  }, {
    key: "navigateForwards",
    value: function navigateForwards() {
      var newPosNr = this.getNewPosNr(this.curConfigStep.posNr, false);
      var ret = this.setActStepByPosNr(newPosNr);
      if (!ret) LOG$e.debug('in der summary geht es nicht mehr weiter!');
    }
  }, {
    key: "navigateBackwards",
    value: function navigateBackwards() {
      var _this$getConfigStepBy;
      var jumpedBackIntoNewTabGroup = false;
      var newPosNr = this.getNewPosNr(this.curConfigStep.posNr, true);
      var tabGroup = (_this$getConfigStepBy = this.getConfigStepByPosNr(newPosNr)) === null || _this$getConfigStepBy === void 0 ? void 0 : _this$getConfigStepBy.tabGroup;
      if ((tabGroup === null || tabGroup === void 0 ? void 0 : tabGroup.key) != this.curTabgroupKey) {
        jumpedBackIntoNewTabGroup = true;
      }
      if (tabGroup && jumpedBackIntoNewTabGroup) {
        newPosNr = this.getFirstPosNrOfTabGroup(tabGroup);
      }
      var ret = this.setActStepByPosNr(newPosNr);
      if (!ret) LOG$e.debug('Von hier aus kann man nicht mehr weiter zurückspringen');
    }
  }, {
    key: "getNewPosNr",
    value: function getNewPosNr(newPosNr, doMinus) {
      var isFirstTimeAlreadyDone = false;
      do {
        var _this$getConfigStepBy2, _this$getConfigStepBy3, _this$getConfigStepBy4, _this$getConfigStepBy5, _this$getConfigStepBy6;
        if (isFirstTimeAlreadyDone) {
          this.visitTabGroup(newPosNr);
        }
        newPosNr = this.getPosNrNotInCurGroupedSteps(doMinus ? newPosNr - 1 : newPosNr + 1, doMinus);
        isFirstTimeAlreadyDone = true;
      } while ((_this$getConfigStepBy2 = this.getConfigStepByPosNr(newPosNr)) !== null && _this$getConfigStepBy2 !== void 0 && _this$getConfigStepBy2.tabGroup.singleOPVOnly || ((_this$getConfigStepBy3 = this.getConfigStepByPosNr(newPosNr)) === null || _this$getConfigStepBy3 === void 0 ? void 0 : _this$getConfigStepBy3.optionStep.mergesToOPG) == true || this.filterOptionByFilterText((_this$getConfigStepBy4 = this.getConfigStepByPosNr(newPosNr)) === null || _this$getConfigStepBy4 === void 0 ? void 0 : _this$getConfigStepBy4.optionStep) || (_this$getConfigStepBy5 = this.getConfigStepByPosNr(newPosNr)) !== null && _this$getConfigStepBy5 !== void 0 && _this$getConfigStepBy5.configStep.notDisplayedInMainConfigTree || (_this$getConfigStepBy6 = this.getConfigStepByPosNr(newPosNr)) !== null && _this$getConfigStepBy6 !== void 0 && _this$getConfigStepBy6.configStep.isDisabledByDiverseParentOption);
      return newPosNr;
    }
  }, {
    key: "filterOptionByFilterText",
    value: function filterOptionByFilterText(option) {
      var _this$tabGroupState;
      var filterText = (_this$tabGroupState = this.tabGroupState) === null || _this$tabGroupState === void 0 ? void 0 : _this$tabGroupState.filterTabgroup;
      if (!filterText || !option) return false;
      if (option.codex.toLocaleLowerCase().includes(filterText) || option.title.toLocaleLowerCase().includes(filterText)) return false;
      return true;
    }
  }, {
    key: "getPosNrNotInCurGroupedSteps",
    value: function getPosNrNotInCurGroupedSteps(newPosNr, doMinus) {
      var newConfigStep = this.getConfigStepByPosNr(newPosNr);
      // from shelfSteps to not confuse the user when navigate never jump a step which was already visited. In bouth directions: back and forward
      if (this.navigateStepByStep() || newConfigStep && !this.stepIsBoundedWithOtherOption(newConfigStep.configStep) && (newConfigStep.optionStep.subTabGroup !== this.curOption.subTabGroup || newConfigStep.configStep.key.startsWith('SHELFARTICLE_') || this.curConfigStep.key.startsWith('SHELFARTICLE_'))) return newPosNr;
      if (newConfigStep && (this.configStepIsInCurGroupedConfigSteps(newConfigStep.configStep.key) || this.stepIsBoundedWithOtherOption(newConfigStep.configStep))) {
        newPosNr = this.getPosNrNotInCurGroupedSteps(doMinus ? newPosNr - 1 : newPosNr + 1, doMinus);
      }
      return newPosNr;
    }
  }, {
    key: "getFirstPosNrOfTabGroup",
    value: function getFirstPosNrOfTabGroup(tabGroup) {
      var optionsFirstKey = Object.keys(tabGroup.options)[0];
      var configFirstKey = Object.keys(tabGroup.options[optionsFirstKey].configSteps)[0];
      return tabGroup.options[optionsFirstKey].configSteps[configFirstKey].posNr;
    }
  }, {
    key: "configStepIsInCurGroupedConfigSteps",
    value: function configStepIsInCurGroupedConfigSteps(key) {
      if (this.curTabGroup.type == 'OPTION') for (var i = 0; i < this.actGroupedConfigSteps.length; i++) {
        var configStep = this.actGroupedConfigSteps[i];
        if (configStep.key == key) {
          return true;
        }
      }
      return false;
    }
  }, {
    key: "visitTabGroup",
    value: function visitTabGroup(newPosNr) {
      var ret = this.getConfigStepByPosNr(newPosNr);
      if (ret) {
        var tabGroupKey = ret.tabGroup.key;
        var targetArticles = this.getTargetArticleOfConfigStepArr(this.getConfigStepsOfTabGroup(ret.tabGroup));
        var options = this.getOptionsOfTabGroup(ret.tabGroup);
        this.addTabGrousVisitedWithTargetArticlesAndOptions(tabGroupKey, targetArticles, options);
      }
    }
  }, {
    key: "configStepKeyIsActive",
    value: function configStepKeyIsActive(key) {
      if (this.curConfigStepKey == key) return true;
      return this.configStepIsInCurGroupedConfigSteps(key);
    }
  }, {
    key: "TabGrousVisitedWithTargetArticles",
    get: function get() {
      return this.tabGrousVisitedWithTargetArticles;
    }
  }, {
    key: "addAllPreviousTabGroupsAsVisited",
    value: function addAllPreviousTabGroupsAsVisited(configStepKey) {
      var _this$getConfigStepBy7;
      var tabGroupKey = (_this$getConfigStepBy7 = this.getConfigStepByKey(configStepKey)) === null || _this$getConfigStepBy7 === void 0 ? void 0 : _this$getConfigStepBy7.tabGroup.key;
      if (!tabGroupKey) return;
      var allTabGroups = Object.keys(this.allTabGroups);
      for (var i = 0; i < allTabGroups.length; i++) {
        if (tabGroupKey == allTabGroups[i]) break;
        this.addTabGrousVisitedWithTargetArticlesAndOptions(allTabGroups[i], ['ALL'], this.getOptionsOfTabGroup(this.allTabGroups[allTabGroups[i]]));
      }
    }
  }, {
    key: "addTabGrousVisitedWithTargetArticlesAndOptions",
    value: function addTabGrousVisitedWithTargetArticlesAndOptions(tabGroupKey, targetArticles, options) {
      if (!this.tabGrousVisitedWithTargetArticles[tabGroupKey]) this.tabGrousVisitedWithTargetArticles[tabGroupKey] = {};
      var tabGroup = this.tabGrousVisitedWithTargetArticles[tabGroupKey];
      for (var i = 0; i < targetArticles.length; i++) {
        if (tabGroup[targetArticles[i]] == undefined) {
          tabGroup[targetArticles[i]] = [];
        }
        for (var j = 0; j < options.length; j++) {
          if (!tabGroup[targetArticles[i]].includes(options[j])) tabGroup[targetArticles[i]].push(options[j]);
        }
      }
    }
  }, {
    key: "getOptionsOfCurTabGroup",
    value: function getOptionsOfCurTabGroup() {
      return Object.keys(this.curTabGroup.options);
    }
  }, {
    key: "getOptionsOfTabGroup",
    value: function getOptionsOfTabGroup(tabGroup) {
      return Object.keys(tabGroup.options);
    }
  }, {
    key: "navigateToTabgroupByType",
    value: function navigateToTabgroupByType(type) {
      for (var key in this.configuratorSturucture) {
        if (type == this.configuratorSturucture[key].type && (!this.hideSingleOPVOnlyAlways || !this.configuratorSturucture[key].singleOPVOnly)) {
          var firstOption = this.configuratorSturucture[key].options[Object.keys(this.configuratorSturucture[key].options)[0]];
          this.setActStepByPosNr(firstOption.configSteps[Object.keys(firstOption.configSteps)[0]].posNr);
          return true;
        }
      }
      return false;
    }
  }, {
    key: "navigateToConfigStepByConfigStepKey",
    value: function navigateToConfigStepByConfigStepKey(configStepKey) {
      for (var iKey in this.configuratorSturucture) {
        var tabgroup = this.configuratorSturucture[iKey];
        for (var jkey in tabgroup.options) {
          var optionStep = tabgroup.options[jkey];
          for (var mKey in optionStep.configSteps) {
            var configStep = optionStep.configSteps[mKey];
            if (configStep.key == configStepKey) {
              if (this.stepIsBoundedWithOtherOption(configStep)) {
                var altenativeConfigStep = this.getFirstConfigStepByTabgroup(tabgroup.key);
                var group = this.getConifgStepsToGroup(altenativeConfigStep.key);
                if (altenativeConfigStep && group && group.find(function (c) {
                  return c.key === configStepKey;
                })) {
                  this.setActStepByPosNr(altenativeConfigStep.posNr);
                  return;
                }
              }
              this.setActStepByPosNr(configStep.posNr);
            }
          }
        }
      }
    }
  }, {
    key: "navigateToFirstOption",
    value: function navigateToFirstOption() {
      for (var iKey in this.configuratorSturucture) {
        if (this.configuratorSturucture[iKey].type == 'OPTION') {
          if (this.configuratorSturucture[iKey].posNr != this.curTabGroup.posNr) {
            this.navigateToTabgroupByPosNr(this.configuratorSturucture[iKey].posNr);
          }
          break;
        }
      }
    }
  }, {
    key: "selectArticleInOption",
    value: function selectArticleInOption(basketId) {
      var newActStep = this.getConfigStepInOptionByBasketId(basketId);
      if (newActStep == null) {
        var i = 0,
          allOptions = this.getOptionsOfCurTabGroup();
        while (newActStep == null && i < allOptions.length) {
          var configStep = this.curTabGroup.options[allOptions[i]].configSteps;
          for (var key in configStep) {
            if (configStep[key].targetArticles.includes(basketId)) {
              newActStep = configStep[key];
            }
          }
          i++;
        }
        if (newActStep == null) return false;
      }
      this.setActStepByPosNr(newActStep.posNr);
      return true;
    }
  }, {
    key: "updateConfiguratorStructure",
    value: function updateConfiguratorStructure(configuratorSturucture) {
      var _configuratorStuructu;
      this.configuratorSturucture = configuratorSturucture;
      //Update cur Values
      var newTabGroup = (_configuratorStuructu = configuratorSturucture[this.curTabGroup.key]) !== null && _configuratorStuructu !== void 0 ? _configuratorStuructu : Object.values(configuratorSturucture)[0];
      var oldTabgroup = this.curTabGroup;
      var oldOption = this.curOption;
      this.callExitFunctions(newTabGroup);
      if (this.curTabGroup.type == 'OPTION' || this.curTabGroup.type == 'ACC_PLANNER' || this.curTabGroup.type == 'ACC' || this.curTabGroup.type == 'SHELFPLANNER' || this.curTabGroup.type == 'SUMMARY') {
        this.curTabGroup = newTabGroup;
        var newOption = newTabGroup.options[this.curConfigStep.option];
        if (newOption) {
          this.curOption = newOption;
          var newConfigStep = newOption.configSteps[this.curConfigStep.key];
          if (newConfigStep) this.curConfigStep = newConfigStep;
        }
      }
      this.callEnterFunctions(oldTabgroup, oldOption);
      this.updateView();
    }
  }, {
    key: "setActStepByPosNrFromSummary",
    value: function setActStepByPosNrFromSummary(newActStepPosNr) {
      this.setActStepByPosNr(newActStepPosNr);
    }
  }, {
    key: "getFirstConfigStepByOptionCodex",
    value: function getFirstConfigStepByOptionCodex(optionCodex) {
      for (var iKey in this.configuratorSturucture) {
        for (var jkey in this.configuratorSturucture[iKey].options) {
          var option = this.configuratorSturucture[iKey].options[jkey];
          if (option.codex == optionCodex) {
            return option.configSteps[Object.keys(option.configSteps)[0]];
          }
        }
      }
      return undefined;
    }
  }, {
    key: "getFirstConfigStepByVisibleOptionValueCodex",
    value: function getFirstConfigStepByVisibleOptionValueCodex(optionValueCodex) {
      for (var iKey in this.configuratorSturucture) {
        for (var jkey in this.configuratorSturucture[iKey].options) {
          var option = this.configuratorSturucture[iKey].options[jkey];
          for (var kkey in option.configSteps) {
            var _configStep$optionsta;
            var configStep = option.configSteps[kkey];
            if ((_configStep$optionsta = configStep.optionstate) !== null && _configStep$optionsta !== void 0 && (_configStep$optionsta = _configStep$optionsta.visibleOPVKeys) !== null && _configStep$optionsta !== void 0 && _configStep$optionsta.includes(optionValueCodex)) {
              return configStep;
            }
          }
        }
      }
      return undefined;
    }
  }, {
    key: "getFirstConfigStepByTabgroup",
    value: function getFirstConfigStepByTabgroup(tabGroup) {
      var options = this.configuratorSturucture[tabGroup].options;
      var configsteps = options[Object.keys(options)[0]].configSteps;
      return configsteps[Object.keys(configsteps)[0]];
    }
  }, {
    key: "setActStepByPosNr",
    value: function setActStepByPosNr(newActStepPosNr) {
      var tmp = this.getConfigStepByPosNr(newActStepPosNr);
      if (tmp != null) {
        var newTabGroup = tmp.tabGroup;
        var oldTabgroup = this.curTabGroup;
        var oldOption = this.curOption;
        this.callExitFunctions(newTabGroup);
        this.curTabGroup = tmp.tabGroup;
        this.curOption = tmp.optionStep;
        if (this.curOption.subTabGroup) {
          var _this$tabGroupState2;
          (_this$tabGroupState2 = this.tabGroupState) === null || _this$tabGroupState2 === void 0 || _this$tabGroupState2.setCurrentSubTabGroupFolded(this.curOption.subTabGroup);
        }
        this.curConfigStep = tmp.configStep;
        this.callEnterFunctions(oldTabgroup, oldOption);
        this.updateView();
        return true;
      } else {
        LOG$e.debug('der neue ConfigStep: ' + newActStepPosNr + ' konnte nicht gefunden werden!');
        return false;
      }
    }
  }, {
    key: "callEnterFunctions",
    value: function callEnterFunctions(oldTabGroup, oldOption) {
      var _this$tabGroupState7;
      if ((oldTabGroup === null || oldTabGroup === void 0 ? void 0 : oldTabGroup.type) != 'PLANNER' && this.curTabGroup.type == 'PLANNER' || (oldTabGroup === null || oldTabGroup === void 0 ? void 0 : oldTabGroup.type) != 'PRODUCTS' && this.curTabGroup.type == 'PRODUCTS' || (oldTabGroup === null || oldTabGroup === void 0 ? void 0 : oldTabGroup.type) != 'PROMOTED' && this.curTabGroup.type == 'PROMOTED' || (oldTabGroup === null || oldTabGroup === void 0 ? void 0 : oldTabGroup.type) != 'ACC_PLANNER' && this.curTabGroup.type == 'ACC_PLANNER') {
        //Enter planner Step
        LOG$e.businessEvent('onEnterPlanner', 'Planner', 'Der Planner Schritt wurde betreten');
        this.onEnterPlannerMode();
      } else if ((oldTabGroup === null || oldTabGroup === void 0 ? void 0 : oldTabGroup.type) != 'SUMMARY' && this.curTabGroup.type == 'SUMMARY') {
        var _this$tabGroupState3;
        (_this$tabGroupState3 = this.tabGroupState) === null || _this$tabGroupState3 === void 0 || _this$tabGroupState3.onEnterSummaryMode();
      } else if ((oldTabGroup === null || oldTabGroup === void 0 ? void 0 : oldTabGroup.type) != 'SHELFPLANNER' && this.curTabGroup.type == 'SHELFPLANNER') {
        var _this$tabGroupState4;
        (_this$tabGroupState4 = this.tabGroupState) === null || _this$tabGroupState4 === void 0 || _this$tabGroupState4.onEnterShelfplannerMode();
      } else if ((oldTabGroup === null || oldTabGroup === void 0 ? void 0 : oldTabGroup.type) != 'OPTION' && this.curTabGroup.type == 'OPTION') {
        var _this$tabGroupState5;
        (_this$tabGroupState5 = this.tabGroupState) === null || _this$tabGroupState5 === void 0 || _this$tabGroupState5.onEnterConfiguratorMode();
      } else if ((oldTabGroup === null || oldTabGroup === void 0 ? void 0 : oldTabGroup.type) != 'ACC' && this.curTabGroup.type == 'ACC') {
        var _this$tabGroupState6;
        (_this$tabGroupState6 = this.tabGroupState) === null || _this$tabGroupState6 === void 0 || _this$tabGroupState6.onEnterAccessoriesMode();
      }
      if ((oldOption === null || oldOption === void 0 ? void 0 : oldOption.codex) !== this.curOption.codex) (_this$tabGroupState7 = this.tabGroupState) === null || _this$tabGroupState7 === void 0 || _this$tabGroupState7.navigatedInConfigurator(this.curConfigStepKey);
    }
  }, {
    key: "callExitFunctions",
    value: function callExitFunctions(newTabGroup) {
      if (this.curTabGroup === undefined) return;
      if (this.curTabGroup.type == 'PLANNER' && newTabGroup.type != 'PLANNER' || this.curTabGroup.type == 'PRODUCTS' && newTabGroup.type != 'PRODUCTS' || this.curTabGroup.type == 'PROMOTED' && newTabGroup.type != 'PROMOTED' || this.curTabGroup.type == 'ACC_PLANNER' && newTabGroup.type != 'ACC_PLANNER') {
        var _this$tabGroupState8;
        //Leave planner step
        (_this$tabGroupState8 = this.tabGroupState) === null || _this$tabGroupState8 === void 0 || _this$tabGroupState8.onExitPlannerMode();
      } else if (this.curTabGroup.type == 'SUMMARY' && newTabGroup.type != 'SUMMARY') {
        var _this$tabGroupState9;
        (_this$tabGroupState9 = this.tabGroupState) === null || _this$tabGroupState9 === void 0 || _this$tabGroupState9.onExitSummaryMode();
      } else if (this.curTabGroup.type == 'SHELFPLANNER' && newTabGroup.type != 'SHELFPLANNER') {
        var _this$tabGroupState10;
        (_this$tabGroupState10 = this.tabGroupState) === null || _this$tabGroupState10 === void 0 || _this$tabGroupState10.onExitShelfPlannerMode();
      } else if (this.curTabGroup.type == 'OPTION' && newTabGroup.type != 'OPTION') {
        var _this$tabGroupState11;
        (_this$tabGroupState11 = this.tabGroupState) === null || _this$tabGroupState11 === void 0 || _this$tabGroupState11.onExitConfiguratorMode();
      } else if (this.curTabGroup.type == 'ACC' && newTabGroup.type != 'ACC') {
        var _this$tabGroupState12;
        //EXIT ACCESORIES
        (_this$tabGroupState12 = this.tabGroupState) === null || _this$tabGroupState12 === void 0 || _this$tabGroupState12.onExitAccessoriesMode();
      }
    }
  }, {
    key: "getConfigStepInOptionByBasketId",
    value: function getConfigStepInOptionByBasketId(basketId) {
      for (var key in this.curOption.configSteps) {
        if (this.curOption.configSteps[key].targetArticles.includes(basketId)) {
          return this.curOption.configSteps[key];
        }
      }
      return null;
    }
  }, {
    key: "getTabByPosNr",
    value: function getTabByPosNr(posNr) {
      for (var key in this.configuratorSturucture) {
        if (this.configuratorSturucture[key].posNr == posNr) {
          return this.configuratorSturucture[key];
        }
      }
      return null;
    }
  }, {
    key: "getConfigStepByPosNr",
    value: function getConfigStepByPosNr(posNr) {
      for (var iKey in this.configuratorSturucture) {
        for (var jkey in this.configuratorSturucture[iKey].options) {
          for (var mKey in this.configuratorSturucture[iKey].options[jkey].configSteps) {
            if (this.configuratorSturucture[iKey].options[jkey].configSteps[mKey].posNr == posNr) {
              return {
                tabGroup: this.configuratorSturucture[iKey],
                optionStep: this.configuratorSturucture[iKey].options[jkey],
                configStep: this.configuratorSturucture[iKey].options[jkey].configSteps[mKey]
              };
            }
          }
        }
      }
      return null;
    }
  }, {
    key: "getConfigStepByKey",
    value: function getConfigStepByKey(key) {
      for (var iKey in this.configuratorSturucture) {
        for (var jkey in this.configuratorSturucture[iKey].options) {
          for (var mKey in this.configuratorSturucture[iKey].options[jkey].configSteps) {
            if (this.configuratorSturucture[iKey].options[jkey].configSteps[mKey].key == key) {
              return {
                tabGroup: this.configuratorSturucture[iKey],
                optionStep: this.configuratorSturucture[iKey].options[jkey],
                configStep: this.configuratorSturucture[iKey].options[jkey].configSteps[mKey]
              };
            }
          }
        }
      }
      return null;
    }
  }, {
    key: "getConifgStepsToGroup",
    value: function getConifgStepsToGroup(key, dontSetState) {
      var ret = this.getConfigStepByKey(key);
      if (ret != null) {
        var tabGroup = ret.tabGroup;
        var configStep = ret.configStep;
        var configStepKeyType = configStep.keyType;
        var targetArticles = configStep.targetArticles;
        var partlistArticles = configStep.partlistArticles;
        var groupedConfigSteps = [];
        if (this.navigateStepByStep()) {
          groupedConfigSteps.push(configStep);
        } else {
          for (var iKey in tabGroup.options) {
            var myOption = tabGroup.options[iKey];
            var optionsConfigSteps = void 0;
            if (myOption.mergeToSameOption) {
              optionsConfigSteps = this.mergeOptionsWhoAreTheSame(iKey, tabGroup);
            } else {
              optionsConfigSteps = myOption.configSteps;
            }
            if (myOption.mergesToOPG == undefined || myOption.mergesToOPG != true) {
              for (var lKey in optionsConfigSteps) {
                var myConfigStep = optionsConfigSteps[lKey];
                if (!myConfigStep.key.startsWith('SHELFARTICLE_') || !!myConfigStep.differentTabgroupType || configStep.key === myConfigStep.key) {
                  if (configStepKeyType == 'ALL' && myConfigStep.keyType == 'ALL') {
                    groupedConfigSteps.push(myConfigStep);
                  }
                  if (configStepKeyType == 'BasketId' && myConfigStep.keyType == 'BasketId') {
                    if (JSON.stringify(targetArticles) === JSON.stringify(myConfigStep.targetArticles) || this.isPartlistArtilceOfTargetArticles(targetArticles, myConfigStep, partlistArticles)) {
                      groupedConfigSteps.push(myConfigStep);
                    }
                  }
                }
              }
            }
          }
        }
        if (!dontSetState) {
          var option = this.actGroupedConfigSteps[0].option;
          this.actGroupedConfigSteps = groupedConfigSteps;
          if (this.curTabGroup.options[option]) {
            //tabGroup didnt Change
            for (var j = 0; j < this.actGroupedConfigSteps.length; j++) {
              var doesAlreadyExist = false;
              for (var i = 0; i < this._loadedConfigStepsOfActTabGroup.length; i++) {
                if (this.actGroupedConfigSteps[j].key == this._loadedConfigStepsOfActTabGroup[i].key && this.actGroupedConfigSteps[j].posNr == this._loadedConfigStepsOfActTabGroup[i].posNr) {
                  doesAlreadyExist = true;
                  break;
                }
              }
              if (!doesAlreadyExist) {
                this._loadedConfigStepsOfActTabGroup.push(this.actGroupedConfigSteps[j]);
              }
            }
          } else {
            this._loadedConfigStepsOfActTabGroup = this.actGroupedConfigSteps;
          }
        }
        return groupedConfigSteps;
      }
      return null;
    }
  }, {
    key: "isPartlistArtilceOfTargetArticles",
    value: function isPartlistArtilceOfTargetArticles(targetArticles, configStep, partlistArticles) {
      var _configStep$partlistA;
      return (partlistArticles === null || partlistArticles === void 0 ? void 0 : partlistArticles.length) && !configStep.targetArticles.find(function (p) {
        return !partlistArticles.includes(p);
      }) || ((_configStep$partlistA = configStep.partlistArticles) === null || _configStep$partlistA === void 0 ? void 0 : _configStep$partlistA.length) && !targetArticles.find(function (p) {
        return !configStep.partlistArticles.includes(p);
      });
    }
  }, {
    key: "mergeOptionsWhoAreTheSame",
    value: function mergeOptionsWhoAreTheSame(optionCodex, tabGroup) {
      var optionsToMergeWithCurrentOption = tabGroup.options[optionCodex].mergeToSameOption;
      var ret = {};
      for (var i = 0; i < optionsToMergeWithCurrentOption.length; i++) {
        for (var key in (_tabGroup$options$opt = tabGroup.options[optionsToMergeWithCurrentOption[i]]) === null || _tabGroup$options$opt === void 0 ? void 0 : _tabGroup$options$opt.configSteps) {
          var _tabGroup$options$opt;
          ret[key] = tabGroup.options[optionsToMergeWithCurrentOption[i]].configSteps[key];
        }
      }
      return ret;
    }
  }, {
    key: "getConfigStepsOfTabGroup",
    value: function getConfigStepsOfTabGroup(tabGroup) {
      var configSteps = [];
      for (var iKey in tabGroup.options) {
        for (var jKey in tabGroup.options[iKey].configSteps) {
          configSteps.push(tabGroup.options[iKey].configSteps[jKey]);
        }
      }
      return configSteps;
    }
  }, {
    key: "getArticlesOfConfigStep",
    value: function getArticlesOfConfigStep(key) {
      var ret = this.getConfigStepByKey(key);
      if (ret) {
        return ret.configStep.targetArticles;
      }
      return null;
    }
  }, {
    key: "getTargetArticleOfConfigStepArr",
    value: function getTargetArticleOfConfigStepArr(configSteps) {
      var targetArticles = [];
      for (var i = 0; i < configSteps.length; i++) {
        for (var j = 0; j < configSteps[i].targetArticles.length; j++) {
          if (!targetArticles.includes(configSteps[i].targetArticles[j])) {
            targetArticles.push(configSteps[i].targetArticles[j]);
          }
        }
      }
      return targetArticles;
    }
  }, {
    key: "getAllArticlesOfTabGroup",
    value: function getAllArticlesOfTabGroup(key) {
      var ret = this.getConfigStepByKey(key);
      var allArticlesOfTabGroup = [];
      if (ret != null) {
        var tabGroup = ret.tabGroup;
        for (var iKey in tabGroup.options) {
          var myOption = tabGroup.options[iKey];
          for (var lKey in myOption.configSteps) {
            var myConfigStep = myOption.configSteps[lKey];
            for (var i = 0; i < myConfigStep.targetArticles.length; i++) {
              if (!allArticlesOfTabGroup.includes(myConfigStep.targetArticles[i])) {
                allArticlesOfTabGroup.push(myConfigStep.targetArticles[i]);
              }
            }
          }
        }
      }
      return allArticlesOfTabGroup;
    }
  }, {
    key: "getActiveArticlesOfTabGroup",
    value: function getActiveArticlesOfTabGroup(key, configSingle) {
      var ret = this.getConfigStepByKey(key);
      var allArticlesOfTabGroup = [];
      if (ret != null) {
        var optionStep = ret.optionStep;
        if (optionStep.selectionmode == 2 && configSingle || optionStep.selectionmode == 3) {
          var configStep = ret.configStep;
          allArticlesOfTabGroup = configStep.targetArticles;
        } else {
          allArticlesOfTabGroup = this.getAllArticlesOfTabGroup(key);
        }
      }
      return allArticlesOfTabGroup;
    }
  }, {
    key: "getTargetArticleOfCurConfigStepArr",
    value: function getTargetArticleOfCurConfigStepArr() {
      var curKey = this.curConfigStepKey;
      var groupedConfigSteps = this.getConifgStepsToGroup(curKey);
      if (groupedConfigSteps) return this.getTargetArticleOfConfigStepArr(groupedConfigSteps);
      return undefined;
    }
  }, {
    key: "getTabgroupCount",
    value: function getTabgroupCount() {
      var count = 0;
      for (var key in this.configuratorSturucture) {
        if (key != 'Planner' && key != 'Summary') {
          count++;
        }
      }
      return count;
    }
  }, {
    key: "getCurTabgroupCount",
    value: function getCurTabgroupCount() {
      var count = 0;
      for (var key in this.configuratorSturucture) {
        if (key != 'Planner' && key != 'Summary') {
          count++;
        }
        if (this.curTabGroup.key == key) {
          return count;
        }
      }
      return -1;
    }
  }, {
    key: "getTabgroupByKey",
    value: function getTabgroupByKey(key) {
      return this.configuratorSturucture[key];
    }
  }, {
    key: "allTabGroups",
    get: function get() {
      return this.configuratorSturucture;
    }
  }, {
    key: "currentTabGroup",
    get: function get() {
      return this.curTabGroup;
    }
  }, {
    key: "isFirstStep",
    get: function get() {
      return this.curConfigStep.posNr == 1;
    }
  }, {
    key: "isLastStep",
    get: function get() {
      if (this.getConfigStepByPosNr(this.curConfigStep.posNr + 1) == null) return true;else return false;
    }
  }, {
    key: "actTargetArticles",
    get: function get() {
      return this.curConfigStep.targetArticles;
    }
  }, {
    key: "curConfigStepKey",
    get: function get() {
      return this.curConfigStep.key;
    }
  }, {
    key: "curConfigStepPosNr",
    get: function get() {
      return this.curConfigStep.posNr;
    }
  }, {
    key: "CurConfigStep",
    get: function get() {
      return this.curConfigStep;
    }
  }, {
    key: "curOptionCodex",
    get: function get() {
      return this.curConfigStep.option;
    }
  }, {
    key: "curOCD",
    get: function get() {
      return this.curConfigStep.ocd;
    }
  }, {
    key: "curOptionTitle",
    get: function get() {
      return this.curOption.title;
    }
  }, {
    key: "actGroupedConfigStepsCount",
    get: function get() {
      return this.actGroupedConfigSteps.length;
    }
  }, {
    key: "loadedConfigStepsOfActTabGroup",
    get: function get() {
      return this._loadedConfigStepsOfActTabGroup;
    }
  }, {
    key: "actGroupedOptionsCodexes",
    get: function get() {
      var ret = [];
      for (var i = 0; i < this.actGroupedConfigSteps.length; i++) {
        ret.push(this.actGroupedConfigSteps[i].option);
      }
      return ret;
    }
  }, {
    key: "curTabgroupType",
    get: function get() {
      var _this$curTabGroup;
      return (_this$curTabGroup = this.curTabGroup) === null || _this$curTabGroup === void 0 ? void 0 : _this$curTabGroup.type;
    }
  }, {
    key: "curTabgroupTitle",
    get: function get() {
      var _this$curTabGroup2;
      return (_this$curTabGroup2 = this.curTabGroup) === null || _this$curTabGroup2 === void 0 ? void 0 : _this$curTabGroup2.title;
    }
  }, {
    key: "curTabgroupAlternativeRendering",
    get: function get() {
      var _this$curTabGroup3;
      return (_this$curTabGroup3 = this.curTabGroup) === null || _this$curTabGroup3 === void 0 ? void 0 : _this$curTabGroup3.alternateRenderings;
    }
  }, {
    key: "curTabgroupKey",
    get: function get() {
      return this.curTabGroup.key;
    }

    /*If a step is bounded with another option dont navigate to this step. Navigate to the bounded Step*/
  }, {
    key: "stepIsBoundedWithOtherOption",
    value: function stepIsBoundedWithOtherOption(configStep) {
      return configStep.differentTabgroupType;
    }
  }, {
    key: "updateView",
    value: function updateView() {
      var _this$tabGroupState13;
      (_this$tabGroupState13 = this.tabGroupState) === null || _this$tabGroupState13 === void 0 || _this$tabGroupState13.updateState(this.configuratorSturucture, this.curTabGroup, this.curOption, this.curConfigStep);
    }
  }, {
    key: "lastConfigStep",
    get: function get() {
      var lastTabGroup = undefined;
      for (var key in this.allTabGroups) {
        lastTabGroup = this.allTabGroups[key];
      }
      if (lastTabGroup) {
        var _this$getConfigStepBy8;
        var configSteps = this.getConfigStepsOfTabGroup(lastTabGroup);
        var posNr = configSteps.reduce(function (last, step) {
          return last = last > step.posNr ? last : step.posNr;
        }, 0);
        return (_this$getConfigStepBy8 = this.getConfigStepByPosNr(posNr)) === null || _this$getConfigStepBy8 === void 0 ? void 0 : _this$getConfigStepBy8.configStep;
      }
      return undefined;
    }
  }, {
    key: "configstepsDisplaySeparatOnTabgroup",
    get: function get() {
      var allTabGroups = this.configuratorSturucture;
      var configStepsOnTargetArticle = {};
      for (var iKey in allTabGroups) {
        for (var jKey in allTabGroups[iKey].options) {
          for (var mKey in allTabGroups[iKey].options[jKey].configSteps) {
            var configStep = allTabGroups[iKey].options[jKey].configSteps[mKey];
            if (configStep.displaySeparatOnTabgroup) {
              if (!configStepsOnTargetArticle[configStep.displaySeparatOnTabgroup]) {
                configStepsOnTargetArticle[configStep.displaySeparatOnTabgroup] = {};
              }
              var curTabGroup = configStepsOnTargetArticle[configStep.displaySeparatOnTabgroup];
              if (!curTabGroup[configStep.targetArticles[0]]) {
                curTabGroup[configStep.targetArticles[0]] = [];
              }
              curTabGroup[configStep.targetArticles[0]].push(configStep);
            }
          }
        }
      }
      return configStepsOnTargetArticle;
    }
  }, {
    key: "visitedOptions",
    get: function get() {
      var visitedOptions = {};
      for (var key in this.TabGrousVisitedWithTargetArticles) {
        var tabGroup = this.TabGrousVisitedWithTargetArticles[key];
        for (var basketId in tabGroup) {
          if (!visitedOptions[basketId]) visitedOptions[basketId] = [];
          visitedOptions[basketId].push(key);
          visitedOptions[basketId] = visitedOptions[basketId].concat(tabGroup[basketId]);
        }
      }
      return visitedOptions;
    }
  }, {
    key: "checkIfAnyConfigStepHasWarning",
    value: function checkIfAnyConfigStepHasWarning() {
      for (var iKey in this.allTabGroups) {
        for (var jKey in this.allTabGroups[iKey].options) {
          for (var mKey in this.allTabGroups[iKey].options[jKey].configSteps) {
            if (this.allTabGroups[iKey].options[jKey].configSteps[mKey].warning) return {
              tabgroup: this.allTabGroups[iKey],
              option: this.allTabGroups[iKey].options[jKey],
              configStep: this.allTabGroups[iKey].options[jKey].configSteps[mKey]
            };
          }
        }
      }
      return false;
    }
  }]);
}();

function ownKeys$g(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$g(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$g(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$g(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var PlannerPalette$1 = /*#__PURE__*/function () {
  // selected element to filter for is saved here when the planner hast no yet got all the elements to filter
  function PlannerPalette(settingsState, uiState) {
    _classCallCheck(this, PlannerPalette);
    _defineProperty(this, "allCatalogPlannerArticleItems", void 0);
    _defineProperty(this, "startFilterType", void 0);
    _defineProperty(this, "startHideArticles", void 0);
    _defineProperty(this, "doShowAdditionalPlannerElements", void 0);
    _defineProperty(this, "doShowAdditionalPlannerElementsFirstTime", true);
    _defineProperty(this, "plannerStep", void 0);
    _defineProperty(this, "settingsState", void 0);
    _defineProperty(this, "uiState", void 0);
    _defineProperty(this, "dpIsSelected", false);
    _defineProperty(this, "elementIsSelected", false);
    _defineProperty(this, "selectedElementOnInit", void 0);
    this.settingsState = settingsState;
    this.uiState = uiState;
    this.doShowAdditionalPlannerElements = false;
    this.allCatalogPlannerArticleItems = [];
    this.startFilterType = [];
    this.startHideArticles = [];
    this.plannerStep = {
      filterType: {},
      startFilter: [],
      startHideArticles: [],
      catalogPlannerArticles: [],
      plannerConfiguration: {
        hideAccessoriesInPlanner: false,
        dockAccOnAvoOnly: false,
        showAccOnDpOnly: false,
        ignoreLayerAVC: false,
        ignoreLayerAVOAVU: false,
        ignoreLayerAVNAVS: false
      }
    };
  }
  return _createClass(PlannerPalette, [{
    key: "setCatalogPlannerStep",
    value: function setCatalogPlannerStep(plannerStep) {
      this.plannerStep = plannerStep;
      var plannerFilter = _objectSpread$g({}, this.settingsState.plannerFilter);
      var dockFilter = _objectSpread$g({}, plannerFilter.dockFilter);
      plannerFilter.curFilteredTypes = this.plannerStep.filterType;
      this.startFilterType = plannerStep.startFilter;
      this.startHideArticles = plannerStep.startHideArticles;
      for (var key in plannerFilter.curFilteredTypes) {
        if (plannerFilter.curFilteredTypes[key]) {
          dockFilter[key] = plannerStep.startFilter.includes(key);
        }
      }
      var itemsArr = plannerStep.catalogPlannerArticles;
      for (var i = 0; i < itemsArr.length; i++) {
        var item = itemsArr[i];
        item.sizeInfo = DivaUtils.sizeFormatter(item, this.uiState.i18nInstance);
      }
      this.allCatalogPlannerArticleItems = itemsArr;
      this.settingsState.updatePlannerFilter(_objectSpread$g(_objectSpread$g({}, plannerFilter), {}, {
        dockFilter: dockFilter
      }));
    }
  }, {
    key: "DoShowAdditionalPlannerElements",
    get: function get() {
      return this.doShowAdditionalPlannerElements;
    }
  }, {
    key: "DoShowAdditionalPlannerElementsFirstTime",
    get: function get() {
      if (this.doShowAdditionalPlannerElementsFirstTime) {
        this.doShowAdditionalPlannerElementsFirstTime = false;
        return true;
      }
      return this.doShowAdditionalPlannerElementsFirstTime;
    },
    set: function set(resetBool) {
      this.doShowAdditionalPlannerElementsFirstTime = resetBool;
    }
  }, {
    key: "UpdatePaletteAfterPlanningChanged",
    get: function get() {
      return this.plannerStep.plannerConfiguration.updatePaletteAfterPlanningChanged;
    }
  }, {
    key: "getAllItems",
    value: function getAllItems(paletteId) {
      return this.allCatalogPlannerArticleItems;
    }
  }, {
    key: "doShowFilterTypes",
    value: function doShowFilterTypes() {
      var count = 0;
      for (var key in this.settingsState.plannerFilter.curFilteredTypes) if (this.settingsState.plannerFilter.curFilteredTypes[key]) count++;
      return count > 1;
    }
  }, {
    key: "doShowFilterType",
    value: function doShowFilterType(filterType) {
      return this.settingsState.plannerFilter.curFilteredTypes[filterType];
    }
  }, {
    key: "isFilterActive",
    value: function isFilterActive(filter) {
      return this.settingsState.plannerFilter.dockFilter[filter];
    }

    /**
     * This function checks if the docking Point filter do match.
     * @param filter1
     * @param filter2
     */
  }, {
    key: "checkDPFilterDoMatch",
    value: function checkDPFilterDoMatch(filter1, filter2) {
      if (filter1 == undefined && filter2 == undefined) return true;
      if (filter1 == undefined || filter2 == undefined) return false;
      if (filter1.length == 0 && filter2.length == 0) return true; //Diese überprüfung bin ich mir nicht sicher, dass es so passt.
      if (filter1 && filter2) for (var i = 0; i < filter1.length; i++) {
        for (var j = 0; j < filter2.length; j++) {
          if (filter1[i] == filter2[j]) {
            return true;
          }
        }
      }
      return false;
    }
  }, {
    key: "updateSearchFreeText",
    value: function updateSearchFreeText(searchText) {
      this.clearFilters(true);
      this.settingsState.updatePlannerFilter(_objectSpread$g(_objectSpread$g({}, this.settingsState.plannerFilter), {}, {
        searchText: searchText
      }));
      if (searchText != '') {
        this.doShowAdditionalPlannerElements = false;
      }
    }
  }, {
    key: "changeDockFilter",
    value: function changeDockFilter(filter) {
      this.settingsState.updatePlannerFilter(_objectSpread$g(_objectSpread$g({}, this.settingsState.plannerFilter), {}, {
        dockFilter: _objectSpread$g(_objectSpread$g({}, this.settingsState.plannerFilter.dockFilter), {}, _defineProperty({}, filter, !this.settingsState.plannerFilter.dockFilter[filter]))
      }));
      this.clearFilters(false);
    }
  }, {
    key: "setFilter",
    value: function setFilter(filter, newValue) {
      this.settingsState.updatePlannerFilter(_objectSpread$g(_objectSpread$g({}, this.settingsState.plannerFilter), {}, {
        dockFilter: _objectSpread$g(_objectSpread$g({}, this.settingsState.plannerFilter.dockFilter), {}, _defineProperty({}, filter, newValue))
      }));
    }

    /**
     * This function returns all articles in the list that are matching the plannerElemnts. This depends on the free docking Points and the DockingFilters
     * @param plannerElements
     * @param dp    When a dp is selected
     */
  }, {
    key: "calcMatchingArticles",
    value: function calcMatchingArticles(plannerElements, dp) {
      var _this$selectedElement, _plannerElements;
      if (!plannerElements.length && (_this$selectedElement = this.selectedElementOnInit) !== null && _this$selectedElement !== void 0 && _this$selectedElement.length) {
        plannerElements = this.selectedElementOnInit;
        this.selectedElementOnInit = undefined;
      }
      this.dpIsSelected = !!dp;
      this.doShowAdditionalPlannerElements = false;
      if (!((_plannerElements = plannerElements) !== null && _plannerElements !== void 0 && _plannerElements.length) || this.allCatalogPlannerArticleItems.length == 0 || this.allCatalogPlannerArticleItems == undefined) {
        var _this$allCatalogPlann;
        if (!((_this$allCatalogPlann = this.allCatalogPlannerArticleItems) !== null && _this$allCatalogPlann !== void 0 && _this$allCatalogPlann.length) && plannerElements.length) this.selectedElementOnInit = plannerElements;
        this.elementIsSelected = false;
        this.settingsState.updatePlannerFilter(_objectSpread$g(_objectSpread$g({}, this.settingsState.plannerFilter), {}, {
          validArticleIdsBySelectedElement: undefined,
          validArticleToReplaceSelectedElement: undefined
        }));
        this.clearFilters(true);
        for (var i = 0; i < this.startFilterType.length; i++) {
          var filterType = this.startFilterType[i];
          this.setFilter(filterType, true);
        }
      } else {
        this.elementIsSelected = true;
        this.clearFilters(true);
        var plannerElement = plannerElements[0];
        var articleIds = this.getArticleIdsOfAddPlannerElement(plannerElement, dp);
        var articleIdsToReplace = this.getArticleIdsOfSwitchPlannerElement(plannerElement, dp);
        if (articleIds.length == 0) {
          this.settingsState.updatePlannerFilter(_objectSpread$g(_objectSpread$g({}, this.settingsState.plannerFilter), {}, {
            validArticleIdsBySelectedElement: undefined,
            validArticleToReplaceSelectedElement: articleIdsToReplace
          }));
          this.doShowAdditionalPlannerElements = false;
          for (var i = 0; i < this.startFilterType.length; i++) {
            var filterType = this.startFilterType[i];
            this.setFilter(filterType, true);
          }
        } else {
          this.settingsState.updatePlannerFilter(_objectSpread$g(_objectSpread$g({}, this.settingsState.plannerFilter), {}, {
            validArticleIdsBySelectedElement: articleIds,
            validArticleToReplaceSelectedElement: articleIdsToReplace
          }));
        }
      }
    }
  }, {
    key: "getArticleIdsOfSwitchPlannerElement",
    value: function getArticleIdsOfSwitchPlannerElement(plannerElement, dp) {
      var articleIds = [];
      var curPlannerElementHasAVL = plannerElement.getDockingPointsByType('AVL').length > 0;
      var curPlannerElementHasAVR = plannerElement.getDockingPointsByType('AVR').length > 0;
      for (var i = 0; i < this.allCatalogPlannerArticleItems.length; i++) {
        var plannerArticle = this.allCatalogPlannerArticleItems[i];
        var articleMatch = true;

        //First check if AVL and AVR is the same

        if (curPlannerElementHasAVL == plannerArticle.hasAVL && curPlannerElementHasAVR == plannerArticle.hasAVR) {
          for (var j = 0; j < plannerElement.getDockingDockingPoints().length; j++) {
            var _dockingDP$OnDP;
            if (!articleMatch) break;
            var dockingDP = plannerElement.getDockingDockingPoints()[j];
            var targetDockingElementFilter = (_dockingDP$OnDP = dockingDP.OnDP) === null || _dockingDP$OnDP === void 0 ? void 0 : _dockingDP$OnDP.DPFilter;
            switch (dockingDP.Type) {
              case 'AVL':
                if (plannerArticle.hasAVL) {
                  if (!this.checkDPFilterDoMatch(targetDockingElementFilter, plannerArticle.avlFilter)) {
                    articleMatch = false;
                  }
                } else {
                  articleMatch = false;
                }
                break;
              case 'AVR':
                if (plannerArticle.hasAVR) {
                  if (!this.checkDPFilterDoMatch(targetDockingElementFilter, plannerArticle.avrFilter)) {
                    articleMatch = false;
                  }
                } else {
                  articleMatch = false;
                }
                break;
              case 'AVO':
                if (!plannerArticle.hasAVO) articleMatch = false;
                break;
              case 'AVU':
                if (!plannerArticle.hasAVU) articleMatch = false;
                break;
              case 'AVC':
                if (!plannerArticle.hasAVC) articleMatch = false;
                break;
              case 'AVN':
                if (!plannerArticle.hasAVN) articleMatch = false;
                break;
              case 'AVS':
                if (!plannerArticle.hasAVS) articleMatch = false;
                break;
            }
          }
          if (articleMatch) {
            articleIds.push(plannerArticle.id);
          }
        } else {
          articleMatch = false;
          continue;
        }
      }
      if (!plannerElement.getFreeDockingPointsByType('AVL').length && !plannerElement.getFreeDockingPointsByType('AVR').length && dp == null && !plannerElement.getIsDockedAccessoire()) {
        this.doShowAdditionalPlannerElements = true;
      }
      return articleIds;
    }
  }, {
    key: "getArticleIdsOfAddPlannerElement",
    value: function getArticleIdsOfAddPlannerElement(plannerElement, dpData) {
      var articleIds = [];
      var doShowFilterTypes = {};
      var hasFreeAVL = false;
      var hasFreeAVR = false;
      var hasFreeAVO = false;
      var hasFreeAVU = false;
      var hasFreeAVC = false;
      var hasFreeAVN = false;
      var hasFreeAVS = false;
      var dp = undefined;
      if ((dpData === null || dpData === void 0 ? void 0 : dpData.ElementId) === plannerElement.getElementId()) {
        dp = dpData;
      }
      if (dp != null) {
        switch (dp.Type) {
          case 'AVL':
            hasFreeAVL = true;
            break;
          case 'AVR':
            hasFreeAVR = true;
            break;
          case 'AVO':
            hasFreeAVO = true;
            break;
          case 'AVU':
            hasFreeAVU = true;
            break;
          case 'AVC':
            hasFreeAVC = true;
            break;
          case 'AVN':
            hasFreeAVN = true;
            break;
          case 'AVS':
            hasFreeAVS = true;
            break;
        }
      } else {
        hasFreeAVL = plannerElement.getFreeDockingPointsByType('AVL').length > 0;
        hasFreeAVR = plannerElement.getFreeDockingPointsByType('AVR').length > 0;
        hasFreeAVO = plannerElement.getFreeDockingPointsByType('AVO').length > 0;
        hasFreeAVU = plannerElement.getFreeDockingPointsByType('AVU').length > 0;
        hasFreeAVN = plannerElement.getFreeDockingPointsByType('AVN').length > 0;
        hasFreeAVS = plannerElement.getFreeDockingPointsByType('AVS').length > 0;
        hasFreeAVC = plannerElement.getFreeDockingPointsByType('AVC').length > 0;
      }
      if (!hasFreeAVL && !hasFreeAVR && dp == null && !plannerElement.getIsDockedAccessoire()) {
        this.doShowAdditionalPlannerElements = true;
      }
      for (var i = 0; i < this.allCatalogPlannerArticleItems.length; i++) {
        var plannerArticle = this.allCatalogPlannerArticleItems[i];
        var articleMatch = false;
        if (hasFreeAVL && plannerArticle.hasAVR) {
          articleMatch = this.checkDPFilterDoMatch(plannerElement.getFilterAVL(), plannerArticle.avrFilter);
        }
        if (!articleMatch && hasFreeAVR && plannerArticle.hasAVL) {
          articleMatch = this.checkDPFilterDoMatch(plannerElement.getFilterAVR(), plannerArticle.avlFilter);
        }
        if (!articleMatch && (hasFreeAVO && plannerArticle.hasAVU || hasFreeAVU && plannerArticle.hasAVO && !this.plannerStep.plannerConfiguration.dockAccOnAvoOnly || hasFreeAVN && plannerArticle.hasAVS || hasFreeAVS && plannerArticle.hasAVN || hasFreeAVC && plannerArticle.hasAVC)) {
          var doCheckLayer = !this.plannerStep.plannerConfiguration.ignoreLayerAVC && !this.plannerStep.plannerConfiguration.ignoreLayerAVOAVU && !this.plannerStep.plannerConfiguration.ignoreLayerAVNAVS;
          if (this.plannerStep.plannerConfiguration.ignoreLayerAVC && hasFreeAVC && plannerArticle.hasAVC || this.plannerStep.plannerConfiguration.ignoreLayerAVOAVU && (hasFreeAVO && plannerArticle.hasAVU || hasFreeAVU && plannerArticle.hasAVO) || this.plannerStep.plannerConfiguration.ignoreLayerAVNAVS && (hasFreeAVN && plannerArticle.hasAVS || hasFreeAVS && plannerArticle.hasAVN)) {
            doCheckLayer = false;
          }
          if (plannerElement && (!doCheckLayer || plannerElement.getLayer() != plannerArticle.layer)) {
            if (plannerArticle.filterType != 'Accessoire' || plannerElement.getAccessoires().includes(plannerArticle.articleCodex)) {
              if (!this.plannerStep.plannerConfiguration.showAccOnDpOnly || plannerArticle.filterType == 'Accessoire' || dp === undefined) articleMatch = true;
            }
          }
        }
        if (articleMatch || this.plannerStep.showAllItemsInPlanner) {
          //Jetzt überprüfen ob die Filter matchen
          articleIds.push(plannerArticle.id);
          if (!doShowFilterTypes[plannerArticle.filterType]) {
            doShowFilterTypes[plannerArticle.filterType] = true;
          }
        }
      }
      for (var key in this.settingsState.plannerFilter.dockFilter) {
        if (!doShowFilterTypes[key]) {
          this.setFilter(key, false);
        } else {
          this.setFilter(key, true);
        }
      }
      var accessoires = plannerElement.getCurrentAccessorires();
      for (var _i = 0; _i < accessoires.length; _i++) {
        articleIds = articleIds.concat(this.getArticleIdsOfAddPlannerElement(accessoires[_i], dp));
      }
      return articleIds;
    }
  }, {
    key: "clearFilters",
    value: function clearFilters(doClearDockFilter) {
      var plannerFilter = this.settingsState.plannerFilter;
      if (doClearDockFilter) {
        for (var key in this.settingsState.plannerFilter.dockFilter) {
          this.settingsState.updatePlannerFilter(_objectSpread$g(_objectSpread$g({}, plannerFilter), {}, {
            dockFilter: _objectSpread$g(_objectSpread$g({}, plannerFilter.dockFilter), {}, _defineProperty({}, key, false))
          }));
        }
      }
      plannerFilter.validArticleIdsBySelectedElement = undefined;
      plannerFilter.validArticleToReplaceSelectedElement = undefined;
      plannerFilter.searchText = '';
      this.settingsState.updatePlannerFilter(plannerFilter);
    }
  }, {
    key: "calcFilter",
    value: function calcFilter(paletteId) {
      var siftFilter = {};

      // filter the matching Articles tovalidArticleIdsBySelectedElement the current selected Element in the Planner (depend on its Free Dockingpoints)

      var artCodexArr;
      if (this.settingsState.plannerReplaceMode) {
        if (this.doShowAdditionalPlannerElements && paletteId == 1) {
          artCodexArr = this.settingsState.plannerFilter.validArticleIdsBySelectedElement;
        } else {
          artCodexArr = this.settingsState.plannerFilter.validArticleToReplaceSelectedElement;
        }
      } else {
        artCodexArr = this.settingsState.plannerFilter.validArticleIdsBySelectedElement;
      }
      var siftIdFilters = [];
      var continuePlanning = paletteId == 2 && !this.settingsState.plannerReplaceMode;
      if (this.settingsState.plannerFilter.searchText != '') {
        siftFilter = this.searchFilter();
      } else if (continuePlanning) {
        var _this$getAllItems;
        siftFilter = (_this$getAllItems = this.getAllItems(paletteId)) !== null && _this$getAllItems !== void 0 && _this$getAllItems.find(function (i) {
          return ['DockL', 'DockNone'].includes(i.filterType);
        }) ? {
          $or: [{
            filterType: {
              $eq: 'DockNone'
            }
          }, {
            filterType: {
              $eq: 'DockL'
            }
          }]
        } : {};
      } else if (artCodexArr != undefined) {
        //Do filter for article codex

        if (artCodexArr.length == 0) {
          //Keine Elemente können mehr angebaut werden!
          this.doShowAdditionalPlannerElements = true;
          siftIdFilters.push({
            id: {
              $eq: 'EMPTY'
            }
          });
        } else {
          for (var i = 0; i < artCodexArr.length; i++) {
            siftIdFilters.push({
              id: {
                $eq: artCodexArr[i]
              }
            });
          }
        }
        if (siftIdFilters.length) siftFilter = {
          $or: siftIdFilters
        };
      } else {
        var _this$startHideArticl, _this$settingsState$a;
        for (var key in this.settingsState.plannerFilter.dockFilter) {
          if (this.settingsState.plannerFilter.dockFilter[key]) {
            siftIdFilters.push({
              filterType: {
                $eq: key
              }
            });
          }
        }
        var andHideArticle = [];
        //only hide startHideArticles if planner is empty
        if ((_this$startHideArticl = this.startHideArticles) !== null && _this$startHideArticl !== void 0 && _this$startHideArticl.length && Object.keys((_this$settingsState$a = this.settingsState.articleInfos) !== null && _this$settingsState$a !== void 0 ? _this$settingsState$a : {}).length == 0) {
          for (var _i2 = 0; _i2 < this.startHideArticles.length; _i2++) {
            andHideArticle.push({
              articleCodex: {
                $ne: this.startHideArticles[_i2]
              }
            });
          }
          siftIdFilters = [{
            $and: [{
              $or: siftIdFilters
            }, {
              $and: andHideArticle
            }]
          }];
        }
        if (siftIdFilters.length !== 0) {
          siftFilter = {
            $or: siftIdFilters
          };
        } else {
          siftFilter = undefined;
        }
      }
      return this.addShowAccOnDpOnlyFilter(siftFilter);
    }
  }, {
    key: "searchFilter",
    value: function searchFilter() {
      this.doShowAdditionalPlannerElements = false;
      var siftOrFilter = [];
      var contains = function contains(filterParam) {
        return ".*".concat(filterParam, ".*");
      };
      var $options = 'i';
      var searchtext = this.settingsState.plannerFilter.searchText;
      searchtext = searchtext.replace(/[\(\)]/g, '\\$&');
      var sRx = {
        $regex: contains(searchtext),
        $options: $options
      };
      siftOrFilter = [{
        articleCodex: sRx
      }, {
        title: sRx
      }, {
        typeNr: sRx
      }, {
        groupName: sRx
      }, {
        partlistcodexes: sRx
      }];
      return {
        $or: siftOrFilter
      };
    }
  }, {
    key: "addShowAccOnDpOnlyFilter",
    value: function addShowAccOnDpOnlyFilter(siftFilter) {
      if (this.plannerStep.plannerConfiguration.showAccOnDpOnly && !this.dpIsSelected && !this.elementIsSelected || this.plannerStep.plannerConfiguration.hideAccessoriesInPlanner) {
        var ret = {
          $and: [{
            $or: [{
              filterType: {
                $ne: 'Accessoire'
              }
            }]
          }]
        };
        if (siftFilter) ret.$and.push(siftFilter);
        return ret;
      }
      return siftFilter;
    }
  }]);
}();

getLogger('Planner', 'ConfigItems');
var ConfigItems = /*#__PURE__*/function () {
  function ConfigItems(i) {
    _classCallCheck(this, ConfigItems);
    //private actDockFilter?: string;
    _defineProperty(this, "configItems", void 0);
    _defineProperty(this, "actColorFilterText", void 0);
    _defineProperty(this, "actSearchFitlerText", void 0);
    _defineProperty(this, "defaultFilterText", void 0);
    _defineProperty(this, "isRendered", void 0);
    _defineProperty(this, "configStepKey", void 0);
    this.configItems = [];
    this.isRendered = false;
    this.configStepKey = '';
  }
  return _createClass(ConfigItems, [{
    key: "IsRendered",
    get: function get() {
      return this.isRendered;
    },
    set: function set(isRendered) {
      this.isRendered = isRendered;
    }
  }, {
    key: "ConfigStepKey",
    get: function get() {
      return this.configStepKey;
    },
    set: function set(configStepKey) {
      this.configStepKey = configStepKey;
    }
  }, {
    key: "setConfigItems",
    value: function setConfigItems(itemsArr) {
      this.configItems = itemsArr;
    }
  }, {
    key: "getAllItems",
    value: function getAllItems(paletteId) {
      return this.configItems;
    }
  }, {
    key: "updateSearchFreeText",
    value: function updateSearchFreeText(searchText) {
      this.clearFilters();
      this.actSearchFitlerText = searchText;
    }
  }, {
    key: "updateColorFilterText",
    value: function updateColorFilterText(codex) {
      this.clearFilters();
      this.getActFilterItems('RAL', codex);
    }
  }, {
    key: "clearFilters",
    value: function clearFilters() {
      this.actSearchFitlerText = undefined;
      this.actColorFilterText = undefined;
    }
  }, {
    key: "getActFilterItems",
    value: function getActFilterItems(filterType, actColorFilterText) {
      if (filterType == 'RAL') {
        this.defaultFilterText = 'Farben';
      }
      if (actColorFilterText) {
        this.actColorFilterText = actColorFilterText;
      } else {
        this.actColorFilterText = this.defaultFilterText;
      }
    }
  }, {
    key: "calcFilter",
    value: function calcFilter(paletteId) {
      var siftFilter = {};
      var contains = function contains(f) {
        return ".*".concat(f, ".*");
      };
      var $options = 'i';
      if (this.actSearchFitlerText) {
        var siftOrFilter = [];
        siftOrFilter.push({
          codex: {
            $regex: contains(this.actSearchFitlerText),
            $options: $options
          }
        });
        siftOrFilter.push({
          codex: {
            $regex: contains(this.actSearchFitlerText.replace(/-/g, '_')),
            $options: $options
          }
        });
        siftOrFilter.push({
          text: {
            $regex: contains(this.actSearchFitlerText),
            $options: $options
          }
        });
        siftOrFilter.push({
          groupName: {
            $regex: contains(this.actSearchFitlerText),
            $options: $options
          }
        });
        siftOrFilter.push({
          longDescription: {
            $regex: contains(this.actSearchFitlerText),
            $options: $options
          }
        });
        siftOrFilter.push({
          vCodex: {
            $regex: contains(this.actSearchFitlerText),
            $options: $options
          }
        });
        siftOrFilter.push({
          ocd: {
            $regex: contains(this.actSearchFitlerText),
            $options: $options
          }
        });
        siftFilter = {
          $or: siftOrFilter
        };
      } else if (this.actColorFilterText && this.actColorFilterText != 'Farben') {
        var _siftOrFilter = [];
        _siftOrFilter.push({
          colorGroups1: {
            $regex: contains(this.actColorFilterText),
            $options: $options
          }
        });
        _siftOrFilter.push({
          colorGroups2: {
            $regex: contains(this.actColorFilterText),
            $options: $options
          }
        });
        _siftOrFilter.push({
          colorGroups3: {
            $regex: contains(this.actColorFilterText),
            $options: $options
          }
        });
        siftFilter = {
          $or: _siftOrFilter
        };
      } else return undefined;
      return siftFilter;
    }
  }]);
}();

var AccItems = /*#__PURE__*/function () {
  function AccItems(uiState) {
    _classCallCheck(this, AccItems);
    _defineProperty(this, "accItems", void 0);
    _defineProperty(this, "uiState", void 0);
    this.accItems = [];
    this.uiState = uiState;
  }
  return _createClass(AccItems, [{
    key: "setAccItems",
    value: function setAccItems(accessoires) {
      for (var i = 0; i < accessoires.length; i++) {
        accessoires[i].formattedSize = DivaUtils.sizeFormatter(accessoires[i], this.uiState.i18nInstance);
      }
      this.accItems = accessoires;
    }
  }, {
    key: "thereArePlannableAcc",
    get: function get() {
      var ret = false;
      for (var i = 0; i < this.accItems.length; i++) {
        if (this.accItems[i].isPlannerElement) {
          ret = true;
          break;
        }
      }
      return ret;
    }
  }, {
    key: "thereAreNotPlannableAcc",
    get: function get() {
      var ret = false;
      for (var i = 0; i < this.accItems.length; i++) {
        if (!this.accItems[i].isPlannerElement) {
          ret = true;
          break;
        }
      }
      return ret;
    }
  }, {
    key: "getAllItems",
    value: function getAllItems(paletteId) {
      return this.accItems;
    }
  }]);
}();

var PlannerPalette = /*#__PURE__*/function () {
  function PlannerPalette() {
    _classCallCheck(this, PlannerPalette);
    _defineProperty(this, "summaryTabItems", void 0);
    _defineProperty(this, "articlePriceFormat", void 0);
    this.summaryTabItems = {};
  }
  return _createClass(PlannerPalette, [{
    key: "setSummaryItems",
    value: function setSummaryItems(summaryTabItems, articlePriceFormat) {
      this.articlePriceFormat = articlePriceFormat;
      this.summaryTabItems = summaryTabItems;
    }
  }, {
    key: "getAllItems",
    value: function getAllItems(paletteId) {
      var ret = [];
      for (var iKey in this.summaryTabItems) {
        ret.push(this.summaryTabItems[iKey]);
        if (this.summaryTabItems[iKey].summaryItems) for (var lKey in this.summaryTabItems[iKey].summaryItems) {
          ret.push(this.summaryTabItems[iKey].summaryItems[lKey]);
        }
      }
      return ret;
    }
  }]);
}();

function ownKeys$f(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$f(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$f(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$f(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$d = getLogger('Planner', 'VisitStepsHandler');
var VisitStepsHandler = /*#__PURE__*/function () {
  function VisitStepsHandler(configuratorStructureHandler, settingsState, getState, defineArticlesOnOptions) {
    _classCallCheck(this, VisitStepsHandler);
    _defineProperty(this, "configuratorStructureHandler", void 0);
    // private configSteps: ConfigItems[];
    _defineProperty(this, "settingsState", void 0);
    _defineProperty(this, "getState", void 0);
    _defineProperty(this, "defineArticlesOnOptions", void 0);
    this.configuratorStructureHandler = configuratorStructureHandler;
    this.settingsState = settingsState;
    // this.configSteps = configSteps;
    this.getState = getState;
    this.defineArticlesOnOptions = defineArticlesOnOptions;
  }

  /**
   * this function sets a Variable called Visited for all the act Visible Steps in the DOM. This information is used know if we have to scroll to a not visited Step or can actually jump to the next tabGroup
   */
  return _createClass(VisitStepsHandler, [{
    key: "visitActVisibleSteps",
    value: function visitActVisibleSteps() {
      var _this = this;
      if (this.configuratorStructureHandler.currentTabGroup == undefined) return;
      var curTabGroupKey = this.configuratorStructureHandler.curTabgroupKey,
        actArtilce;
      switch (this.getState()) {
        case 'OPTION':
        case 'ACC':
        case 'ACC_PLANNER':
        case 'SHELFPLANNER':
          {
            this.settingsState.itemPalettes.forEach(function (item) {
              var _item$selectedElement;
              var step = _this.configuratorStructureHandler.getConfigStepByKey(item.configStepKey);
              if ((_item$selectedElement = item.selectedElements) !== null && _item$selectedElement !== void 0 && (_item$selectedElement = _item$selectedElement[0]) !== null && _item$selectedElement !== void 0 && _item$selectedElement.isFreeText && !(step !== null && step !== void 0 && step.configStep.isDefined)) return;
              if (item.htmlRef && !item.isVisited) {
                var _this$settingsState$s, _this$settingsState$s2, _this$settingsState$s3, _this$settingsState$s4, _this$settingsState$s5, _this$settingsState$s6;
                var position = item.htmlRef.getBoundingClientRect().top - ((_this$settingsState$s = (_this$settingsState$s2 = _this.settingsState.settingsScrollerElement) === null || _this$settingsState$s2 === void 0 ? void 0 : _this$settingsState$s2.getBoundingClientRect().top) !== null && _this$settingsState$s !== void 0 ? _this$settingsState$s : 0);
                var settingsscrollerHeight = (_this$settingsState$s3 = (_this$settingsState$s4 = _this.settingsState.settingsScrollerElement) === null || _this$settingsState$s4 === void 0 ? void 0 : _this$settingsState$s4.getBoundingClientRect().height) !== null && _this$settingsState$s3 !== void 0 ? _this$settingsState$s3 : 0;
                var settingsHeaderHeight = (_this$settingsState$s5 = (_this$settingsState$s6 = _this.settingsState.settingsHeaderElement) === null || _this$settingsState$s6 === void 0 ? void 0 : _this$settingsState$s6.getBoundingClientRect().height) !== null && _this$settingsState$s5 !== void 0 ? _this$settingsState$s5 : 0;
                if (position + 70 + 30 /* 30 px extra so you are forced to see at least a little bit of the first option */ < settingsscrollerHeight + settingsHeaderHeight) {
                  _this.settingsState.itemPalettes.set(item.id, _objectSpread$f(_objectSpread$f({}, item), {}, {
                    isVisited: true
                  }));
                  if (_this.wereAllStepsVisited()) {
                    //all steps were Visited -> Tabgroup Visited
                    var allArticleOfTabGroup = _this.configuratorStructureHandler.getTargetArticleOfCurConfigStepArr();
                    var options = _this.settingsState.itemPaletteArr.map(function (item) {
                      return item[1].optionCodex;
                    });
                    if (allArticleOfTabGroup && options) {
                      _this.configuratorStructureHandler.addTabGrousVisitedWithTargetArticlesAndOptions(curTabGroupKey, allArticleOfTabGroup, options);
                    }
                  }
                }
              } else if (item.isVisited) {
                _this.defineOptionOfConfigStepKey(item.configStepKey);
              }
            });
            if (this.getState() === 'SHELFPLANNER' || this.getState() === 'OPTION') break;
          }
        case 'SUMMARY':
          actArtilce = this.configuratorStructureHandler.getTargetArticleOfCurConfigStepArr();
          if (actArtilce) this.configuratorStructureHandler.addTabGrousVisitedWithTargetArticlesAndOptions(curTabGroupKey, actArtilce, [this.getState()]);
          break;
        case 'PLANNER':
        case 'PRODUCTS':
        case 'PROMOTED':
        case 'STATICROOMLIST':
          this.configuratorStructureHandler.addTabGrousVisitedWithTargetArticlesAndOptions(curTabGroupKey, ['ALL'], [this.getState()]);
      }
    }
  }, {
    key: "defineOptionOfConfigStepKey",
    value: function defineOptionOfConfigStepKey(configStepKey) {
      var tabGroup = this.configuratorStructureHandler.getConfigStepByKey(configStepKey);
      if (tabGroup) {
        var basketIdsOnOptions = {};
        for (var i = 0; i < tabGroup.configStep.targetArticles.length; i++) {
          basketIdsOnOptions[tabGroup.configStep.targetArticles[i]] = [tabGroup.configStep.option];
        }
        this.defineArticlesOnOptions(basketIdsOnOptions);
      }
    }
    /**
     * this function checks if there are steps not already visited
     */
  }, {
    key: "wereAllStepsVisited",
    value: function wereAllStepsVisited() {
      var curConfigStepKey = this.configuratorStructureHandler.curConfigStepKey;
      var currentItem = this.settingsState.itemPaletteArr.find(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2);
          _ref2[0];
          var item = _ref2[1];
        return curConfigStepKey === item.configStepKey;
      });
      var subTabGroup = currentItem === null || currentItem === void 0 ? void 0 : currentItem[1].subTabGroup;
      var noVisitedStep = this.settingsState.itemPaletteArr.find(function (_ref3) {
        var _ref4 = _slicedToArray(_ref3, 2);
          _ref4[0];
          var item = _ref4[1];
        return !item.isVisited && item.visible && !item.isRenderedNotInSettingPalette && (!subTabGroup || item.subTabGroup === subTabGroup);
      });
      LOG$d.debug('noVisitedStep: ', noVisitedStep);
      return !noVisitedStep;
    }
  }, {
    key: "getFirstUnvisitedStepIndex",
    value: function getFirstUnvisitedStepIndex() {
      return this.settingsState.itemPaletteArr.findIndex(function (_ref5) {
        var _ref6 = _slicedToArray(_ref5, 2);
          _ref6[0];
          var item = _ref6[1];
        return !item.isVisited && item.visible;
      });
    }
  }]);
}();

var StaticRoomItems = /*#__PURE__*/function () {
  function StaticRoomItems() {
    _classCallCheck(this, StaticRoomItems);
    _defineProperty(this, "staticRoomItems", void 0);
    _defineProperty(this, "actRoomItem", void 0);
    this.actRoomItem = undefined;
    this.staticRoomItems = [];
  }
  return _createClass(StaticRoomItems, [{
    key: "StaticRoomItems",
    set: function set(newVal) {
      this.staticRoomItems = newVal;
      if (this.staticRoomItems.length > 0) {
        for (var i = 0; i < this.staticRoomItems.length; i++) {
          if (this.staticRoomItems[i].sel) {
            this.actRoomItem = this.staticRoomItems[i];
            break;
          }
        }
      }
    }
  }, {
    key: "getAllItems",
    value: function getAllItems(paletteId) {
      if (paletteId == 1) {
        return this.staticRoomItems;
      } else {
        if (this.actRoomItem) {
          return this.actRoomItem.VP;
        }
        return [];
      }
    }
  }]);
}();

var ShelfItem = /*#__PURE__*/function () {
  function ShelfItem() {
    _classCallCheck(this, ShelfItem);
    _defineProperty(this, "shelfAccessoiresItems", []);
    _defineProperty(this, "grids", void 0);
    _defineProperty(this, "currentPaletteId", 1);
    _defineProperty(this, "actSearchFitlerText", void 0);
  }
  return _createClass(ShelfItem, [{
    key: "AccItems",
    set: function set(newVal) {
      this.shelfAccessoiresItems = newVal;
    }
  }, {
    key: "Grids",
    get: function get() {
      return this.grids;
    },
    set: function set(grids) {
      this.grids = grids;
    }
  }, {
    key: "getAllItems",
    value: function getAllItems(paletteId) {
      return this.shelfAccessoiresItems;
    }
  }, {
    key: "getActSearchFitlerText",
    value: function getActSearchFitlerText() {
      return this.actSearchFitlerText;
    }
  }, {
    key: "updateSearchFreeText",
    value: function updateSearchFreeText(searchText) {
      this.clearFilters();
      this.actSearchFitlerText = searchText;
    }
  }, {
    key: "clearFilters",
    value: function clearFilters() {
      this.actSearchFitlerText = undefined;
    }
  }, {
    key: "calcFilter",
    value: function calcFilter(paletteId) {
      var siftFilter = {};
      var contains = function contains(f) {
        return ".*".concat(f, ".*");
      };
      var $options = 'i';
      if (this.actSearchFitlerText) {
        var siftOrFilter = [{
          articleCodex: {
            $regex: contains(this.actSearchFitlerText),
            $options: $options
          }
        }, {
          articleCodex: {
            $regex: contains(this.actSearchFitlerText.replace(/-/g, '_')),
            $options: $options
          }
        }, {
          name: {
            $regex: contains(this.actSearchFitlerText),
            $options: $options
          }
        }, {
          partlistcodexes: {
            $regex: contains(this.actSearchFitlerText),
            $options: $options
          }
        }];
        siftFilter = {
          $or: siftOrFilter
        };
      } else return undefined;
      return siftFilter;
    }
  }]);
}();

var SpecialItem = /*#__PURE__*/function () {
  function SpecialItem() {
    _classCallCheck(this, SpecialItem);
    _defineProperty(this, "specialItems", []);
  }
  return _createClass(SpecialItem, [{
    key: "AccItems",
    set: function set(newVal) {
      this.specialItems = newVal;
    }
  }, {
    key: "getAllItems",
    value: function getAllItems(paletteId) {
      return this.specialItems;
    }
  }]);
}();

function _createForOfIteratorHelper$2(r, e) { var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (!t) { if (Array.isArray(r) || (t = _unsupportedIterableToArray$2(r)) || e && r && "number" == typeof r.length) { t && (r = t); var _n = 0, F = function F() {}; return { s: F, n: function n() { return _n >= r.length ? { done: !0 } : { done: !1, value: r[_n++] }; }, e: function e(r) { throw r; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var o, a = !0, u = !1; return { s: function s() { t = t.call(r); }, n: function n() { var r = t.next(); return a = r.done, r; }, e: function e(r) { u = !0, o = r; }, f: function f() { try { a || null == t["return"] || t["return"](); } finally { if (u) throw o; } } }; }
function _unsupportedIterableToArray$2(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray$2(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray$2(r, a) : void 0; } }
function _arrayLikeToArray$2(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
function ownKeys$e(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$e(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$e(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$e(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$c = getLogger('Planner', 'PaletteViewHandler');
var PaletteViewHandler = /*#__PURE__*/function () {
  function PaletteViewHandler(settingsState, productState, uiState, configuratorStructureHandler, getState, updateConfiguratorState, getSelectedElementsArr, getSelectedDockingPoint, updateElementSelectionPlanner, configSingleIsChanged, addPlannerElement, deletePlannerElement, switchPlannerElementLogic, copyPlannerElementLogic, deleteAccessoire, updateAccessoireQuantity, doConfigure, setCurConfigStepKey, setActSet, doAllowSetPlanner, handle3DVisualizationChanged, loadCatalogPlannerArticles, loadAccessoiresOfArticles, loadConfigItemsOfOptions, loadSummaryItems, loadStaticRoomList, loadShelfPlannerItemsLogic, loadDoorPlannerItemsLogic, selectShelfAccArticle, getFreePlacesOnGrids, addAccessorieOnShelfPlanner, moveShelfItem, moveDoors, deleteAccessorieOnShelfPlanner, getBasketIdsOfElementIdLogic, loadPaletteInfos, changeOnFavorites, defineOptions, onClickConfiguratorSettingsForth, getPlanningIsInCompleteOrEmpty, checkNotificationRules, checkBannerRules, actions) {
    var _this = this;
    _classCallCheck(this, PaletteViewHandler);
    _defineProperty(this, "settingsState", void 0);
    _defineProperty(this, "productState", void 0);
    _defineProperty(this, "uiState", void 0);
    _defineProperty(this, "configuratorStructureHandler", void 0);
    _defineProperty(this, "plannerItems", void 0);
    _defineProperty(this, "shelfItem", void 0);
    _defineProperty(this, "specialItem", void 0);
    _defineProperty(this, "accItems", void 0);
    _defineProperty(this, "configSteps", []);
    _defineProperty(this, "summaryItems", []);
    _defineProperty(this, "staticRoomItems", void 0);
    _defineProperty(this, "autoOpenedContentItems", {});
    _defineProperty(this, "visitStepsHandler", void 0);
    _defineProperty(this, "paletteWasLoadedFrom", void 0);
    _defineProperty(this, "notTotallyMatichingOPVs", void 0);
    _defineProperty(this, "filtersToCalc", {});
    //for remembering to calc the filters when the palette is not jet ready
    //functions
    _defineProperty(this, "getState", void 0);
    _defineProperty(this, "updateConfiguratorState", void 0);
    _defineProperty(this, "updatePlannerMsgContainer", void 0);
    _defineProperty(this, "getSelectedElementsArr", void 0);
    _defineProperty(this, "getSelectedDockingPoint", void 0);
    _defineProperty(this, "updateElementSelectionPlanner", void 0);
    //function in Logic!
    _defineProperty(this, "configSingleIsChangedLogic", void 0);
    _defineProperty(this, "addPlannerElementLogic", void 0);
    _defineProperty(this, "deletePlannerElementLogic", void 0);
    _defineProperty(this, "switchPlannerElementLogic", void 0);
    _defineProperty(this, "copyPlannerElementLogic", void 0);
    _defineProperty(this, "onCalculating", false);
    _defineProperty(this, "deleteAccessoireLogic", void 0);
    _defineProperty(this, "updateAccessoireQuantityLogic", void 0);
    _defineProperty(this, "doConfigureLogic", void 0);
    _defineProperty(this, "setCurConfigStepKey", void 0);
    _defineProperty(this, "setActSet", void 0);
    _defineProperty(this, "doAllowSetPlanner", void 0);
    _defineProperty(this, "handle3DVisualizationChanged", void 0);
    _defineProperty(this, "loadCatalogPlannerArticlesLogic", void 0);
    _defineProperty(this, "loadAccessoiresOfArticlesLogic", void 0);
    _defineProperty(this, "loadConfigItemsOfOptionsLogic", void 0);
    _defineProperty(this, "loadSummaryItemsLogic", void 0);
    _defineProperty(this, "loadStaticRoomList", void 0);
    _defineProperty(this, "loadShelfPlannerItemsLogic", void 0);
    _defineProperty(this, "loadDoorPlannerItemsLogic", void 0);
    _defineProperty(this, "selectShelfAccArticle", void 0);
    _defineProperty(this, "getFreePlacesOnGrids", void 0);
    _defineProperty(this, "addAccessorieOnShelfPlanner", void 0);
    _defineProperty(this, "moveShelfItem", void 0);
    _defineProperty(this, "moveDoors", void 0);
    _defineProperty(this, "deleteAccessorieOnShelfPlanner", void 0);
    _defineProperty(this, "getBasketIdsOfElementIdLogic", void 0);
    _defineProperty(this, "loadPaletteInfosLogic", void 0);
    _defineProperty(this, "changeOnFavoritesLogic", void 0);
    _defineProperty(this, "actGroupedConfigSteps", []);
    _defineProperty(this, "defineOptions", void 0);
    _defineProperty(this, "onClickConfiguratorSettingsForth", void 0);
    _defineProperty(this, "getPlanningIsInCompleteOrEmpty", void 0);
    _defineProperty(this, "checkNotificationRules", void 0);
    _defineProperty(this, "checkBannerRules", void 0);
    _defineProperty(this, "actions", void 0);
    /**
     * This function is called when a element should be copied.
     * @param  setId the setId the element to copy belongs too
     * @param  elementId  The elementId that should be copied
     * @param   copyInfo  This objects hold information that is needed for copying. For example when copying an element in the box planner, the information in which direction it should be copied is important.
     */
    _defineProperty(this, "copyPlannerElement", /*#__PURE__*/function () {
      var _ref = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee(setId, elementId, copyInfo) {
        var newConfigStepKey;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              if (_this.productState.isBoxPlanner) {
                _this.configuratorStructureHandler.navigateToFirstOption();
              }
              _context.next = 3;
              return _this.copyPlannerElementLogic(setId, elementId, _this.configuratorStructureHandler.curConfigStepKey, copyInfo);
            case 3:
              newConfigStepKey = _context.sent;
              if (newConfigStepKey) _this.configuratorStructureHandler.navigateToConfigStepByConfigStepKey(newConfigStepKey);
              _this.updateCurPaletteData(false);
              _this.settingsState.ConfigDidChange();
            case 7:
            case "end":
              return _context.stop();
          }
        }, _callee);
      }));
      return function (_x, _x2, _x3) {
        return _ref.apply(this, arguments);
      };
    }());
    /**
     *this function binds a listender for the FreeText Input event
     */
    _defineProperty(this, "bindListenerForTextBox", /*#__PURE__*/function () {
      var _ref2 = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee3(paletteName, actStepName, item) {
        var textBox;
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return DivaUtils.Sleep(200);
            case 2:
              textBox = document.querySelector('#' + item.op + '_' + item.codex);
              if (textBox) {
                textBox.addEventListener('change', /*#__PURE__*/_asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
                  var myValues, configStepKey, targetArticles;
                  return _regeneratorRuntime.wrap(function _callee2$(_context2) {
                    while (1) switch (_context2.prev = _context2.next) {
                      case 0:
                        if (!textBox) {
                          _context2.next = 7;
                          break;
                        }
                        myValues = {};
                        myValues[item.op] = item.codex;
                        myValues[item.op + '_val'] = textBox.value;
                        configStepKey = _this.configuratorStructureHandler.curConfigStepKey, targetArticles = _this.configuratorStructureHandler.getActiveArticlesOfTabGroup(configStepKey, _this.settingsState.doConfigSingle);
                        _context2.next = 7;
                        return _this.doConfigureLogic(myValues, targetArticles);
                      case 7:
                      case "end":
                        return _context2.stop();
                    }
                  }, _callee2);
                })), {
                  passive: true
                });
              } else {
                setTimeout(function () {
                  _this.bindListenerForTextBox(paletteName, actStepName, item);
                }, 100);
              }
            case 4:
            case "end":
              return _context3.stop();
          }
        }, _callee3);
      }));
      return function (_x4, _x5, _x6) {
        return _ref2.apply(this, arguments);
      };
    }());
    this.settingsState = settingsState;
    this.productState = productState;
    this.uiState = uiState;
    this.plannerItems = new PlannerPalette$1(this.settingsState, this.uiState);
    this.shelfItem = new ShelfItem();
    this.specialItem = new SpecialItem();
    this.accItems = new AccItems(this.uiState);
    this.configSteps = [];
    for (var i = 0; i < 10; i++) {
      this.configSteps[i] = new ConfigItems(i);
      this.summaryItems[i] = new PlannerPalette();
    }
    this.staticRoomItems = new StaticRoomItems();
    this.configuratorStructureHandler = configuratorStructureHandler;
    this.visitStepsHandler = new VisitStepsHandler(configuratorStructureHandler, this.settingsState, getState, defineOptions);
    this.defineOptions = defineOptions;
    this.getState = getState;
    this.updateConfiguratorState = updateConfiguratorState;
    this.updatePlannerMsgContainer = productState.updatePlannerMsgContainer;
    this.getSelectedElementsArr = getSelectedElementsArr;
    this.getSelectedDockingPoint = getSelectedDockingPoint;
    this.updateElementSelectionPlanner = updateElementSelectionPlanner;
    //logic Functions
    this.configSingleIsChangedLogic = configSingleIsChanged;
    this.addPlannerElementLogic = addPlannerElement;
    this.deletePlannerElementLogic = deletePlannerElement;
    this.switchPlannerElementLogic = switchPlannerElementLogic;
    this.copyPlannerElementLogic = copyPlannerElementLogic;
    this.deleteAccessoireLogic = deleteAccessoire;
    this.updateAccessoireQuantityLogic = updateAccessoireQuantity;
    this.doConfigureLogic = doConfigure;
    this.setCurConfigStepKey = setCurConfigStepKey;
    this.setActSet = setActSet;
    this.doAllowSetPlanner = doAllowSetPlanner;
    this.handle3DVisualizationChanged = handle3DVisualizationChanged;
    this.loadCatalogPlannerArticlesLogic = loadCatalogPlannerArticles;
    this.loadAccessoiresOfArticlesLogic = loadAccessoiresOfArticles;
    this.loadConfigItemsOfOptionsLogic = loadConfigItemsOfOptions;
    this.loadSummaryItemsLogic = loadSummaryItems;
    this.loadStaticRoomList = loadStaticRoomList;
    this.loadShelfPlannerItemsLogic = loadShelfPlannerItemsLogic;
    this.loadDoorPlannerItemsLogic = loadDoorPlannerItemsLogic;
    this.moveDoors = moveDoors;
    this.selectShelfAccArticle = selectShelfAccArticle;
    this.getFreePlacesOnGrids = getFreePlacesOnGrids;
    this.addAccessorieOnShelfPlanner = addAccessorieOnShelfPlanner;
    this.moveShelfItem = moveShelfItem;
    this.deleteAccessorieOnShelfPlanner = deleteAccessorieOnShelfPlanner;
    this.getBasketIdsOfElementIdLogic = getBasketIdsOfElementIdLogic;
    this.loadPaletteInfosLogic = loadPaletteInfos;
    this.changeOnFavoritesLogic = changeOnFavorites;
    this.onClickConfiguratorSettingsForth = onClickConfiguratorSettingsForth;
    this.getPlanningIsInCompleteOrEmpty = getPlanningIsInCompleteOrEmpty;
    this.checkNotificationRules = checkNotificationRules;
    this.checkBannerRules = checkBannerRules;
    this.actions = actions;
  }

  //clicked Element gets identifyed and addPlannerElement is called
  return _createClass(PaletteViewHandler, [{
    key: "clickedOnPlannerPalette",
    value: function () {
      var _clickedOnPlannerPalette = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee4(item, paletteId) {
        var clickedOnAccessoirePalette;
        return _regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) switch (_context4.prev = _context4.next) {
            case 0:
              clickedOnAccessoirePalette = paletteId == 1 && this.plannerItems.DoShowAdditionalPlannerElements;
              if (!this.settingsState.plannerReplaceMode || clickedOnAccessoirePalette) {
                this.addPlannerElement(item, clickedOnAccessoirePalette);
              } else {
                this.clickOnPlannerElementSwitch(item);
              }
            case 2:
            case "end":
              return _context4.stop();
          }
        }, _callee4, this);
      }));
      function clickedOnPlannerPalette(_x7, _x8) {
        return _clickedOnPlannerPalette.apply(this, arguments);
      }
      return clickedOnPlannerPalette;
    }()
  }, {
    key: "addPlannerElement",
    value: function () {
      var _addPlannerElement = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee5(item, dontUpdateReplaceMode) {
        var _this2 = this,
          _selectedElement$,
          _this$getSelectedDock;
        var initparams, selectedElement, partlistCodex, _item$partlistcodexes;
        return _regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) switch (_context5.prev = _context5.next) {
            case 0:
              initparams = {};
              selectedElement = this.getSelectedElementsArr(true);
              partlistCodex = undefined;
              if (this.settingsState.plannerFilter.searchText) {
                partlistCodex = (_item$partlistcodexes = item.partlistcodexes) === null || _item$partlistcodexes === void 0 ? void 0 : _item$partlistcodexes.split(';').find(function (pCodex) {
                  return pCodex.toLowerCase() == _this2.settingsState.plannerFilter.searchText.toLowerCase();
                });
              }
              initparams = {
                partlistCodexToInit: partlistCodex,
                dockingPointInfos: [{
                  targetElemIdToDock: (_selectedElement$ = selectedElement[0]) === null || _selectedElement$ === void 0 ? void 0 : _selectedElement$.getElementId(),
                  targetDockingPointId: (_this$getSelectedDock = this.getSelectedDockingPoint()) === null || _this$getSelectedDock === void 0 ? void 0 : _this$getSelectedDock.DpId
                }],
                svgJsonFormula: item.jsonFormula ? item.jsonFormula : undefined
              };
              _context5.next = 7;
              return this.addPlannerElementLogic(item.articleCodex, initparams);
            case 7:
              if (!dontUpdateReplaceMode) this.checkPlanningReplaceModeAfterChange(!this.plannerItems.UpdatePaletteAfterPlanningChanged);
              if (this.plannerItems.UpdatePaletteAfterPlanningChanged) this.updateCurPaletteData(false);
              this.updateConfiguratorState(false);
              this.settingsState.ConfigDidChange();
            case 11:
            case "end":
              return _context5.stop();
          }
        }, _callee5, this);
      }));
      function addPlannerElement(_x9, _x10) {
        return _addPlannerElement.apply(this, arguments);
      }
      return addPlannerElement;
    }()
  }, {
    key: "checkPlanningReplaceModeAfterChange",
    value: function checkPlanningReplaceModeAfterChange(updatePaletteData) {
      if (!this.settingsState.plannerReplaceModeAutoSwitch) return;
      var isInCompleteOrEmpty = this.getPlanningIsInCompleteOrEmpty();
      if (this.settingsState.plannerReplaceMode) {
        if (isInCompleteOrEmpty) {
          this.setPlannerReplaceMode(false, updatePaletteData);
        }
      } else {
        if (!isInCompleteOrEmpty) {
          this.setPlannerReplaceMode(true, updatePaletteData);
        }
      }
    }

    /**
     * This function is called all when a Planner Element gets Deleted in th PlannerMode
     */
  }, {
    key: "deletePlannerElement",
    value: (function () {
      var _deletePlannerElement = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee6(setId, elementId) {
        return _regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) switch (_context6.prev = _context6.next) {
            case 0:
              _context6.next = 2;
              return this.deletePlannerElementLogic(setId, elementId);
            case 2:
              this.checkPlanningReplaceModeAfterChange(false);
              this.updateCurPaletteData(false);
              this.settingsState.ConfigDidChange();
            case 5:
            case "end":
              return _context6.stop();
          }
        }, _callee6, this);
      }));
      function deletePlannerElement(_x11, _x12) {
        return _deletePlannerElement.apply(this, arguments);
      }
      return deletePlannerElement;
    }())
  }, {
    key: "clickOnPlannerElementSwitch",
    value: function () {
      var _clickOnPlannerElementSwitch = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee7(item) {
        var _this3 = this;
        var elementToDelete, element, elementId, setId, dps, dockingPointInfos, i, _dps$i$OnDP, partlistCodex, _item$partlistcodexes2;
        return _regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) switch (_context7.prev = _context7.next) {
            case 0:
              if (!this.onCalculating) {
                _context7.next = 2;
                break;
              }
              return _context7.abrupt("return");
            case 2:
              this.onCalculating = true;
              //the current selected Element gets switched out with a now one
              elementToDelete = this.getSelectedElementsArr(true);
              if (!elementToDelete.length) {
                _context7.next = 18;
                break;
              }
              element = elementToDelete[0];
              elementId = element.getElementId();
              setId = element.getSetId();
              dps = element.getDockingDockingPoints();
              dockingPointInfos = [];
              for (i = 0; i < dps.length; i++) {
                dockingPointInfos.push({
                  targetElemIdToDock: dps[i].OnElementId,
                  targetDockingPointId: (_dps$i$OnDP = dps[i].OnDP) === null || _dps$i$OnDP === void 0 ? void 0 : _dps$i$OnDP.DpId
                });
              }
              partlistCodex = undefined;
              if (this.settingsState.plannerFilter.searchText) {
                partlistCodex = (_item$partlistcodexes2 = item.partlistcodexes) === null || _item$partlistcodexes2 === void 0 ? void 0 : _item$partlistcodexes2.split(';').find(function (pCodex) {
                  return pCodex.toLowerCase() == _this3.settingsState.plannerFilter.searchText.toLowerCase();
                });
              }
              _context7.next = 15;
              return this.switchPlannerElementLogic(setId, elementId, item.articleCodex, {
                dockingPointInfos: dockingPointInfos,
                partlistCodexToInit: partlistCodex,
                posNr: element.getPosNr()
              });
            case 15:
              this.onCalculating = false;
              this.updateCurPaletteData(false);
              this.settingsState.ConfigDidChange();
            case 18:
            case "end":
              return _context7.stop();
          }
        }, _callee7, this);
      }));
      function clickOnPlannerElementSwitch(_x13) {
        return _clickOnPlannerElementSwitch.apply(this, arguments);
      }
      return clickOnPlannerElementSwitch;
    }()
  }, {
    key: "setPlannerReplaceMode",
    value: function setPlannerReplaceMode(val, updateCurPaletteData) {
      if (!this.settingsState.plannerReplaceModeAvailable || !this.settingsState.doShowReplaceMode) return;
      this.settingsState.setPlannerReplaceMode(val);
      var selection = this.getSelectedElementsArr(false);
      this.plannerItems.calcMatchingArticles(selection);
      if (updateCurPaletteData) this.updateCurPaletteData(false);
    }
  }, {
    key: "clickedOnAccessoirePaletteWrapper",
    value: //clicked Accessorie & newQuantity gets identifyed an is called loadAccessoiresOfActArticles
    function () {
      var _clickedOnAccessoirePaletteWrapper = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee8(clickedAcc, type, forceWithOutSVG) {
        var ret;
        return _regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) switch (_context8.prev = _context8.next) {
            case 0:
              if (!this.onCalculating) {
                _context8.next = 2;
                break;
              }
              return _context8.abrupt("return");
            case 2:
              this.onCalculating = true;
              _context8.next = 5;
              return this.clickedOnAccessoirePalette(clickedAcc, type, forceWithOutSVG);
            case 5:
              ret = _context8.sent;
              this.onCalculating = false;
              return _context8.abrupt("return", ret);
            case 8:
            case "end":
              return _context8.stop();
          }
        }, _callee8, this);
      }));
      function clickedOnAccessoirePaletteWrapper(_x14, _x15, _x16) {
        return _clickedOnAccessoirePaletteWrapper.apply(this, arguments);
      }
      return clickedOnAccessoirePaletteWrapper;
    }()
  }, {
    key: "clickedOnAccessoirePalette",
    value: function () {
      var _clickedOnAccessoirePalette = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee9(clickedAcc, type, forceWithOutSVG) {
        var newQuantity, relatedParentArticle, somethingChanged, _selectedElement$2, selectedElement, ret;
        return _regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) switch (_context9.prev = _context9.next) {
            case 0:
              newQuantity = type == 'minus' ? clickedAcc.actQuantity - 1 : clickedAcc.actQuantity + 1;
              if (type == 'minus' && clickedAcc.minQuantity && newQuantity < clickedAcc.minQuantity) newQuantity = 0;
              if (type == 'plus' && clickedAcc.minQuantity && newQuantity < clickedAcc.minQuantity) newQuantity = clickedAcc.minQuantity;
              this.settingsState.ConfigDidChange();
              relatedParentArticle = this.configuratorStructureHandler.actTargetArticles[0];
              if (!(clickedAcc.isPlannerElement && !forceWithOutSVG)) {
                _context9.next = 36;
                break;
              }
              if (!(type == 'minus')) {
                _context9.next = 20;
                break;
              }
              if (!(clickedAcc.actQuantity > 0)) {
                _context9.next = 17;
                break;
              }
              _context9.next = 10;
              return this.deleteAccessoireLogic(clickedAcc.articleCodex, relatedParentArticle);
            case 10:
              somethingChanged = _context9.sent;
              if (!somethingChanged) {
                _context9.next = 14;
                break;
              }
              _context9.next = 14;
              return this.updateCurPaletteData(false);
            case 14:
              return _context9.abrupt("return", 'AccessoireAdded');
            case 17:
              return _context9.abrupt("return", 'MinQuantityReached');
            case 18:
              _context9.next = 34;
              break;
            case 20:
              if (!(clickedAcc.actQuantity < clickedAcc.maxQuantity || clickedAcc.maxQuantity == 0)) {
                _context9.next = 33;
                break;
              }
              //verplanen
              selectedElement = this.getSelectedElementsArr(true);
              _context9.next = 24;
              return this.addPlannerElementLogic(clickedAcc.articleCodex, {
                dockingPointInfos: [{
                  targetElemIdToDock: (_selectedElement$2 = selectedElement[0]) === null || _selectedElement$2 === void 0 ? void 0 : _selectedElement$2.getElementId()
                }],
                relatedToBasketId: relatedParentArticle,
                quantity: newQuantity - clickedAcc.actQuantity
              }, false);
            case 24:
              ret = _context9.sent;
              if (ret == 'AccessoireNotVisualized') {
                this.clickedOnAccessoirePalette(clickedAcc, type, true);
              }
              _context9.next = 28;
              return this.updateCurPaletteData(false);
            case 28:
              if (!(ret == 'ElementAdded')) {
                _context9.next = 30;
                break;
              }
              return _context9.abrupt("return", 'AccessoireAdded');
            case 30:
              return _context9.abrupt("return", ret);
            case 33:
              return _context9.abrupt("return", 'MaxQuantityReached');
            case 34:
              _context9.next = 57;
              break;
            case 36:
              if (!(type == 'minus')) {
                _context9.next = 48;
                break;
              }
              if (!(clickedAcc.actQuantity > 0)) {
                _context9.next = 45;
                break;
              }
              _context9.next = 40;
              return this.updateAccessoireQuantityLogic(clickedAcc, relatedParentArticle, newQuantity, forceWithOutSVG);
            case 40:
              _context9.next = 42;
              return this.updateCurPaletteData(false);
            case 42:
              return _context9.abrupt("return", 'AccessoireAdded');
            case 45:
              return _context9.abrupt("return", 'MinQuantityReached');
            case 46:
              _context9.next = 57;
              break;
            case 48:
              if (!(clickedAcc.actQuantity != clickedAcc.maxQuantity)) {
                _context9.next = 56;
                break;
              }
              _context9.next = 51;
              return this.updateAccessoireQuantityLogic(clickedAcc, relatedParentArticle, newQuantity, forceWithOutSVG);
            case 51:
              _context9.next = 53;
              return this.updateCurPaletteData(false);
            case 53:
              return _context9.abrupt("return", 'AccessoireAdded');
            case 56:
              return _context9.abrupt("return", 'MaxQuantityReached');
            case 57:
            case "end":
              return _context9.stop();
          }
        }, _callee9, this);
      }));
      function clickedOnAccessoirePalette(_x17, _x18, _x19) {
        return _clickedOnAccessoirePalette.apply(this, arguments);
      }
      return clickedOnAccessoirePalette;
    }()
  }, {
    key: "selectShelfAccessorie",
    value: function () {
      var _selectShelfAccessorie = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee10(articleCodex, basketId) {
        var grids, _grids$find, accessoire, stepOfAccessoire;
        return _regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) switch (_context10.prev = _context10.next) {
            case 0:
              grids = this.shelfItem.Grids;
              if (!(grids !== null && grids !== void 0 && grids.length)) {
                _context10.next = 13;
                break;
              }
              if (!basketId) {
                _context10.next = 11;
                break;
              }
              accessoire = (_grids$find = grids.find(function (g) {
                return g.getAccessoireByBasketId(basketId);
              })) === null || _grids$find === void 0 ? void 0 : _grids$find.getAccessoireByBasketId(basketId);
              stepOfAccessoire = accessoire === null || accessoire === void 0 ? void 0 : accessoire.relatedToConfigStep;
              if (!(stepOfAccessoire && !this.configuratorStructureHandler.configStepKeyIsActive(stepOfAccessoire))) {
                _context10.next = 11;
                break;
              }
              this.configuratorStructureHandler.navigateToConfigStepByConfigStepKey(stepOfAccessoire);
              //TODO after navigation select the accessorie again
              _context10.next = 9;
              return this.updateCurPaletteData(true);
            case 9:
              this.selectShelfAccessorie(articleCodex, basketId);
              return _context10.abrupt("return");
            case 11:
              _context10.next = 13;
              return this.clickedOnShelfAccessorie(grids, articleCodex, basketId);
            case 13:
              this.settingsState.setSelectedShelfAccessoire(basketId);
              this.updateCurPaletteData(true);
            case 15:
            case "end":
              return _context10.stop();
          }
        }, _callee10, this);
      }));
      function selectShelfAccessorie(_x20, _x21) {
        return _selectShelfAccessorie.apply(this, arguments);
      }
      return selectShelfAccessorie;
    }()
  }, {
    key: "clickedOnShelfAccessorie",
    value: function () {
      var _clickedOnShelfAccessorie = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee11(grids, itemCodex, basketId, triggerUpdate) {
        var _this$settingsState$i;
        var targetArticles, configStepKey;
        return _regeneratorRuntime.wrap(function _callee11$(_context11) {
          while (1) switch (_context11.prev = _context11.next) {
            case 0:
              targetArticles = grids.map(function (g) {
                return g.basketId;
              });
              configStepKey = (_this$settingsState$i = this.settingsState.itemPalettes.get(this.shelfItem.currentPaletteId)) === null || _this$settingsState$i === void 0 ? void 0 : _this$settingsState$i.configStepKey;
              this.selectShelfAccArticle(configStepKey !== null && configStepKey !== void 0 ? configStepKey : this.configuratorStructureHandler.curConfigStepKey, targetArticles, itemCodex);
              if (itemCodex === undefined) this.settingsState.setFreePlacesOnGrids([]);
              if (this.productState.shelfPlannerRTV) this.productState.onChangePlannerView('RTV');
              if (triggerUpdate) this.updateCurPaletteData(true);
            case 6:
            case "end":
              return _context11.stop();
          }
        }, _callee11, this);
      }));
      function clickedOnShelfAccessorie(_x22, _x23, _x24, _x25) {
        return _clickedOnShelfAccessorie.apply(this, arguments);
      }
      return clickedOnShelfAccessorie;
    }()
  }, {
    key: "updateFreePlacesOnGrids",
    value: function () {
      var _updateFreePlacesOnGrids = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee12(shelfId, from) {
        var _yield$this$getFreePl, freeShelfGrids, noFreeSpace, _this$shelfItem, _this$shelfItem$getAl, _this$uiState$i18nIns, _this$uiState$i18nIns2, selectedAcc, selectedShelAccessoire, msg;
        return _regeneratorRuntime.wrap(function _callee12$(_context12) {
          while (1) switch (_context12.prev = _context12.next) {
            case 0:
              if (this.productState.shelfPlannerRTV) {
                _context12.next = 2;
                break;
              }
              return _context12.abrupt("return");
            case 2:
              _context12.next = 4;
              return this.getFreePlacesOnGrids(shelfId, from);
            case 4:
              _yield$this$getFreePl = _context12.sent;
              freeShelfGrids = _yield$this$getFreePl.freeShelfGrids;
              noFreeSpace = freeShelfGrids && !freeShelfGrids.find(function (a) {
                return Object.values(a.grids).find(function (g) {
                  return g.freeCells.length > 0;
                });
              });
              if (noFreeSpace) {
                selectedAcc = (_this$shelfItem = this.shelfItem) === null || _this$shelfItem === void 0 || (_this$shelfItem = _this$shelfItem.Grids) === null || _this$shelfItem === void 0 || (_this$shelfItem = _this$shelfItem.find(function (g) {
                  return g.ActAccessoire;
                })) === null || _this$shelfItem === void 0 ? void 0 : _this$shelfItem.ActAccessoire;
                selectedShelAccessoire = (_this$shelfItem$getAl = this.shelfItem.getAllItems()) === null || _this$shelfItem$getAl === void 0 ? void 0 : _this$shelfItem$getAl.find(function (item) {
                  return item.articleCodex == selectedAcc;
                });
                msg = (_this$uiState$i18nIns = (_this$uiState$i18nIns2 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns2 === void 0 ? void 0 : _this$uiState$i18nIns2.t('configurator.noSpaceForArticle', {
                  name: selectedShelAccessoire === null || selectedShelAccessoire === void 0 ? void 0 : selectedShelAccessoire.name
                })) !== null && _this$uiState$i18nIns !== void 0 ? _this$uiState$i18nIns : '';
                this.updatePlannerMsgContainer('SHELFPLANNER_MSG', true, 'INFO', msg, true, 8000);
              } else {
                this.updatePlannerMsgContainer('SHELFPLANNER_MSG', false, 'INFO', '', false);
              }
            case 8:
            case "end":
              return _context12.stop();
          }
        }, _callee12, this);
      }));
      function updateFreePlacesOnGrids(_x26, _x27) {
        return _updateFreePlacesOnGrids.apply(this, arguments);
      }
      return updateFreePlacesOnGrids;
    }() //clicked OptionValue gets Identifyed an doConfigure is called
  }, {
    key: "clickedOnConfigPalette",
    value: function () {
      var _clickedOnConfigPalette = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee13(item, from, paletteId) {
        var _item$ocd,
          _palette$configStepKe,
          _palette$configAllMod,
          _this4 = this,
          _item$contentItemId;
        var op, codex, palette, configStepKey, myValues, targetArticles, notifications, _this$configuratorStr, _this$configuratorStr2, groupValCodex, groupOptionCodex;
        return _regeneratorRuntime.wrap(function _callee13$(_context13) {
          while (1) switch (_context13.prev = _context13.next) {
            case 0:
              op = (item.parentOptionCodex ? item.parentOptionCodex + '.' : '') + item.op;
              codex = (_item$ocd = item.ocd) !== null && _item$ocd !== void 0 ? _item$ocd : item.codex;
              palette = paletteId ? this.settingsState.itemPalettes.get(paletteId) : undefined;
              configStepKey = (_palette$configStepKe = palette === null || palette === void 0 ? void 0 : palette.configStepKey) !== null && _palette$configStepKe !== void 0 ? _palette$configStepKe : this.configuratorStructureHandler.curConfigStepKey;
              myValues = {};
              targetArticles = this.configuratorStructureHandler.getActiveArticlesOfTabGroup(configStepKey, this.settingsState.doConfigSingle || !!(palette !== null && palette !== void 0 && palette.isRenderedInAccessorieStep));
              if (palette !== null && palette !== void 0 && (_palette$configAllMod = palette.configAllMode) !== null && _palette$configAllMod !== void 0 && _palette$configAllMod.length) targetArticles = palette.configAllMode;

              //show freeText Dialog
              if (!(!!item.isFreeText && from != 'inputbox')) {
                _context13.next = 10;
                break;
              }
              this.settingsState.updateInputBoxValues(true, item);
              return _context13.abrupt("return");
            case 10:
              if (!(from !== 'notification')) {
                _context13.next = 15;
                break;
              }
              notifications = this.checkNotificationRules(targetArticles, this.configuratorStructureHandler.visitedOptions, {
                type: 'config',
                data: {
                  optionCodex: op,
                  optionValueCodex: codex
                }
              });
              if (!(notifications !== null && notifications !== void 0 && notifications.length)) {
                _context13.next = 15;
                break;
              }
              this.manageNotifications(notifications, function () {
                _this4.clickedOnConfigPalette(item, 'notification', paletteId);
              });
              return _context13.abrupt("return");
            case 15:
              if (!item.disabledByConstructionRule) {
                _context13.next = 18;
                break;
              }
              this.updatePlannerMsgContainer('DISABLED_BY_CONSTRUCTION', true, 'INFO', item.disabledByConstructionRule, true);
              return _context13.abrupt("return");
            case 18:
              if (((_item$contentItemId = item.contentItemId) !== null && _item$contentItemId !== void 0 ? _item$contentItemId : item.groupContentItemId) || this.settingsState.conetentItemOfOPVOpen) {
                if (item.groupContentItemId) this.settingsState.setContentItemId(item.groupContentItemId, true, item.groupName + (item.parentName ? '_' + item.parentName : ''));else if (item.contentItemId) this.settingsState.setContentItemId(item.contentItemId, true, item.codex);
              }
              if (item.isMultiSelect) codex = this.handleMultiselectCodex(codex, item, palette);
              if (!(op && targetArticles)) {
                _context13.next = 29;
                break;
              }
              myValues[op] = codex;
              if (item.freeText !== undefined) {
                myValues[item.op + '_val'] = item.freeText;
                this.visitStepsHandler.defineOptionOfConfigStepKey(configStepKey);
              }

              //evtl. IDM Gruppeninfos holen (z.B. OPT2 wenn auf OPT3 geklickt wurde)
              if (item.idmOpgValCodex) {
                groupValCodex = item.idmOpgValCodex;
                groupOptionCodex = item.idmOpgOptionCodex;
                if (groupOptionCodex && groupValCodex) {
                  myValues[groupOptionCodex] = groupValCodex;
                }
              }
              _context13.next = 26;
              return this.doConfigureLogic(myValues, targetArticles);
            case 26:
              this.doDefineOptions(myValues, targetArticles);
              if (!item.isMultiSelect && this.uiState.ViewConfig.navigateOnClick && !((_this$configuratorStr = this.configuratorStructureHandler.tabGroupState) !== null && _this$configuratorStr !== void 0 && _this$configuratorStr.filterTabgroup) && !((_this$configuratorStr2 = this.configuratorStructureHandler.CurConfigStep.optionstate) !== null && _this$configuratorStr2 !== void 0 && _this$configuratorStr2.isDifferentOption)) {
                this.onClickConfiguratorSettingsForth();
              } else {
                this.updateCurPaletteData(false, configStepKey);
              }
              this.settingsState.ConfigDidChange();
            case 29:
              this.updateBanner(op, codex);
            case 30:
            case "end":
              return _context13.stop();
          }
        }, _callee13, this);
      }));
      function clickedOnConfigPalette(_x28, _x29, _x30) {
        return _clickedOnConfigPalette.apply(this, arguments);
      }
      return clickedOnConfigPalette;
    }()
  }, {
    key: "updateBanner",
    value: function updateBanner(op, codex) {
      this.productState.setCurrentBanner(this.checkBannerRules(this.configuratorStructureHandler.visitedOptions, {
        type: 'config',
        data: {
          optionCodex: op,
          optionValueCodex: codex
        }
      }));
    }
  }, {
    key: "manageNotifications",
    value: function manageNotifications(notifications, _callback) {
      var _this5 = this;
      var _loop = function _loop() {
          var _this5$uiState$i18nIn;
          var note = notifications[i];
          var lang = (_this5$uiState$i18nIn = _this5.uiState.i18nInstance) === null || _this5$uiState$i18nIn === void 0 ? void 0 : _this5$uiState$i18nIn.language;
          if (note.type === 'notification' && note.onlyShowOnce) {
            if (_this5.settingsState.notificationsShown.includes(note.index)) {
              _callback();
              return 0; // continue
            }
          }
          if (note.type === 'notification' && !note.showInModal) {
            _this5.updatePlannerMsgContainer('DISABLED_BY_CONSTRUCTION', true, 'INFO', note.message[lang === null || lang === void 0 ? void 0 : lang.toUpperCase()], true);
            _this5.settingsState.notificationsShown.push(note.index);
          } else {
            var _this5$actions, _note$messageTitle;
            (_this5$actions = _this5.actions) === null || _this5$actions === void 0 || _this5$actions.openModal({
              accept: true,
              callback: function callback(accepted) {
                if (accepted) {
                  _callback();
                  _this5.settingsState.notificationsShown.push(note.index);
                }
              },
              text: note.message[lang === null || lang === void 0 ? void 0 : lang.toUpperCase()],
              title: (_note$messageTitle = note.messageTitle) === null || _note$messageTitle === void 0 ? void 0 : _note$messageTitle[lang === null || lang === void 0 ? void 0 : lang.toUpperCase()],
              reject: true,
              rejectByIcon: true
            });
          }
          return 1; // break
        },
        _ret;
      for (var i = 0; i < notifications.length; i++) {
        _ret = _loop();
        if (_ret === 0) continue;
        if (_ret === 1) break;
      }
    }
  }, {
    key: "handleMultiselectCodex",
    value: function handleMultiselectCodex(codex, item, palette) {
      var _palette$selectedElem, _palette$selectedElem2, _item$ocd2;
      var codexes = (_palette$selectedElem = palette === null || palette === void 0 || (_palette$selectedElem2 = palette.selectedElements) === null || _palette$selectedElem2 === void 0 ? void 0 : _palette$selectedElem2.map(function (opv) {
        var _opv$ocd;
        return (_opv$ocd = opv.ocd) !== null && _opv$ocd !== void 0 ? _opv$ocd : opv.codex;
      })) !== null && _palette$selectedElem !== void 0 ? _palette$selectedElem : [];
      if (codexes.includes(codex)) {
        codexes.splice(codexes.indexOf(codex), 1);
      } else {
        codexes.push(codex);
      }
      if (codexes.length && codexes.find(function (c) {
        return c.startsWith('OPVOPV_NICHT_RELEVANT');
      })) {
        codexes.splice(codexes.findIndex(function (c) {
          return c.startsWith('OPVOPV_NICHT_RELEVANT');
        }), 1);
      }
      if (codexes.length && !((_item$ocd2 = item.ocd) !== null && _item$ocd2 !== void 0 ? _item$ocd2 : codex).startsWith('OPVOPV_NICHT_RELEVANT')) {
        codex = codexes.join(';');
      } else {
        //Default select first item
        codex = 'EMPTY_MULTISELECT';
      }
      return codex;
    }
  }, {
    key: "onFreeTextLangChanged",
    value: function () {
      var _onFreeTextLangChanged = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee14(lang, text, item, paletteId) {
        var _item$freeTextLang$DE, _item$freeTextLang, _item$freeTextLang$FR, _item$freeTextLang2, _item$freeTextLang$IT, _item$freeTextLang3;
        var op, palette, configStepKey, targetArticles, value, myValues;
        return _regeneratorRuntime.wrap(function _callee14$(_context14) {
          while (1) switch (_context14.prev = _context14.next) {
            case 0:
              op = item.op;
              palette = paletteId ? this.settingsState.itemPalettes.get(paletteId) : undefined;
              configStepKey = palette === null || palette === void 0 ? void 0 : palette.configStepKey;
              if (!configStepKey) {
                configStepKey = this.configuratorStructureHandler.curConfigStepKey;
              }
              targetArticles = this.configuratorStructureHandler.getActiveArticlesOfTabGroup(configStepKey, this.settingsState.doConfigSingle);
              value = (lang == 'DE' ? text : (_item$freeTextLang$DE = (_item$freeTextLang = item.freeTextLang) === null || _item$freeTextLang === void 0 ? void 0 : _item$freeTextLang['DE']) !== null && _item$freeTextLang$DE !== void 0 ? _item$freeTextLang$DE : '') + '##' + (lang == 'FR' ? text : (_item$freeTextLang$FR = (_item$freeTextLang2 = item.freeTextLang) === null || _item$freeTextLang2 === void 0 ? void 0 : _item$freeTextLang2['FR']) !== null && _item$freeTextLang$FR !== void 0 ? _item$freeTextLang$FR : '') + '##' + (lang == 'IT' ? text : (_item$freeTextLang$IT = (_item$freeTextLang3 = item.freeTextLang) === null || _item$freeTextLang3 === void 0 ? void 0 : _item$freeTextLang3['IT']) !== null && _item$freeTextLang$IT !== void 0 ? _item$freeTextLang$IT : '');
              myValues = _defineProperty({}, op, value);
              _context14.next = 9;
              return this.doConfigureLogic(myValues, targetArticles);
            case 9:
              this.updateCurPaletteData(false, configStepKey);
            case 10:
            case "end":
              return _context14.stop();
          }
        }, _callee14, this);
      }));
      function onFreeTextLangChanged(_x31, _x32, _x33, _x34) {
        return _onFreeTextLangChanged.apply(this, arguments);
      }
      return onFreeTextLangChanged;
    }()
  }, {
    key: "doDefineOptions",
    value: function doDefineOptions(myValues, targetArticles) {
      var optionsWithArticles = targetArticles.reduce(function (optionsWithArticles, item) {
        optionsWithArticles[item] = Object.keys(myValues);
        return optionsWithArticles;
      }, {});
      this.defineOptions(optionsWithArticles);
    }
  }, {
    key: "changeOnCheckoutConfig",
    value: function () {
      var _changeOnCheckoutConfig = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee15(myValues) {
        return _regeneratorRuntime.wrap(function _callee15$(_context15) {
          while (1) switch (_context15.prev = _context15.next) {
            case 0:
              _context15.next = 2;
              return this.doConfigureLogic(myValues, Object.values(this.settingsState.articleInfos).map(function (a) {
                return a.basketId;
              }));
            case 2:
              this.updateCurPaletteData(false);
            case 3:
            case "end":
              return _context15.stop();
          }
        }, _callee15, this);
      }));
      function changeOnCheckoutConfig(_x35) {
        return _changeOnCheckoutConfig.apply(this, arguments);
      }
      return changeOnCheckoutConfig;
    }()
  }, {
    key: "onRangeClickChange",
    value: function () {
      var _onRangeClickChange = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee16(palette, item, value, onSlide) {
        var _freeText, _freeText2;
        var freeText, myValues, configStepKey, targetArticles;
        return _regeneratorRuntime.wrap(function _callee16$(_context16) {
          while (1) switch (_context16.prev = _context16.next) {
            case 0:
              freeText = value;
              myValues = {};
              configStepKey = palette.configStepKey;
              targetArticles = this.configuratorStructureHandler.getActiveArticlesOfTabGroup(configStepKey, this.settingsState.doConfigSingle);
              myValues[item.op] = item.codex;
              freeText = (_freeText = freeText) === null || _freeText === void 0 ? void 0 : _freeText.toString();
              myValues[item.op + '_val'] = (_freeText2 = freeText) !== null && _freeText2 !== void 0 ? _freeText2 : '';
              this.visitStepsHandler.defineOptionOfConfigStepKey(configStepKey);
              _context16.next = 10;
              return this.doConfigureLogic(myValues, targetArticles, onSlide);
            case 10:
              if (!onSlide) this.updateCurPaletteData(false, configStepKey);
              this.settingsState.ConfigDidChange();
            case 12:
            case "end":
              return _context16.stop();
          }
        }, _callee16, this);
      }));
      function onRangeClickChange(_x36, _x37, _x38, _x39) {
        return _onRangeClickChange.apply(this, arguments);
      }
      return onRangeClickChange;
    }()
  }, {
    key: "clickedOnFavIcon",
    value: function () {
      var _clickedOnFavIcon = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee17(item) {
        var _item$ocd3;
        var remove, myValues, groupValCodex, groupOptionCodex, _key;
        return _regeneratorRuntime.wrap(function _callee17$(_context17) {
          while (1) switch (_context17.prev = _context17.next) {
            case 0:
              remove = item.isFav;
              myValues = {};
              myValues[item.optionOcd || item.op] = (_item$ocd3 = item.ocd) !== null && _item$ocd3 !== void 0 ? _item$ocd3 : item.codex;
              //evtl. IDM Gruppeninfos holen (z.B. OPT2 wenn auf OPT3 geklickt wurde)
              if (item.idmOpgValCodex) {
                groupValCodex = item.idmOpgValCodex;
                groupOptionCodex = item.idmOpgOptionCodex;
                if (groupOptionCodex && groupValCodex) {
                  myValues[groupOptionCodex] = groupValCodex;
                }
              }
              for (_key in myValues) {
                this.changeOnFavoritesLogic(_key, [myValues[_key]], this.configuratorStructureHandler.CurConfigStep.targetArticles, remove);
              }
            case 5:
            case "end":
              return _context17.stop();
          }
        }, _callee17, this);
      }));
      function clickedOnFavIcon(_x40) {
        return _clickedOnFavIcon.apply(this, arguments);
      }
      return clickedOnFavIcon;
    }() //clicked Edit Button gets Identifyed an navigates there
  }, {
    key: "clickedOnSummaryPalette",
    value: function () {
      var _clickedOnSummaryPalette = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee18(item) {
        return _regeneratorRuntime.wrap(function _callee18$(_context18) {
          while (1) switch (_context18.prev = _context18.next) {
            case 0:
              this.setActSet(item.setId);
              this.configuratorStructureHandler.setActStepByPosNrFromSummary(item.posNr);
              this.updateConfiguratorState(true);
              _context18.next = 5;
              return DivaUtils.Sleep(200);
            case 5:
              this.scrollToConfigStepByPosNr(item.posNr, true);
            case 6:
            case "end":
              return _context18.stop();
          }
        }, _callee18, this);
      }));
      function clickedOnSummaryPalette(_x41) {
        return _clickedOnSummaryPalette.apply(this, arguments);
      }
      return clickedOnSummaryPalette;
    }() //clicked Edit Button gets Identifyed an navigates there
  }, {
    key: "clickedOnStaticRoomPalette",
    value: function () {
      var _clickedOnStaticRoomPalette = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee19(item, dontUpdateView) {
        return _regeneratorRuntime.wrap(function _callee19$(_context19) {
          while (1) switch (_context19.prev = _context19.next) {
            case 0:
              // temporally hide the orbit cube. This variable will later be set to the correct value
              this.productState.setPlannerRenderingIsStaticBG(true);
              //StaticRoom List
              this.handle3DVisualizationChanged('StaticRoom', {
                staticRoom: {
                  codex: item.codex
                }
              });
              this.loadStaticRoomListItems(dontUpdateView);
              LOG$c.businessEvent('onRoomVisualization', 'Planner', 'Raum Visualisierung wurde gewächselt', {
                roomCodex: item.codex,
                roomName: item.N
              });
            case 4:
            case "end":
              return _context19.stop();
          }
        }, _callee19, this);
      }));
      function clickedOnStaticRoomPalette(_x42, _x43) {
        return _clickedOnStaticRoomPalette.apply(this, arguments);
      }
      return clickedOnStaticRoomPalette;
    }()
  }, {
    key: "clickedOnStaticVPPalette",
    value: function () {
      var _clickedOnStaticVPPalette = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee20(item, dontUpdateView) {
        return _regeneratorRuntime.wrap(function _callee20$(_context20) {
          while (1) switch (_context20.prev = _context20.next) {
            case 0:
              //Viewpointlist
              if (item.M == 'Orbit') this.handle3DVisualizationChanged('StaticVP', {
                staticVP: {
                  codex: item.codex
                }
              }, true);else this.handle3DVisualizationChanged('StaticVP', {
                staticVP: {
                  codex: item.codex
                }
              }, false);
              this.loadStaticRoomListItems(dontUpdateView);
            case 2:
            case "end":
              return _context20.stop();
          }
        }, _callee20, this);
      }));
      function clickedOnStaticVPPalette(_x44, _x45) {
        return _clickedOnStaticVPPalette.apply(this, arguments);
      }
      return clickedOnStaticVPPalette;
    }()
  }, {
    key: "clickAddAccessorieOnShelfPlanner",
    value: function () {
      var _clickAddAccessorieOnShelfPlanner = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee21(grid, cellIndex) {
        var _this6 = this,
          _this$uiState$i18nIns4,
          _this$uiState$i18nIns5,
          _this$uiState$i18nIns6,
          _this$uiState$i18nIns7,
          _this$uiState$i18nIns8,
          _this$uiState$i18nIns9,
          _this$uiState$i18nIns10;
        var selCell, ret, _this$settingsState$i2, _this$settingsState$i3, partlistCodex, _this$shelfItem$AccIt, _this$uiState$i18nIns3, msg, count, cellsMsg, i, articleNames, _i;
        return _regeneratorRuntime.wrap(function _callee21$(_context21) {
          while (1) switch (_context21.prev = _context21.next) {
            case 0:
              selCell = grid.Cells[cellIndex];
              ret = false;
              if (!(!selCell.IsDisabled && selCell.InUse == -1)) {
                _context21.next = 11;
                break;
              }
              partlistCodex = undefined;
              if (this.shelfItem.getActSearchFitlerText()) {
                partlistCodex = (_this$shelfItem$AccIt = this.shelfItem.AccItems.find(function (a) {
                  return a.articleCodex === grid.ActAccessoire;
                })) === null || _this$shelfItem$AccIt === void 0 || (_this$shelfItem$AccIt = _this$shelfItem$AccIt.partlistcodexes) === null || _this$shelfItem$AccIt === void 0 ? void 0 : _this$shelfItem$AccIt.split(';').find(function (pCodex) {
                  var _this6$shelfItem$getA;
                  return pCodex.toLowerCase() == ((_this6$shelfItem$getA = _this6.shelfItem.getActSearchFitlerText()) === null || _this6$shelfItem$getA === void 0 ? void 0 : _this6$shelfItem$getA.toLowerCase());
                });
              }
              _context21.next = 7;
              return this.addAccessorieOnShelfPlanner(this.shelfItem.currentPaletteId, grid.ShelfIndex, cellIndex, grid.basketId, (_this$settingsState$i2 = (_this$settingsState$i3 = this.settingsState.itemPalettes.get(this.shelfItem.currentPaletteId)) === null || _this$settingsState$i3 === void 0 ? void 0 : _this$settingsState$i3.configStepKey) !== null && _this$settingsState$i2 !== void 0 ? _this$settingsState$i2 : this.configuratorStructureHandler.curConfigStepKey, partlistCodex);
            case 7:
              ret = _context21.sent;
              this.settingsState.ConfigDidChange();
              _context21.next = 13;
              break;
            case 11:
              //select the clicked article
              if (selCell.InUse != -1 && !selCell.IsDisabled && selCell.Article) {
                this.clickedOnShelfAccessorie([grid], selCell.Article.articleCodex);
              }
              if (selCell.IsDisabled) {
                msg = ((_this$uiState$i18nIns3 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns3 === void 0 ? void 0 : _this$uiState$i18nIns3.t('configurator.SHELFPLANNER_DISABLED_AREA')) + '';
                this.updatePlannerMsgContainer('SHELFPLANNER_MSG', true, 'WARNING', msg, true);
              }
            case 13:
              if (!(ret !== false)) {
                _context21.next = 35;
                break;
              }
              if (ret.accessoire) {
                this.selectShelfAccessorie(ret.accessoire.articleCodex, ret.accessoire.basketId);
              }
              _context21.t0 = ret.type;
              _context21.next = _context21.t0 === 'notAllowedByParentRule' ? 18 : _context21.t0 === 'notAllowedByMaxQuantityReached' ? 24 : _context21.t0 === 'doesNotFit' ? 26 : _context21.t0 === 'fitsOnParentRuleButOriginalNotAllowed' ? 28 : _context21.t0 === 'notAllowedByAccessoireRule' ? 30 : 35;
              break;
            case 18:
              count = ret.canOnlyFitOn.length;
              cellsMsg = ' ';
              for (i = count - 1; i >= 0; i--) {
                cellsMsg = cellsMsg + ret.canOnlyFitOn[i];
                if (i != 0) {
                  cellsMsg = cellsMsg + ', ';
                }
              }
              cellsMsg = cellsMsg + ' ';
              this.updatePlannerMsgContainer('SHELFPLANNER_MSG', true, 'WARNING', ((_this$uiState$i18nIns4 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns4 === void 0 ? void 0 : _this$uiState$i18nIns4.t('configurator.CAN_ONLY_FIT_ON_RH_1')) + cellsMsg + ((_this$uiState$i18nIns5 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns5 === void 0 ? void 0 : _this$uiState$i18nIns5.t('configurator.CAN_ONLY_FIT_ON_RH_2')), true);
              return _context21.abrupt("break", 35);
            case 24:
              this.updatePlannerMsgContainer('SHELFPLANNER_MSG', true, 'WARNING', (_this$uiState$i18nIns6 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns6 === void 0 ? void 0 : _this$uiState$i18nIns6.t('configurator.maxQuantityReached'), true);
              return _context21.abrupt("break", 35);
            case 26:
              this.updatePlannerMsgContainer('SHELFPLANNER_MSG', true, 'WARNING', ((_this$uiState$i18nIns7 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns7 === void 0 ? void 0 : _this$uiState$i18nIns7.t('configurator.REGALPLANER_NO_SPACE_1')) + ' ' + ret.size.toString() + ' ' + ((_this$uiState$i18nIns8 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns8 === void 0 ? void 0 : _this$uiState$i18nIns8.t('configurator.REGALPLANER_NO_SPACE_2')), true);
              return _context21.abrupt("break", 35);
            case 28:
              this.updatePlannerMsgContainer('SHELFPLANNER_MSG', true, 'INFO', (_this$uiState$i18nIns9 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns9 === void 0 ? void 0 : _this$uiState$i18nIns9.t('configurator.REGALPLANER_PLACED_ON_OTHER_RH'), true);
              return _context21.abrupt("break", 35);
            case 30:
              articleNames = '';
              if (ret.canOnlyFitOn) for (_i = 0; _i < ret.canOnlyFitOn.length; _i++) {
                articleNames += ret.canOnlyFitOn + ', ';
              }
              articleNames = articleNames.substring(0, articleNames.length - 2);
              this.updatePlannerMsgContainer('SHELFPLANNER_MSG', true, 'WARNING', ((_this$uiState$i18nIns10 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns10 === void 0 ? void 0 : _this$uiState$i18nIns10.t('configurator.NOT_ALLOWED_BY_ON_ACCESSOIRE_RULE')) + articleNames, true);
              return _context21.abrupt("break", 35);
            case 35:
              this.updateFreePlacesOnGrids(grid.gridGroup, 'click add accessorie');
            case 36:
            case "end":
              return _context21.stop();
          }
        }, _callee21, this);
      }));
      function clickAddAccessorieOnShelfPlanner(_x46, _x47) {
        return _clickAddAccessorieOnShelfPlanner.apply(this, arguments);
      }
      return clickAddAccessorieOnShelfPlanner;
    }()
  }, {
    key: "addAccessorieOnShelfPlannerView",
    value: function () {
      var _addAccessorieOnShelfPlannerView = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee22(gridIndex, cellIndex, parentBasketId) {
        var _this$settingsState$i4, _this$settingsState$i5;
        var configStepKey, result;
        return _regeneratorRuntime.wrap(function _callee22$(_context22) {
          while (1) switch (_context22.prev = _context22.next) {
            case 0:
              configStepKey = (_this$settingsState$i4 = (_this$settingsState$i5 = this.settingsState.itemPalettes.get(this.shelfItem.currentPaletteId)) === null || _this$settingsState$i5 === void 0 ? void 0 : _this$settingsState$i5.configStepKey) !== null && _this$settingsState$i4 !== void 0 ? _this$settingsState$i4 : this.configuratorStructureHandler.curConfigStepKey;
              _context22.next = 3;
              return this.addAccessorieOnShelfPlanner(this.shelfItem.currentPaletteId, gridIndex, cellIndex, parentBasketId, configStepKey);
            case 3:
              result = _context22.sent;
              if (result !== false && result.accessoire) {
                this.settingsState.setSelectedShelfAccessoire(result.accessoire.basketId);
              } else {
                if (this.shelfItem.Grids) this.updateFreePlacesOnGrids(configStepKey, 'add accessoire');
              }
              this.settingsState.ConfigDidChange();
            case 6:
            case "end":
              return _context22.stop();
          }
        }, _callee22, this);
      }));
      function addAccessorieOnShelfPlannerView(_x48, _x49, _x50) {
        return _addAccessorieOnShelfPlannerView.apply(this, arguments);
      }
      return addAccessorieOnShelfPlannerView;
    }()
  }, {
    key: "deleteAccessorieOnShelfPlannerView",
    value: function () {
      var _deleteAccessorieOnShelfPlannerView = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee23(gridIndex, cellIndex, parentBasketId) {
        var _this$settingsState$i6, _this$settingsState$i7;
        var configStepKey;
        return _regeneratorRuntime.wrap(function _callee23$(_context23) {
          while (1) switch (_context23.prev = _context23.next) {
            case 0:
              configStepKey = (_this$settingsState$i6 = (_this$settingsState$i7 = this.settingsState.itemPalettes.get(this.shelfItem.currentPaletteId)) === null || _this$settingsState$i7 === void 0 ? void 0 : _this$settingsState$i7.configStepKey) !== null && _this$settingsState$i6 !== void 0 ? _this$settingsState$i6 : this.configuratorStructureHandler.curConfigStepKey;
              _context23.next = 3;
              return this.deleteAccessorieOnShelfPlanner(this.shelfItem.currentPaletteId, gridIndex, cellIndex, parentBasketId, configStepKey);
            case 3:
              if (this.shelfItem.Grids) {
                _context23.next = 5;
                break;
              }
              return _context23.abrupt("return");
            case 5:
              this.settingsState.ConfigDidChange();
            case 6:
            case "end":
              return _context23.stop();
          }
        }, _callee23, this);
      }));
      function deleteAccessorieOnShelfPlannerView(_x51, _x52, _x53) {
        return _deleteAccessorieOnShelfPlannerView.apply(this, arguments);
      }
      return deleteAccessorieOnShelfPlannerView;
    }()
  }, {
    key: "shelfItemMoved",
    value: function () {
      var _shelfItemMoved = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee24(grid, initialIndex, finalIndex) {
        var _this$settingsState$i8, _this$settingsState$i9;
        return _regeneratorRuntime.wrap(function _callee24$(_context24) {
          while (1) switch (_context24.prev = _context24.next) {
            case 0:
              _context24.next = 2;
              return this.moveShelfItem(this.shelfItem.currentPaletteId, grid, initialIndex, finalIndex, (_this$settingsState$i8 = (_this$settingsState$i9 = this.settingsState.itemPalettes.get(this.shelfItem.currentPaletteId)) === null || _this$settingsState$i9 === void 0 ? void 0 : _this$settingsState$i9.configStepKey) !== null && _this$settingsState$i8 !== void 0 ? _this$settingsState$i8 : this.configuratorStructureHandler.curConfigStepKey);
            case 2:
              this.updateFreePlacesOnGrids(grid.gridGroup, 'move');
              this.updateConfigStepOfProduct();
            case 4:
            case "end":
              return _context24.stop();
          }
        }, _callee24, this);
      }));
      function shelfItemMoved(_x54, _x55, _x56) {
        return _shelfItemMoved.apply(this, arguments);
      }
      return shelfItemMoved;
    }()
  }, {
    key: "clickDeleteAccessorieOnShelfPlanner",
    value: function () {
      var _clickDeleteAccessorieOnShelfPlanner = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee25(grid, cellIndex) {
        var _this$settingsState$i10, _this$settingsState$i11;
        return _regeneratorRuntime.wrap(function _callee25$(_context25) {
          while (1) switch (_context25.prev = _context25.next) {
            case 0:
              _context25.next = 2;
              return this.deleteAccessorieOnShelfPlanner(this.shelfItem.currentPaletteId, grid.ShelfIndex, cellIndex, grid.basketId, (_this$settingsState$i10 = (_this$settingsState$i11 = this.settingsState.itemPalettes.get(this.shelfItem.currentPaletteId)) === null || _this$settingsState$i11 === void 0 ? void 0 : _this$settingsState$i11.configStepKey) !== null && _this$settingsState$i10 !== void 0 ? _this$settingsState$i10 : this.configuratorStructureHandler.curConfigStepKey);
            case 2:
              this.settingsState.ConfigDidChange();
              this.updateFreePlacesOnGrids(grid.gridGroup, 'delete shelf accessoire');
            case 4:
            case "end":
              return _context25.stop();
          }
        }, _callee25, this);
      }));
      function clickDeleteAccessorieOnShelfPlanner(_x57, _x58) {
        return _clickDeleteAccessorieOnShelfPlanner.apply(this, arguments);
      }
      return clickDeleteAccessorieOnShelfPlanner;
    }() //after Element in PLanner changed the Planner PLaette needs to be updated witf filder and possibly second alternative Planner palette
  }, {
    key: "updatePlannerPaletteAfterElementSelectionChanged",
    value: function updatePlannerPaletteAfterElementSelectionChanged(plannerElements, dp) {
      this.plannerItems.calcMatchingArticles(plannerElements, dp);
      this.emptySearchBar(1);
      this.calcFilterItemPalette(1);
      //additional Plannerpalette anzeigen
      if (this.plannerItems.DoShowAdditionalPlannerElements) {
        this.loadCatalogPlannerArticlesLogic(2, true, 'ART', '', 'weiter Planen', 'PLANNER', this.getSelectedElementForReplacement());
        this.calcFilterItemPalette(2);
      } else if (this.settingsState.plannerReplaceMode) {
        this.loadPlannerItems(false);
      } else {
        var palette = this.settingsState.itemPalettes.get(2);
        if (palette) this.settingsState.updateItemPalette(2, _objectSpread$e(_objectSpread$e({}, palette), {}, {
          visible: false
        }));
      }
      this.getUpdatePaletteinfos('ALL');
    }
  }, {
    key: "getSelectedElementForReplacement",
    value: function getSelectedElementForReplacement() {
      var _this$getSelectedElem;
      if (!this.settingsState.plannerReplaceMode) return undefined;
      return (_this$getSelectedElem = this.getSelectedElementsArr()[0]) === null || _this$getSelectedElem === void 0 ? void 0 : _this$getSelectedElem.getArticleCodex();
    }

    /**********************************Update Palette without calling Logic (Filter the Palette)*****************************/

    /**
     * this function is called when the search text was changed
     * @param e event for DOM
     */
  }, {
    key: "onSearchTextChanged",
    value: function onSearchTextChanged(searchValue, paletteId) {
      var curItems = this.getCurItems(paletteId);
      if (curItems instanceof PlannerPalette$1 || curItems instanceof ConfigItems || curItems instanceof ShelfItem) {
        curItems.updateSearchFreeText(searchValue);
        this.calcFilterItemPalette(paletteId);
        if (curItems instanceof PlannerPalette$1) {
          var palette = this.settingsState.itemPalettes.get(2);
          if (palette) this.settingsState.updateItemPalette(2, _objectSpread$e(_objectSpread$e({}, palette), {}, {
            visible: false
          }));
        }
      }
    }
  }, {
    key: "onTitleFavClick",
    value: function onTitleFavClick(paletteId) {
      var curItems = this.getCurItems(paletteId),
        // Testen ob das noch funktioniert
        remove = false;
      if (curItems instanceof ConfigItems) {
        var items = curItems.getAllItems(paletteId),
          op = items[0].optionOcd || items[0].op,
          codexes = [];
        for (var i = 0; i < items.length; i++) {
          codexes.push(items[i].codex);
        }
        items[0].codex;
        this.changeOnFavoritesLogic(op, codexes, this.configuratorStructureHandler.CurConfigStep.targetArticles, remove);
        this.updateCurPaletteData(false);
      }
    }

    /**
     * this function is called when a Filter on the Palette was changed
     * @param e event for DOM
     */
  }, {
    key: "onSelectFilter",
    value: function onSelectFilter(paletteId, codex, filterType) {
      var curItems = this.getCurItems(paletteId);
      switch (filterType) {
        case 'RAL':
          {
            var palette = this.settingsState.itemPalettes.get(paletteId);
            if (palette) {
              this.settingsState.updateItemPalette(paletteId, _objectSpread$e(_objectSpread$e({}, palette), {}, {
                filterValue: codex
              }));
              if (curItems instanceof ConfigItems) {
                this.emptySearchBar(paletteId); //                                  Suche leeren
                curItems.updateColorFilterText(codex); //                           im Datenmodell der ConifgPalette den zu Filternden Text setzten
                this.calcFilterItemPalette(paletteId); //                           Items In der Palette Filtern
              }
            }
            break;
          }
        case 'DockFilter':
          {
            if (curItems instanceof PlannerPalette$1) {
              curItems.changeDockFilter(codex);
              this.calcFilterItemPalette(paletteId);
              this.emptySearchBar(paletteId);
            }
            break;
          }
      }
    }

    /**
     * takes the Filter of the specific Palette and filters its DataSource
     * @param paletteId palette Identifyer
     */
  }, {
    key: "calcFilterItemPalette",
    value: function calcFilterItemPalette(paletteId) {
      var curItems = this.getCurItems(paletteId);
      if (curItems instanceof PlannerPalette$1 || curItems instanceof ConfigItems || curItems instanceof ShelfItem) {
        var palette = this.settingsState.itemPalettes.get(paletteId);
        if (palette) {
          this.settingsState.updateItemPalette(paletteId, _objectSpread$e(_objectSpread$e({}, palette), {}, {
            currentItemsFilter: curItems.calcFilter(paletteId)
          }));
        } else {
          this.filtersToCalc[paletteId] = true;
        }
      }
    }

    /********************************* Update Palette -> Calling to Logic ***************************************************/

    /**
     * this functions calls the spesific loadPalette Function depending on the actState
     */
  }, {
    key: "updateCurPaletteData",
    value: (function () {
      var _updateCurPaletteData = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee26(wasNaviagtion) {
        var _this$shelfItem2;
        var configStepKey,
          curState,
          configSKey,
          _args26 = arguments;
        return _regeneratorRuntime.wrap(function _callee26$(_context26) {
          while (1) switch (_context26.prev = _context26.next) {
            case 0:
              configStepKey = _args26.length > 1 && _args26[1] !== undefined ? _args26[1] : this.configuratorStructureHandler.curConfigStepKey;
              curState = this.getState();
              _context26.next = 4;
              return this.setCurConfigStepKey(configStepKey);
            case 4:
              _context26.t0 = curState;
              _context26.next = _context26.t0 === 'PLANNER' ? 7 : _context26.t0 === 'PRODUCTS' ? 9 : _context26.t0 === 'PROMOTED' ? 11 : _context26.t0 === 'ACC' ? 13 : _context26.t0 === 'ACC_PLANNER' ? 13 : _context26.t0 === 'OPTION' ? 17 : _context26.t0 === 'SUMMARY' ? 21 : _context26.t0 === 'STATICROOMLIST' ? 24 : _context26.t0 === 'SHELFPLANNER' ? 26 : _context26.t0 === 'DOORPLANNER' ? 30 : 32;
              break;
            case 7:
              this.loadPlannerItems(!wasNaviagtion);
              return _context26.abrupt("break", 34);
            case 9:
              this.loadProducts();
              return _context26.abrupt("break", 34);
            case 11:
              this.loadPromoted();
              return _context26.abrupt("break", 34);
            case 13:
              this.settingsState.setDoShowActElementInfo(true);
              _context26.next = 16;
              return this.loadAccessoiresOfActArticles();
            case 16:
              return _context26.abrupt("break", 34);
            case 17:
              this.settingsState.setDoShowActElementInfo(true);
              _context26.next = 20;
              return this.loadConfigItemsOfTabGroup(configStepKey, wasNaviagtion ? 'byNavigation' : 'doConfigure');
            case 20:
              return _context26.abrupt("break", 34);
            case 21:
              //Set in the plannerHandler all elements not selected
              if (!this.doAllowSetPlanner()) this.updateElementSelectionPlanner([], false);
              this.loadSummaryItems();
              return _context26.abrupt("break", 34);
            case 24:
              this.loadStaticRoomListItems();
              return _context26.abrupt("break", 34);
            case 26:
              this.settingsState.setDoShowActElementInfo(true);
              configSKey = configStepKey.startsWith('SHELFARTICLE_') ? configStepKey : (_this$shelfItem2 = this.shelfItem) === null || _this$shelfItem2 === void 0 || (_this$shelfItem2 = _this$shelfItem2.Grids) === null || _this$shelfItem2 === void 0 ? void 0 : _this$shelfItem2[0].gridGroup;
              this.loadShelfPlannerItems(1, configSKey);
              return _context26.abrupt("break", 34);
            case 30:
              this.loadDoorPlannerItems(configStepKey);
              return _context26.abrupt("break", 34);
            case 32:
              LOG$c.error(new DivaError('tabgroup Type: ' + curState + ' not implemented', {
                code: 'WC_0011'
              }));
              return _context26.abrupt("break", 34);
            case 34:
              if (wasNaviagtion) this.updateContentItemAfterNavigation();
            case 35:
            case "end":
              return _context26.stop();
          }
        }, _callee26, this);
      }));
      function updateCurPaletteData(_x59) {
        return _updateCurPaletteData.apply(this, arguments);
      }
      return updateCurPaletteData;
    }())
  }, {
    key: "updateContentItemAfterNavigation",
    value: function updateContentItemAfterNavigation(configStepKey) {
      var _this$configuratorStr3;
      var configSteps = (_this$configuratorStr3 = this.configuratorStructureHandler.loadedConfigStepsOfActTabGroup) !== null && _this$configuratorStr3 !== void 0 ? _this$configuratorStr3 : [];
      var contentItemId = undefined;
      var fromItem = undefined;
      if (!this.settingsState.autoOpenContentItems && !this.settingsState.isContentItemVisible || this.uiState.isInMobileView) {
        this.settingsState.onClickContentItemId(contentItemId, fromItem);
        return;
      }
      for (var i = 0; i < configSteps.length; i++) {
        var _this$settingsState$g, _palette$selectedElem3;
        var step = configSteps[i];
        if (configStepKey) {
          if (configStepKey == step.key) configStepKey = undefined;else continue;
        }
        if (Array.isArray(this.settingsState.autoOpenContentItems) && !this.settingsState.autoOpenContentItems.includes(step.option)) continue;
        var palette = (_this$settingsState$g = this.settingsState.getItemPaletteByConfigStepKey(step.key)) === null || _this$settingsState$g === void 0 ? void 0 : _this$settingsState$g[1];
        var selectedElement = palette === null || palette === void 0 || (_palette$selectedElem3 = palette.selectedElements) === null || _palette$selectedElem3 === void 0 ? void 0 : _palette$selectedElem3[0];
        if (selectedElement !== null && selectedElement !== void 0 && selectedElement.contentItemId) {
          contentItemId = selectedElement.contentItemId;
          fromItem = selectedElement.codex;
          break;
        } else if (selectedElement !== null && selectedElement !== void 0 && selectedElement.groupContentItemId) {
          contentItemId = selectedElement.groupContentItemId;
          fromItem = selectedElement.groupName + (selectedElement.parentName ? '_' + selectedElement.parentName : '');
          break;
        } else if (palette !== null && palette !== void 0 && palette.optionContentItemId) {
          contentItemId = palette === null || palette === void 0 ? void 0 : palette.optionContentItemId;
          fromItem = palette.optionCodex;
          break;
        }
      }
      if (fromItem && this.autoOpenedContentItems[fromItem] && !this.settingsState.isContentItemVisible) return;
      if (contentItemId !== this.settingsState.contentItemId || !this.settingsState.contentItemVisible) if (fromItem) this.autoOpenedContentItems[fromItem] = true;
      this.settingsState.onClickContentItemId(contentItemId, fromItem);
    }

    /**
     * this Function loads plannerPaletteItems from logik
     */
  }, {
    key: "loadPlannerItems",
    value: function loadPlannerItems(isUpdate) {
      this.loadCatalogPlannerArticlesLogic(1, isUpdate, 'ART', 'DockFilter', 'Planen', 'PLANNER', this.getSelectedElementForReplacement());
      if (this.plannerItems.DoShowAdditionalPlannerElements) {
        var _this$uiState$i18nIns11;
        this.loadCatalogPlannerArticlesLogic(2, isUpdate, 'ART', '', (_this$uiState$i18nIns11 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns11 === void 0 ? void 0 : _this$uiState$i18nIns11.t('configurator.continue_planning'), 'PLANNER', this.getSelectedElementForReplacement());
      } else {
        var palette = this.settingsState.itemPalettes.get(2);
        if (palette) this.settingsState.updateItemPalette(2, _objectSpread$e(_objectSpread$e({}, palette), {}, {
          visible: false
        }));
      }
      this.actGroupedConfigSteps = [];
      this.getUpdatePaletteinfos('ALL');
    }
  }, {
    key: "loadProducts",
    value: function loadProducts() {
      var _this$uiState$i18nIns12, _this$uiState$i18nIns13;
      this.settingsState.setPaletteHeaderStepName((_this$uiState$i18nIns12 = (_this$uiState$i18nIns13 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns13 === void 0 ? void 0 : _this$uiState$i18nIns13.t('configurator.products')) !== null && _this$uiState$i18nIns12 !== void 0 ? _this$uiState$i18nIns12 : '');
      this.updateItemPalette('PRODUCTS', '', 'PRODUCTS', 1, false, 'PRODUCTS', true);
      this.settingsState.hideNotUsedPalettes(1);
      this.getUpdatePaletteinfos('ALL');
    }
  }, {
    key: "loadPromoted",
    value: function loadPromoted() {
      var _this$uiState$i18nIns14, _this$uiState$i18nIns15;
      this.settingsState.setPaletteHeaderStepName((_this$uiState$i18nIns14 = (_this$uiState$i18nIns15 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns15 === void 0 ? void 0 : _this$uiState$i18nIns15.t('configurator.promoted')) !== null && _this$uiState$i18nIns14 !== void 0 ? _this$uiState$i18nIns14 : '');
      this.updateItemPalette('PROMOTED', '', 'PROMOTED', 1, false, 'PROMOTED', true);
      this.settingsState.hideNotUsedPalettes(1);
      this.getUpdatePaletteinfos('ALL');
    }

    /**
     * this Function loads AccessoiresItems from logik
     */
  }, {
    key: "loadAccessoiresOfActArticles",
    value: (function () {
      var _loadAccessoiresOfActArticles = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee27() {
        return _regeneratorRuntime.wrap(function _callee27$(_context27) {
          while (1) switch (_context27.prev = _context27.next) {
            case 0:
              this.loadAccessoiresOfArticlesLogic(this.configuratorStructureHandler.actTargetArticles);

              //paletteInfos Updaten
              this.getUpdatePaletteinfos(this.configuratorStructureHandler.curConfigStepKey);
              this.actGroupedConfigSteps = [];
            case 3:
            case "end":
              return _context27.stop();
          }
        }, _callee27, this);
      }));
      function loadAccessoiresOfActArticles() {
        return _loadAccessoiresOfActArticles.apply(this, arguments);
      }
      return loadAccessoiresOfActArticles;
    }()
    /**
     * The function: gets all the ConfigSteps to Group then loads each of them from Logik
     * sets the TabGroup name
     * gets the Palette Infos (Element Legend in ConfigHeader)
     * if the TabGroup gets changed all the configsteps that are closed get reopened
     * if the TabGroup was already visited and no Configstep appears new all ConfigItems get set visited true.
     *
     * @param curConfigStepKey identyfier of ConfigStep
     */
    )
  }, {
    key: "loadConfigItemsOfTabGroup",
    value: (function () {
      var _loadConfigItemsOfTabGroup = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee28(curConfigStepKey, calledFrom) {
        var newGroupedConfigSteps, configStepWasAlreadyLoaded, i, j, isVisited, itemPalette, stepsNotRenderdCount, _i2, configStep, _this$configuratorStr4, _this$getSelectedElem2, optionName, visible;
        return _regeneratorRuntime.wrap(function _callee28$(_context28) {
          while (1) switch (_context28.prev = _context28.next) {
            case 0:
              this.paletteWasLoadedFrom = calledFrom;
              newGroupedConfigSteps = this.configuratorStructureHandler.getConifgStepsToGroup(curConfigStepKey); //Alle selektierbar
              if (!(newGroupedConfigSteps != null)) {
                _context28.next = 47;
                break;
              }
              this.settingsState.hideNotUsedPalettes(newGroupedConfigSteps.length);

              //Tabgroup-Name kann immer gesetzt werden. Wird evtl. vom Konfiguratorschritt später überschrieben falls der Konfiguratorschritt die erlaubniss hat. wird in der Funktion getPaletteNameOfPaletteId() entschieden
              this.settingsState.setPaletteHeaderStepName(this.configuratorStructureHandler.curTabgroupTitle);

              //paletteInfos Updaten
              this.getUpdatePaletteinfos(curConfigStepKey);

              //falls ein konfiguratorschritt erst jetzt eingeblendet wird dort isUpdate doch auf false setzten!
              configStepWasAlreadyLoaded = [];
              i = 0;
            case 8:
              if (!(i < newGroupedConfigSteps.length)) {
                _context28.next = 26;
                break;
              }
              if (!this.configSteps[i]) {
                this.configSteps[i] = new ConfigItems(i);
              }
              this.configSteps[i].IsRendered = false;
              configStepWasAlreadyLoaded[i] = false;
              j = 0;
            case 13:
              if (!(j < this.actGroupedConfigSteps.length)) {
                _context28.next = 20;
                break;
              }
              if (!(this.actGroupedConfigSteps[j].key == newGroupedConfigSteps[i].key)) {
                _context28.next = 17;
                break;
              }
              configStepWasAlreadyLoaded[i] = true;
              return _context28.abrupt("break", 20);
            case 17:
              j++;
              _context28.next = 13;
              break;
            case 20:
              //alle Schritte auf Visited false setzten falls es kein Update ist, oder neue Schritte dazugekommen sind
              //falls die TabGroup bereits besucht wurde werden alle auf isVisited = true gesetzt
              // das hier soll erst später passieren wenn die daten ankommen!
              isVisited = this.configuratorStructureHandler.tabGroupAlreadyVisited();
              itemPalette = this.settingsState.itemPalettes.get(i + 1);
              if (!configStepWasAlreadyLoaded[i] && itemPalette && !itemPalette.configStepKey.startsWith('SHELFARTICLE_')) {
                this.settingsState.itemPalettes.set(itemPalette.id, _objectSpread$e(_objectSpread$e({}, itemPalette), {}, {
                  isVisited: isVisited
                }));
              }
            case 23:
              i++;
              _context28.next = 8;
              break;
            case 26:
              stepsNotRenderdCount = 0;
              _i2 = 0;
            case 28:
              if (!(_i2 < newGroupedConfigSteps.length)) {
                _context28.next = 46;
                break;
              }
              configStep = newGroupedConfigSteps[_i2]; //die einzelnen ConfigPaletten laden. Evtl. können es auch andere arten von Paletten sein (Regalplanner oder Accessoires)
              if (!configStep.differentTabgroupType) {
                _context28.next = 34;
                break;
              }
              this.loadShelfPlannerItemsLogic(_i2 + 1 - stepsNotRenderdCount, configStep.key, 'loadShelfPlannerItemsLogic');
              _context28.next = 43;
              break;
            case 34:
              if (!(configStep.option == 'OPTPLANNERSWITCH')) {
                _context28.next = 39;
                break;
              }
              optionName = (_this$configuratorStr4 = this.configuratorStructureHandler.getConfigStepByKey(configStep.key)) === null || _this$configuratorStr4 === void 0 ? void 0 : _this$configuratorStr4.optionStep.title;
              this.loadCatalogPlannerArticlesLogic(_i2 + 1 - stepsNotRenderdCount, false, 'PLANNERSWITCH', '', optionName, 'PLANNER', (_this$getSelectedElem2 = this.getSelectedElementsArr()[0]) === null || _this$getSelectedElem2 === void 0 ? void 0 : _this$getSelectedElem2.getArticleCodex());
              _context28.next = 43;
              break;
            case 39:
              visible = true;
              if (this.uiState.ViewConfig.hideFirstStepWhenSingleOPVOnly && configStep.singleOPVOnly && _i2 === 0 || this.uiState.ViewConfig.hideSingleOPVOnlyAlways && configStep.singleOPVOnly) {
                visible = false;
              }
              _context28.next = 43;
              return this.loadConfigItemsOfOptionsLogic(configStep.key, _i2 + 1 - stepsNotRenderdCount, configStepWasAlreadyLoaded[_i2], visible);
            case 43:
              _i2++;
              _context28.next = 28;
              break;
            case 46:
              this.actGroupedConfigSteps = _toConsumableArray(newGroupedConfigSteps);
            case 47:
            case "end":
              return _context28.stop();
          }
        }, _callee28, this);
      }));
      function loadConfigItemsOfTabGroup(_x60, _x61) {
        return _loadConfigItemsOfTabGroup.apply(this, arguments);
      }
      return loadConfigItemsOfTabGroup;
    }()
    /**
     * this Function loads SummaryItems from logik
     */
    )
  }, {
    key: "loadSummaryItems",
    value: function loadSummaryItems() {
      this.settingsState.setPaletteHeaderStepName('Summary');
      this.loadSummaryItemsLogic();
      this.actGroupedConfigSteps = [];
      this.getUpdatePaletteinfos('ALL');
    }

    /**
     * this Function loads StaticRoomListItems from logik
     */
  }, {
    key: "loadStaticRoomListItems",
    value: function loadStaticRoomListItems(dontUpdateView) {
      this.loadStaticRoomList(dontUpdateView);
      this.actGroupedConfigSteps = [];
    }
  }, {
    key: "loadShelfPlannerItems",
    value: function loadShelfPlannerItems(paletteId, configStepKey) {
      this.settingsState.hideNotUsedPalettes(paletteId);
      this.getUpdatePaletteinfos(configStepKey);
      this.loadShelfPlannerItemsLogic(paletteId, configStepKey, 'loadShelfPlannerItems');
      this.actGroupedConfigSteps = [];
    }
  }, {
    key: "loadDoorPlannerItems",
    value: function loadDoorPlannerItems(configStepKey) {
      this.settingsState.hideNotUsedPalettes(1);
      this.loadDoorPlannerItemsLogic(configStepKey);
      this.actGroupedConfigSteps = [];
    }
    /**
     * this function gets the Palette Infos From Logik
     * this includes which articles are active at the moment
     * which articles are selectable at the moment
     * and all the specific infos of each of these articles like (image, singlePrice, name, quantity, posNr)
     * @param curConfigStepKey identyfier of ConfigStep
     */
  }, {
    key: "getUpdatePaletteinfos",
    value: function getUpdatePaletteinfos(curConfigStepKey) {
      var articlesOfConfigStep, allArticlesOfTabGroup;
      if (curConfigStepKey != 'ALL') {
        articlesOfConfigStep = this.configuratorStructureHandler.getTargetArticleOfCurConfigStepArr(); //alle selektierten
        allArticlesOfTabGroup = this.configuratorStructureHandler.getAllArticlesOfTabGroup(curConfigStepKey);
      } else {
        var elements = this.getSelectedElementsArr();
        var elemIds = [];
        if (elements.length) {
          for (var i = 0; i < elements.length; i++) {
            elemIds.push(elements[i].getElementId());
          }
        }
        articlesOfConfigStep = this.getBasketIdsOfElementIdLogic(elemIds);
      }
      this.loadPaletteInfosLogic(curConfigStepKey, articlesOfConfigStep, allArticlesOfTabGroup);
    }

    /**
     * Updates all the Palette Infos once. This Funktion Is called just once when something changed.
     * It doesnt matter how many ConifgSteps there are Displayed
     * @param articleInfos  articleInfos contains SinglePrice nam Image etc.
     * @param doShowConfigSingle doShowConfigSingle is the ConfigSingle Checkbox visible
     * @param isConfigSingle is the ConfigSingle Checkbox value
     */
  }, {
    key: "updatePaletteInfos",
    value: function updatePaletteInfos(articleInfos, doShowConfigSingle, isConfigSingle) {
      this.settingsState.setArticleInfos(articleInfos);
      var doShow = undefined;
      //if doShowConfigSingle was set false by the rules of OPVsDontMatch dont show it again.

      doShow = doShowConfigSingle;
      this.settingsState.updateConfigSingle(doShow, isConfigSingle);
      this.updateConfigStepOfProduct();
    }

    //clicked articlePosNr -> navigate there
  }, {
    key: "articleInfoSelected",
    value: function articleInfoSelected(posNr) {
      var article = this.settingsState.articleInfos[posNr];
      if (article) this.configuratorStructureHandler.selectArticleInOption(article.basketId);
      this.updateConfiguratorState(false);
    }

    /**
     * this Function is called when the checkbox conifgsingle is clicked
     * Now we got to:
     * Update the Configstructure
     * Update the actConfigStep -> because it changes from BE_*BasketId* to BE_ALL or
     * Update the act Palette Items
     */
  }, {
    key: "configSingleIsChanged",
    value: (function () {
      var _configSingleIsChanged = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee29() {
        var groupedConfigOptions, allArticlesOfTabGroup, i, configStepKey;
        return _regeneratorRuntime.wrap(function _callee29$(_context29) {
          while (1) switch (_context29.prev = _context29.next) {
            case 0:
              groupedConfigOptions = [];
              allArticlesOfTabGroup = this.configuratorStructureHandler.getAllArticlesOfTabGroup(this.configuratorStructureHandler.curConfigStepKey);
              for (i = 0; i < this.actGroupedConfigSteps.length; i++) {
                groupedConfigOptions.push(this.actGroupedConfigSteps[i].option);
              }
              _context29.next = 5;
              return this.configSingleIsChangedLogic(!this.settingsState.doConfigSingle, this.configuratorStructureHandler.curConfigStepKey, groupedConfigOptions, allArticlesOfTabGroup, this.configuratorStructureHandler.curOptionCodex, this.configuratorStructureHandler.curTabgroupKey);
            case 5:
              configStepKey = _context29.sent;
              this.configuratorStructureHandler.navigateToConfigStepByConfigStepKey(configStepKey);
              this.loadConfigItemsOfTabGroup(configStepKey, 'configSingleIsChanged');
            case 8:
            case "end":
              return _context29.stop();
          }
        }, _callee29, this);
      }));
      function configSingleIsChanged() {
        return _configSingleIsChanged.apply(this, arguments);
      }
      return configSingleIsChanged;
    }()
    /*********************************Palette Updates -> Are Comming From Logic*********************************** */
    /**
     * Updates the Article-List(on PLannerStep) with articles that are comming for logic
     * @param plannerItems articles
     */
    )
  }, {
    key: "updateCatalogPlannerArticles",
    value: function updateCatalogPlannerArticles(plannerItems, paletteId, isUpdate, type, dockFilter, actStepName, stepKey) {
      var _currentPlannerItems$, _plannerItems$catalog;
      var firstTime = false;
      if (this.plannerItems.getAllItems(1).length == 0) {
        firstTime = true;
      }
      var currentPlannerItems = this.plannerItems.getAllItems(paletteId);
      if (!(currentPlannerItems !== null && currentPlannerItems !== void 0 && currentPlannerItems.length) || JSON.stringify(currentPlannerItems.map(function (p) {
        return p.articleCodex;
      })) !== JSON.stringify(plannerItems.catalogPlannerArticles.map(function (p) {
        return p.articleCodex;
      })) || ((_currentPlannerItems$ = currentPlannerItems.find(function (c) {
        return !!c.sel;
      })) === null || _currentPlannerItems$ === void 0 ? void 0 : _currentPlannerItems$.codex) !== ((_plannerItems$catalog = plannerItems.catalogPlannerArticles.find(function (c) {
        return !!c.sel;
      })) === null || _plannerItems$catalog === void 0 ? void 0 : _plannerItems$catalog.codex)) {
        //only update article list if really something changed
        this.plannerItems.setCatalogPlannerStep(plannerItems);
      }
      this.calcFilterItemPalette(1);
      if (this.plannerItems.DoShowAdditionalPlannerElements) this.calcFilterItemPalette(2);
      if (firstTime) this.plannerItems.calcMatchingArticles([]);
      this.settingsState.hideNotUsedPalettes(this.plannerItems.DoShowAdditionalPlannerElements ? 2 : 1);
      this.settingsState.setPaletteHeaderStepName('Planen');
      this.updateItemPalette(type, dockFilter, actStepName, paletteId, isUpdate, stepKey, true);
      if (this.filtersToCalc[paletteId]) {
        delete this.filtersToCalc[paletteId];
        this.calcFilterItemPalette(paletteId);
      }
    }

    /**
     * Updates the Accessoire-List(on AccessoireStep)  with accessoires from the actArticle
     * @param accessoires accessorie Items
     */
  }, {
    key: "updateAccessoiresPalette",
    value: function updateAccessoiresPalette(accessoires) {
      var accType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'ACC';
      this.accItems.setAccItems(accessoires);
      this.updateConfigStepsOfAccessoires(accessoires);
      this.settingsState.setPaletteHeaderStepName('ZUSATZARTIKEL');
      this.updateItemPalette(accType, 'None', 'ZUSATZARTIKEL', 1, false, 'ACCESSOIRES', true);
    }
  }, {
    key: "updateConfigStepsOfAccessoires",
    value: function updateConfigStepsOfAccessoires(accessoires) {
      var count = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
      for (var i = 0; i < accessoires.length; i++) {
        var acc = accessoires[i];
        if (acc.configStepKeys) {
          for (var j = 0; j < acc.configStepKeys.length; j++) {
            var _this$settingsState$i12;
            var stepKey = acc.configStepKeys[j];
            count++;
            if (!this.configSteps[count]) this.configSteps[count] = new ConfigItems(count);
            stepKey.id = count;
            var isUpdate = ((_this$settingsState$i12 = this.settingsState.itemPalettes.get(stepKey.id)) === null || _this$settingsState$i12 === void 0 ? void 0 : _this$settingsState$i12.configStepKey) == stepKey.key;
            this.loadConfigItemsOfOptionsLogic(stepKey.key, count, isUpdate, true);
          }
        }
      }
      this.settingsState.hideNotUsedPalettes(count);
    }
  }, {
    key: "setSelectedShelfAccessoire",
    value: function setSelectedShelfAccessoire(basketId) {
      this.settingsState.setSelectedShelfAccessoire(basketId);
      this.updateConfigStepOfProduct();
    }
  }, {
    key: "updateConfigStepOfProduct",
    value: function updateConfigStepOfProduct() {
      var configSteps = [];
      var configstepsDisplaySeparatOnTabgroup = this.configuratorStructureHandler.configstepsDisplaySeparatOnTabgroup;
      var currentSelectedArticles = [].concat(_toConsumableArray(this.settingsState.getSelectedBasketIds), _toConsumableArray(this.settingsState.selectedShelfAccessoire ? [this.settingsState.selectedShelfAccessoire] : []));
      var currentTabGroupType = this.configuratorStructureHandler.curTabgroupType;
      if (currentTabGroupType && configstepsDisplaySeparatOnTabgroup[currentTabGroupType]) {
        var tabgroup = configstepsDisplaySeparatOnTabgroup[currentTabGroupType];
        for (var i = 0; i < currentSelectedArticles.length; i++) {
          var stepsOfArticle = tabgroup[currentSelectedArticles[i]];
          if (stepsOfArticle) configSteps = configSteps.concat(stepsOfArticle);
        }
        var count = 2;
        for (var _i3 = 0; _i3 < configSteps.length; _i3++) {
          var _this$settingsState$i13;
          var configstep = configSteps[_i3];
          count++;
          if (!this.configSteps[count]) this.configSteps[count] = new ConfigItems(count);
          var isUpdate = ((_this$settingsState$i13 = this.settingsState.itemPalettes.get(count)) === null || _this$settingsState$i13 === void 0 ? void 0 : _this$settingsState$i13.configStepKey) == configstep.key;
          this.loadConfigItemsOfOptionsLogic(configstep.key, count, isUpdate, true);
        }
        this.settingsState.hideNotUsedPalettes(count);
      }
    }

    /**
     * Updates all on one specific ConfigPalette (can be more than one!)
     * @param configItems selectable / Visible ConfigItems
     * @param actStepName Title of the ConfigPalette
     * @param paletteType Palette Template e.g: GAL for Gallery
     * @param filterType Filter Type e.g: RAL for Colorfilter
     * @param paletteId iD of the ConfigPalette
     */
  }, {
    key: "updateConfigItems",
    value: function updateConfigItems(configStepKey, configItems, actStepName, paletteType, filterType, paletteId, isUpdate, visible, stepInfo, isRenderedInAccessorieStep, isRenderedNotInSettingPalette, optionCodex, notTotallyMatichingOPVs, subTabGroup, pairWithOptionsToGroup, selectedItems, isMultiselect, groupsDefaultClosed, contentItemId) {
      this.configSteps[paletteId - 1].setConfigItems(configItems);
      this.configSteps[paletteId - 1].ConfigStepKey = configStepKey;
      this.updateItemPalette(paletteType, filterType, actStepName, paletteId, isUpdate, configStepKey, visible, stepInfo, undefined, undefined, isRenderedInAccessorieStep, isRenderedNotInSettingPalette, optionCodex, subTabGroup, pairWithOptionsToGroup, selectedItems, isMultiselect, groupsDefaultClosed, contentItemId);
      if (notTotallyMatichingOPVs) {
        if (this.paletteWasLoadedFrom == 'byNavigation') {
          if (!this.notTotallyMatichingOPVs && !this.settingsState.doConfigSingle) {
            this.notTotallyMatichingOPVs = optionCodex;
            this.rerenderPaletteWithConfigSingeIfOPVsDontMatch();
          }
          this.getCurItems(paletteId).IsRendered = true;
          return;
        } else if (this.paletteWasLoadedFrom == 'configSingleIsChanged') {
          this.updatePlannerMsgContainer('OPVS_OF_ARTICLES_DONT_MATCH', true, 'WARNING', 'OPVS_OF_ARTICLES_DONT_MATCH', true);
        }
      } else if (this.notTotallyMatichingOPVs && this.notTotallyMatichingOPVs !== optionCodex) {
        if (this.paletteWasLoadedFrom == 'byNavigation' && this.settingsState.doConfigSingle) {
          this.notTotallyMatichingOPVs = undefined;
          this.resetConfigSingeIfOPVsMatchAgain();
          this.getCurItems(paletteId).IsRendered = true;
        }
      }
    }

    /**
     * Updates the summary Items
     * @param summaryItems Items in the Summary
     */
  }, {
    key: "updateSummaryItems",
    value: function updateSummaryItems(ret, articlePriceFormat, stepInfo, checkoutConfig) {
      var addToIndex = 0;
      if (checkoutConfig !== null && checkoutConfig !== void 0 && checkoutConfig.buttons.filter(function (b) {
        return b.visibleInOverview !== false;
      }).length) {
        this.updateItemPalette('CHECKOUT', '', 'Checkout', 1, false, 'SUMMARY', true, stepInfo, this.productState.formatedTotalPrice);
        addToIndex = addToIndex + 1;
      }
      for (var i = 0; i < ret.length; i++) {
        var summaryPalette = ret[i];
        var ajustedIndex = i + addToIndex;
        if (!this.summaryItems[ajustedIndex]) {
          this.summaryItems[ajustedIndex] = new PlannerPalette();
        }
        this.summaryItems[ajustedIndex].setSummaryItems(summaryPalette.items, articlePriceFormat);
        this.updateItemPalette(summaryPalette.type, '', 'Summary', ajustedIndex + 1, false, 'SUMMARY', true, stepInfo);
      }
      this.settingsState.hideNotUsedPalettes(ret.length + addToIndex);
    }
  }, {
    key: "updateStaticRoomListItems",
    value: function updateStaticRoomListItems(roomListItems, dontUpdateView) {
      this.staticRoomItems.StaticRoomItems = roomListItems;
      if (!dontUpdateView) {
        this.updateItemPalette('STATICROOM', '', 'Raumauswahl', 1, false, 'STATICROOM', true);
        this.updateItemPalette('VIEWPOINT', '', 'Blickpunkte', 2, false, 'VIEWPOINT', true);
        this.settingsState.hideNotUsedPalettes(2);
      }
      if (roomListItems.length > 0) this.settingsState.roomListCountUp();
    }
  }, {
    key: "updateShelfPlannerItems",
    value: function updateShelfPlannerItems(paletteId, accessoires, grids, configStepKey, from) {
      this.shelfItem.AccItems = accessoires;
      this.shelfItem.Grids = grids;
      this.shelfItem.currentPaletteId = paletteId;
      this.updateItemPalette('SHELFPLANNER', '', 'Regalplanner', paletteId, false, configStepKey, true, undefined, undefined, undefined, undefined, undefined, grids[0].gridGroup);
      this.updateFreePlacesOnGrids(configStepKey, from);
      this.updateConfigStepsOfAccessoires(accessoires, paletteId);
    }
  }, {
    key: "updateDoorPlannerItems",
    value: function updateDoorPlannerItems(doorPlannerItems, configStepKey) {
      this.specialItem.AccItems = doorPlannerItems;
      this.updateItemPalette('DOOR_PLANNER', '', 'DoorPlanner', 1, false, configStepKey, true);
    }

    /**
     * This Function renders spesific Items in one of the many possibe ItemPalettes. Its also handles the Title, Filters and a part of the Visit logic(first visible Steps get Visited here)
     * @param paletteType   decides which Template is Used for the Items
     * @param filterType    decides which and if a Filter is renderd in View
     * @param actStepName   title of step
     * @param paletteId     indication of ItemPalette
     * @param isUpdate      was the Palette loaded already bevore and it is just an Update (for example new selected element) -> filters dont need to be renderd again. if not it means Items and Filter wasnt here before
     * @param selectedItem  is the selected Item. Is used for example at textures to show the selected item next to the title
     */
  }, {
    key: "updateItemPalette",
    value: function updateItemPalette(paletteType, filterType, actStepName, paletteId, isUpdate, configStepKey, visible, stepInfo, price, quantity, isRenderedInAccessorieStep, isRenderedNotInSettingPalette, optionCodex, subTabGroup, pairWithOptionsToGroup, selectedElements, isMultiselect, groupsDefaultClosed, contentItemId) {
      var _this7 = this,
        _step$configStep$opti,
        _step$configStep$opti3;
      var existingItemPalette = this.settingsState.itemPalettes.get(paletteId);
      var items = [];
      var itemPalette;
      var isConfigHeader = (this.uiState.ViewConfig.headerInConfigPalette == 'NameOfOption' || this.uiState.ViewConfig.headerInConfigPalette == 'NameOfTabGroup' || actStepName == this.configuratorStructureHandler.curTabgroupTitle) && this.configuratorStructureHandler.curTabgroupType == 'OPTION' || this.configuratorStructureHandler.curTabgroupType != 'OPTION';
      if (isUpdate && existingItemPalette && existingItemPalette.configStepKey === configStepKey) {
        var _selectedElements$;
        itemPalette = _objectSpread$e(_objectSpread$e({}, existingItemPalette), {}, {
          id: paletteId,
          filterComponent: filterType,
          visible: visible,
          configStepKey: configStepKey,
          type: paletteType,
          hideHeader: false,
          price: price,
          isRenderedNotInSettingPalette: !!isRenderedNotInSettingPalette,
          isRenderedInAccessorieStep: !!isRenderedInAccessorieStep,
          quantity: quantity,
          info: undefined,
          subTabGroup: subTabGroup,
          pairWithOptionsToGroup: pairWithOptionsToGroup,
          selectedElements: selectedElements,
          numberInput: selectedElements !== null && selectedElements !== void 0 && (_selectedElements$ = selectedElements[0]) !== null && _selectedElements$ !== void 0 && _selectedElements$.isFreeText ? Number(selectedElements[0].freeText) : undefined,
          isMultiselect: isMultiselect,
          optionContentItemId: contentItemId
        });
      } else {
        var _selectedElements$2;
        itemPalette = {
          filterComponent: filterType,
          filterVisible: false,
          isArrowFolded: false,
          groups: [],
          name: '',
          id: paletteId,
          searchVisible: false,
          info: undefined,
          type: paletteType,
          isHeader: false,
          currentItemsFilter: configStepKey === 'PLANNER' ? this.plannerItems.calcFilter(paletteId) : undefined,
          visible: visible,
          htmlInputElement: existingItemPalette === null || existingItemPalette === void 0 ? void 0 : existingItemPalette.htmlInputElement,
          isVisited: !!subTabGroup || !!(existingItemPalette !== null && existingItemPalette !== void 0 && existingItemPalette.isVisited) && existingItemPalette.configStepKey === configStepKey,
          htmlRef: existingItemPalette === null || existingItemPalette === void 0 ? void 0 : existingItemPalette.htmlRef,
          configStepKey: configStepKey,
          filterValue: '',
          optionCodex: optionCodex,
          hideHeader: false,
          price: price,
          isRenderedNotInSettingPalette: !!isRenderedNotInSettingPalette,
          isRenderedInAccessorieStep: !!isRenderedInAccessorieStep,
          quantity: quantity,
          subTabGroup: subTabGroup,
          pairWithOptionsToGroup: pairWithOptionsToGroup,
          selectedElements: selectedElements,
          numberInput: selectedElements !== null && selectedElements !== void 0 && (_selectedElements$2 = selectedElements[0]) !== null && _selectedElements$2 !== void 0 && _selectedElements$2.isFreeText ? Number(selectedElements[0].freeText) : undefined,
          isMultiselect: isMultiselect,
          searchText: undefined,
          optionContentItemId: contentItemId
        };
      }
      var paletteName = this.getPaletteNameOfPaletteId(paletteId, filterType, actStepName, visible);
      itemPalette.isHeader = paletteName === 'Header' && isConfigHeader;
      itemPalette.hideHeader = this.getHidePaletteHeader(paletteId, visible, itemPalette);
      this.setStepInfo(itemPalette, stepInfo);
      var curItems = this.getCurItems(itemPalette);
      if (curItems) {
        items = curItems.getAllItems(paletteId);
        if (curItems instanceof ConfigItems) {
          curItems.IsRendered = true;
          if (this.checkIfAllItemsAreRendered()) {
            setTimeout(function () {
              _this7.visitActVisibleSteps();
            }, 100);
          }
        }
        itemPalette.name = actStepName;
        if (paletteType == 'TEXTBOX' && curItems instanceof ConfigItems) {
          var freeTextItem = curItems.getAllItems(paletteId);
          if (freeTextItem[0]) {
            this.bindListenerForTextBox(paletteName, actStepName, freeTextItem[0]);
          }
        }
        if (filterType == 'RAL' && itemPalette.type == 'GAL' && curItems.getAllItems(paletteId).length <= 8 || filterType == 'RAL' && itemPalette.type != 'GAL' && curItems.getAllItems(paletteId).length <= 5 || itemPalette.type == 'GAL_SMALL_GROUP_DROPDOWN' || itemPalette.type == 'GAL_GROUP_DROPDOWN') {
          filterType = '';
        }
      }
      itemPalette.filterComponent = filterType;
      if (itemPalette.filterComponent && itemPalette.filterComponent != 'None' && itemPalette.type != 'GAL_SMALL_GROUP_DROPDOWN' && itemPalette.type != 'GAL_GROUP_DROPDOWN') {
        itemPalette.filterVisible = true;
        itemPalette.searchVisible = true;
      }
      if (itemPalette.type == 'GAL') {
        itemPalette.searchVisible = true;
      }
      if (itemPalette.type == 'SHELFPLANNER') {
        itemPalette.searchVisible = true;
      }
      if (this.uiState.ViewConfig.alwaysShowSearchBar && ['ACC', 'ART', 'GAL', 'GAL_BIG', 'GAL_GROUP_DROPDOWN', 'GAL_SMALL_GROUP_DROPDOWN', 'SMALL', 'VERY_SMALL'].includes(itemPalette.type)) {
        itemPalette.searchVisible = true;
      }
      var step = this.configuratorStructureHandler.getConfigStepByKey(configStepKey);
      if ((step === null || step === void 0 || (_step$configStep$opti = step.configStep.optionstate) === null || _step$configStep$opti === void 0 ? void 0 : _step$configStep$opti.option.showSearchbar) !== undefined) {
        var _step$configStep$opti2;
        itemPalette.searchVisible = step === null || step === void 0 || (_step$configStep$opti2 = step.configStep.optionstate) === null || _step$configStep$opti2 === void 0 ? void 0 : _step$configStep$opti2.option.showSearchbar;
      }
      if ((step === null || step === void 0 || (_step$configStep$opti3 = step.configStep.optionstate) === null || _step$configStep$opti3 === void 0 ? void 0 : _step$configStep$opti3.option.showFilter) !== undefined) {
        var _step$configStep$opti4;
        itemPalette.filterVisible = step === null || step === void 0 || (_step$configStep$opti4 = step.configStep.optionstate) === null || _step$configStep$opti4 === void 0 ? void 0 : _step$configStep$opti4.option.showFilter;
      }
      var groupsObj = [];
      var _iterator = _createForOfIteratorHelper$2(items),
        _step;
      try {
        var _loop2 = function _loop2() {
          var item = _step.value;
          var groupName = item.groupName;
          if (!groupName) {
            groupName = 'none';
          }
          var groupNameKey = groupName;
          var parentName = item.parentName;
          if (parentName) {
            groupNameKey = groupNameKey + '_' + parentName;
          }
          var oldGroup = itemPalette.groups.find(function (g) {
            return g.key === groupNameKey;
          });
          var group = groupsObj.find(function (g) {
            return g.key === groupNameKey;
          });
          if (!group) {
            var _oldGroup$isFolded, _this7$uiState$i18nIn, _this7$uiState$i18nIn2;
            groupsObj.push({
              isFolded: (_oldGroup$isFolded = oldGroup === null || oldGroup === void 0 ? void 0 : oldGroup.isFolded) !== null && _oldGroup$isFolded !== void 0 ? _oldGroup$isFolded : groupsDefaultClosed || !!_this7.isGroupFolded(undefined, items.length, groupsObj),
              items: [item],
              name: (_this7$uiState$i18nIn = (_this7$uiState$i18nIn2 = _this7.uiState.i18nInstance) === null || _this7$uiState$i18nIn2 === void 0 ? void 0 : _this7$uiState$i18nIn2.t('configuration.group.' + groupName, groupName)) !== null && _this7$uiState$i18nIn !== void 0 ? _this7$uiState$i18nIn : groupName,
              parentName: parentName,
              contentItemId: item.groupContentItemId,
              key: groupNameKey
            });
          } else {
            var groupContentItemId = item.groupContentItemId;
            if (groupContentItemId && !group.contentItemId) group.contentItemId = groupContentItemId;
            group.items.push(item);
          }
        };
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          _loop2();
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
      itemPalette.groups = groupsObj;
      if (itemPalette.type == 'SHELFPLANNER') {
        itemPalette.shelfPlannerGrids = this.shelfItem.Grids;
      }
      this.settingsState.updateItemPalette(paletteId, itemPalette);
      if (!(curItems instanceof ConfigItems)) {
        this.visitActVisibleSteps();
      }
    }
  }, {
    key: "setStepInfo",
    value: function setStepInfo(itemPalette, stepInfo) {
      if (stepInfo === undefined) return;
      switch (stepInfo) {
        case 'SHOW_NOT_VISUALIZED_WARNINIG':
          {
            var _this$uiState$i18nIns16, _this$uiState$i18nIns17;
            itemPalette.info = {
              type: 'INFO',
              text: (_this$uiState$i18nIns16 = (_this$uiState$i18nIns17 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns17 === void 0 ? void 0 : _this$uiState$i18nIns17.t('configurator.TEXTURE_NOT_DIGITALIZED_WARNING_SHORT')) !== null && _this$uiState$i18nIns16 !== void 0 ? _this$uiState$i18nIns16 : ''
            };
            break;
          }
        case 'requiresUserInput':
          {
            var _this$uiState$i18nIns18, _this$uiState$i18nIns19;
            itemPalette.info = {
              type: 'ERROR',
              text: (_this$uiState$i18nIns18 = (_this$uiState$i18nIns19 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns19 === void 0 ? void 0 : _this$uiState$i18nIns19.t('configurator.requiresUserInput')) !== null && _this$uiState$i18nIns18 !== void 0 ? _this$uiState$i18nIns18 : ''
            };
            break;
          }
        case 'measureWarning':
          {
            var _this$uiState$i18nIns20, _this$uiState$i18nIns21;
            itemPalette.info = {
              hideDuringUserInput: this.uiState.ViewConfig.hideWarningDuringUserInput,
              text: (_this$uiState$i18nIns20 = (_this$uiState$i18nIns21 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns21 === void 0 ? void 0 : _this$uiState$i18nIns21.t('configurator.measureWarning', {
                min: itemPalette.selectedElements[0].measureMinimum,
                max: itemPalette.selectedElements[0].measureMaximum,
                measure: ', in ' + itemPalette.selectedElements[0].measureUnit + ', '
              })) !== null && _this$uiState$i18nIns20 !== void 0 ? _this$uiState$i18nIns20 : '',
              type: 'ERROR'
            };
            break;
          }
        default:
          {
            if (itemPalette.type == 'VARIANTS') {
              var _this$uiState$i18nIns22, _this$uiState$i18nIns23;
              itemPalette.info = {
                type: 'ERROR',
                text: stepInfo == 'ERROR' ? (_this$uiState$i18nIns22 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns22 === void 0 ? void 0 : _this$uiState$i18nIns22.t('configurator.variants_error') : ((_this$uiState$i18nIns23 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns23 === void 0 ? void 0 : _this$uiState$i18nIns23.t('configurator.variants_option_error')) + '<br/><br/>' + stepInfo
              };
              break;
            }
            itemPalette.info = {
              type: 'ERROR',
              text: stepInfo
            };
            break;
          }
      }
    }
    /**
     * this function defineds which groups are folded on initializiation
     * keep only one group open due to performance issues. But only when there are more than 150 items in total
     * @param paletteId
     * @returns
     */
  }, {
    key: "isGroupFolded",
    value: function isGroupFolded(paletteId, length, groupsObj) {
      var isNotFirstGroup = (groupsObj === null || groupsObj === void 0 ? void 0 : groupsObj.length) != 0;
      if (this.uiState.ViewConfig.keepOnlyOneGroupOpen) {
        if (length && length > 150) return true;
        if (paletteId !== undefined) {
          var palette = this.settingsState.itemPalettes.get(paletteId);
          if (palette) {
            var itemsCount = 0;
            palette.groups.map(function (g) {
              return itemsCount = g.items.length + itemsCount;
            });
            if (itemsCount > 150) return isNotFirstGroup;
          }
        }
      }
      return false;
    }
  }, {
    key: "checkIfAllItemsAreRendered",
    value:
    /**
     * this function checks if all Steps are rendered
     */
    function checkIfAllItemsAreRendered() {
      var curConfigPaletteCount = this.configuratorStructureHandler.actGroupedConfigStepsCount;
      var allAreRendered = true;
      for (var i = 0; i < curConfigPaletteCount; i++) {
        if (!this.configSteps[i].IsRendered) {
          allAreRendered = false;
        }
      }
      return allAreRendered;
    }
  }, {
    key: "rerenderPaletteWithConfigSingeIfOPVsDontMatch",
    value: function () {
      var _rerenderPaletteWithConfigSingeIfOPVsDontMatch = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee30() {
        var i;
        return _regeneratorRuntime.wrap(function _callee30$(_context30) {
          while (1) switch (_context30.prev = _context30.next) {
            case 0:
              i = 0;
            case 1:
              if (!(!this.checkIfAllItemsAreRendered() && i < 50)) {
                _context30.next = 7;
                break;
              }
              _context30.next = 4;
              return DivaUtils.Sleep(20);
            case 4:
              i++;
              _context30.next = 1;
              break;
            case 7:
              this.configSingleIsChanged();
            case 8:
            case "end":
              return _context30.stop();
          }
        }, _callee30, this);
      }));
      function rerenderPaletteWithConfigSingeIfOPVsDontMatch() {
        return _rerenderPaletteWithConfigSingeIfOPVsDontMatch.apply(this, arguments);
      }
      return rerenderPaletteWithConfigSingeIfOPVsDontMatch;
    }()
  }, {
    key: "resetConfigSingeIfOPVsMatchAgain",
    value: function () {
      var _resetConfigSingeIfOPVsMatchAgain = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee31() {
        var i;
        return _regeneratorRuntime.wrap(function _callee31$(_context31) {
          while (1) switch (_context31.prev = _context31.next) {
            case 0:
              i = 0;
            case 1:
              if (!(!this.checkIfAllItemsAreRendered() && i < 50)) {
                _context31.next = 7;
                break;
              }
              _context31.next = 4;
              return DivaUtils.Sleep(20);
            case 4:
              i++;
              _context31.next = 1;
              break;
            case 7:
              this.configSingleIsChanged();
            case 8:
            case "end":
              return _context31.stop();
          }
        }, _callee31, this);
      }));
      function resetConfigSingeIfOPVsMatchAgain() {
        return _resetConfigSingeIfOPVsMatchAgain.apply(this, arguments);
      }
      return resetConfigSingeIfOPVsMatchAgain;
    }() //HelperFunctions
    /**
     * the PaletteName can be "1","2" ... or "Header"
     * @param paletteId
     */
  }, {
    key: "getPaletteNameOfPaletteId",
    value: function getPaletteNameOfPaletteId(paletteId, filterType, actStepName, visible) {
      var firstVisiblePaletteId = 1;
      while (paletteId >= firstVisiblePaletteId && !(visible && paletteId == firstVisiblePaletteId || (_this$settingsState$i14 = this.settingsState.itemPalettes.get(firstVisiblePaletteId)) !== null && _this$settingsState$i14 !== void 0 && _this$settingsState$i14.visible && paletteId !== firstVisiblePaletteId)) {
        var _this$settingsState$i14;
        firstVisiblePaletteId++;
      }
      var paletteName = paletteId.toString();
      if (paletteId == firstVisiblePaletteId) {
        switch (this.getState()) {
          case 'OPTION':
            if (filterType != 'RAL') paletteName = 'Header';
          case 'PRODUCTS':
          case 'PROMOTED':
          case 'PLANNER':
          case 'SUMMARY':
            if (!this.doAllowSetPlanner()) {
              paletteName = 'Header';
            }
          case 'ACC_PLANNER':
          case 'ACC':
            if (actStepName == this.configuratorStructureHandler.curTabgroupTitle) {
              paletteName = 'Header';
            }
            break;
          case 'DOORPLANNER':
          case 'SHELFPLANNER':
          case 'STATICROOMLIST':
            {
              paletteName = 'Header';
            }
        }
      }
      return this.uiState.isInMobileView && this.uiState.ViewConfig.hideTopHeaderInMobile ? paletteId.toString() : paletteName;
    }
  }, {
    key: "getHidePaletteHeader",
    value: function getHidePaletteHeader(paletteId, visible, itemPalette) {
      var afterToggelHeader_NextHeaderHidden = 'dontNeedTo';
      if (itemPalette.isRenderedInAccessorieStep) return true;
      for (var i = 1; i <= paletteId; i++) {
        var _this$settingsState$i15;
        if (((_this$settingsState$i15 = this.settingsState.itemPalettes.get(i)) === null || _this$settingsState$i15 === void 0 ? void 0 : _this$settingsState$i15.type) == 'TOGGLE_HEADER') {
          if (afterToggelHeader_NextHeaderHidden == 'dontNeedTo') afterToggelHeader_NextHeaderHidden = false;
          continue;
        }
        if (afterToggelHeader_NextHeaderHidden == false) {
          var _this$settingsState$i16;
          if ((_this$settingsState$i16 = this.settingsState.itemPalettes.get(i)) !== null && _this$settingsState$i16 !== void 0 && _this$settingsState$i16.visible || i == paletteId && visible) {
            var _this$settingsState$i17, _this$settingsState$i18;
            afterToggelHeader_NextHeaderHidden = true;
            if (((_this$settingsState$i17 = this.settingsState.itemPalettes.get(i)) === null || _this$settingsState$i17 === void 0 ? void 0 : _this$settingsState$i17.type) != 'DROPDOWN_TEXT' || ((_this$settingsState$i18 = this.settingsState.itemPalettes.get(i)) === null || _this$settingsState$i18 === void 0 ? void 0 : _this$settingsState$i18.type) != 'TOGGLE') {
              if (i == paletteId) {
                return true;
              }
            }
          }
        }
      }
      return false;
    }

    /**
     * this functions get the spesific paletteItems depending on the actState and paletteId
     *
     * @param paletteId palette identifyer
     */
  }, {
    key: "getCurItems",
    value: function getCurItems(palette) {
      var itemPalette = typeof palette == 'number' ? this.settingsState.itemPalettes.get(palette) : palette;
      if (!itemPalette) return undefined;
      var paletteId = typeof palette == 'number' ? palette : itemPalette.id;
      if (itemPalette.isRenderedNotInSettingPalette) {
        return this.configSteps[paletteId - 1];
      }
      if (itemPalette.type == 'PLANNERSWITCH') {
        return this.plannerItems;
      }
      switch (itemPalette.type) {
        case 'PRODUCTS':
        case 'PROMOTED':
          return undefined;
        case 'ART':
          return this.plannerItems;
        case 'ACC':
        case 'ACC_CONFIG':
          if (paletteId != 1) return this.configSteps[paletteId - 1];
          return this.accItems;
        case 'SUMMARY_ARTICLES':
        case 'SUMMARY_OPTIONTREE':
        case 'SUMMARY_PARTLIST':
        case 'VARIANTS':
          return this.summaryItems[paletteId - 1];
        case 'STATICROOM':
        case 'VIEWPOINT':
          return this.staticRoomItems;
        case 'SHELFPLANNER':
          return this.shelfItem;
        case 'DOOR_PLANNER':
          return this.specialItem;
        case 'SMALL':
        case 'GAL':
        case 'GAL_BIG':
        case 'GAL_SMALL':
        case 'GAL_BIG_TEXT':
        case 'GAL_GROUP_DROPDOWN':
        case 'GAL_SMALL_GROUP_DROPDOWN':
        case 'GAL_SMALL_TEXT':
        case 'GAL_TEXT':
        default:
          return this.configSteps[paletteId - 1];
      }
      return undefined;
    }
  }, {
    key: "getRoomList",
    value: function getRoomList() {
      return this.staticRoomItems.getAllItems(1);
    }
  }, {
    key: "getRoomViewList",
    value: function getRoomViewList() {
      return this.staticRoomItems.getAllItems(0);
    }

    /**
     * this function scrolls to the Top of the ItemPalette
     */
  }, {
    key: "scrollToTopInItemPalette",
    value: function scrollToTopInItemPalette(smooth) {
      var _this$settingsState$s;
      var props = {
        left: 0,
        top: 0
      };
      if (smooth) {
        props['behavior'] = 'smooth';
      }
      (_this$settingsState$s = this.settingsState.settingsScrollerElement) === null || _this$settingsState$s === void 0 || _this$settingsState$s.scrollTo(props);
    }

    /**
     * this function gets the index of the Step by ConfigStepByPosNr waits till everything is rendered then calls scrollToConfigStep
     * @param posNr ConfigStepByPosNr of the step to which we have to scroll
     * @param withDelay if true waits 400 ms and then calls scrollToConfigStep
     */
  }, {
    key: "scrollToConfigStepByPosNr",
    value: (function () {
      var _scrollToConfigStepByPosNr = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee32(posNr, withDelay) {
        var i, ret, j;
        return _regeneratorRuntime.wrap(function _callee32$(_context32) {
          while (1) switch (_context32.prev = _context32.next) {
            case 0:
              ret = this.configuratorStructureHandler.getConfigStepByPosNr(posNr);
              i = 0;
            case 2:
              if (!(i < this.configuratorStructureHandler.actGroupedConfigStepsCount)) {
                _context32.next = 8;
                break;
              }
              if (!(this.configSteps[i].ConfigStepKey == (ret === null || ret === void 0 ? void 0 : ret.configStep.key))) {
                _context32.next = 5;
                break;
              }
              return _context32.abrupt("break", 8);
            case 5:
              i++;
              _context32.next = 2;
              break;
            case 8:
              j = 0;
            case 9:
              if (!(!this.checkIfAllItemsAreRendered() && j < 100)) {
                _context32.next = 15;
                break;
              }
              j++;
              _context32.next = 13;
              return DivaUtils.Sleep(50);
            case 13:
              _context32.next = 9;
              break;
            case 15:
              if (!withDelay) {
                _context32.next = 18;
                break;
              }
              _context32.next = 18;
              return DivaUtils.Sleep(400);
            case 18:
              this.scrollToConfigStep(i + 1);
            case 19:
            case "end":
              return _context32.stop();
          }
        }, _callee32, this);
      }));
      function scrollToConfigStepByPosNr(_x62, _x63) {
        return _scrollToConfigStepByPosNr.apply(this, arguments);
      }
      return scrollToConfigStepByPosNr;
    }()
    /**
     * this function actually scrolls to the configStep in the View
     * @param step index of Step to which we have to scroll
     */
    )
  }, {
    key: "scrollToConfigStep",
    value: function scrollToConfigStep(step) {
      var paletteItem = this.settingsState.itemPalettes.get(step);
      if (paletteItem) {
        var _paletteItem$htmlRef$, _paletteItem$htmlRef, _this$settingsState$s2, _this$settingsState$s3, _this$settingsState$s4, _this$settingsState$s5, _this$settingsState$s6;
        var scrollTo = ((_paletteItem$htmlRef$ = (_paletteItem$htmlRef = paletteItem.htmlRef) === null || _paletteItem$htmlRef === void 0 ? void 0 : _paletteItem$htmlRef.getBoundingClientRect().top) !== null && _paletteItem$htmlRef$ !== void 0 ? _paletteItem$htmlRef$ : 0) - ((_this$settingsState$s2 = (_this$settingsState$s3 = this.settingsState.settingsScrollerElement) === null || _this$settingsState$s3 === void 0 ? void 0 : _this$settingsState$s3.getBoundingClientRect().top) !== null && _this$settingsState$s2 !== void 0 ? _this$settingsState$s2 : 0) + ((_this$settingsState$s4 = (_this$settingsState$s5 = this.settingsState.settingsScrollerElement) === null || _this$settingsState$s5 === void 0 ? void 0 : _this$settingsState$s5.scrollTop) !== null && _this$settingsState$s4 !== void 0 ? _this$settingsState$s4 : 0);
        (_this$settingsState$s6 = this.settingsState.settingsScrollerElement) === null || _this$settingsState$s6 === void 0 || _this$settingsState$s6.scrollTo({
          left: 0,
          top: scrollTo,
          behavior: 'smooth'
        });
      }
      this.updateContentItemAfterNavigation(paletteItem === null || paletteItem === void 0 ? void 0 : paletteItem.configStepKey);
    }

    /**
     * this function emptys the searchbar of an specific itemPalette
     * @param paletteId identifyer
     */
  }, {
    key: "emptySearchBar",
    value: function emptySearchBar(paletteId) {
      var palette = this.settingsState.itemPalettes.get(paletteId);
      if (palette) this.settingsState.updateItemPalette(paletteId, _objectSpread$e(_objectSpread$e({}, palette), {}, {
        searchText: undefined
      }));
    }
  }, {
    key: "focusSearch",
    value: function focusSearch(paletteId) {
      var _this$settingsState$i19;
      var palette = paletteId !== undefined ? this.settingsState.itemPalettes.get(paletteId) : (_this$settingsState$i19 = this.settingsState.itemPaletteArr.find(function (i) {
        return i[1].visible;
      })) === null || _this$settingsState$i19 === void 0 ? void 0 : _this$settingsState$i19[1];
      if (!(palette !== null && palette !== void 0 && palette.htmlInputElement)) return false;
      palette.htmlInputElement.focus();
      return true;
    }

    // clicked Selected Preview Optionvalue gets identifyed and scrolls there
  }, {
    key: "scrollToOPV",
    value: function scrollToOPV(paletteId, codex) {
      var _element$getBoundingC, _this$settingsState$s7, _this$settingsState$s8, _this$settingsState$s9, _this$settingsState$s10, _this$settingsState$s11;
      this.settingsState.openGroupWithCodex(paletteId, codex);
      var element = document.getElementById(codex);
      var scrollTo = ((_element$getBoundingC = element === null || element === void 0 ? void 0 : element.getBoundingClientRect().top) !== null && _element$getBoundingC !== void 0 ? _element$getBoundingC : 0) - ((_this$settingsState$s7 = (_this$settingsState$s8 = this.settingsState.settingsScrollerElement) === null || _this$settingsState$s8 === void 0 ? void 0 : _this$settingsState$s8.getBoundingClientRect().top) !== null && _this$settingsState$s7 !== void 0 ? _this$settingsState$s7 : 0) + ((_this$settingsState$s9 = (_this$settingsState$s10 = this.settingsState.settingsScrollerElement) === null || _this$settingsState$s10 === void 0 ? void 0 : _this$settingsState$s10.scrollTop) !== null && _this$settingsState$s9 !== void 0 ? _this$settingsState$s9 : 0);
      (_this$settingsState$s11 = this.settingsState.settingsScrollerElement) === null || _this$settingsState$s11 === void 0 || _this$settingsState$s11.scrollTo({
        left: 0,
        top: scrollTo,
        behavior: 'smooth'
      });
    }

    /***********************************Weiterleitung zum VisitedHandler ************************************************/
  }, {
    key: "visitOptions",
    value: function visitOptions(options) {
      for (var i = 0; i < options.length; i++) {
        var option = options[i];
        var configStep = this.configuratorStructureHandler.getFirstConfigStepByOptionCodex(option);
        if (!configStep) continue;
        var steps = this.configuratorStructureHandler.getConfigStepByKey(configStep === null || configStep === void 0 ? void 0 : configStep.key);
        if (steps === null) continue;
        this.configuratorStructureHandler.addTabGrousVisitedWithTargetArticlesAndOptions(steps.tabGroup.key, configStep.targetArticles, [option]);
      }
    }
  }, {
    key: "visitActVisibleSteps",
    value: function visitActVisibleSteps() {
      this.visitStepsHandler.visitActVisibleSteps();
    }
  }, {
    key: "wereAllStepsVisited",
    value: function wereAllStepsVisited() {
      return this.visitStepsHandler.wereAllStepsVisited();
    }
  }, {
    key: "getFirstUnvisitedStepIndex",
    value: function getFirstUnvisitedStepIndex() {
      return this.visitStepsHandler.getFirstUnvisitedStepIndex();
    }
  }]);
}();

function ownKeys$d(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$d(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$d(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$d(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$b = getLogger('Planner', ' PlannerUI');
var PlannerUI = /*#__PURE__*/function () {
  function PlannerUI(viewPlannerUIHandler, getCurPlannerViewMode) {
    var _this = this;
    _classCallCheck(this, PlannerUI);
    _defineProperty(this, "uiItems", []);
    _defineProperty(this, "uiOutline3DItems", []);
    _defineProperty(this, "grid2DUIElement", undefined);
    _defineProperty(this, "planner2DBackgroundInfo", {
      position: new Vector2(0, 0),
      zoom: 1
    });
    _defineProperty(this, "curSelectedDP", undefined);
    // private image3DElement: HTMLDivElement | undefined;
    _defineProperty(this, "viewPlannerUIHandler", void 0);
    _defineProperty(this, "viewConfig", void 0);
    _defineProperty(this, "getCurPlannerViewMode", void 0);
    _defineProperty(this, "lastKey", 0);
    //-------------------------------------------- FUNCTIONS CALLED FROM REACT -------------------------------------------------------//
    //Functions called from components
    _defineProperty(this, "getItems", function () {
      var items = new Map();
      for (var i = 0; i < _this.uiItems.length; i++) {
        var item = _this.uiItems[i];
        if (!items.has(item.setId)) items.set(item.setId, new Map());
        if (!items.get(item.setId).has(item.elementId)) items.get(item.setId).set(item.elementId, new Array());
        items.get(item.setId).get(item.elementId).push(item);
      }
      return items;
    });
    makeAutoObservable(this);
    this.viewPlannerUIHandler = viewPlannerUIHandler;
    this.getCurPlannerViewMode = getCurPlannerViewMode;
  }
  return _createClass(PlannerUI, [{
    key: "getKey",
    value: function getKey() {
      this.lastKey++;
      return this.lastKey;
    }
  }, {
    key: "getPosNrByBasketId",
    value: function getPosNrByBasketId(basketId) {
      return this.viewPlannerUIHandler.getPosNrByBasketId(basketId);
    }
  }, {
    key: "setViewPlannerConfig",
    value: function setViewPlannerConfig(viewConfig) {
      this.viewConfig = viewConfig;
    }
  }, {
    key: "clear2D3DViewerUI",
    value: function clear2D3DViewerUI() {
      LOG$b.debug('ClearUI', 'PlannerUI');
      var helperArr = [];
      for (var i = 0; i < this.uiItems.length; i++) {
        if (this.uiItems[i].type == 'Rot3DGrid' || this.uiItems[i].type == 'Move3DGrid') {
          helperArr.push(this.uiItems[i]);
        }
      }
      this.uiOutline3DItems = [];
      this.uiItems = helperArr;
    }
  }, {
    key: "clear3DGridUI",
    value: function clear3DGridUI() {
      var helperArr = [];
      for (var i = 0; i < this.uiItems.length; i++) {
        if (this.uiItems[i].type != 'Rot3DGrid' && this.uiItems[i].type != 'Move3DGrid') {
          helperArr.push(this.uiItems[i]);
        }
      }
      this.uiItems = helperArr;
    }
  }, {
    key: "destroyUI",
    value: function destroyUI() {
      this.clear2D3DViewerUI();
      this.clear3DGridUI();
      this.curSelectedDP = undefined;
    }
  }, {
    key: "addPosNrUI",
    value: function addPosNrUI(setId, elementId, isDockedAccessoire, posOnScreen, rotOfElement, zoom, posNr, state, plannerMode) {
      this.uiItems.push({
        key: this.getKey(),
        type: 'PosNr',
        position: posOnScreen,
        rotOfElement: rotOfElement,
        posNr: posNr,
        setId: setId,
        elementId: elementId.toString(),
        elemState: state,
        iconZoom: zoom,
        plannerMode: plannerMode,
        isDockedAccessoire: isDockedAccessoire,
        clickFunction: state == 'selectable' ? this.viewPlannerUIHandler.onElementSelected : undefined
      });
    }
  }, {
    key: "addDistanceFromGround",
    value: function addDistanceFromGround(setId, elementId, distanceVal, posOnScreen) {
      this.uiItems.push({
        key: this.getKey(),
        type: 'DistanceFromGround',
        position: posOnScreen,
        setId: setId,
        elementId: elementId,
        plannerMode: '3D',
        dimensionVal: distanceVal
      });
    }
  }, {
    key: "addTrashIconUI",
    value: function addTrashIconUI(setId, elementId, rotOfElement, posOnScreen, plannerType) {
      this.uiItems.push({
        key: this.getKey(),
        type: 'TrashIcon',
        rotOfElement: rotOfElement,
        position: posOnScreen,
        setId: setId,
        elementId: elementId.toString(),
        plannerMode: plannerType,
        clickFunction: this.viewPlannerUIHandler.onElementDeleted
      });
    }
  }, {
    key: "addRotationIconUI",
    value: function addRotationIconUI(setId, elementId, rotOfElement, rotDirection, posOnScreen, plannerMode) {
      this.uiItems.push({
        key: this.getKey(),
        type: rotDirection,
        position: posOnScreen,
        setId: setId,
        elementId: elementId.toString(),
        rotOfElement: rotOfElement,
        plannerMode: plannerMode,
        clickFunction: rotDirection == 'RotateC' ? this.viewPlannerUIHandler.onElementRotationClockwise : this.viewPlannerUIHandler.onElementRotationCounterClockwise
      });
    }
  }, {
    key: "addElemOutlineUI",
    value: function addElemOutlineUI(setId, elementId, rotOfElement, posOnScreen, size, state, onDragStart, onDragMove, onDragEnd, plannerMode) {
      this.uiItems.push({
        key: this.getKey(),
        type: 'Outline',
        setId: setId,
        elementId: elementId.toString(),
        rotOfElement: rotOfElement,
        position: posOnScreen,
        size: size,
        elemState: state,
        clickFunction: state == 'selectable' ? this.viewPlannerUIHandler.onElementSelected : undefined,
        onDragStartFunction: onDragStart,
        onDragMoveFunction: onDragMove,
        onDragEndFunction: onDragEnd,
        plannerMode: plannerMode
      });
      //LOG.log("ADD ELEMEN OUTLINE UI " + elementId + " with position: " + posOnScreen.x + "; " + posOnScreen.y);
    }
  }, {
    key: "addDP",
    value: function addDP(setId, elementId, dpId, dpType, dockingInfo, dockToElementId, rotOfElement, iconRotDeg, posOnScreen, plannerMode) {
      this.uiItems.push({
        key: this.getKey(),
        type: dpType == 'AVL' ? 'AVL' : dpType == 'AVR' ? 'AVR' : 'AccDP',
        elementId: elementId.toString(),
        dockingInfo: dockingInfo,
        dockToElementId: dockToElementId,
        setId: setId,
        rotOfElement: rotOfElement,
        position: posOnScreen,
        dpId: dpId,
        iconRotDeg: iconRotDeg,
        plannerMode: plannerMode,
        clickFunction: this.viewPlannerUIHandler.onDockingPointInteraction
      });
    }
  }, {
    key: "addCopy",
    value: function addCopy(setId, elementId, copyType, posOnScreen, plannerMode) {
      if (copyType == 'CopyElement' && !this.viewConfig.enableElementCopy) return;
      this.uiItems.push({
        key: this.getKey(),
        setId: setId,
        type: copyType,
        elementId: elementId.toString(),
        plannerMode: plannerMode,
        position: posOnScreen,
        clickFunction: this.viewPlannerUIHandler.onElementCopy
      });
    }
  }, {
    key: "addMove3DGrid",
    value: function addMove3DGrid(setId, elementId, posOnScreen, onDragStart, onDragMove, onDragEnd) {
      this.uiItems.push({
        key: this.getKey(),
        setId: setId,
        elementId: elementId.toString(),
        type: 'Move3DGrid',
        plannerMode: '3DGrid',
        position: posOnScreen,
        clickFunction: undefined,
        onDragMoveFunction: onDragMove,
        onDragEndFunction: onDragEnd,
        onDragStartFunction: onDragStart
      });
    }
  }, {
    key: "addRot3DGrid",
    value: function addRot3DGrid(setId, elementId, posOnScreen, onDragStart, onDragMove, onDragEnd) {
      this.uiItems.push({
        key: this.getKey(),
        setId: setId,
        elementId: elementId.toString(),
        type: 'Rot3DGrid',
        plannerMode: '3DGrid',
        position: posOnScreen,
        clickFunction: undefined,
        onDragMoveFunction: onDragMove,
        onDragEndFunction: onDragEnd,
        onDragStartFunction: onDragStart
      });
    }
  }, {
    key: "addOutline3D",
    value: function addOutline3D(setId, elementId, polygonPoints, avgCamDist, state, drawContours) {
      //Correct z-Index
      LOG$b.debug('addOutline3D setId: ' + setId + ', elementId: ' + elementId);
      this.uiOutline3DItems.push({
        key: this.getKey(),
        setId: setId,
        elementId: elementId.toString(),
        type: drawContours ? 'Outline3D_Contour' : 'Outline3D',
        plannerMode: '3D',
        polygonPoints: polygonPoints,
        position: new Vector2(0, 0),
        avgCamDist: avgCamDist,
        elemState: state,
        clickFunction: state == 'selectable' ? this.viewPlannerUIHandler.onElementSelected : undefined
      });
      this.correctOutline3DZIndex();
    }

    /**
     * This function corrects the zIndex information of the outline3D Elements
     */
  }, {
    key: "correctOutline3DZIndex",
    value: function correctOutline3DZIndex() {
      this.uiOutline3DItems = this.uiOutline3DItems.slice().sort(function (a, b) {
        return b.avgCamDist - a.avgCamDist;
      });
      var zIndex = 100; //start with 100
      for (var i = 0; i < this.uiOutline3DItems.length; i++) {
        this.uiOutline3DItems[i].zIndex = zIndex;
        zIndex++;
      }
    }
  }, {
    key: "update2DGridBackground",
    value: function update2DGridBackground(backgroundState) {
      this.planner2DBackgroundInfo = backgroundState;
    }
  }, {
    key: "getCanvas2DGridSize",
    value: function getCanvas2DGridSize() {
      if (this.grid2DUIElement) {
        return new Vector2(this.grid2DUIElement.clientWidth, this.grid2DUIElement.clientHeight);
      }
      return new Vector2(0, 0);
    }
  }, {
    key: "getImage3DSize",
    value: function getImage3DSize() {
      // if (this.image3DElement) {
      //   return new Vector2(this.image3DElement.clientWidth, this.image3DElement.clientHeight);
      // }
      return new Vector2(0, 0);
    }
  }, {
    key: "hasOutline3DItems",
    value: function hasOutline3DItems() {
      return this.uiOutline3DItems && this.uiOutline3DItems.length > 0;
    }
  }, {
    key: "getItemsOutline3D",
    value: function getItemsOutline3D() {
      return this.uiOutline3DItems;
    }
  }, {
    key: "setGrid2DUIElement",
    value: function setGrid2DUIElement(grid2DElement) {
      this.grid2DUIElement = grid2DElement;
    }
  }, {
    key: "get2DGridBackgroundState",
    value: function get2DGridBackgroundState() {
      return this.planner2DBackgroundInfo;
    }
  }, {
    key: "getViewConfig",
    value: function getViewConfig() {
      return this.viewConfig;
    }
  }, {
    key: "getDPUIISSelected",
    value: function getDPUIISSelected(dpUI) {
      if (this.curSelectedDP == undefined) return false;
      if (this.curSelectedDP.elementId == dpUI.elementId && this.curSelectedDP.setId == dpUI.setId && this.curSelectedDP.dpId == dpUI.dpId) return true;
      return false;
    }
  }, {
    key: "setDPUISelectionChanged",
    value: function setDPUISelectionChanged(dpUI, isSelected) {
      if (dpUI.type == 'AVR' || dpUI.type == 'AVL' || dpUI.type == 'AccDP') {
        if (!isSelected) {
          this.curSelectedDP = undefined;
        } else {
          this.curSelectedDP = dpUI;
        }
      }
    }

    /**
     * This function updates the onHover information on a specific item.
     * @param doHover
     * @param item
     * @param updateOnType    Update the hover information on the icon with type e.g. PosNr
     */
  }, {
    key: "updateOnHoverOutline3D",
    value: function updateOnHoverOutline3D(doHover, item, updateOnType) {
      var index = -1;
      for (var i = 0; i < this.uiItems.length; i++) {
        if (this.uiItems[i].setId == item.setId && this.uiItems[i].elementId == item.elementId && this.uiItems[i].type == updateOnType) {
          index = i;
          break;
        }
      }
      if (index > -1) {
        var posNrItem = this.uiItems[i];
        this.uiItems[i] = _objectSpread$d(_objectSpread$d({}, posNrItem), {}, {
          onHover: doHover
        }); //copy instance of item and set one property
      }
    }
  }]);
}();

function _createForOfIteratorHelper$1(r, e) { var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (!t) { if (Array.isArray(r) || (t = _unsupportedIterableToArray$1(r)) || e && r && "number" == typeof r.length) { t && (r = t); var _n = 0, F = function F() {}; return { s: F, n: function n() { return _n >= r.length ? { done: !0 } : { done: !1, value: r[_n++] }; }, e: function e(r) { throw r; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var o, a = !0, u = !1; return { s: function s() { t = t.call(r); }, n: function n() { var r = t.next(); return a = r.done, r; }, e: function e(r) { u = !0, o = r; }, f: function f() { try { a || null == t["return"] || t["return"](); } finally { if (u) throw o; } } }; }
function _unsupportedIterableToArray$1(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray$1(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray$1(r, a) : void 0; } }
function _arrayLikeToArray$1(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
var ViewPlannerHandler = /*#__PURE__*/function () {
  function ViewPlannerHandler(onElemSelectionChanged, getConfiguratorState, getPlannerRenderingSize, doShowConfigPalette, update3DImgSrcInView, forceViewToEnterPlannerMode, getOrbitInteractionAPI, onDoShowNav3DChanged, doShow2DGridInViewModel, setPlannerRenderingIsStaticBG, getPosNrByBasketId) {
    var _this = this;
    _classCallCheck(this, ViewPlannerHandler);
    _defineProperty(this, "planner3DImageHandler", void 0);
    _defineProperty(this, "planner2DGridHandler", void 0);
    _defineProperty(this, "plannerUIHandler", void 0);
    _defineProperty(this, "curPlannerMode", void 0);
    _defineProperty(this, "initPlannerMode", void 0);
    _defineProperty(this, "initOrbitCube", void 0);
    _defineProperty(this, "selectedElementsDict", void 0);
    //Contains all articles elementId that are selected
    _defineProperty(this, "selectableElementsDict", void 0);
    //Contains all articles elementId that can be selected in the current config state
    _defineProperty(this, "setAllElementsSelectable", false);
    _defineProperty(this, "selectedDockingPoint", undefined);
    //Contains the dockingpoint that is currently selected
    _defineProperty(this, "allPlannedElementsDict", void 0);
    _defineProperty(this, "allVisualizedNotPlannedElements", void 0);
    //Functions in the DivaViewHandler
    _defineProperty(this, "informViewElemSelectionChanged", void 0);
    _defineProperty(this, "deletePlannerElement", void 0);
    _defineProperty(this, "copyPlannerElement", void 0);
    _defineProperty(this, "getPlannerRenderingSize", void 0);
    _defineProperty(this, "doShowConfigPalette", void 0);
    _defineProperty(this, "doShow2DGridInViewModel", void 0);
    _defineProperty(this, "getConfiguratorState", void 0);
    _defineProperty(this, "getOrbitInteractionAPI", void 0);
    _defineProperty(this, "planner_view_config", void 0);
    _defineProperty(this, "setPlannerRenderingIsStaticBG", void 0);
    _defineProperty(this, "getPosNrByBasketId", void 0);
    _defineProperty(this, "updatePlannerView", void 0);
    _defineProperty(this, "forceViewToEnterPlannerMode", void 0);
    _defineProperty(this, "update3DImgSrcInView", void 0);
    _defineProperty(this, "onDoShowNav3DChanged", void 0);
    _defineProperty(this, "lastSetIdUpdated", void 0);
    _defineProperty(this, "productState", void 0);
    _defineProperty(this, "getPlannerElementByElemId", function (setId, elementId) {
      if (_this.allPlannedElementsDict.has(setId)) {
        var elements = _this.allPlannedElementsDict.get(setId);
        for (var i = 0; i < elements.length; i++) {
          if (elements[i].getElementId() == elementId) return elements[i];
        }
      }
      return undefined;
    });
    /*
     * Search the baseElement of an Accessoire. It searches it recursively.
     * @param curElement
     * @param allSelectedElements
     * @returns
     */
    _defineProperty(this, "getAllPlannedElements", function () {
      return _this.allPlannedElementsDict;
    });
    /**
     * This function is called to clear the view - the 3D image is cleared and the 3D Grid is disabld. as well as the 2D Canvas is cleard
     */
    _defineProperty(this, "clearView", function () {
      var _this$planner3DImageH, _this$planner2DGridHa;
      (_this$planner3DImageH = _this.planner3DImageHandler) === null || _this$planner3DImageH === void 0 || _this$planner3DImageH.clearView(_this.plannerUIHandler);
      (_this$planner2DGridHa = _this.planner2DGridHandler) === null || _this$planner2DGridHa === void 0 || _this$planner2DGridHa.clearView();
      _this.allPlannedElementsDict = new Map();
      _this.allVisualizedNotPlannedElements = new Map();
      _this.selectedElementsDict = new Map();
      _this.selectableElementsDict = new Map();
    });
    /**
     * This function returns the current plannerViewConfig. This configuration file is saved for each client
     */
    _defineProperty(this, "getPlannerViewConfig", function () {
      return _this.planner_view_config;
    });
    /**
     * Returns true if the 3D Grid is enabled
     */
    _defineProperty(this, "getPlanner3DGridEnabled", function () {
      var _this$planner3DImageH2;
      return (_this$planner3DImageH2 = _this.planner3DImageHandler) === null || _this$planner3DImageH2 === void 0 ? void 0 : _this$planner3DImageH2.getPlanner3DGridEnabled();
    });
    _defineProperty(this, "getPlanner3DGridIsInTransformMode", function () {
      var _this$planner3DImageH3;
      return (_this$planner3DImageH3 = _this.planner3DImageHandler) === null || _this$planner3DImageH3 === void 0 ? void 0 : _this$planner3DImageH3.getPlanner3DGridIsInTransformMode();
    });
    /**
     * This function is called when an element is deleted
     * @param setId
     * @param elementId
     */
    _defineProperty(this, "onElementDeleted", function (setId, elementId) {
      _this.deletePlannerElement(setId, elementId);
    });
    /**
     * This function is called when the user selects manually a new element in the planner.
     * The UI is updated and the configurator is informed
     * @param elementId
     */
    _defineProperty(this, "onElementSelected", function (setId, elementId, dp) {
      if (setId == undefined || elementId == undefined) return;
      var selectedElement = _this.getPlannerElementByElemId(setId, elementId);
      if (selectedElement === undefined) return;
      /*
      //In the case of the selected element is an accessoire. In Planner mode you can only select the base element
      if ((selectedElement?.getIsDockedAccessoire() && this.getConfiguratorState() == 'PLANNER') || this.getConfiguratorState() == 'PRODUCTS') {
        //Force selecting baseParent
        selectedElement = this.getBaseParentOfAccessoires(setId, elementId);
        if (selectedElement == undefined) return;
        setId = selectedElement?.getSetId();
        elementId = selectedElement?.getElementId();
      }
      */
      if (_this.selectedElementsDict.has(setId)) {
        //This set was already selected. Make others in the set selectable
        var selectedElements = _this.selectedElementsDict.get(setId);
        for (var i = 0; i < selectedElements.length; i++) {
          var found = false;
          if (_this.selectableElementsDict.has(setId)) {
            //Check if the last selected element is already in the selectable array
            var allSelectableElements = _this.selectableElementsDict.get(setId);
            for (var j = 0; j < allSelectableElements.length; j++) {
              if (allSelectableElements[j].getElementId() == selectedElements[i].getElementId()) {
                found = true;
                break;
              }
            }
          }
          if (!found) {
            var _this$selectableEleme;
            if (!_this.selectableElementsDict.has(setId)) _this.selectableElementsDict.set(setId, []);
            (_this$selectableEleme = _this.selectableElementsDict.get(setId)) === null || _this$selectableEleme === void 0 || _this$selectableEleme.push(selectedElements[i]);
          }
        }
      }
      _this.selectedElementsDict.clear();
      if (selectedElement != undefined) {
        _this.selectedElementsDict.set(setId, [selectedElement]);
      }
      if (!_this.setAllElementsSelectable && !_this.selectableElementsDict.has(setId)) return;
      _this.updateUI();
      _this.informViewElemSelectionChanged(_this.selectedElementsDict.get(setId), _this.selectedDockingPoint);
    });
    _defineProperty(this, "onElementRotationClockwise", function (setId, elementId) {
      var _this$planner2DGridHa2;
      (_this$planner2DGridHa2 = _this.planner2DGridHandler) === null || _this$planner2DGridHa2 === void 0 || _this$planner2DGridHa2.handleRotation(setId, elementId, 'CLOCKWISE');
    });
    _defineProperty(this, "onElementRotationCounterClockwise", function (setId, elementId) {
      var _this$planner2DGridHa3;
      (_this$planner2DGridHa3 = _this.planner2DGridHandler) === null || _this$planner2DGridHa3 === void 0 || _this$planner2DGridHa3.handleRotation(setId, elementId, 'COUNTERCLOCKWISE');
    });
    /**
     * This function should be called if no element is selected
     */
    _defineProperty(this, "onClearElementSelection", function () {
      if (['PLANNER', 'PRODUCTS', 'PROMOTED', 'ACC_PLANNER'].includes(_this.getConfiguratorState()) && _this.selectedElementsDict.size > 0) {
        //If in planner mode clear Element Selection
        _this.selectedElementsDict.clear();
        _this.setAllElementsSelectable = true;
        _this.selectedDockingPoint = undefined;
        _this.updateUI();
        _this.informViewElemSelectionChanged([]);
      }
    });
    /**
     * This function is called when the user interacts with the dockingPoints in the scene.
     * When a AVL or AVR is selected then the Article list is filtered and an element is docked only to this DP.
     * When the AVL or AVR was already selected, the selection is cleared.
     * When a AVO/AVU/AVC is selected that was not docked - the list is filtered and the selected element is docked only to this DP.
     * When a AVO/AVU/AVC is selected that is docked - delete Ref element
     * @param elementId     The elementId of the planner element the user is interacting with
     * @param dpId          The id of the docking Point the user is interacting with
     */
    _defineProperty(this, "onDockingPointInteraction", function (setId, elementId, dpId, dpIsSelected, onDockingElementId) {
      var element = _this.getPlannerElementByElemId(setId, elementId);
      var dp = element === null || element === void 0 ? void 0 : element.getDockingPointById(dpId);
      if (dp != undefined) {
        _this.doShowConfigPalette();
        if (!dp.IsDocking || dp.Type == 'AVR' || dp.Type == 'AVL') {
          if (dp.IsDocking) return;
          if (dpIsSelected) {
            _this.selectedDockingPoint = dp;
            _this.onElementSelected(setId, elementId, dp);
          } else {
            _this.selectedDockingPoint = undefined;
            _this.onElementSelected(setId, elementId);
          }
        } else if (onDockingElementId != undefined) {
          //Handle delete Element ACC
          _this.deletePlannerElement(setId, onDockingElementId);
        }
      }
    });
    /**
     * This function is called to copy an Element.
     * If a direction is specified it is copied to the left or right side. If Standard is set then it is placed depending on other rules
     * @param setId
     * @param elementId
     * @param plannerType Box or Standard Planner
     * @param copyDir
     */
    _defineProperty(this, "onElementCopy", function (setId, elementId, copyDir) {
      var copyInfo = {
        dpType: copyDir == 'LEFT' ? 'AVL' : copyDir == 'RIGHT' ? 'AVR' : copyDir == 'TOP' ? 'AVO' : 'NO',
        copyType: 'STD'
      };
      _this.copyPlannerElement(setId, elementId, copyInfo);
    });
    /**
     * This function is called every time the configurator enters the planner mode
     * It updates the element selection - in the planner mode, only one element can be selected at a time.
     */
    _defineProperty(this, "enterPlannerMode", function () {
      var _this$planner2DGridHa4, _this$planner3DImageH4;
      if (_this.getPlannerViewConfig().autoActivatePlanner3DMode) {
        _this.handlePlanner3DGridChanged(false, true);
        _this.updatePlannerView('ENTER_PLANNER_MODE');
      }
      if ((_this$planner2DGridHa4 = _this.planner2DGridHandler) !== null && _this$planner2DGridHa4 !== void 0 && _this$planner2DGridHa4.resetSelectedElementsOnEnterPlannerMode() && (_this$planner3DImageH4 = _this.planner3DImageHandler) !== null && _this$planner3DImageH4 !== void 0 && _this$planner3DImageH4.resetSelectedElementsOnEnterPlannerMode()) {
        _this.selectedElementsDict.clear();
      } else if (_this.selectedElementsDict.size > 0) {
        var setId = _this.selectedElementsDict.keys().next().value;
        if (_this.selectedElementsDict.size > 1 || setId && _this.selectedElementsDict.get(setId).length > 1) {
          //Remove the others
          var helpDict = new Map();
          var selectionFound = false;
          _this.selectedElementsDict.forEach(function (elements, setId) {
            for (var i = 0; i < elements.length; i++) {
              if (!elements[i].getIsDockedAccessoire()) {
                if (!selectionFound) {
                  helpDict.set(setId, [elements[i]]);
                  selectionFound = true;
                }
              }
            }
          });
          _this.selectedElementsDict = helpDict;
        }
      }
      _this.selectableElementsDict.clear();
      _this.setAllElementsSelectable = true;
      _this.selectedDockingPoint = undefined;
      _this.updateUI();
      var allSelectedElements = [];
      _this.selectedElementsDict.forEach(function (elements, setId) {
        for (var i = 0; i < elements.length; i++) {
          allSelectedElements.push(elements[i]);
        }
      });
      _this.informViewElemSelectionChanged(allSelectedElements);
    });
    /**
     * This function returns all selected elements by their elementID
     */
    _defineProperty(this, "getSelectedElements", function () {
      return _this.selectedElementsDict;
    });
    _defineProperty(this, "getSelectedElementsArr", function (forceSingleElement) {
      if (forceSingleElement) {
        var elements = [];
        if (_this.selectedElementsDict.size > 0) {
          //select first if multiple are Selected
          var setId = _this.selectedElementsDict.keys().next().value;
          if (setId && _this.selectedElementsDict.get(setId).length > 0) {
            elements.push(_this.selectedElementsDict.get(setId)[0]);
          }
        }
        return elements;
      } else {
        var plannerElements = [];
        _this.selectedElementsDict.forEach(function (elements, setId) {
          plannerElements = plannerElements.concat(elements);
        });
        return plannerElements;
      }
    });
    /**
     * This function returns the currently selected DP. If none is selected the result is null.
     */
    _defineProperty(this, "getSelectedDockingPoint", function () {
      return _this.selectedDockingPoint;
    });
    /**
     * This function is called when a new rendering is loaded
     * @param rendering the rendering that should be displayed in the view
     * @alternateRenderingsExist is true, if the catalog has alternate renderings. False if no alternate renderings exist
     */
    _defineProperty(this, "updatePlanner3DImage", function (rendering) {
      var _this$planner3DImageH5;
      _this.setPlannerRenderingIsStaticBG(rendering.getHasStaticBackground());
      (_this$planner3DImageH5 = _this.planner3DImageHandler) === null || _this$planner3DImageH5 === void 0 || _this$planner3DImageH5.updatePlannerView(_this.plannerUIHandler, rendering, _this.allPlannedElementsDict, _this.selectedElementsDict, _this.getPlannerRenderingSize());
      _this.updateUI();
    });
    _defineProperty(this, "setRotationOfOrbitCube", function (forceOrbitTo, rotHorizontal, rotVertical) {
      var api = _this.getOrbitInteractionAPI();
      if (!api) {
        _this.updateOrbitCubeInitParams(rotHorizontal, rotVertical);
      }
      if (api) {
        if (!forceOrbitTo) _this.productState.setIgnoreUpdateRenderingOnOrbitChanged(true);
        api.rotateTo({
          vertical: rotVertical,
          horizontal: rotHorizontal,
          smooth: false
        });
      } else {
        _this.initOrbitCube.rotHorizontal = rotHorizontal;
        _this.initOrbitCube.rotVertical = rotVertical;
      }
    });
    _defineProperty(this, "getOrbitCubeInitParams", function () {
      return _this.initOrbitCube;
    });
    _defineProperty(this, "updateOrbitCubeInitParams", function (rotHorizontal, rotVertical) {
      _this.initOrbitCube.rotHorizontal = rotHorizontal;
      _this.initOrbitCube.rotVertical = rotVertical;
    });
    /**
     * This function is called when the user wants to save the current 2D Planning in a JPG file
     */
    _defineProperty(this, "get2DPlanning2DAsJPG", function () {
      var plannerMode = _this.getInitPlannerViewMode();
      if (plannerMode == 'No' || _this.productState.isBoxPlanner) return '';
      return _this.planner2DGridHandler == undefined ? '' : _this.planner2DGridHandler.getContentAsJPG();
    });
    /**
     * This function is called when the view should update the 2D View
     */
    _defineProperty(this, "updatePlanner2DGrid", function (boundingBox) {
      var _this$planner2DGridHa5;
      if (!['2DGrid', 'RTViewer'].includes(_this.getCurPlannerViewMode())) _this.doShow2DGridInViewModel(true);
      _this.handlePlanner3DGridChanged(true);
      (_this$planner2DGridHa5 = _this.planner2DGridHandler) === null || _this$planner2DGridHa5 === void 0 || _this$planner2DGridHa5.updatePlannerView(_this.PlannerUIHandler, _this.allPlannedElementsDict, boundingBox);
      _this.updateUI();
      if (!['2DGrid', 'RTViewer'].includes(_this.getCurPlannerViewMode())) _this.doShow2DGridInViewModel(false);
    });
    _defineProperty(this, "updatePlannerElements", function (elements, visualizedNotPlannedElements, setId, elementId) {
      _this.allPlannedElementsDict = elements;
      _this.allVisualizedNotPlannedElements = visualizedNotPlannedElements;
      if (setId && elementId) {
        _this.onElementSelected(setId, elementId);
      }
    });
    /**
     * This function is called when the articles (selected and selectable) are changed. This is done when the configurator has changed
     * If we work with the box planner - in one set only one BoxPlanner can be selected. If multiple should be selected - select the latest one
     * @param articleInfos
     */
    _defineProperty(this, "updateElementSelectionConfig", function (articleInfos, allSelectable) {
      //updatePlannerUI

      var lastSelectedElementBoxPlanner = new Map(_this.selectedElementsDict);
      _this.selectedElementsDict.clear();
      _this.selectableElementsDict.clear();
      _this.setAllElementsSelectable = !!allSelectable;

      //Sort the articles to be set wise. Each set can be treated differently based on the planner type (STD or BOX)
      var setBasedArticles = new Map();
      for (var posNum in articleInfos) {
        var _setBasedArticles$get;
        var articleInfo = articleInfos[posNum];
        if (!setBasedArticles.has(articleInfo.setId)) {
          setBasedArticles.set(articleInfo.setId, []);
        }
        (_setBasedArticles$get = setBasedArticles.get(articleInfo.setId)) === null || _setBasedArticles$get === void 0 || _setBasedArticles$get.push(articleInfo);
        _this.lastSetIdUpdated = articleInfo.setId;
      }
      setBasedArticles.forEach(function (articlesInSet, setId) {
        var selectionFinished = false;
        var isBoxPlanner = false;
        var boxPlannerSelectableElements = [];
        for (var i = 0; i < articlesInSet.length; i++) {
          var _articleInfo = articlesInSet[i];
          if (!_articleInfo.elementId || _articleInfo.elementId <= 0) continue;
          var element = _this.getPlannerElementByElemId(_articleInfo.setId, _articleInfo.elementId);
          if (element == undefined) continue;
          if (_articleInfo.isActive) {
            if (!_this.selectedElementsDict.has(_articleInfo.setId)) _this.selectedElementsDict.set(_articleInfo.setId, []);
            if (element.getPlannerType() == 'BOX') {
              isBoxPlanner = true;
              var elementWasSelectedPrev = false;
              if (lastSelectedElementBoxPlanner.has(element.getSetId())) {
                var lastSelectedElements = lastSelectedElementBoxPlanner.get(element.getSetId());
                for (var j = 0; j < lastSelectedElements.length; j++) {
                  if (lastSelectedElements[j].getElementId() == element.getElementId()) {
                    _this.selectedElementsDict.get(_articleInfo.setId).push(element);
                    selectionFinished = true;
                    elementWasSelectedPrev = true;
                  }
                }
              }
              if (!elementWasSelectedPrev) {
                boxPlannerSelectableElements.push(element);
              }
            } else {
              _this.selectedElementsDict.get(_articleInfo.setId).push(element);
              selectionFinished = true;
            }
          } else {
            var _this$selectableEleme2;
            if (!_this.selectableElementsDict.has(_articleInfo.setId)) _this.selectableElementsDict.set(_articleInfo.setId, []);
            (_this$selectableEleme2 = _this.selectableElementsDict.get(_articleInfo.setId)) === null || _this$selectableEleme2 === void 0 || _this$selectableEleme2.push(element);
          }
        }
        if (articlesInSet.length > 0 && isBoxPlanner == true && boxPlannerSelectableElements.length > 0) {
          //A new element is selected. No element that was previously already selected. Simply select the first possible element
          if (!_this.selectedElementsDict.has(boxPlannerSelectableElements[0].getSetId())) _this.selectedElementsDict.set(boxPlannerSelectableElements[0].getSetId(), []);
          for (var i = 0; i < boxPlannerSelectableElements.length; i++) {
            if (!selectionFinished && i == 0) {
              _this.selectedElementsDict.get(boxPlannerSelectableElements[0].getSetId()).push(boxPlannerSelectableElements[i]);
            } else {
              var _this$selectableEleme3;
              if (!_this.selectableElementsDict.has(boxPlannerSelectableElements[i].getSetId())) _this.selectableElementsDict.set(boxPlannerSelectableElements[i].getSetId(), []);
              (_this$selectableEleme3 = _this.selectableElementsDict.get(boxPlannerSelectableElements[i].getSetId())) === null || _this$selectableEleme3 === void 0 || _this$selectableEleme3.push(boxPlannerSelectableElements[i]);
            }
          }
        }
      });
      _this.updateUI();
      //this.planner3DImageHandler.updateUI(this.selectedElements, this.selectableElements, this.setAllElementsSelectable);
    });
    /**
     * This function is called from the logic when the logic decides a new element has to be selected.
     * @param selectedElementSetIds The setIds of the corresponsting elements ids. [0] of set ids corresponds to [0] in element id
     * @param selectedElements  The elements that are now selected
     * @param setOthersSelectable   If true - all not selected elements are selectable
     */
    _defineProperty(this, "updateElementSelectionPlanner", function (selectedElementSetIds, selectedElementIds, setOthersSelectable) {
      var somethingChanged = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
      var _iterator = _createForOfIteratorHelper$1(selectedElementSetIds),
        _step;
      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var _setId = _step.value;
          if (_this.lastSetIdUpdated != _setId) somethingChanged = true;
          _this.lastSetIdUpdated = _setId;
          if (_this.getElementCountInDict(_this.selectedElementsDict) != selectedElementIds.length) somethingChanged = true;
          for (var i = 0; i < selectedElementIds.length; i++) {
            if (_this.selectedElementsDict.has(_setId)) {
              var curSelectedElements = _this.selectedElementsDict.get(_setId);
              var elemFound = false;
              for (var j = 0; j < (curSelectedElements === null || curSelectedElements === void 0 ? void 0 : curSelectedElements.length); j++) {
                if (curSelectedElements[j].getElementId() == selectedElementIds[i]) {
                  elemFound = true;
                  break;
                }
              }
              if (!elemFound) {
                somethingChanged = true;
                break;
              }
            }
          }
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
      if ((selectedElementSetIds === null || selectedElementSetIds === void 0 ? void 0 : selectedElementSetIds.length) == 0 && _this.selectedElementsDict.size > 0) {
        somethingChanged = true;
      }
      // this should not influence the current selected articles
      // if (!somethingChanged) {
      //   somethingChanged = setOthersSelectable != this.setAllElementsSelectable;
      // }

      if (somethingChanged) {
        _this.selectedElementsDict.clear();
        var allSelectedElements = [];
        for (var i = 0; i < selectedElementSetIds.length; i++) {
          var setId = selectedElementSetIds[i];
          var _iterator2 = _createForOfIteratorHelper$1(selectedElementIds),
            _step2;
          try {
            for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
              var elemId = _step2.value;
              if (!_this.selectedElementsDict.has(setId)) {
                _this.selectedElementsDict.set(setId, []);
              }
              var _IDivaViewPlannerElement = _this.getPlannerElementByElemId(setId, elemId);
              if (_IDivaViewPlannerElement != undefined) {
                var _this$selectedElement;
                (_this$selectedElement = _this.selectedElementsDict.get(setId)) === null || _this$selectedElement === void 0 || _this$selectedElement.push(_IDivaViewPlannerElement);
                allSelectedElements.push(_IDivaViewPlannerElement);
              }
            }
          } catch (err) {
            _iterator2.e(err);
          } finally {
            _iterator2.f();
          }
        }
        _this.selectableElementsDict.clear();
        _this.setAllElementsSelectable = setOthersSelectable;
        _this.selectedDockingPoint = undefined;
        _this.updateUI();
        _this.informViewElemSelectionChanged(allSelectedElements);
      } else if (selectedElementIds.length == 0 && _this.allPlannedElementsDict.size == 0) {
        _this.informViewElemSelectionChanged([]);
      }
    });
    /**
     * Returns the planner view mode that is currectly used for visualization.
     * No means - no planner view is shown at all
     */
    _defineProperty(this, "getCurPlannerViewMode", function () {
      if (_this.curPlannerMode == 'BoxPlanner') return '3DImage';
      return _this.curPlannerMode;
    });
    _defineProperty(this, "getInitPlannerViewMode", function () {
      return _this.initPlannerMode;
    });
    this.getConfiguratorState = getConfiguratorState;
    this.planner_view_config = {
      posTrashRotIcon: 'POSNUM_TOP',
      //Dieser Parameter gibt an, wo die Positionsnummer positioniert werden muss.
      posNumIcon: 'TOP_LEFT_BACK',
      iconPosition2DGrid: 'BORDER',
      //Gibt an ob die Planner Icons außerhald des SVgs gezeichnet werden. Beispiel Pfister
      deltaRotDeg2D: 45,
      posDPIcon2D: 'CORNER',
      enableElementCopy: false,
      autoActivatePlanner3DMode: false
    };
    this.plannerUIHandler = new PlannerUI(this, this.getCurPlannerViewMode);
    this.allPlannedElementsDict = new Map();
    this.allVisualizedNotPlannedElements = new Map();
    this.selectedElementsDict = new Map();
    this.selectableElementsDict = new Map();
    this.onDoShowNav3DChanged = onDoShowNav3DChanged;
    this.forceViewToEnterPlannerMode = forceViewToEnterPlannerMode;
    this.update3DImgSrcInView = update3DImgSrcInView;
    this.informViewElemSelectionChanged = onElemSelectionChanged;
    this.getPlannerRenderingSize = getPlannerRenderingSize;
    this.doShowConfigPalette = doShowConfigPalette;
    this.doShow2DGridInViewModel = doShow2DGridInViewModel;
    this.getOrbitInteractionAPI = getOrbitInteractionAPI;
    this.setPlannerRenderingIsStaticBG = setPlannerRenderingIsStaticBG;
    this.getPosNrByBasketId = getPosNrByBasketId;
    this.curPlannerMode = '3DImage';
    this.initPlannerMode = '3DImage';
    this.initOrbitCube = {
      rotHorizontal: 0,
      rotVertical: 0
    };
  }
  return _createClass(ViewPlannerHandler, [{
    key: "initFunctions",
    value: function initFunctions(deletePlannerElement, copyPlannerElement) {
      this.deletePlannerElement = deletePlannerElement;
      this.copyPlannerElement = copyPlannerElement;
    }
    /**
     * This function is called to initialize everything in the 3D image handler and the 2D Grid
     * @param updatePlannerTrans
     * @param useBoxPlanner True if the current planner is a Box Planner.
     */
  }, {
    key: "initPlannerView",
    value: (function () {
      var _initPlannerView = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee(viewConfig, productState, updatePlannerTrans, updatePlannerView, plannerPaletteViewInfo) {
        var _viewConfig$plannerCo, _productState$planner, _this$planner3DImageH6, _this$planner2DGridHa6;
        var plannerMode, _yield$import, Planner3DImage, _yield$import2, Planner2DGrid;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              this.planner_view_config = (_viewConfig$plannerCo = viewConfig.plannerConfig) !== null && _viewConfig$plannerCo !== void 0 ? _viewConfig$plannerCo : {
                posTrashRotIcon: 'TOP_RIGHT_BACK',
                posNumIcon: 'TOP_LEFT_BACK',
                iconPosition2DGrid: 'OUTSIDE',
                posDPIcon2D: 'CORNER',
                deltaRotDeg2D: 45,
                enableElementCopy: false,
                autoActivatePlanner3DMode: false
              };
              plannerMode = (_productState$planner = productState.plannerMode) !== null && _productState$planner !== void 0 ? _productState$planner : '3DImage';
              this.productState = productState;
              this.curPlannerMode = plannerMode;
              this.initPlannerMode = plannerMode;
              this.updatePlannerView = updatePlannerView;
              _context.next = 8;
              return import('./Planner3DImage-d717152c.js');
            case 8:
              _yield$import = _context.sent;
              Planner3DImage = _yield$import["default"];
              _context.next = 12;
              return import('./Planner2DGrid-8daf37cb.js');
            case 12:
              _yield$import2 = _context.sent;
              Planner2DGrid = _yield$import2["default"];
              this.planner3DImageHandler = new Planner3DImage(this.getConfiguratorState, this.update3DImgSrcInView, this.getPlannerViewConfig, this.onDoShowNav3DChanged, this.getAllPlannedElements, plannerPaletteViewInfo);
              this.planner2DGridHandler = new Planner2DGrid(this.getConfiguratorState, this.getPlannerElementByElemId, this.getPlannerViewConfig, this.forceViewToEnterPlannerMode, plannerPaletteViewInfo);
              this.plannerUIHandler.setViewPlannerConfig(this.planner_view_config);
              (_this$planner3DImageH6 = this.planner3DImageHandler) === null || _this$planner3DImageH6 === void 0 || _this$planner3DImageH6.initialize('3DImage-Container', updatePlannerTrans);
              (_this$planner2DGridHa6 = this.planner2DGridHandler) === null || _this$planner2DGridHa6 === void 0 || _this$planner2DGridHa6.initialize(updatePlannerTrans);
              // this.plannerUIHandler.clearUI();
            case 19:
            case "end":
              return _context.stop();
          }
        }, _callee, this);
      }));
      function initPlannerView(_x, _x2, _x3, _x4, _x5) {
        return _initPlannerView.apply(this, arguments);
      }
      return initPlannerView;
    }())
  }, {
    key: "updateUI",
    value:
    /**
     * This function calls the update UI function in the currently displaying planner mode.
     */
    function updateUI() {
      var _this$planner3DImageH7, _this$planner2DGridHa7;
      //DAS MÜSSEN DIE KOMPONENTEN ENTSCHEIDEN this.plannerUIHandler.clearUI();

      switch (this.getCurPlannerViewMode()) {
        case '3DImage':
        case 'BoxPlanner':
        case '3DCOMB':
        case 'RTViewer':
          (_this$planner3DImageH7 = this.planner3DImageHandler) === null || _this$planner3DImageH7 === void 0 || _this$planner3DImageH7.updateUI(this.plannerUIHandler, this.allPlannedElementsDict, this.allVisualizedNotPlannedElements, this.selectedElementsDict, this.selectableElementsDict, this.setAllElementsSelectable, this.selectedDockingPoint, this.getPlannerRenderingSize());
          break;
        case '2DGrid':
          (_this$planner2DGridHa7 = this.planner2DGridHandler) === null || _this$planner2DGridHa7 === void 0 || _this$planner2DGridHa7.updateUI(this.plannerUIHandler, this.allPlannedElementsDict, this.selectedElementsDict, this.selectableElementsDict, this.setAllElementsSelectable);
          break;
      }
    }
  }, {
    key: "handleUIInteraction",
    value:
    /**
     * This function is called when the user clicks on the planner view.
     * It evaluates if elements are selected or if the element selection should be cleared
     * @param e the event
     */
    function handleUIInteraction(e, tabGroupType) {}

    //This Function was generalized with updatePlannerMsgContainer
    // /**
    //  * This function is called when the logic detects a change in the collision of planner Elements
    //  * @param state     Describes if two elements do now collide or not collide
    //  * @param collisionPairs    An array of pairs that describes which elements do collide
    //  */
    // public onCollisionChanged(state: "COLLISION" | "NO_COLLISION", collisionPairs: { elementId1: number; elementId2: number }[]) {
    //   if (state == "COLLISION") {
    //     this.updatePlannerMsgContainer(true, "WARNING", "COLLISION_MSG", false);
    //   } else if (state == "NO_COLLISION") {
    //     this.updatePlannerMsgContainer(false, "WARNING", "COLLISION_MSG", false);
    //   }
    // }

    /**
     * This function is called when the 3D Grid should be displayed/Disabled.
     * If forceClose is set - the grid is closed! it is called every time the planner step is exited
     * @param forceClose
     */
  }, {
    key: "handlePlanner3DGridChanged",
    value: function handlePlanner3DGridChanged(forceClose, forceActive) {
      var _this$planner3DImageH8;
      if (this.getCurPlannerViewMode() == '3DImage' || this.getCurPlannerViewMode() == '3DCOMB' || forceClose) (_this$planner3DImageH8 = this.planner3DImageHandler) === null || _this$planner3DImageH8 === void 0 || _this$planner3DImageH8.handlePlanner3DGridChanged(this.plannerUIHandler, this.getPlannerRenderingSize(), forceClose, forceActive);
    }
  }, {
    key: "getElementCountInDict",
    value: function getElementCountInDict(elementDict) {
      var elemCount = 0;
      elementDict.forEach(function (elements, setId) {
        elemCount += elements.length;
      });
      return elemCount;
    }
  }, {
    key: "initPlannerViewMode",
    value: function initPlannerViewMode(mode) {
      this.initPlannerMode = mode;
      this.curPlannerMode = mode;
    }
  }, {
    key: "setCurPlannerViewMode",
    value: function setCurPlannerViewMode(mode) {
      if (this.curPlannerMode != mode) {
        this.PlannerUIHandler.clear2D3DViewerUI();
        this.PlannerUIHandler.clear3DGridUI();
      }
      this.curPlannerMode = mode;
      if (this.getPlannerViewConfig().autoActivatePlanner3DMode && ['PLANNER', 'PRODUCTS', 'PROMOTED', 'ACC_PLANNER'].includes(this.getConfiguratorState())) {
        //If we always have to activate the 3DGrid - forca activate it
        this.handlePlanner3DGridChanged(false, true);
      }
    }
  }, {
    key: "PlannerUIHandler",
    get: function get() {
      return this.plannerUIHandler;
    }
  }, {
    key: "SelectedElementsPosNrs",
    get: function get() {
      var ret = [];
      this.selectedElementsDict.forEach(function (elements, setId) {
        for (var i = 0; i < elements.length; i++) {
          ret.push(elements[i].getPosNr().toString());
        }
      });
      return ret;
    }
  }]);
}();

var ViewHelper = /*#__PURE__*/function () {
  function ViewHelper(getState, navigateToTabGroup, updateConfiguratorState) {
    var _this = this;
    _classCallCheck(this, ViewHelper);
    _defineProperty(this, "paletteViewHandler", void 0);
    _defineProperty(this, "configuratorStructureHandler", void 0);
    _defineProperty(this, "divaWebPlanner", void 0);
    _defineProperty(this, "getState", void 0);
    _defineProperty(this, "updateConfiguratorState", void 0);
    _defineProperty(this, "navigateToTabGroup", void 0);
    //helper variables for events
    _defineProperty(this, "clickWasMadeViaTouch", false);
    _defineProperty(this, "actUserView", 'desktop');
    _defineProperty(this, "viewConfig", void 0);
    _defineProperty(this, "productState", void 0);
    _defineProperty(this, "tabGroupState", void 0);
    _defineProperty(this, "settingsState", void 0);
    /**
     * This function is called when the planner detects that the view must enter the Planner Mode.
     * @param doShowInfo    Informs the user to switch tabgroup
     */
    _defineProperty(this, "forceViewToEnterPlannerMode", function (doShowInfo) {
      _this.navigateToTabGroup('PLANNER');
      _this.productState.updatePlannerMsgContainer('JUMP_BACK_TO_PLANNER', true, 'INFO', 'WARNING_FORCE_ENTER_PLANNERMODE', true);
    });
    /**
     * This function is called in the viewPlannerHandler, when the user clicks on another selectable element
     * @param plannerElements Array of all currently selected PlannerElements
     */
    _defineProperty(this, "plannerElementSelectionChanged", /*#__PURE__*/function () {
      var _ref = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee(plannerElements, dp) {
        var setSelectionChanged, _this$tabGroupState$c;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              setSelectionChanged = false;
              if (plannerElements && plannerElements.length == 1) {
                setSelectionChanged = _this.divaWebPlanner.onElementSelectionChanged(plannerElements[0].getSetId(), plannerElements[0].getElementId(), (_this$tabGroupState$c = _this.tabGroupState.curTabGroup) === null || _this$tabGroupState$c === void 0 ? void 0 : _this$tabGroupState$c.type);
              }
              if (!dp && plannerElements && plannerElements.length == 1) {
                _this.settingsState.setPlannerReplaceModeAvailable(true);
                _this.paletteViewHandler.checkPlanningReplaceModeAfterChange(false);
              } else {
                _this.settingsState.setPlannerReplaceModeAvailable(false);
              }
              _context.t0 = _this.getState();
              _context.next = _context.t0 === 'PRODUCTS' ? 6 : _context.t0 === 'PROMOTED' ? 6 : _context.t0 === 'PLANNER' ? 8 : _context.t0 === 'ACC_PLANNER' ? 8 : _context.t0 === 'ACC' ? 10 : _context.t0 === 'SHELFPLANNER' ? 12 : _context.t0 === 'OPTION' ? 12 : 14;
              break;
            case 6:
              if ((plannerElements === null || plannerElements === void 0 ? void 0 : plannerElements.length) > 0) _this.divaWebPlanner.loadPaletteInfos('PRODUCTS', [plannerElements[0].getBasketId()], [plannerElements[0].getBasketId()]);else _this.divaWebPlanner.loadPaletteInfos('PRODUCTS');
              return _context.abrupt("break", 14);
            case 8:
              _this.paletteViewHandler.updatePlannerPaletteAfterElementSelectionChanged(plannerElements, dp);
              return _context.abrupt("break", 14);
            case 10:
              //falls nur ein Arikel selektiert wurde zu dessen Konfiguratorschritt springen
              if ((plannerElements === null || plannerElements === void 0 ? void 0 : plannerElements.length) == 1 && !setSelectionChanged) {
                _this.paletteViewHandler.articleInfoSelected(plannerElements[0].getPosNr());
              }
              return _context.abrupt("break", 14);
            case 12:
              //falls nur ein Arikel selektiert wurde zu dessen Konfiguratorschritt springen
              if ((plannerElements === null || plannerElements === void 0 ? void 0 : plannerElements.length) == 1 && !setSelectionChanged) {
                if (plannerElements[0].getIsDockedAccessoire()) {
                  _this.configuratorStructureHandler.selectArticleInOption(plannerElements[0].getBasketId());
                  _this.updateConfiguratorState(false);
                } else {
                  if (!_this.configuratorStructureHandler.CurConfigStep.targetArticles.includes(plannerElements[0].getBasketId())) {
                    _this.paletteViewHandler.articleInfoSelected(plannerElements[0].getPosNr());
                  }
                }
              }
              return _context.abrupt("break", 14);
            case 14:
            case "end":
              return _context.stop();
          }
        }, _callee);
      }));
      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    }());
    this.getState = getState;
    this.navigateToTabGroup = navigateToTabGroup;
    this.updateConfiguratorState = updateConfiguratorState;
  }
  return _createClass(ViewHelper, [{
    key: "initialize",
    value: function initialize(paletteViewHandler, configuratorStructureHandler, divaWebPlanner, productState, tabGroupState, settingsState) {
      this.paletteViewHandler = paletteViewHandler;
      this.configuratorStructureHandler = configuratorStructureHandler;
      this.divaWebPlanner = divaWebPlanner;
      this.productState = productState;
      this.settingsState = settingsState;
      this.tabGroupState = tabGroupState;
    }
  }]);
}();

function ownKeys$c(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$c(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$c(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$c(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var UIState = /*#__PURE__*/function () {
  function UIState() {
    var _this = this,
      _ref2;
    _classCallCheck(this, UIState);
    _defineProperty(this, "checkedMobileFirstTime", false);
    _defineProperty(this, "isCheckCurrentlyCheckingInMobileView", false);
    _defineProperty(this, "config", void 0);
    _defineProperty(this, "updatePlannerView", void 0);
    _defineProperty(this, "visitActVisibleSteps", void 0);
    _defineProperty(this, "updateCurTabGroupType", void 0);
    _defineProperty(this, "settingsState", void 0);
    _defineProperty(this, "getReturnTo", void 0);
    _defineProperty(this, "barrierTextClass", void 0);
    _defineProperty(this, "barrierCreateActionOrder", void 0);
    _defineProperty(this, "resizeObserver", void 0);
    _defineProperty(this, "configPalettescrollEventCounter", 0);
    _defineProperty(this, "tabGroupscrollEventCounter", 0);
    _defineProperty(this, "checkoutConfig", void 0);
    _defineProperty(this, "templateConfigKeys", undefined);
    _defineProperty(this, "templateConfig", undefined);
    /*****************************************************************Mobile, Expand and fold state *****************************************************************/
    _defineProperty(this, "configuratorRef", null);
    _defineProperty(this, "coreMobile", false);
    _defineProperty(this, "i18n", undefined);
    _defineProperty(this, "user", undefined);
    _defineProperty(this, "organization", undefined);
    _defineProperty(this, "firstVisit", true);
    _defineProperty(this, "isLoading", true);
    _defineProperty(this, "configuratorDisabled", false);
    _defineProperty(this, "configuratorHidden", false);
    _defineProperty(this, "doShowConfigPalette", function () {
      if (_this.configSettingsIsHidden && !_this.configSettingsIsHiddenInitialized) {
        _this.configSettingsIsHidden = false;
        _this.updatePlannerView('DO_SHOW_CONFIG_PALETTE');
      }
    });
    //responsive CSS Classes booleans
    _defineProperty(this, "isMobilePortrait", false);
    _defineProperty(this, "isMobileLandscape", false);
    /*expandItemsPalette*/
    /*Die ConfiguratorPlaette wird um 2/3 der größe des Bildschrims erweitert -> Erweitert*/
    _defineProperty(this, "configSettingsIsExpanded", false);
    /*Expand Weiterentwicklung mit ganz auf und ganz zuklappen */
    _defineProperty(this, "configSettingsIsMaxExpanded", false);
    _defineProperty(this, "configSettingsIsMinExpanded", false);
    _defineProperty(this, "doShowConfSettingsExpander", undefined);
    _defineProperty(this, "doShowConfSettingsMaxMinExpander", undefined);
    _defineProperty(this, "onClickExpandConfigSettings", function () {
      _this.configSettingsIsExpanded = !_this.configSettingsIsExpanded;
      //nun können evtl. gewisse Schritte sichtbar sein. Diese werden nun besucht.
      _this.visitSteps();
    });
    _defineProperty(this, "onClickExpandMaxConfigSettings", function () {
      if (_this.configSettingsIsMinExpanded) {
        //von min zurück auf normalansicht
        _this.configSettingsIsMinExpanded = false;
      } else if (!_this.configSettingsIsExpanded) {
        //von normalansicht auf expanded
        _this.configSettingsIsExpanded = true;
      } else {
        //von expandend auf Max expanded
        _this.configSettingsIsExpanded = false;
        _this.configSettingsIsMaxExpanded = true;
      }
      _this.visitSteps();
    });
    _defineProperty(this, "onClickExpandMinConfigSettings", function () {
      if (_this.configSettingsIsMaxExpanded) {
        //von max zurück auf expanded
        _this.configSettingsIsMaxExpanded = false;
        _this.configSettingsIsExpanded = true;
      } else if (_this.configSettingsIsExpanded) {
        //von expanded zurück auf Normal
        _this.configSettingsIsExpanded = false;
      } else {
        //von normal auf min expanded
        _this.configSettingsIsMinExpanded = true;
      }
      _this.visitSteps();
    });
    _defineProperty(this, "visitSteps", function () {
      setTimeout(function () {
        _this.visitActVisibleSteps();
      }, 450);
    });
    /*hideItemsPalette*/
    /*ConfiguratorPalette/ ConfiguratorSettings werden ausgeblendet falls configSettingsIsHidden = true
                der rest( 3D Bild) wird unfolded (aufgeklappt) und somit 100%*/
    _defineProperty(this, "configSettingsIsHidden", false);
    //configSettingsIsFolded
    _defineProperty(this, "configSettingsIsHiddenInitialized", false);
    _defineProperty(this, "onClickHideConfigSettings", /*#__PURE__*/_asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            if (_this.configSettingsIsHiddenInitialized) _this.configSettingsIsHiddenInitialized = false;
            if (_this.configSettingsIsFoldedUp) {
              // falls der Configurator Ganz aufgeklappt ist nicht ganz runter klappen sondern wieder zurück zu ausgangsnage klappen (alles auf false)
              _this.configSettingsIsFoldedUp = false;
              _this.updatePlannerView('ON_CLICK_HIDE_CONFIG_SETTINGS');
            } else {
              _this.configSettingsIsHidden = !_this.configSettingsIsHidden;
              _this.updatePlannerView('ON_CLICK_HIDE_CONFIG_SETTINGS');
            }
            //tabmenu bekommt evtl eine neue Breite
            _context.next = 4;
            return DivaUtils.Sleep(450 + 450);
          case 4:
          case "end":
            return _context.stop();
        }
      }, _callee);
    })));
    /*FoldUp configSettings*/
    //ConfiguratorPalette/ ConfiguratorSettings wird zu Fullscreen aufgeklappt bei der Portraitansicht um volle übersicht über die Answahlmöglcihkeiten zu haben im Planner / Configurator
    _defineProperty(this, "configSettingsIsFoldedUp", false);
    _defineProperty(this, "onClickFoldupConfigSettings", function () {
      _this.configSettingsIsFoldedUp = !_this.configSettingsIsFoldedUp;
      _this.updatePlannerView('ON_FOLDUP_CONFIG_SETTINGS');
      //tabmenu bekommt evtl eine neue Breite
    });
    _defineProperty(this, "hidePrices", false);
    _defineProperty(this, "activeCatalogSection", (_ref2 = localStorage.getItem('lastWindow')) !== null && _ref2 !== void 0 ? _ref2 : '3D/2D');
    _defineProperty(this, "navigateStepByStep", function () {
      return !!(_this.ViewConfig.navigateStepByStep || _this.isInMobileView && _this.ViewConfig.navigateStepByStepInMobile);
    });
    _defineProperty(this, "barrierActions", undefined);
    _defineProperty(this, "setBarrierActions", function (barrierActions) {
      _this.barrierActions = barrierActions;
    });
    makeObservable(this, {
      configuratorRef: observable,
      setConfiguratorRef: action,
      configuratorHidden: observable,
      setConfiguratorHidden: action,
      configuratorDisabled: observable,
      setConfiguratorDisabled: action,
      isLoading: observable,
      setIsLoading: action,
      firstVisit: observable,
      setFirstVisit: action,
      ItemsPaletteInProductVisible: computed,
      doShowConfigPalette: action,
      initTemplateConfig: action,
      templateConfig: observable,
      templateConfigKeys: observable,
      configSettingsIsHidden: observable,
      configSettingsIsFoldedUp: observable,
      onClickHideConfigSettings: action,
      onClickFoldupConfigSettings: action,
      configSettingsIsExpanded: observable,
      setConfigSettingsIsExpanded: action,
      configSettingsIsMaxExpanded: observable,
      setConfigSettingsIsMaxExpanded: action,
      configSettingsIsMinExpanded: observable,
      setConfigSettingsIsMinExpanded: action,
      doShowConfSettingsExpander: observable,
      doShowConfSettingsMaxMinExpander: observable,
      onClickExpandConfigSettings: action,
      onClickExpandMaxConfigSettings: action,
      onClickExpandMinConfigSettings: action,
      initFromConfig: action,
      reset: action,
      config: observable,
      hidePrices: observable,
      setHidePrices: action,
      setCheckoutConfig: action,
      checkoutConfig: observable,
      isInMobileView: computed,
      isMobilePortrait: observable,
      isMobileLandscape: observable,
      checkIsInMobileView: action,
      activeCatalogSection: observable,
      setActiveCatalogSection: action,
      barrierActions: observable,
      setBarrierActions: action
    });
    this.config = {
      viewConfig: {}
    };
  }
  return _createClass(UIState, [{
    key: "initFunctions",
    value: function initFunctions(updatePlannerView, visitActVisibleSteps, updateCurTabGroupType, settingsState, getReturnTo, configSettingsIsHidden) {
      this.configSettingsIsHidden = !!configSettingsIsHidden;
      this.configSettingsIsHiddenInitialized = !!configSettingsIsHidden;
      this.updatePlannerView = updatePlannerView;
      this.visitActVisibleSteps = visitActVisibleSteps;
      this.updateCurTabGroupType = updateCurTabGroupType;
      this.settingsState = settingsState;
      this.getReturnTo = getReturnTo;
      //extra ausprogrammierter eventhandler für touchevents
      if (this.isInMobileView) {
        document.addEventListener('contextmenu', function (event) {
          return event.preventDefault();
        });
      }
    }
    /********************************************************************* Template STATES *********************************************************************/
  }, {
    key: "initTemplateConfig",
    value: function initTemplateConfig(t, advancedconfigurator) {
      var templateConfig = _objectSpread$c({}, t);
      var templates = {};
      if (!templateConfig) throw new Error('templateConfig is undefined');
      if (!templateConfig['IsActive']) templateConfig['IsActive'] = 'standard';
      if (!templateConfig['PlannerMsg']) templateConfig['PlannerMsg'] = 'standard';
      if (!templateConfig['DropDownText']) templateConfig['DropDownText'] = 'standard';
      if (!templateConfig['PaletteGroup']) templateConfig['PaletteGroup'] = 'standard';
      if (!templateConfig['RoomListSelection']) templateConfig['RoomListSelection'] = 'standard';
      if (advancedconfigurator) {
        templateConfig['Product'] = 'catalog';
      }
      for (var key in templateConfig) {
        switch (key) {
          case 'Layout':
            templates['Layout'] = templateConfig[key];
            break;
          case 'BasketButton':
            templates['Action/Basket'] = templateConfig[key];
            break;
          case 'NavigationButton':
            templates['Action/Navigation'] = templateConfig[key];
            break;
          case 'Product':
            templates['Product/Product'] = templateConfig[key];
            break;
          case 'PlannerUIVariants':
            templates['Product/ProductCenter/PlannerUI/PlannerUIVariants'] = templateConfig[key];
            break;
          case 'PlannerMsg':
            templates['Product/ProductCenter/PlannerMsgContainer'] = templateConfig[key];
            break;
          case 'ProductHeader':
            templates['Product/ProductHeader/ProductHeader'] = templateConfig[key];
            break;
          case 'ProductHeaderFunction':
            templates['Product/ProductHeader/Functions'] = templateConfig[key];
            break;
          case 'ProductFooter':
            templates['Product/ProductFooter'] = templateConfig[key];
            break;
          case 'NavBar3D':
            templates['Product/ProductCenter/Planner3DImage/NavBar'] = templateConfig[key];
            break;
          case 'OrbitInteraction':
            templates['Product/ProductCenter/Planner3DImage/OrbitInteraction'] = templateConfig[key];
            break;
          case 'RoomListSelection':
            templates['Product/ProductCenter/RoomListSelection'] = templateConfig[key];
            break;
          case 'SettingsHeader':
            templates['Settings/SettingsHeaderContainer/SettingsItemHeader/Header'] = templateConfig[key];
            break;
          case 'PaletteGroup':
            templates['Settings/SettingsScroller/SettingsItem/PaletteGroup'] = templateConfig[key];
            break;
          case 'TabGroupVertical':
            templates['TabGroup/TabGroupVertical/ItemVariants'] = templateConfig[key];
            break;
          case 'TabGroupHorizontal':
            templates['TabGroup/TabGroupHorizontal/item'] = templateConfig[key];
            break;
          case 'ItemAccessoire':
            templates['Settings/SettingsScroller/SettingsItem/ItemPalette/Accessoires'] = templateConfig[key];
            break;
          case 'DropDownText':
            templates['Settings/SettingsScroller/SettingsItem/ItemPalette/HeaderAndOption/DropDownText'] = templateConfig[key];
            break;
          case 'ItemGallery':
            templates['Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Gallery'] = templateConfig[key];
            break;
          case 'ItemDefault':
            templates['Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Default'] = templateConfig[key];
            break;
          case 'ItemRoom':
            templates['Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Room/RoomItem'] = templateConfig[key];
            break;
          case 'SummaryOptionTree':
            templates['Settings/SettingsScroller/SettingsItem/ItemPalette/Summary/OptionTree'] = templateConfig[key];
            break;
          case 'IsActive':
            templates['Settings/SettingsScroller/SettingsItem/ItemPalette/Options/IsActive'] = templateConfig[key];
            break;
          case 'BarrierActions':
            templates['GlobalElements/BarrierActions'] = templateConfig[key];
            break;
        }
      }
      this.templateConfigKeys = templateConfig;
      this.templateConfig = templates;
    }
  }, {
    key: "setConfiguratorRef",
    value: function setConfiguratorRef(val) {
      this.configuratorRef = val;
    }
  }, {
    key: "setCoreMobile",
    value: function setCoreMobile(val) {
      this.coreMobile = val;
    }
  }, {
    key: "i18nInstance",
    get: function get() {
      return this.i18n;
    },
    set: function set(i18n) {
      this.i18n = i18n;
    }
  }, {
    key: "ItemsPaletteInProductVisible",
    get: function get() {
      return this.settingsState.itemPaletteInProduct.length && !this.configSettingsIsExpanded && !this.configSettingsIsMaxExpanded && !this.isMobileLandscape;
    }
  }, {
    key: "setUser",
    value: function setUser(user) {
      this.user = user;
    }
  }, {
    key: "setOrganization",
    value: function setOrganization(organization) {
      this.organization = organization;
      this.setCheckoutConfig();
    }
  }, {
    key: "setFirstVisit",
    value: function setFirstVisit(val) {
      this.firstVisit = val;
    }
  }, {
    key: "setIsLoading",
    value: function setIsLoading(val) {
      this.isLoading = val;
    }
  }, {
    key: "setConfiguratorDisabled",
    value: function setConfiguratorDisabled(val) {
      this.configuratorDisabled = val;
    }
  }, {
    key: "setConfiguratorHidden",
    value: function setConfiguratorHidden(val) {
      this.configuratorHidden = val;
    }
  }, {
    key: "checkIsInMobileView",
    value: function () {
      var _checkIsInMobileView = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee2(dimensions) {
        var _this$ViewConfig$mobi;
        var width, height, mobileDirectionChaned, _this$getReturnTo;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) switch (_context2.prev = _context2.next) {
            case 0:
              if (!this.isCheckCurrentlyCheckingInMobileView) {
                _context2.next = 2;
                break;
              }
              return _context2.abrupt("return");
            case 2:
              this.isCheckCurrentlyCheckingInMobileView = true;
              width = dimensions.width;
              height = dimensions.height;
              if (!(!width || !height)) {
                _context2.next = 8;
                break;
              }
              this.isCheckCurrentlyCheckingInMobileView = false;
              return _context2.abrupt("return");
            case 8:
              mobileDirectionChaned = false;
              if (/Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) || width < ((_this$ViewConfig$mobi = this.ViewConfig.mobileThreshhold) !== null && _this$ViewConfig$mobi !== void 0 ? _this$ViewConfig$mobi : 800)) {
                if (width < height) {
                  if (this.isMobileLandscape || !this.isMobilePortrait) mobileDirectionChaned = true;
                  this.isMobilePortrait = true;
                  this.isMobileLandscape = false;
                } else if (!/iPad/i.test(navigator.userAgent) && !this.ViewConfig.disableLandscape && !((_this$getReturnTo = this.getReturnTo()) !== null && _this$getReturnTo !== void 0 && _this$getReturnTo.includes('OPEN_SELLER_BASKET'))) {
                  if (this.isMobilePortrait || !this.isMobileLandscape) mobileDirectionChaned = true;
                  this.isMobilePortrait = false;
                  this.isMobileLandscape = true;
                } else {
                  this.isMobilePortrait = false;
                  this.isMobileLandscape = false;
                }
              } else {
                this.isMobilePortrait = false;
                this.isMobileLandscape = false;
              }
              if (this.isInMobileView) {
                this.configSettingsIsExpanded = false;
              }
              //fix for safari ios devices when rotate https://stackoverflow.com/questions/62717621/white-space-at-page-bottom-after-device-rotation-in-ios-safari
              if (mobileDirectionChaned) {
                this.configSettingsIsMaxExpanded = false;
                this.configSettingsIsMinExpanded = false;
                if (/iPhone|iPad|iPod/i.test(navigator.userAgent)) {
                  setTimeout(function () {
                    window.scrollTo(10, 0);
                    window.scrollTo(0, 0);
                  }, 200);
                }
              }

              // const settingsWidth = this.settingsState.settingsScrollerElement?.getBoundingClientRect().width;

              this.settingsState.setSettingsWidth(width);
              this.updateCurTabGroupType();
              if (!this.checkedMobileFirstTime) {
                this.checkedMobileFirstTime = true;
                //this.setIsLoading(false);
                this.configuratorHidden = false;
                DivaUtils.endSpanOfApm('initialize Configurator');
              }
              this.isCheckCurrentlyCheckingInMobileView = false;
            case 16:
            case "end":
              return _context2.stop();
          }
        }, _callee2, this);
      }));
      function checkIsInMobileView(_x) {
        return _checkIsInMobileView.apply(this, arguments);
      }
      return checkIsInMobileView;
    }()
  }, {
    key: "isInMobileView",
    get: function get() {
      return this.isMobilePortrait || this.isMobileLandscape;
    }
  }, {
    key: "setConfigSettingsIsExpanded",
    value: function setConfigSettingsIsExpanded(val) {
      this.configSettingsIsExpanded = val;
    }
  }, {
    key: "setConfigSettingsIsMaxExpanded",
    value: function setConfigSettingsIsMaxExpanded(val) {
      this.configSettingsIsMaxExpanded = val;
    }
  }, {
    key: "setConfigSettingsIsMinExpanded",
    value: function setConfigSettingsIsMinExpanded(val) {
      this.configSettingsIsMinExpanded = val;
    }
  }, {
    key: "ViewConfig",
    get: function get() {
      return this.config.viewConfig;
    }
  }, {
    key: "AllConfig",
    get: function get() {
      return this.config;
    }
  }, {
    key: "initFromConfig",
    value: function initFromConfig(config) {
      this.config = config;
      this.doShowConfSettingsExpander = config.viewConfig.doShowConfSettingsExpander;
      this.doShowConfSettingsMaxMinExpander = config.viewConfig.doShowConfSettingsMaxMinExpander;
      this.setCheckoutConfig();
    }
  }, {
    key: "reset",
    value: function reset() {
      this.configSettingsIsExpanded = false;
      if (!this.configSettingsIsHiddenInitialized) this.configSettingsIsHidden = false;
      this.configSettingsIsMinExpanded = false;
    }
  }, {
    key: "buildResizer",
    value: /************************Resize Logic ***************************************/

    function buildResizer() {
      var _this2 = this;
      //@ts-ignore
      this.resizeObserver = new ResizeObserver(function () {
        _this2.onResize();
      });
      var element = document.querySelectorAll('#wcConfigurator');
      if (element[0]) this.resizeObserver.observe(element[0]);
    }
  }, {
    key: "onResize",
    value: function onResize() {
      if (this.configPalettescrollEventCounter > 4) {
        this.configPalettescrollEventCounter = 0;
        this.visitActVisibleSteps();
      } else {
        this.configPalettescrollEventCounter++;
      }
      if (this.tabGroupscrollEventCounter > 4) {
        this.tabGroupscrollEventCounter = 0;
      } else {
        this.tabGroupscrollEventCounter++;
      }
    }
  }, {
    key: "resizeCore",
    value: function () {
      var _resizeCore = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee3(dimensions) {
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return this.checkIsInMobileView(dimensions);
            case 2:
              this.updatePlannerView('RESIZE_PLANNER');
            case 3:
            case "end":
              return _context3.stop();
          }
        }, _callee3, this);
      }));
      function resizeCore(_x2) {
        return _resizeCore.apply(this, arguments);
      }
      return resizeCore;
    }()
  }, {
    key: "setHidePrices",
    value: function setHidePrices(hidden) {
      this.hidePrices = hidden;
    }
  }, {
    key: "buildResizeObserver",
    value: function buildResizeObserver() {
      if (this.resizeObserver) {
        this.resizeObserver.disconnect();
        this.resizeObserver = undefined;
      }
      if ('ResizeObserver' in self) {
        this.buildResizer();
      } else {
        DivaUtils.loadScript('https://cdn.jsdelivr.net/npm/resize-observer-polyfill@1.5.1/dist/ResizeObserver.min.js', this.buildResizer.bind(this));
      }
    }
  }, {
    key: "setUpSettingsScrollEvent",
    value: function setUpSettingsScrollEvent(element) {
      var _this3 = this;
      //to cleanup
      var onScroll = function onScroll() {
        // this.viewModel.golbalToolTip.hide();
        if (_this3.configPalettescrollEventCounter > 4) {
          _this3.configPalettescrollEventCounter = 0;
          //every 4 events visits the act Visible steps
          _this3.visitActVisibleSteps();
        } else {
          _this3.configPalettescrollEventCounter++;
        }
      };

      //detects Scrolling on the ConfigBar
      element.addEventListener('scroll', onScroll);
      return function () {
        return element.removeEventListener('scroll', onScroll);
      };
    }
  }, {
    key: "setUpTabGropsScrollEvent",
    value: function setUpTabGropsScrollEvent(element) {
      var _this4 = this;
      //detects Scrolling on the ConfigBar
      var onScroll = function onScroll() {
        if (_this4.tabGroupscrollEventCounter > 4) {
          _this4.tabGroupscrollEventCounter = 0;
        } else {
          _this4.tabGroupscrollEventCounter++;
        }
      };
      element.addEventListener('scroll', onScroll);
      return function () {
        return element.removeEventListener('scroll', onScroll);
      };
    }
  }, {
    key: "setActiveCatalogSection",
    value: function () {
      var _setActiveCatalogSection = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee4(val) {
        var somethingChanged;
        return _regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) switch (_context4.prev = _context4.next) {
            case 0:
              somethingChanged = val !== this.activeCatalogSection;
              this.activeCatalogSection = val;
              if (!somethingChanged) {
                _context4.next = 5;
                break;
              }
              _context4.next = 5;
              return DivaUtils.Sleep(2000);
            case 5:
            case "end":
              return _context4.stop();
          }
        }, _callee4, this);
      }));
      function setActiveCatalogSection(_x3) {
        return _setActiveCatalogSection.apply(this, arguments);
      }
      return setActiveCatalogSection;
    }()
  }, {
    key: "setCheckoutConfig",
    value: function setCheckoutConfig() {
      var c = this.ViewConfig.checkout;
      if (c) {
        var _this$organization, _c$url, _this$getReturnTo2;
        if (c && (!((_this$organization = this.organization) !== null && _this$organization !== void 0 && _this$organization.shareBaseUrl) && JSON.stringify((_c$url = c === null || c === void 0 ? void 0 : c.url) !== null && _c$url !== void 0 ? _c$url : '').includes('organization.shareBaseUrl') || (_this$getReturnTo2 = this.getReturnTo()) !== null && _this$getReturnTo2 !== void 0 && _this$getReturnTo2.includes('FAVORITS'))) {
          this.checkoutConfig = undefined;
        } else {
          this.checkoutConfig = c;
        }
      }
    }
  }]);
}();

function ownKeys$b(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$b(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$b(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$b(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$a = getLogger('Planner', 'Product');
var ProductState = /*#__PURE__*/function () {
  function ProductState(initializeView, logAnalytics, getState, eventIsManagedByClient, forceViewToEnterPlannerMode, toggleLoading) {
    var _this = this;
    _classCallCheck(this, ProductState);
    _defineProperty(this, "divaWebPlanner", void 0);
    _defineProperty(this, "settingsState", void 0);
    _defineProperty(this, "initializeView", void 0);
    _defineProperty(this, "toggleLoading", void 0);
    _defineProperty(this, "logAnalytics", void 0);
    _defineProperty(this, "setCurPlannerViewMode", void 0);
    _defineProperty(this, "onClickAddToBasket", void 0);
    _defineProperty(this, "getState", void 0);
    _defineProperty(this, "eventIsManagedByClient", void 0);
    _defineProperty(this, "handlePlanner3DGridChanged", void 0);
    _defineProperty(this, "forceViewToEnterPlannerMode", void 0);
    _defineProperty(this, "initPlannerViewMode", void 0);
    _defineProperty(this, "changeOnCheckoutConfig", void 0);
    _defineProperty(this, "loadRTVForAr", false);
    _defineProperty(this, "activateAR", undefined);
    _defineProperty(this, "isArCompatible", undefined);
    _defineProperty(this, "glbUrl", undefined);
    _defineProperty(this, "plannerMode", undefined);
    _defineProperty(this, "isBoxPlanner", false);
    /**************************************PRODUCT Header****************************************************/
    _defineProperty(this, "webConifguratorTitle", '');
    _defineProperty(this, "formatedTotalPrice", '');
    _defineProperty(this, "formatedBruttoPrice", undefined);
    _defineProperty(this, "formatedCurSetPrice", '');
    _defineProperty(this, "customDimensions", undefined);
    _defineProperty(this, "currentBanner", undefined);
    /*************************************PRODUCT Functions***************************************************/
    //Share and co. Functions
    _defineProperty(this, "doShowFunctions", false);
    _defineProperty(this, "doShowUserSaveButton", false);
    _defineProperty(this, "showArQrCode", undefined);
    _defineProperty(this, "onClickReset", function () {
      var _this$toggleLoading;
      (_this$toggleLoading = _this.toggleLoading) === null || _this$toggleLoading === void 0 || _this$toggleLoading.call(_this, 'Planner_Loading', false, true);
      _this.logAnalytics('onReset', 'Click on Reset Button');
      _this.initializeView(false, true);
      LOG$a.debug('die Gesammte Planung wird resetet');
    });
    _defineProperty(this, "onClickHelp", function () {
      var _this$divaWebPlanner;
      (_this$divaWebPlanner = _this.divaWebPlanner) === null || _this$divaWebPlanner === void 0 || _this$divaWebPlanner.callEvent('onWebPlanerHelp');
      LOG$a.debug('Hilfe anzeigen');
      _this.logAnalytics('onHelpPlanner', 'Click on Help Button');
    });
    _defineProperty(this, "onClickShare", function () {
      _this.onClickAddToBasket(undefined, 'onWebPlanerShare');
      LOG$a.debug('Planung Teilen');
      _this.logAnalytics('onSharePlanner', 'Click on Share Button');
      LOG$a.businessEvent('onShareStep', 'Planner', 'Teilen der Planung', {});
    });
    _defineProperty(this, "onClickPrint", function () {
      _this.onClickAddToBasket(undefined, 'onWebPlanerPrint');
      LOG$a.debug('Planung Drucken / PDF generieren');
      _this.logAnalytics('onPrintPlanner', 'Click on Print Button');
    });
    _defineProperty(this, "onClickSave", function () {
      _this.onClickAddToBasket(undefined, 'onWebPlanerSave');
      LOG$a.debug('Planung speichern');
      _this.logAnalytics('onSavePlanner', 'Click on Save Button');
    });
    _defineProperty(this, "onRequestConsulation", function () {
      var _this$divaWebPlanner2;
      (_this$divaWebPlanner2 = _this.divaWebPlanner) === null || _this$divaWebPlanner2 === void 0 || _this$divaWebPlanner2.showModal({
        title: 'modal.requestConsulation.title',
        text: 'modal.requestConsulation.content',
        rejectByIcon: true,
        reject: 'modal.requestConsulation.rejectText',
        accept: 'modal.requestConsulation.acceptText',
        callback: function callback(accepted) {
          if (accepted) {
            _this.onClickAddToBasket(undefined, 'onRequestConsulation');
            LOG$a.debug('Planung speichern / Beratungstermin anfragen');
            _this.logAnalytics('onRequestConsulation', 'Click on Request Consulation');
            LOG$a.businessEvent('onConsultancyStep', 'Planner', 'Beratungstermin anfragen', {});
          }
        }
      });
    });
    _defineProperty(this, "onClickSave2", function () {
      var _this$divaWebPlanner3;
      (_this$divaWebPlanner3 = _this.divaWebPlanner) === null || _this$divaWebPlanner3 === void 0 || _this$divaWebPlanner3.showModal({
        title: 'modal.onSave.title',
        text: 'modal.onSave.content',
        rejectByIcon: true,
        accept: 'modal.onSave.acceptText',
        reject: 'modal.onSave.rejectText',
        callback: function callback(accepted) {
          if (accepted) {
            _this.onClickAddToBasket(undefined, 'onSave');
            LOG$a.debug('Planung speichern Variante Ralph');
            _this.logAnalytics('onSave', 'Click on Save Variant Ralph');
          }
        }
      });
    });
    /**************************************PRODUCT Center****************************************************/
    //3D Grid
    _defineProperty(this, "allowRendering", true);
    _defineProperty(this, "doShowRenderImage", false);
    _defineProperty(this, "diva3dImgSrc", 'data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==');
    //2D Grid
    _defineProperty(this, "_allowGrid", true);
    _defineProperty(this, "doShowGrid", false);
    _defineProperty(this, "GridImageURL", '');
    //RTV Real Time Viewer
    _defineProperty(this, "allowRTV", false);
    _defineProperty(this, "allowQRCodeAR", false);
    _defineProperty(this, "doShowRTV", false);
    _defineProperty(this, "allow3DComb", false);
    _defineProperty(this, "doShow3DComb", false);
    _defineProperty(this, "shelfPlannerRTV", false);
    _defineProperty(this, "pricePreview", undefined);
    _defineProperty(this, "hide2DPlanner", false);
    //Article Image
    _defineProperty(this, "articleImage", '');
    _defineProperty(this, "selectedPromotionIndex", undefined);
    _defineProperty(this, "setSelectedPromitionIndex", function (index) {
      _this.selectedPromotionIndex = index;
    });
    _defineProperty(this, "configChangedByPromoOpvs", false);
    _defineProperty(this, "setConfigChangedByPromoOpvs", function (val) {
      _this.configChangedByPromoOpvs = val;
    });
    _defineProperty(this, "configFirstPromo", function () {
      var _this$divaWebPlanner4, _this$settingsState;
      var promotedOpvs = (_this$divaWebPlanner4 = _this.divaWebPlanner) === null || _this$divaWebPlanner4 === void 0 || (_this$divaWebPlanner4 = _this$divaWebPlanner4.actSet()) === null || _this$divaWebPlanner4 === void 0 || (_this$divaWebPlanner4 = _this$divaWebPlanner4.CatalogData) === null || _this$divaWebPlanner4 === void 0 ? void 0 : _this$divaWebPlanner4.promotedOpvs;
      var selectedArticleCodex = (_this$settingsState = _this.settingsState) === null || _this$settingsState === void 0 || (_this$settingsState = _this$settingsState.selectedArticles[0]) === null || _this$settingsState === void 0 ? void 0 : _this$settingsState.codex;
      if (!promotedOpvs || !selectedArticleCodex) return;
      var promo = promotedOpvs.articles[selectedArticleCodex][0];
      _this.onPromotionClick(promo.variantValues, 0);
    });
    _defineProperty(this, "onPromotionClick", function (variantValues, i) {
      _this.setSelectedPromitionIndex(i);
      _this.setConfigChangedByPromoOpvs(true);
      _this.changeOnCheckoutConfig(variantValues);
    });
    //3D Navbr
    _defineProperty(this, "doShowNav3D", false);
    _defineProperty(this, "doShowNav3DCOMB", false);
    _defineProperty(this, "onDoShowNav3DChange", function (doShow) {
      _this.doShowNav3D = doShow;
      _this.doShowNav3DCOMB = doShow;
    });
    _defineProperty(this, "doShowNav3DPlannerMode", true);
    _defineProperty(this, "doShow3DZoom", true);
    _defineProperty(this, "doEnableToShow3DDimensionLines", false);
    //Button is shown for current model
    _defineProperty(this, "hidePlannerUiIcons", false);
    _defineProperty(this, "doShow3DDimensionLines", false);
    //Button is activated
    _defineProperty(this, "doShowARQRCode", false);
    _defineProperty(this, "arCodeWasVisibleOnce", !!DivaUtils$1.getCookie('arCodeWasVisibleOnce'));
    _defineProperty(this, "getDoShow3DDimensionLines", function () {
      return _this.doShow3DDimensionLines;
    });
    _defineProperty(this, "setDoShow3DDimensionLines", function (val) {
      _this.doShow3DDimensionLines = val;
    });
    _defineProperty(this, "plannerRenderingIsStaticBG", true);
    _defineProperty(this, "setPlannerRenderingIsStaticBG", function (val) {
      _this.plannerRenderingIsStaticBG = val;
    });
    _defineProperty(this, "plannerRenderingIsStart", true);
    _defineProperty(this, "setPlannerRenderingIsStart", function (val) {
      _this.plannerRenderingIsStart = val;
    });
    _defineProperty(this, "curAlternateRendering", undefined);
    _defineProperty(this, "roomListExpanded", false);
    _defineProperty(this, "lastTimeoutRoomlistExpanded", undefined);
    _defineProperty(this, "doShow2DGridInViewModel", function (doShow) {
      if (doShow) {
        _this.doShowGrid = true;
        _this.doShowRTV = false;
        _this.doShow3DComb = false;
        _this.doShowRenderImage = false;
      } else {
        _this.doShowGrid = false;
        _this.doShowRTV = false;
        _this.doShow3DComb = false;
        _this.doShowRenderImage = false;
        if (_this.allow3DComb) {
          _this.doShow3DComb = true;
          _this.doShowRenderImage = false;
        } else {
          _this.doShowRenderImage = true;
          _this.doShow3DComb = false;
        }
      }
    });
    _defineProperty(this, "onChangePlannerView", function (type, alternateRendering) {
      var _this$divaWebPlanner5, _this$divaWebPlanner6, _this$divaWebPlanner7, _this$divaWebPlanner9;
      switch (type) {
        case '3DCOMB':
          _this.doShowRenderImage = false;
          _this.doShowRTV = false;
          _this.doShowGrid = false;
          _this.doShow3DComb = true;
          _this.setCurPlannerViewMode('3DCOMB');
          (_this$divaWebPlanner5 = _this.divaWebPlanner) === null || _this$divaWebPlanner5 === void 0 || _this$divaWebPlanner5.updatePlannerView('3DCOMB');
          _this.logAnalytics('onShow3DCOMB', 'Click on 3DCOMB Button');
          break;
        case '3D':
          _this.doShowRenderImage = true;
          _this.doShowRTV = false;
          _this.doShow3DComb = false;
          _this.doShowGrid = false;
          _this.curAlternateRendering = alternateRendering;
          _this.setCurPlannerViewMode('3DImage');
          (_this$divaWebPlanner6 = _this.divaWebPlanner) === null || _this$divaWebPlanner6 === void 0 || _this$divaWebPlanner6.updatePlannerView('ENTER_3D_VIEW');
          _this.logAnalytics('onShow3D', 'Click on 3D Button');
          break;
        case '2D':
          _this.doShowRenderImage = false;
          _this.doShowRTV = false;
          _this.doShow3DComb = false;
          _this.doShowGrid = true;
          _this.setCurPlannerViewMode('2DGrid');
          (_this$divaWebPlanner7 = _this.divaWebPlanner) === null || _this$divaWebPlanner7 === void 0 || _this$divaWebPlanner7.updatePlannerView('2D_PLANNER');
          setTimeout(function () {
            var _this$divaWebPlanner8;
            (_this$divaWebPlanner8 = _this.divaWebPlanner) === null || _this$divaWebPlanner8 === void 0 || _this$divaWebPlanner8.updatePlannerView('2D_PLANNER');
          }, 100);
          _this.logAnalytics('onShow2D', 'Click on 2D Button');
          break;
        case 'RTV':
          _this.doShowRenderImage = false;
          _this.doShowGrid = false;
          _this.doShowRTV = true;
          _this.doShow3DComb = false;
          _this.setCurPlannerViewMode('RTViewer');
          (_this$divaWebPlanner9 = _this.divaWebPlanner) === null || _this$divaWebPlanner9 === void 0 || _this$divaWebPlanner9.updatePlannerView('RTV');
          _this.logAnalytics('onShowRTV', 'Click on RTV Button');
          break;
      }
      LOG$a.businessEvent('onDimensoinVisualization', 'Planner', 'Wechsel der Visualisierung Des Planners', {
        dimensionVisualization: type
      });
    });
    //Planner MsgContainer
    _defineProperty(this, "plannerMessages", new Map());
    _defineProperty(this, "onPlannerMsgArrowClick", function (codex) {
      var item = _this.plannerMessages.get(codex);
      if (item) {
        _this.plannerMessages.set(codex, _objectSpread$b(_objectSpread$b({}, item), {}, {
          collapsed: !item.collapsed
        }));
      }
    });
    /**
     * This function is used to display a info message in the planner.
     * @param doShowPlannerMsg  Gibt an, ob die meldung ein oder ausgeblendet werden soll
     * @param type              Gibt an, ob es sich um eine Warnung oder Info handelt.
     * @param content           Gibt den Text an, der ausgegebn werden soll
     * @param autoHide            Gibt an ob die Warnung nach einiger ZEit wieder verschwinden soll
     */
    _defineProperty(this, "updatePlannerMsgContainer", function (codex, doShowPlannerMsg, type, content, autoHide, timeoutDuration) {
      if (timeoutDuration == undefined || timeoutDuration <= 0) timeoutDuration = 5000;
      if (doShowPlannerMsg) {
        _this.plannerMessages.set(codex, {
          doShowPlannerMsg: doShowPlannerMsg,
          type: type,
          content: content,
          autoHide: autoHide,
          timeoutDuration: timeoutDuration,
          collapsed: false
        });
      } else {
        if (_this.plannerMessages.get(codex)) _this.plannerMessages["delete"](codex);
      }
    });
    _defineProperty(this, "onNav3DMenuSelect", function (codex) {
      var _this$divaWebPlanner10, _this$divaWebPlanner11, _this$divaWebPlanner12, _this$divaWebPlanner13, _this$divaWebPlanner18;
      switch (codex) {
        case 'Nav3DZoomIn':
          (_this$divaWebPlanner10 = _this.divaWebPlanner) === null || _this$divaWebPlanner10 === void 0 || _this$divaWebPlanner10.handle3DVisualizationChanged('Nav3DZoom', {
            nav3DZoom: {
              doZoomIn: true
            }
          });
          break;
        case 'Nav3DZoomOut':
          (_this$divaWebPlanner11 = _this.divaWebPlanner) === null || _this$divaWebPlanner11 === void 0 || _this$divaWebPlanner11.handle3DVisualizationChanged('Nav3DZoom', {
            nav3DZoom: {
              doZoomOut: true
            }
          });
          break;
        case 'Nav3DViews':
          _this.setPlannerRenderingIsStart(true);
          (_this$divaWebPlanner12 = _this.divaWebPlanner) === null || _this$divaWebPlanner12 === void 0 || _this$divaWebPlanner12.handle3DVisualizationChanged('Nav3DSetVPByName', {
            nav3DSetVPByName: {
              vpName: 'Start',
              clearNav: true
            }
          });
          break;
        case 'Nav3DFullScreen':
          if (!_this.eventIsManagedByClient('onWebPlanerZoomClick')) {
            var _this$toggleLoading2;
            (_this$toggleLoading2 = _this.toggleLoading) === null || _this$toggleLoading2 === void 0 || _this$toggleLoading2.call(_this, 'Planner_Rendering', false, true);
          }
          (_this$divaWebPlanner13 = _this.divaWebPlanner) === null || _this$divaWebPlanner13 === void 0 || _this$divaWebPlanner13.handle3DVisualizationChanged('Nav3DFullScreen', {
            nav3DFullScreen: {
              renderWidth: 2000,
              renderHeight: 1000
            }
          });
          break;
        case 'Nav3DPlannerMode':
          if (['PLANNER', 'PRODUCTS', 'PROMOTED', 'ACC_PLANNER'].includes(_this.getState())) {
            var _this$divaWebPlanner14, _this$divaWebPlanner15;
            _this.handlePlanner3DGridChanged();
            (_this$divaWebPlanner14 = _this.divaWebPlanner) === null || _this$divaWebPlanner14 === void 0 || _this$divaWebPlanner14.handle3DVisualizationChanged('Nav3DPlannerMode', {});
            (_this$divaWebPlanner15 = _this.divaWebPlanner) === null || _this$divaWebPlanner15 === void 0 || _this$divaWebPlanner15.updatePlannerView('NAV_3D');
            break;
          } else {
            var _this$divaWebPlanner16, _this$divaWebPlanner17;
            //Erster in den Planer zurückspringen. - Meldung ausgeben.
            _this.forceViewToEnterPlannerMode(true);
            _this.handlePlanner3DGridChanged();
            (_this$divaWebPlanner16 = _this.divaWebPlanner) === null || _this$divaWebPlanner16 === void 0 || _this$divaWebPlanner16.handle3DVisualizationChanged('Nav3DPlannerMode', {});
            (_this$divaWebPlanner17 = _this.divaWebPlanner) === null || _this$divaWebPlanner17 === void 0 || _this$divaWebPlanner17.updatePlannerView('NAV_3D');
          }
          break;
        case 'Nav3DDimLine':
          _this.doShow3DDimensionLines = !_this.doShow3DDimensionLines;
          (_this$divaWebPlanner18 = _this.divaWebPlanner) === null || _this$divaWebPlanner18 === void 0 || _this$divaWebPlanner18.updatePlannerView('NAV_3D');
          break;
      }
      LOG$a.businessEvent('onVisualizationChange', 'Planner', 'Click on 3D Nav Button', {
        visualization: codex
      });
    });
    _defineProperty(this, "setDoShowARQRCode", function (val) {
      _this.doShowARQRCode = val;
    });
    _defineProperty(this, "setArCodeWasVisibleOnce", function () {
      var _this$showArQrCode$co, _this$showArQrCode;
      _this.arCodeWasVisibleOnce = true;
      // if the ar-qr-code was visible once we need to keep updating it
      if (_this.divaWebPlanner) _this.divaWebPlanner.doLoadHQR = true;
      DivaUtils$1.setCookie('arCodeWasVisibleOnce', '1', (_this$showArQrCode$co = (_this$showArQrCode = _this.showArQrCode) === null || _this$showArQrCode === void 0 ? void 0 : _this$showArQrCode.cookieDays) !== null && _this$showArQrCode$co !== void 0 ? _this$showArQrCode$co : 1);
    });
    _defineProperty(this, "productElement", null);
    _defineProperty(this, "image3DElement", null);
    _defineProperty(this, "planner2DGridElement", null);
    _defineProperty(this, "planner3DGridElement", null);
    _defineProperty(this, "planner3DImageSvgArea", null);
    _defineProperty(this, "orbitCubeElement", null);
    _defineProperty(this, "orbitHidden", true);
    _defineProperty(this, "ignoreUpdateRenderingOnOrbitChanged", false);
    this.initializeView = initializeView;
    this.toggleLoading = toggleLoading;
    this.logAnalytics = logAnalytics;
    this.getState = getState;
    this.eventIsManagedByClient = eventIsManagedByClient;
    this.forceViewToEnterPlannerMode = forceViewToEnterPlannerMode;
    makeObservable(this, {
      initialize: action,
      plannerMode: observable,
      isBoxPlanner: observable,
      webConifguratorTitle: observable,
      setWebConifguratorTitle: action,
      formatedTotalPrice: observable,
      formatedBruttoPrice: observable,
      formatedCurSetPrice: observable,
      setFormatedPrices: action,
      currentBanner: observable,
      setCurrentBanner: action,
      initViewConfig: action,
      doShowFunctions: observable,
      doShowUserSaveButton: observable,
      _allowGrid: observable,
      hide2DPlanner: observable,
      setAllowGrid: action,
      doShowGrid: observable,
      doShowRTV: observable,
      doShow3DComb: observable,
      allowRTV: observable,
      allow3DComb: observable,
      allowQRCodeAR: observable,
      shelfPlannerRTV: observable,
      GridImageURL: observable,
      allowRendering: observable,
      setAllowRendering: action,
      footerVisible: computed,
      doShowRenderImage: observable,
      diva3dImgSrc: observable,
      setDiva3dImgSrc: action,
      articleImage: observable,
      setArticleImage: action,
      updateSetImage: action,
      doShowNav3D: observable,
      doShowNav3DCOMB: observable,
      onDoShowNav3DChange: action,
      doShowNav3DPlannerMode: observable,
      doShow3DZoom: observable,
      doEnableToShow3DDimensionLines: observable,
      hidePlannerUiIcons: observable,
      doShow3DDimensionLines: observable,
      setDoShow3DDimensionLines: action,
      doShowARQRCode: observable,
      setDoShowARQRCode: action,
      arCodeWasVisibleOnce: observable,
      setArCodeWasVisibleOnce: action,
      plannerRenderingIsStaticBG: observable,
      setPlannerRenderingIsStaticBG: action,
      plannerRenderingIsStart: observable,
      setPlannerRenderingIsStart: action,
      roomListExpanded: observable,
      setRoomListExpanded: action,
      curAlternateRendering: observable,
      setCurAlternateRendering: action,
      onChangePlannerView: action,
      doShow2DGridInViewModel: action,
      plannerMessages: observable,
      plannerMessagesArr: computed,
      onPlannerMsgArrowClick: action,
      updatePlannerMsgContainer: action,
      onNav3DMenuSelect: action,
      initConfiguratorMode: action,
      productElement: observable,
      setProductElement: action,
      image3DElement: observable,
      planner2DGridElement: observable,
      planner3DGridElement: observable,
      planner3DImageSvgArea: observable,
      orbitCubeElement: observable,
      setImage3DElement: action,
      setPlanner2DGridElement: action,
      setPlanner3DGridElement: action,
      setPlanner3DImageSvgArea: action,
      setOrbitCubeElement: action,
      orbitHidden: observable,
      setOrbitIsHidden: action,
      ignoreUpdateRenderingOnOrbitChanged: observable,
      promotionVisible: computed,
      getPlannerRenderingSize: observable,
      selectedPromotionIndex: observable,
      setSelectedPromitionIndex: action,
      configChangedByPromoOpvs: observable,
      setConfigChangedByPromoOpvs: action,
      activateAR: observable,
      isArCompatible: observable,
      setRTVActivateAR: action,
      loadRTVForAr: observable,
      glbUrl: observable
    });
  }
  return _createClass(ProductState, [{
    key: "initialize",
    value: function initialize(divaWebPlanner, settingsState, setCurPlannerViewMode, handlePlanner3DGridChanged, initPlannerViewMode, onClickAddToBasket, changeOnCheckoutConfig) {
      this.divaWebPlanner = divaWebPlanner;
      this.settingsState = settingsState;
      this.setCurPlannerViewMode = setCurPlannerViewMode;
      this.handlePlanner3DGridChanged = handlePlanner3DGridChanged;
      this.initPlannerViewMode = initPlannerViewMode;
      this.onClickAddToBasket = onClickAddToBasket;
      this.changeOnCheckoutConfig = changeOnCheckoutConfig;
      this.doShow3DDimensionLines = false;
      this.plannerRenderingIsStaticBG = true;
      this.setWebConifguratorTitle('');
    }
  }, {
    key: "setWebConifguratorTitle",
    value: function setWebConifguratorTitle(val) {
      this.webConifguratorTitle = DivaUtils$1.stripTags(val !== null && val !== void 0 ? val : '');
    }
  }, {
    key: "setFormatedPrices",
    value: function setFormatedPrices(totalPrice, setPrice, bruttoPrice, customDimensions) {
      this.formatedTotalPrice = totalPrice;
      this.formatedCurSetPrice = setPrice;
      this.formatedBruttoPrice = bruttoPrice;
      this.customDimensions = customDimensions;
    }
  }, {
    key: "setCurrentBanner",
    value: function setCurrentBanner(banner) {
      this.currentBanner = banner;
    }
  }, {
    key: "initViewConfig",
    value: function initViewConfig(doShowFunctions, doShowUserSaveButton, showArQrCode) {
      this.doShowFunctions = doShowFunctions;
      this.doShowUserSaveButton = doShowUserSaveButton;
      this.showArQrCode = showArQrCode;
    }
  }, {
    key: "syncOnClickShare",
    value: function () {
      var _syncOnClickShare = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return this.onClickAddToBasket(undefined, 'onWebPlanerShare');
            case 2:
              LOG$a.debug('Planung Teilen');
              this.logAnalytics('onSharePlanner', 'Click on Share Button');
              LOG$a.businessEvent('onShareStep', 'Planner', 'Teilen der Planung', {});
            case 5:
            case "end":
              return _context.stop();
          }
        }, _callee, this);
      }));
      function syncOnClickShare() {
        return _syncOnClickShare.apply(this, arguments);
      }
      return syncOnClickShare;
    }()
  }, {
    key: "setAllowRendering",
    value: function setAllowRendering(val) {
      this._allowGrid = val;
    }
  }, {
    key: "setDiva3dImgSrc",
    value: function setDiva3dImgSrc(val) {
      this.diva3dImgSrc = val;
    }
  }, {
    key: "allowGrid",
    get: function get() {
      return this._allowGrid && !this.hide2DPlanner;
    }
  }, {
    key: "setAllowGrid",
    value: function setAllowGrid(val) {
      this._allowGrid = val;
    }
  }, {
    key: "footerVisible",
    get: function get() {
      var visibleFooter = {
        allowRendering: this.allowRendering,
        allowGrid: this.allowGrid,
        allowRTV: this.allowRTV,
        allow3DComb: this.allow3DComb
      };
      return Object.keys(visibleFooter).filter(function (f) {
        return visibleFooter[f];
      });
    }
  }, {
    key: "setRTVActivateAR",
    value: function setRTVActivateAR(isArCompatible, activateAR) {
      this.isArCompatible = isArCompatible;
      this.activateAR = activateAR;
    }
  }, {
    key: "browserSupportAR",
    get: function get() {
      var userAgent = navigator.userAgent.toLowerCase();
      return !(userAgent.includes('fxios') || userAgent.includes('firefox') || userAgent.includes('edgios') && userAgent.includes('iphone'));
    }
  }, {
    key: "setArticleImage",
    value: function setArticleImage(url) {
      this.articleImage = url;
    }
  }, {
    key: "updateSetImage",
    value: function updateSetImage(url) {
      this.articleImage = url;
      this.diva3dImgSrc = url;
    }

    //promoted Optionvalues
  }, {
    key: "promotionVisible",
    get: function get() {
      var _this$divaWebPlanner19, _this$settingsState2, _promotedOpvs$article;
      var promotedOpvs = (_this$divaWebPlanner19 = this.divaWebPlanner) === null || _this$divaWebPlanner19 === void 0 || (_this$divaWebPlanner19 = _this$divaWebPlanner19.actSet()) === null || _this$divaWebPlanner19 === void 0 || (_this$divaWebPlanner19 = _this$divaWebPlanner19.CatalogData) === null || _this$divaWebPlanner19 === void 0 ? void 0 : _this$divaWebPlanner19.promotedOpvs;
      var selectedArticleCodex = (_this$settingsState2 = this.settingsState) === null || _this$settingsState2 === void 0 || (_this$settingsState2 = _this$settingsState2.selectedArticles) === null || _this$settingsState2 === void 0 || (_this$settingsState2 = _this$settingsState2[0]) === null || _this$settingsState2 === void 0 ? void 0 : _this$settingsState2.codex;
      var hidden = !selectedArticleCodex || !promotedOpvs || !((_promotedOpvs$article = promotedOpvs.articles[selectedArticleCodex]) !== null && _promotedOpvs$article !== void 0 && _promotedOpvs$article.length);
      return !hidden;
    }
  }, {
    key: "setCurAlternateRendering",
    value: function setCurAlternateRendering(val) {
      this.curAlternateRendering = val;
    }
  }, {
    key: "setRoomListExpanded",
    value: function setRoomListExpanded(expanded) {
      this.roomListExpanded = expanded;
      var that = this;
      if (this.lastTimeoutRoomlistExpanded != undefined) {
        window.clearTimeout(this.lastTimeoutRoomlistExpanded);
      }
      this.lastTimeoutRoomlistExpanded = setTimeout(function () {
        that.roomListExpanded = false;
      }, 8000);
    }
  }, {
    key: "plannerMessagesArr",
    get: function get() {
      return _toConsumableArray(this.plannerMessages.entries());
    }
  }, {
    key: "initConfiguratorMode",
    value:
    /**
     * this function tells the View if only 2D is avabile or 3D and 2D or neither of those
    
     */
    function initConfiguratorMode(plannerMode) {
      var _this$divaWebPlanner20, _this$divaWebPlanner21, _this$divaWebPlanner22, _this$divaWebPlanner27, _this$divaWebPlanner28, _this$divaWebPlanner29, _this$divaWebPlanner30, _this$divaWebPlanner31, _this$divaWebPlanner32, _this$divaWebPlanner33, _this$divaWebPlanner34;
      this.plannerMode = plannerMode;
      this.isBoxPlanner = plannerMode === 'BoxPlanner';
      this.doEnableToShow3DDimensionLines = !!((_this$divaWebPlanner20 = this.divaWebPlanner) !== null && _this$divaWebPlanner20 !== void 0 && _this$divaWebPlanner20.doShow3DDimensionLines());
      this.doShow3DDimensionLines = !!((_this$divaWebPlanner21 = this.divaWebPlanner) !== null && _this$divaWebPlanner21 !== void 0 && _this$divaWebPlanner21.defaultDimensionsActive());
      this.hidePlannerUiIcons = !!((_this$divaWebPlanner22 = this.divaWebPlanner) !== null && _this$divaWebPlanner22 !== void 0 && _this$divaWebPlanner22.hidePlannerUiIcons());
      this.doShowARQRCode = false;
      if (this.settingsState) {
        var _this$divaWebPlanner$, _this$divaWebPlanner23, _this$divaWebPlanner24, _this$divaWebPlanner25, _this$divaWebPlanner26;
        this.settingsState.autoOpenContentItems = (_this$divaWebPlanner$ = (_this$divaWebPlanner23 = this.divaWebPlanner) === null || _this$divaWebPlanner23 === void 0 ? void 0 : _this$divaWebPlanner23.autoOpenContentItem()) !== null && _this$divaWebPlanner$ !== void 0 ? _this$divaWebPlanner$ : !!((_this$divaWebPlanner24 = this.divaWebPlanner) !== null && _this$divaWebPlanner24 !== void 0 && (_this$divaWebPlanner24 = _this$divaWebPlanner24.getViewConfig()) !== null && _this$divaWebPlanner24 !== void 0 && _this$divaWebPlanner24.autoOpenContentItem);
        this.settingsState.doShowReplaceMode = !!((_this$divaWebPlanner25 = this.divaWebPlanner) !== null && _this$divaWebPlanner25 !== void 0 && _this$divaWebPlanner25.doShowPlannerReplaceMode());
        this.settingsState.plannerReplaceModeAutoSwitch = !!((_this$divaWebPlanner26 = this.divaWebPlanner) !== null && _this$divaWebPlanner26 !== void 0 && _this$divaWebPlanner26.doPlannerReplaceAutoSwitch());
      }
      this.pricePreview = (_this$divaWebPlanner27 = this.divaWebPlanner) === null || _this$divaWebPlanner27 === void 0 ? void 0 : _this$divaWebPlanner27.getPricePreview();
      this.hide2DPlanner = (_this$divaWebPlanner28 = this.divaWebPlanner) === null || _this$divaWebPlanner28 === void 0 ? void 0 : _this$divaWebPlanner28.hide2DPlanner();
      this.allowRTV = !!((_this$divaWebPlanner29 = this.divaWebPlanner) !== null && _this$divaWebPlanner29 !== void 0 && _this$divaWebPlanner29.getAllowRTV());
      this.allow3DComb = !!((_this$divaWebPlanner30 = this.divaWebPlanner) !== null && _this$divaWebPlanner30 !== void 0 && _this$divaWebPlanner30.getEnableUnityRtvKombi());
      this.allowQRCodeAR = !!((_this$divaWebPlanner31 = this.divaWebPlanner) !== null && _this$divaWebPlanner31 !== void 0 && _this$divaWebPlanner31.getAllowQRCodeAR());
      this.shelfPlannerRTV = !!((_this$divaWebPlanner32 = this.divaWebPlanner) !== null && _this$divaWebPlanner32 !== void 0 && _this$divaWebPlanner32.getShelfPlannerRTV());
      this.doShowNav3DPlannerMode = !((_this$divaWebPlanner33 = this.divaWebPlanner) !== null && _this$divaWebPlanner33 !== void 0 && _this$divaWebPlanner33.getHidePlanner()) && this.plannerMode != 'BoxPlanner';
      if (this.shelfPlannerRTV) this.plannerMode = 'RTViewer';
      if ((_this$divaWebPlanner34 = this.divaWebPlanner) !== null && _this$divaWebPlanner34 !== void 0 && _this$divaWebPlanner34.getEnableUnityRtvKombi()) this.plannerMode = '3DCOMB';
      this.plannerRenderingIsStaticBG = this.plannerMode == '3DCOMB' ? false : this.plannerRenderingIsStaticBG;
      this.initPlannerViewMode(this.plannerMode);
      switch (this.plannerMode) {
        case 'ProductList':
          this.doShowRenderImage = false;
          this.doShowGrid = false;
          this.doShowNav3D = false;
          this.doShowNav3DCOMB = false;
          this.doShowRTV = false;
          this.doShow3DComb = false;
          this.doShow3DZoom = true;
          break;
        case '2DGrid':
          this.setPlannerRenderingIsStaticBG(false);
          this.doShow3DZoom = true;
          this.doShowRenderImage = false;
          this.doShowGrid = true;
          this.doShowNav3D = false;
          this.doShowNav3DCOMB = false;
          this.doShowRTV = false;
          this.allowRendering = false;
          this._allowGrid = true;
          this.doShow3DComb = false;
          break;
        case 'BoxPlanner':
          this.doShow3DZoom = true;
          this.doShowRenderImage = true;
          this.doShowRTV = false;
          this.allowRendering = true;
          this._allowGrid = false;
          this.allowRTV = false;
          this.doShowNav3DCOMB = false;
          this.allow3DComb = false;
          this.doShowGrid = false;
          this.doShowNav3D = true;
          this.doShow3DComb = false;
          break;
        case '3DCOMB':
          this.doShow3DZoom = false;
          this.doShowRenderImage = false;
          this.doShowGrid = false;
          this.doShowNav3DPlannerMode = false;
          this.doShowNav3DCOMB = true;
          this.doShowNav3D = true;
          this.doShowRTV = false;
          this.allowRendering = false;
          this._allowGrid = !this.isBoxPlanner;
          this.doShow3DComb = true;
          break;
        case '3DImage':
          this.doShow3DZoom = true;
          this.doShowRenderImage = true;
          this.doShowRTV = false;
          this.allowRendering = true;
          this._allowGrid = true;
          this.doShowGrid = false;
          this.doShowNav3DCOMB = false;
          this.doShowNav3D = true;
          this.doShow3DComb = false;
          break;
        case 'No':
          this.doShow3DZoom = false;
          this.doShowRenderImage = false;
          this.doShowGrid = false;
          this.doShowRTV = false;
          this.doShowNav3D = false;
          this.allowRendering = false;
          this._allowGrid = false;
          this.doShowNav3DCOMB = false;
          this.doShow3DComb = false;
          break;
        case 'RTViewer':
          this.doShowRenderImage = false;
          this.doShowRTV = true;
          this.doShowGrid = false;
          this.doShowNav3D = false;
          this.doShowNav3DCOMB = false;
          this.allowRendering = false;
          this._allowGrid = true;
          this.doShow3DComb = false;
          this.doShow3DZoom = false;
          break;
      }
    }
  }, {
    key: "setProductElement",
    value: function setProductElement(val) {
      this.productElement = val;
    }
  }, {
    key: "setImage3DElement",
    value: function setImage3DElement(val) {
      this.image3DElement = val;
    }
  }, {
    key: "setPlanner2DGridElement",
    value: function setPlanner2DGridElement(val) {
      this.planner2DGridElement = val;
    }
  }, {
    key: "setPlanner3DGridElement",
    value: function setPlanner3DGridElement(val) {
      this.planner3DGridElement = val;
    }
  }, {
    key: "setPlanner3DImageSvgArea",
    value: function setPlanner3DImageSvgArea(val) {
      this.planner3DImageSvgArea = val;
    }
  }, {
    key: "setOrbitCubeElement",
    value: function setOrbitCubeElement(val) {
      this.orbitCubeElement = val;
    }
  }, {
    key: "setOrbitIsHidden",
    value: function setOrbitIsHidden(val) {
      this.orbitHidden = val;
    }
  }, {
    key: "setIgnoreUpdateRenderingOnOrbitChanged",
    value: function setIgnoreUpdateRenderingOnOrbitChanged(val) {
      this.ignoreUpdateRenderingOnOrbitChanged = val;
    }

    /**
     * This function is returns the act size of the 3D Container / 2D Container
     */
  }, {
    key: "getPlannerRenderingSize",
    value: function getPlannerRenderingSize() {
      var _this$image3DElement, _rect$width, _rect$height;
      var rect = (_this$image3DElement = this.image3DElement) === null || _this$image3DElement === void 0 ? void 0 : _this$image3DElement.getBoundingClientRect();
      var width = Math.round((_rect$width = rect === null || rect === void 0 ? void 0 : rect.width) !== null && _rect$width !== void 0 ? _rect$width : 0);
      var height = Math.round((_rect$height = rect === null || rect === void 0 ? void 0 : rect.height) !== null && _rect$height !== void 0 ? _rect$height : 0);
      return {
        imgWidth: width,
        imgHeight: height
      };
    }
  }]);
}();

function ownKeys$a(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$a(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$a(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$a(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var SettingsState = /*#__PURE__*/function () {
  function SettingsState() {
    var _this = this;
    _classCallCheck(this, SettingsState);
    _defineProperty(this, "configChanged", 0);
    _defineProperty(this, "ConfigDidChange", function () {
      _this.configChanged++;
    });
    /**Itempalettes */
    _defineProperty(this, "itemPalettes", new Map());
    //render specific stuff
    _defineProperty(this, "forceSettingsHeaderInScrollContainer", false);
    /**Roomlist initzializer for Rooms in Product  */
    _defineProperty(this, "roomListInitialized", 0);
    //Title
    _defineProperty(this, "paletteHeaderStepName", '');
    /**Config Single */
    _defineProperty(this, "doShowConfigSingle", false);
    _defineProperty(this, "doConfigSingle", false);
    /**Article Info  */
    _defineProperty(this, "doShowActElementInfo", false);
    _defineProperty(this, "articleInfos", {});
    /***HTML REFS */
    _defineProperty(this, "settingsScrollerElement", undefined);
    _defineProperty(this, "settingsHeaderElement", null);
    /**planner Filter */
    _defineProperty(this, "plannerFilter", {
      searchText: '',
      dockFilter: {},
      curFilteredTypes: {}
    });
    _defineProperty(this, "doShowReplaceMode", false);
    _defineProperty(this, "plannerReplaceMode", false);
    _defineProperty(this, "plannerReplaceModeAutoSwitch", false);
    _defineProperty(this, "plannerReplaceModeAvailable", false);
    /**Inputwindow */
    _defineProperty(this, "inputBoxValue", '');
    _defineProperty(this, "setInputBoxValue", function (val) {
      _this.inputBoxValue = val;
    });
    _defineProperty(this, "inputBoxVisible", false);
    _defineProperty(this, "inputBoxItem", undefined);
    _defineProperty(this, "infoBoxContent", undefined);
    /**SettingsWidth */
    _defineProperty(this, "settingsWidth", 0);
    /**
     * shelfplanner free spaces
     */
    _defineProperty(this, "freePlacesOnGrids", undefined);
    _defineProperty(this, "setFreePlacesOnGrids", function (free) {
      _this.freePlacesOnGrids = free;
      if (free === undefined) _this.setSelectedShelfAccessoire();
    });
    _defineProperty(this, "plannedPlacesOnGrids", undefined);
    _defineProperty(this, "setGridsData", function (free, planned) {
      _this.setFreePlacesOnGrids(free);
      _this.plannedPlacesOnGrids = planned;
    });
    _defineProperty(this, "selectedShelfAccessoire", undefined);
    _defineProperty(this, "autoOpenContentItems", false);
    _defineProperty(this, "contentItemId", undefined);
    _defineProperty(this, "clickedFrom", undefined);
    _defineProperty(this, "contentItemVisible", true);
    _defineProperty(this, "conetentItemOfOPVOpen", false);
    _defineProperty(this, "notificationsShown", []);
    // array of notification Indexes that where visible
    _defineProperty(this, "reset", function () {
      _this.updateInputBoxValues(false, undefined);
      _this.setGridsData();
      _this.roomListInitialized = 0;
      _this.doShowReplaceMode = false;
      _this.articleInfos = {};
      _this.notificationsShown = [];
      _this.plannerFilter = {
        searchText: '',
        dockFilter: {},
        curFilteredTypes: {}
      };
      _this.plannerReplaceMode = false;
      _this.plannerReplaceModeAvailable = false;
      _this.plannerReplaceModeAutoSwitch = false;
      _this.contentItemId = undefined;
      _this.contentItemVisible = false;
    });
    _defineProperty(this, "partlistArticlesInOverview", {});
    makeObservable(this, {
      configChanged: observable,
      ConfigDidChange: action,
      itemPalettes: observable,
      openGroupWithCodex: action,
      closeGroup: action,
      openGroup: action,
      onAllGroups: action,
      itemPaletteArr: computed,
      itemPaletteInProduct: computed,
      updateItemPalette: action,
      hideNotUsedPalettes: action,
      paletteHeaderStepName: observable,
      setPaletteHeaderStepName: action,
      forceSettingsHeaderInScrollContainer: observable,
      setForceSettingsHeaderInScrollContainer: action,
      //plannerFilter
      plannerFilter: observable,
      updatePlannerFilter: action,
      doShowReplaceMode: observable,
      plannerReplaceMode: observable,
      plannerReplaceModeAutoSwitch: observable,
      setPlannerReplaceMode: action,
      plannerReplaceModeAvailable: observable,
      setPlannerReplaceModeAvailable: action,
      //configsingle
      doShowConfigSingle: observable,
      doConfigSingle: observable,
      updateConfigSingle: action,
      //inputwindow
      inputBoxValue: observable,
      setInputBoxValue: action,
      inputBoxItem: observable,
      inputBoxVisible: observable,
      updateInputBoxValues: action,
      //roomlist
      roomListInitialized: observable,
      roomListCountUp: action,
      //articles Info
      doShowActElementInfo: observable,
      setDoShowActElementInfo: action,
      articleInfos: observable,
      getSelectedBasketIds: computed,
      setArticleInfos: action,
      initForUpdate: action,
      //infobox
      infoBoxContent: observable,
      updateInfoBox: action,
      //settingsWidth
      settingsWidth: observable,
      setSettingsWidth: action,
      settingsScrollerElement: observable,
      setScrollerElement: action,
      settingsHeaderElement: observable,
      setHeaderElement: action,
      //shelfplanner free Cells
      freePlacesOnGrids: observable,
      setFreePlacesOnGrids: action,
      setGridsData: action,
      selectedShelfAccessoire: observable,
      setSelectedShelfAccessoire: action,
      //contentItem
      contentItemId: observable,
      setContentItemId: action,
      contentItemVisible: observable,
      setContentItemVisible: action,
      clickedFrom: observable,
      setClickedFrom: action,
      isContentItemVisible: computed,
      partlistArticlesInOverview: observable,
      setPartlistArticlesInOverview: action
    });
  }
  return _createClass(SettingsState, [{
    key: "itemPaletteArr",
    get: function get() {
      return _toConsumableArray(this.itemPalettes.entries()).sort(function (a, b) {
        return a[0] < b[0] ? -1 : 1;
      });
    }
  }, {
    key: "itemPaletteInProduct",
    get: function get() {
      var newItemPalette = [];
      for (var i = 0; i < this.itemPaletteArr.length; i++) {
        var itemPalette = this.itemPaletteArr[i][1];
        if (itemPalette.isRenderedNotInSettingPalette && itemPalette.visible && !itemPalette.isRenderedInAccessorieStep) {
          newItemPalette.push({
            palette: itemPalette,
            id: itemPalette.id
          });
        }
      }
      return newItemPalette;
    }
  }, {
    key: "updateItemPalette",
    value: function updateItemPalette(paletteId, itemPalette) {
      this.itemPalettes.set(paletteId, itemPalette);
    }
  }, {
    key: "getItemPaletteByConfigStepKey",
    value: function getItemPaletteByConfigStepKey(configStepKey) {
      return this.itemPaletteArr.find(function (p) {
        return p[1].configStepKey === configStepKey;
      });
    }
  }, {
    key: "hideNotUsedPalettes",
    value: function hideNotUsedPalettes(index) {
      this.itemPalettes.forEach(function (val, key) {
        if (key > index) val.visible = false;
      });
    }
  }, {
    key: "openGroupWithCodex",
    value: function openGroupWithCodex(paletteId, codex) {
      var _palette$groups;
      var palette = this.itemPalettes.get(paletteId);
      if (!palette) return;
      var group = palette === null || palette === void 0 || (_palette$groups = palette.groups) === null || _palette$groups === void 0 ? void 0 : _palette$groups.find(function (g) {
        return g.items.find(function (i) {
          return i.codex === codex;
        });
      });
      if (group) group.isFolded = false;
      this.updateItemPalette(paletteId, palette);
    }
  }, {
    key: "openGroup",
    value: function openGroup(paletteId, groupKey, allOthersClosed) {
      var palette = this.itemPalettes.get(paletteId);
      if (!palette) return;
      palette = _objectSpread$a({}, palette);
      var oldGroups = palette.groups;
      var group = oldGroups === null || oldGroups === void 0 ? void 0 : oldGroups.find(function (g) {
        return g.key == groupKey;
      });
      if (group) group.isFolded = false;
      if (allOthersClosed) {
        oldGroups.map(function (g) {
          if (g.key !== (group === null || group === void 0 ? void 0 : group.key)) {
            g.isFolded = true;
          }
        });
      }
      palette.groups = _toConsumableArray(oldGroups);
      this.updateItemPalette(paletteId, palette);
    }
  }, {
    key: "onAllGroups",
    value: function onAllGroups(paletteId, closed) {
      var palette = this.itemPalettes.get(paletteId);
      if (!palette) return;
      palette = _objectSpread$a({}, palette);
      palette.groups.map(function (g) {
        g.isFolded = closed;
      });
      this.updateItemPalette(paletteId, palette);
    }
  }, {
    key: "closeGroup",
    value: function closeGroup(paletteId, groupKey) {
      var _palette;
      var palette = this.itemPalettes.get(paletteId);
      if (!palette) return;
      palette = _objectSpread$a({}, palette);
      var group = (_palette = palette) === null || _palette === void 0 || (_palette = _palette.groups) === null || _palette === void 0 ? void 0 : _palette.find(function (g) {
        return g.key == groupKey;
      });
      if (group) group.isFolded = true;
      this.updateItemPalette(paletteId, palette);
    }
  }, {
    key: "setForceSettingsHeaderInScrollContainer",
    value: function setForceSettingsHeaderInScrollContainer(val) {
      this.forceSettingsHeaderInScrollContainer = val;
    }
  }, {
    key: "roomListCountUp",
    value: function roomListCountUp() {
      this.roomListInitialized = this.roomListInitialized + 1;
    }
  }, {
    key: "setPaletteHeaderStepName",
    value: function setPaletteHeaderStepName(val) {
      this.paletteHeaderStepName = val;
    }
  }, {
    key: "updateConfigSingle",
    value: function updateConfigSingle(show, state) {
      if (show !== undefined) this.doShowConfigSingle = show;
      if (state !== undefined) this.doConfigSingle = state;
    }
  }, {
    key: "setDoShowActElementInfo",
    value: function setDoShowActElementInfo(val) {
      this.doShowActElementInfo = val;
    }
  }, {
    key: "setArticleInfos",
    value: function setArticleInfos(articleInfos) {
      var keys = Object.keys(articleInfos);
      keys.sort();
      for (var i = 0; i < keys.length; i++) {
        var key = keys[i],
          thereIsAWidth = articleInfos[key].width && articleInfos[key].width != -1,
          thereIsADepth = articleInfos[key].width && articleInfos[key].depth != -1,
          thereIsAHeight = articleInfos[key].width && articleInfos[key].height != -1;
        if (thereIsAWidth) {
          articleInfos[key].formattedSize = articleInfos[key].width + (thereIsAHeight || thereIsADepth ? ' cm | ' : ' cm');
        }
        if (thereIsADepth) {
          if (articleInfos[key].formattedSize) {
            articleInfos[key].formattedSize += articleInfos[key].depth + (thereIsAHeight ? ' cm | ' : ' cm');
          } else {
            articleInfos[key].formattedSize = articleInfos[key].depth + (thereIsAHeight ? ' cm | ' : ' cm');
          }
        }
        if (thereIsAHeight) {
          if (articleInfos[key].formattedSize) {
            articleInfos[key].formattedSize += articleInfos[key].height + ' cm ';
          } else {
            articleInfos[key].formattedSize = articleInfos[key].height + ' cm ';
          }
        }
        articleInfos[key].formattedPrice = DivaUtils.formatPrice(articleInfos[key].singlePrice);
        this.articleInfos = articleInfos;
      }
    }
  }, {
    key: "getSelectedBasketIds",
    get: function get() {
      var basketIds = [];
      for (var posNr in this.articleInfos) {
        if (this.articleInfos[posNr].isActive) {
          basketIds.push(this.articleInfos[posNr].basketId);
        }
      }
      return basketIds;
    }
  }, {
    key: "allBaksetIds",
    get: function get() {
      var basketIds = [];
      for (var posNr in this.articleInfos) {
        basketIds.push(this.articleInfos[posNr].basketId);
      }
      return basketIds;
    }

    /**
     * this function resets/cleans all ItemPalette ViewParameters
     */
  }, {
    key: "initForUpdate",
    value: function initForUpdate() {
      this.doShowActElementInfo = false;
      this.doShowConfigSingle = false;
      this.infoBoxContent = undefined;
    }
  }, {
    key: "selectedArticles",
    get: function get() {
      var articles = [];
      for (var posNr in this.articleInfos) {
        var article = this.articleInfos[posNr];
        if (article.isActive) {
          articles.push(article);
        }
      }
      return articles;
    }
  }, {
    key: "setScrollerElement",
    value: function setScrollerElement(ref) {
      this.settingsScrollerElement = ref;
    }
  }, {
    key: "setHeaderElement",
    value: function setHeaderElement(ref) {
      this.settingsHeaderElement = ref;
    }
  }, {
    key: "setPlannerReplaceMode",
    value: function setPlannerReplaceMode(val) {
      this.plannerReplaceMode = val;
    }
  }, {
    key: "setPlannerReplaceModeAvailable",
    value: function setPlannerReplaceModeAvailable(val) {
      if (!val && this.plannerReplaceMode) this.setPlannerReplaceMode(false);
      this.plannerReplaceModeAvailable = val;
    }
  }, {
    key: "updatePlannerFilter",
    value: function updatePlannerFilter(plannerFilter) {
      this.plannerFilter = plannerFilter;
    }
  }, {
    key: "updateInputBoxValues",
    value: function updateInputBoxValues(visible, item) {
      this.inputBoxItem = item;
      this.inputBoxVisible = visible;
    }
  }, {
    key: "updateInfoBox",
    value: function updateInfoBox(newVal) {
      this.infoBoxContent = newVal;
    }
  }, {
    key: "setSettingsWidth",
    value: function setSettingsWidth(val) {
      this.settingsWidth = val;
    }
  }, {
    key: "setSelectedShelfAccessoire",
    value: function setSelectedShelfAccessoire(basketId) {
      this.selectedShelfAccessoire = basketId;
    }
  }, {
    key: "setContentItemId",
    value: function setContentItemId(id, isOfOPV, clickedFrom) {
      this.contentItemId = id;
      this.conetentItemOfOPVOpen = !!isOfOPV && !!id;
      if (id === undefined) {
        this.setContentItemVisible(false);
      }
      if (clickedFrom) this.setClickedFrom(clickedFrom);
    }
  }, {
    key: "setClickedFrom",
    value: function setClickedFrom(val) {
      this.clickedFrom = val;
    }
  }, {
    key: "setContentItemVisible",
    value: function setContentItemVisible(visible) {
      this.contentItemVisible = visible;
    }
  }, {
    key: "isContentItemVisible",
    get: function get() {
      return this.contentItemId && this.contentItemVisible;
    }
  }, {
    key: "onClickContentItemId",
    value: function onClickContentItemId(id, clickedFrom) {
      var visible = !!id && (id != this.contentItemId || !this.contentItemVisible || !!clickedFrom && clickedFrom != this.clickedFrom);
      this.setClickedFrom(clickedFrom);
      this.setContentItemVisible(visible);
      this.setContentItemId(visible ? id : undefined, true);
    }
  }, {
    key: "setPartlistArticlesInOverview",
    value: function setPartlistArticlesInOverview(val) {
      this.partlistArticlesInOverview = val;
    }
  }]);
}();

function ownKeys$9(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$9(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$9(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$9(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$9 = DivaUtils$1.initLog('Planner', 'WEB-PLANNER-REACT');
var TabGroupState = /*#__PURE__*/function () {
  function TabGroupState(uiState, configuratorStructureHandler, viewPlannerHandler, productState, settingsState, updateConfiguratorState, onEvent) {
    var _this = this;
    _classCallCheck(this, TabGroupState);
    _defineProperty(this, "uiState", void 0);
    _defineProperty(this, "configuratorStructureHandler", void 0);
    _defineProperty(this, "viewPlannerHandler", void 0);
    _defineProperty(this, "divaWebPlanner", void 0);
    _defineProperty(this, "productState", void 0);
    _defineProperty(this, "settingsState", void 0);
    _defineProperty(this, "configuratorSturucture", {});
    _defineProperty(this, "curTabGroup", undefined);
    _defineProperty(this, "curOption", undefined);
    _defineProperty(this, "curConfigStep", undefined);
    _defineProperty(this, "updateConfiguratorState", void 0);
    _defineProperty(this, "onEvent", void 0);
    _defineProperty(this, "tabGroupItems", new Map());
    _defineProperty(this, "subTabGroupItems", new Map());
    _defineProperty(this, "filterTabgroup", undefined);
    _defineProperty(this, "currentSubTabGroupFolded", undefined);
    _defineProperty(this, "tabGroupScroller", null);
    _defineProperty(this, "curTabGroupType", 'horizontal');
    /****Events called by navigatiion********/
    /**
     * This function is called all the time the configurator exits the planner mode
     */
    _defineProperty(this, "onExitPlannerMode", function () {
      _this.viewPlannerHandler.handlePlanner3DGridChanged(true);
      if (!_this.divaWebPlanner.getPlanningIsComplete()) {
        //SHOW Warning
        _this.productState.updatePlannerMsgContainer('PLANNING_NOT_COMPLETE_MSG', true, 'WARNING', 'PLANNING_NOT_COMPLETE_MSG', true);
      }
      LOG$9.businessEvent('onExitPlanner', 'Planner', 'Der Planner Schritt wurde verlassen');
    });
    /**
     * This function is called every time the configurator enters the summary step
     * In this case the configruator has to force to load all renderings new in order to save them.
     */
    _defineProperty(this, "onEnterSummaryMode", function () {
      var _this$divaWebPlanner$;
      if (!_this.uiState.isInMobileView && (_this$divaWebPlanner$ = _this.divaWebPlanner.ReturnTo) !== null && _this$divaWebPlanner$ !== void 0 && _this$divaWebPlanner$.includes('FAVORITS')) _this.uiState.setConfigSettingsIsExpanded(true);
      _this.divaWebPlanner.handle3DVisualizationChanged('Nav3DResetNavParams', {}, true); //Reset view to start
      var dimensionLinesAreVisible = _this.productState.doShow3DDimensionLines;
      _this.productState.setDoShow3DDimensionLines(false);
      _this.divaWebPlanner.forceUpdatePlannerView('PERMANENT', true);
      if (dimensionLinesAreVisible) _this.productState.setDoShow3DDimensionLines(true);
      LOG$9.businessEvent('onEnterSummary', 'Planner', 'Es wurde in die Zusammenfassung gewechselt');
    });
    _defineProperty(this, "onExitSummaryMode", function () {
      _this.productState.setCurAlternateRendering();
      LOG$9.businessEvent('onExitSummary', 'Planner', 'Die Zusammenfassung wurde verlassen');
    });
    _defineProperty(this, "onEnterConfiguratorMode", function () {
      LOG$9.businessEvent('onEnterConfigurator', 'Planner', 'Es wurde in den Konfigurator gewechselt');
    });
    _defineProperty(this, "onEnterAccessoriesMode", function () {
      LOG$9.businessEvent('onEnterAccessories', 'Planner', 'Es wurde in den Zubehör Modus gewechselt');
    });
    _defineProperty(this, "onEnterShelfplannerMode", function () {
      if (_this.productState.shelfPlannerRTV) {
        _this.productState.setAllowGrid(false);
        if (!_this.productState.doShowRTV) _this.productState.onChangePlannerView('RTV');
      }
    });
    _defineProperty(this, "onExitShelfPlannerMode", function () {
      if (_this.productState.shelfPlannerRTV) {
        _this.productState.setAllowGrid(true);
      }
      _this.settingsState.setFreePlacesOnGrids();
    });
    _defineProperty(this, "onExitConfiguratorMode", function () {
      LOG$9.businessEvent('onExitConfigurator', 'Planner', 'Es wurde in den Konfigurator gewechselt');
    });
    _defineProperty(this, "onExitAccessoriesMode", function () {
      LOG$9.businessEvent('onExitAccessories', 'Planner', 'Es wurde in den Zubehör Modus gewechselt');
    });
    _defineProperty(this, "navigatedInConfigurator", function (configStepKey) {
      var step = _this.configuratorStructureHandler.getConfigStepByKey(configStepKey);
      LOG$9.businessEvent('onTabGroupStep', 'Planner', 'TabGroupStep', {
        name: step === null || step === void 0 ? void 0 : step.tabGroup.key,
        step: step === null || step === void 0 ? void 0 : step.tabGroup.posNr,
        type: step === null || step === void 0 ? void 0 : step.tabGroup.type
      });
      if (step) {
        var _this$onEvent;
        var firstOptions = Object.keys(step.tabGroup.options)[0];
        (_this$onEvent = _this.onEvent) === null || _this$onEvent === void 0 || _this$onEvent.call(_this, {
          type: 'navigatedToOption',
          data: {
            optionCodex: step.tabGroup.options[firstOptions].codex
          }
        });
      }
    });
    this.uiState = uiState;
    this.configuratorStructureHandler = configuratorStructureHandler;
    this.viewPlannerHandler = viewPlannerHandler;
    this.productState = productState;
    this.settingsState = settingsState;
    this.updateConfiguratorState = updateConfiguratorState;
    this.onEvent = onEvent;
    makeObservable(this, {
      tabGroupItems: observable,
      setTabGroupItems: action,
      subTabGroupItems: observable,
      setSubTabGroupItems: action,
      currentSubTabGroupFolded: observable,
      setCurrentSubTabGroupFolded: action,
      updateCurTabGroupType: action,
      curTabGroupType: observable,
      updateState: action,
      curTabGroup: observable,
      curOption: observable,
      curConfigStep: observable,
      configuratorSturucture: observable,
      tabGroupScroller: observable,
      setTabGroupScroller: action,
      allTabGroups: computed,
      hasPlanner: computed,
      filterTabgroup: observable,
      setFilterTabgroup: action
    });
    this.updateCurTabGroupType();
  }
  return _createClass(TabGroupState, [{
    key: "initFunctions",
    value: function initFunctions(divaWebPlanner) {
      this.divaWebPlanner = divaWebPlanner;
    }
  }, {
    key: "setTabGroupItems",
    value: function setTabGroupItems(key, element) {
      this.tabGroupItems.set(key, element);
    }
  }, {
    key: "setSubTabGroupItems",
    value: function setSubTabGroupItems(id, element) {
      this.subTabGroupItems.set(id, element);
    }
  }, {
    key: "navigateToTabgroupByPosNr",
    value: function navigateToTabgroupByPosNr(posNr) {
      this.configuratorStructureHandler.navigateToTabgroupByPosNr(posNr);
      this.updateConfiguratorState(true);
    }
  }, {
    key: "allTabGroups",
    get: function get() {
      var allTabGroups = _objectSpread$9({}, this.configuratorSturucture);
      var visibleTabGroups = {};
      if (this.filterTabgroup) {
        var filterText = this.filterTabgroup.toLocaleLowerCase();
        for (var key in allTabGroups) {
          var tabGroup = _objectSpread$9({}, allTabGroups[key]);
          var visibleOptions = {};
          for (var opt in tabGroup.options) {
            var option = tabGroup.options[opt];
            if (option.codex.toLocaleLowerCase().includes(filterText) || option.title.toLocaleLowerCase().includes(filterText)) {
              visibleOptions[opt] = option;
            }
          }
          tabGroup.options = visibleOptions;
          allTabGroups[key] = tabGroup;
        }
      }
      for (var _key in allTabGroups) {
        var tabgroup = allTabGroups[_key];
        if (!tabgroup.hidden && (!this.uiState.ViewConfig.hideSingleOPVOnlyAlways || !tabgroup.singleOPVOnly) && Object.keys(tabgroup.options).length) {
          visibleTabGroups[_key] = tabgroup;
        }
      }
      return visibleTabGroups;
    }
  }, {
    key: "hasPlanner",
    get: function get() {
      return !!this.configuratorSturucture['Planner'];
    }
  }, {
    key: "currentStepIsVisible",
    get: function get() {
      var allTabGroups = this.allTabGroups;
      for (var iKey in allTabGroups) {
        var tabgroup = allTabGroups[iKey];
        for (var jKey in tabgroup.options) {
          var option = tabgroup.options[jKey];
          for (var mKey in option.configSteps) {
            if (this.configuratorStructureHandler.curConfigStepKey === mKey) return true;
          }
        }
      }
      return false;
    }
  }, {
    key: "firstVisibleConfigStep",
    get: function get() {
      var allTabGroups = this.allTabGroups;
      var tabgroup = allTabGroups[Object.keys(allTabGroups)[0]];
      if (!tabgroup) return undefined;
      var option = tabgroup.options[Object.keys(tabgroup.options)[0]];
      if (!option) return undefined;
      return option.configSteps[Object.keys(option.configSteps)[0]];
    }
  }, {
    key: "setFilterTabgroup",
    value: function setFilterTabgroup(val) {
      this.filterTabgroup = val;
    }
  }, {
    key: "setCurrentSubTabGroupFolded",
    value: function setCurrentSubTabGroupFolded(val) {
      this.currentSubTabGroupFolded = val;
    }
  }, {
    key: "setTabGroupScroller",
    value: function setTabGroupScroller(val) {
      this.tabGroupScroller = val;
    }
  }, {
    key: "updateCurTabGroupType",
    value: function updateCurTabGroupType() {
      if (!this.uiState.isInMobileView && this.uiState.ViewConfig.tabOrientation != undefined) {
        this.curTabGroupType = this.uiState.ViewConfig.tabOrientation;
      } else {
        var _this$uiState$ViewCon;
        this.curTabGroupType = (_this$uiState$ViewCon = this.uiState.ViewConfig.tabOrientationMobile) !== null && _this$uiState$ViewCon !== void 0 ? _this$uiState$ViewCon : 'horizontal';
      }
    }
  }, {
    key: "scrollToCurTabgroup",
    value: function scrollToCurTabgroup() {
      if (!this.curTabGroup) return;
      var curTabGroup = this.tabGroupItems.get(this.curTabGroup.key);
      if (!this.tabGroupScroller || !curTabGroup) return;
      var offsetToTop = this.tabGroupScroller.getBoundingClientRect().height / 100 * 10; //  das Active Div soll 10% in der Breite nach links positioniert sein
      var actDivPos = curTabGroup.getBoundingClientRect().top - this.tabGroupScroller.getBoundingClientRect().top; //     dies ist die Position des activen Divs relative dazu wie gerade gescrollt ist
      var actScrollVal = this.tabGroupScroller.scrollTop; //           soweit ist aktuell gescrollt
      var toScrollTop = actDivPos - offsetToTop + actScrollVal;
      this.tabGroupScroller.scrollTo({
        left: 0,
        top: toScrollTop,
        behavior: 'smooth'
      });
    }
  }, {
    key: "updateState",
    value: function updateState(configuratorSturucture, curTabGroup, curOption, curConfigStep) {
      this.configuratorSturucture = configuratorSturucture;
      this.curTabGroup = curTabGroup;
      this.curOption = curOption;
      this.curConfigStep = curConfigStep;
    }
  }]);
}();

function ownKeys$8(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$8(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$8(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$8(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var ConfigState = /*#__PURE__*/function () {
  function ConfigState() {
    _classCallCheck(this, ConfigState);
    _defineProperty(this, "_currentInitializedProductId", void 0);
    _defineProperty(this, "lang", void 0);
    _defineProperty(this, "initProductFolded", void 0);
    _defineProperty(this, "paramsSendViaAdapt", void 0);
    _defineProperty(this, "organizationId", void 0);
    _defineProperty(this, "currentOrganization", undefined);
    _defineProperty(this, "currentUser", undefined);
    _defineProperty(this, "apiConfig", void 0);
    _defineProperty(this, "isIDMData", void 0);
    _defineProperty(this, "packageId", void 0);
    _defineProperty(this, "serieId", void 0);
    _defineProperty(this, "accountId", void 0);
    _defineProperty(this, "plannerIsLoading", void 0);
    _defineProperty(this, "smSupplierNr", void 0);
    _defineProperty(this, "jwt", void 0);
    _defineProperty(this, "forceReload", undefined);
    _defineProperty(this, "divaNr", undefined);
    _defineProperty(this, "productId", undefined);
    _defineProperty(this, "customProductId", undefined);
    _defineProperty(this, "catalogCodex", undefined);
    _defineProperty(this, "catalogGuid", undefined);
    _defineProperty(this, "articleCodexes", undefined);
    _defineProperty(this, "differentOptions", undefined);
    _defineProperty(this, "resetArticle", undefined);
    _defineProperty(this, "variantValues", undefined);
    _defineProperty(this, "basketData", undefined);
    _defineProperty(this, "setId", undefined);
    _defineProperty(this, "loginMode", void 0);
    _defineProperty(this, "initMode", void 0);
    _defineProperty(this, "opt", void 0);
    _defineProperty(this, "initEmptyPlanner", void 0);
    _defineProperty(this, "assignNewDivaNr", void 0);
    _defineProperty(this, "configuratorTitle", void 0);
    _defineProperty(this, "used_logged_in", void 0);
    _defineProperty(this, "useEkPrices", void 0);
    _defineProperty(this, "addDivaNr", void 0);
    _defineProperty(this, "accountGuid", void 0);
    _defineProperty(this, "accountSessionGuid", void 0);
    _defineProperty(this, "supplierColors", void 0);
    _defineProperty(this, "catalogMask", void 0);
    _defineProperty(this, "no3DConnection", void 0);
    _defineProperty(this, "returnTo", undefined);
    _defineProperty(this, "startDivaNr", void 0);
    _defineProperty(this, "settings", void 0);
    _defineProperty(this, "paramters", undefined);
    _defineProperty(this, "actions", void 0);
    _defineProperty(this, "handler", void 0);
    _defineProperty(this, "state", void 0);
    _defineProperty(this, "applicationEnvironment", void 0);
    _defineProperty(this, "configuratorSessionId", void 0);
    _defineProperty(this, "useIDMService", void 0);
    this.paramsSendViaAdapt = false;
    this.organizationId = '';
    this.lang = 'DE';
    this.loginMode = 'offline';
    makeObservable(this, {
      forceReload: observable,
      divaNr: observable,
      productId: observable,
      customProductId: observable,
      catalogCodex: observable,
      isIDMData: observable,
      packageId: observable,
      smSupplierNr: observable,
      serieId: observable,
      accountId: observable,
      articleCodexes: observable,
      catalogGuid: observable,
      differentOptions: observable,
      variantValues: observable,
      basketData: observable,
      setId: observable,
      lang: observable,
      organizationId: observable,
      currentOrganization: observable,
      paramters: observable,
      currentUser: observable,
      returnTo: observable,
      loginMode: observable,
      setConfigParams: action
    });
  }
  return _createClass(ConfigState, [{
    key: "setConfigParams",
    value: function setConfigParams(val) {
      this.loginMode = val.loginMode;
      if (val) {
        var _val$settings$viewCon, _val$settings, _val$settings2, _val$internal;
        this.lang = val.lang.toLocaleUpperCase();
        this.initProductFolded = val.initProductFolded;
        this.paramsSendViaAdapt = val.paramsSendViaAdapt;
        this.organizationId = val.organizationId;
        this.currentOrganization = val.currentOrganization;
        this.currentUser = val.currentUser;
        this.isIDMData = val.isIDMData;
        this.packageId = val.packageId;
        this.serieId = val.serieId;
        this.accountId = val.accountId;
        this.apiConfig = val.apiConfig;
        this.plannerIsLoading = val.plannerIsLoading;
        this.forceReload = val.forceReload;
        this.divaNr = val.divaNr;
        this.productId = val.productId;
        this.customProductId = val.customProductId;
        this.catalogCodex = val.catalogCodex;
        this.catalogGuid = val.catalogGuid;
        this.articleCodexes = val.articleCodexes;
        this.differentOptions = val.differentOptions;
        this.resetArticle = val.resetArticle;
        this.variantValues = val.variantValues;
        this.basketData = val.basketData;
        this.setId = val.setId;
        this.jwt = val.jwt;
        this.initMode = val.initMode;
        this.opt = val.opt;
        this.initEmptyPlanner = val.initEmptyPlanner;
        this.assignNewDivaNr = val.assignNewDivaNr;
        this.configuratorTitle = val.configuratorTitle;
        this.used_logged_in = val.used_logged_in;
        this.useEkPrices = val.useEkPrices;
        this.addDivaNr = val.addDivaNr;
        this.accountGuid = val.accountGuid;
        this.accountSessionGuid = val.accountSessionGuid;
        this.supplierColors = val.supplierColors;
        this.smSupplierNr = val.smSupplierNr;
        this.catalogMask = val.catalogMask;
        this.no3DConnection = val.no3DConnection;
        this.returnTo = val.returnTo;
        this.startDivaNr = val.startDivaNr;
        this.actions = val.actions;
        this.state = val.state;
        this.handler = val.handler;
        this.settings = _objectSpread$8(_objectSpread$8({}, val.settings), {}, {
          viewConfig: (_val$settings$viewCon = (_val$settings = val.settings) === null || _val$settings === void 0 ? void 0 : _val$settings.viewConfig) !== null && _val$settings$viewCon !== void 0 ? _val$settings$viewCon : (_val$settings2 = val.settings) === null || _val$settings2 === void 0 ? void 0 : _val$settings2.ViewConfig // for compatibility with old viewConfig
        });
        this.paramters = val.parameters;
        this.applicationEnvironment = val.applicationEnvironment;
        this.configuratorSessionId = (_val$internal = val.internal) === null || _val$internal === void 0 ? void 0 : _val$internal.configuratorSessionId;
      }
    }
  }, {
    key: "currentInitializedProductId",
    get: function get() {
      return this._currentInitializedProductId;
    },
    set: function set(id) {
      this._currentInitializedProductId = id;
    }
  }]);
}();

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var wrapper$2 = "wrapper-XXCGP";
var subWrapper = "sub-wrapper-4ioWs";
var scrollWrapper = "scroll-wrapper-SRfiH";
var textWrapper = "text-wrapper-bETBh";
var inModalClass = "in-modal-class-TkCQS";
var headerWrapper = "header-wrapper-m09hl";
var inputWrapper = "input-wrapper-prYzj";
var inputWrapper2 = "input-wrapper2-k0rPA";
var input = "input-Imhh-";
var errorText = "error-text-yTOKb";
var boldText = "bold-text-fAQr-";
var text = "text-rVkVr";
var checkboxWrapper = "checkbox-wrapper-sQ-Sp";
var checkbox = "checkbox-oU8gK";
var button = "button-tJIp6";
var iconButton = "icon-button-68ogi";
var iconTextButton = "icon-text-button-DnkUh";
var disabled = "disabled-v27yj";
var inverted = "inverted-0l0El";
var dark = "dark-o5JJe";
var icon$1 = "icon-dDJyC";
var big = "big-LTvFV";
var createLeaflet = "create-leaflet-jEhMa";
var share = "share-1uEOU";
var b = "b-GGIBZ";
var calendar = "calendar-zayqu";
var cart = "cart-edwEp";
var back = "back-qhIiE";
var mail = "mail-VJEvY";
var copy = "copy-aOn72";
var check = "check-lsKWw";
var message = "message-GzlsK";
var url = "url-7RCQe";
var facebook = "facebook-xMM4M";
var linkedin = "linkedin-EWr8J";
var twitter = "twitter-QharN";
var checkGreen = "check-green-C5tRq";
var reset = "reset-QL8tU";
var print = "print-Zo7U-";
var help = "help-1diDf";
var pShare = "p-share-qaR6O";
var pSave = "p-save-U0OTD";
var lCalendar = "l-calendar-jt18N";
var heart = "heart-YJDRN";
var bookmark = "bookmark-ucaBU";
var close = "close-68IIu";
var requiredFieldsInfo = "requiredFieldsInfo-nkkEE";
var nameErrorText = "nameErrorText-QN0MZ";
var css_248z$5 = ".wrapper-XXCGP {\n  display: flex;\n  flex-direction: row;\n  flex-wrap: wrap;\n  gap: 8px;\n  justify-content: center;\n  align-items: center;\n  margin: 12px;\n}\n\n.sub-wrapper-4ioWs {\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  gap: 12px;\n  align-items: center;\n  width: 360px;\n  max-width: 100%;\n}\n.scroll-wrapper-SRfiH {\n  max-height: 600px;\n  overflow: auto;\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  gap: 12px;\n  align-items: center;\n}\n\n.text-wrapper-bETBh {\n  display: flex;\n  align-items: center;\n  width: 100%;\n}\n\n.in-modal-class-TkCQS {\n  width: 360px;\n  max-width: 100%;\n}\n\n.header-wrapper-m09hl {\n  display: flex;\n  justify-content: flex-start;\n  align-items: center;\n  gap: 8px;\n  font-size: 18px;\n  height: 40px;\n}\n\n.input-wrapper-prYzj {\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  width: 100%;\n}\n.input-wrapper2-k0rPA {\n  display: grid;\n  grid-template-columns: repeat(2, 1fr);\n  grid-template-rows: 1fr;\n  grid-column-gap: 16px;\n  width: 100%;\n}\n\n.input-Imhh- {\n  height: 20px;\n  padding: 8px 12px;\n  border-radius: 6px;\n  border: rgb(228, 228, 231) solid 1px;\n  width: calc(100% - 28px);\n  box-sizing: content-box;\n}\n[dir=\"ltr\"] .error-text-yTOKb {\n  text-align: left;\n}\n[dir=\"rtl\"] .error-text-yTOKb {\n  text-align: right;\n}\n.error-text-yTOKb {\n  color: red;\n  font-size: 11px;\n}\n.bold-text-fAQr- {\n  font-size: 15px;\n  font-weight: 600;\n}\n\n.text-rVkVr {\n  font-size: 12px;\n}\n\n.checkbox-wrapper-sQ-Sp {\n  display: flex;\n  flex-direction: row;\n  gap: 12px;\n  width: 100%;\n}\n.checkbox-oU8gK {\n  width: 24px;\n  flex-shrink: 0;\n  height: 16px;\n}\n\n.button-tJIp6 {\n  height: 38px;\n  width: 100%;\n  display: flex;\n  align-items: center;\n  justify-content: center;\n  gap: 8px;\n  margin: 0px;\n  padding: 0px;\n  background-color: #fff;\n  border-radius: 6px;\n  border: solid 1px rgb(228, 228, 231);\n  color: rgb(24, 24, 27);\n  cursor: pointer;\n  font-size: 14px;\n  font-weight: 500;\n  line-height: 20px;\n  text-align: center;\n  transition-duration: 0.3s;\n  transition-property:\n    color,\n    background-color,\n    border-color,\n    text-decoration-color,\n    fill,\n    stroke,\n    -webkit-text-decoration-color;\n  transition-timing-function: ease;\n  white-space-collapse: collapse;\n}\n\n.icon-button-68ogi {\n  width: 40px;\n  height: 40px;\n  border: 0px solid transparent;\n}\n\n.icon-text-button-DnkUh {\n  width: 100%;\n  height: 40px;\n}\n\n.disabled-v27yj {\n  cursor: not-allowed;\n  background-color: var(--diva-theme-palette-primary-light);\n}\n\n.inverted-0l0El {\n  color: #fff;\n  background-color: var(--diva-theme-palette-primary-main);\n}\n\n.button-tJIp6:hover {\n  background-color: #f4f4f4;\n}\n\n.button-tJIp6:hover .inverted-0l0El,\n.inverted-0l0El:hover,\n.dark-o5JJe {\n  background-color: var(--diva-theme-palette-primary-dark);\n}\n\n.icon-dDJyC {\n  width: 20px;\n  height: 20px;\n  background-position: center;\n  background-repeat: no-repeat;\n  background-size: 70%;\n  cursor: pointer;\n  transition:\n    background 0.4s ease,\n    transform 0.4s ease;\n}\n\n.icon-dDJyC.big-LTvFV {\n  width: 32px;\n  height: 32px;\n}\n\n.create-leaflet-jEhMa {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Cpath d%3D%22M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4%22%3E%3C%2Fpath%3E  %3Cpolyline points%3D%227 10 12 15 17 10%22%3E%3C%2Fpolyline%3E  %3Cline x1%3D%2212%22 y1%3D%2215%22 x2%3D%2212%22 y2%3D%223%22%3E%3C%2Fline%3E%3C%2Fsvg%3E\");\n}\n\n.share-1uEOU {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Ccircle cx%3D%2218%22 cy%3D%225%22 r%3D%223%22%3E%3C%2Fcircle%3E  %3Ccircle cx%3D%226%22 cy%3D%2212%22 r%3D%223%22%3E%3C%2Fcircle%3E  %3Ccircle cx%3D%2218%22 cy%3D%2219%22 r%3D%223%22%3E%3C%2Fcircle%3E  %3Cline x1%3D%228.59%22 y1%3D%2213.51%22 x2%3D%2215.42%22 y2%3D%2217.49%22%3E%3C%2Fline%3E  %3Cline x1%3D%2215.41%22 y1%3D%226.51%22 x2%3D%228.59%22 y2%3D%2210.49%22%3E%3C%2Fline%3E%3C%2Fsvg%3E\");\n}\n\n.b-GGIBZ:hover .share-1uEOU,\n.button-tJIp6:hover .share-1uEOU,\n.share-1uEOU:hover {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Ccircle cx%3D%2218%22 cy%3D%225%22 r%3D%223%22 fill%3D%22%23000%22%3E%3C%2Fcircle%3E  %3Ccircle cx%3D%226%22 cy%3D%2212%22 r%3D%223%22 fill%3D%22%23000%22%3E%3C%2Fcircle%3E  %3Ccircle cx%3D%2218%22 cy%3D%2219%22 r%3D%223%22 fill%3D%22%23000%22%3E%3C%2Fcircle%3E  %3Cline x1%3D%228.59%22 y1%3D%2213.51%22 x2%3D%2215.42%22 y2%3D%2217.49%22%3E%3C%2Fline%3E  %3Cline x1%3D%2215.41%22 y1%3D%226.51%22 x2%3D%228.59%22 y2%3D%2210.49%22%3E%3C%2Fline%3E%3C%2Fsvg%3E\");\n}\n\n.save-EMvD0 {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Cpath d%3D%22M19 21H5a2 2 0 0 1-2-2V5a2 2 0 0 1 2-2h11l5 5v11a2 2 0 0 1-2 2z%22%3E%3C%2Fpath%3E  %3Cpolyline points%3D%2217 21 17 13 7 13 7 21%22%3E%3C%2Fpolyline%3E  %3Cpolyline points%3D%227 3 7 8 15 8%22%3E%3C%2Fpolyline%3E%3C%2Fsvg%3E\");\n}\n\n.calendar-zayqu {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Crect x%3D%223%22 y%3D%224%22 width%3D%2218%22 height%3D%2218%22 rx%3D%222%22 ry%3D%222%22%3E%3C%2Frect%3E  %3Cline x1%3D%2216%22 y1%3D%222%22 x2%3D%2216%22 y2%3D%226%22%3E%3C%2Fline%3E  %3Cline x1%3D%228%22 y1%3D%222%22 x2%3D%228%22 y2%3D%226%22%3E%3C%2Fline%3E  %3Cline x1%3D%223%22 y1%3D%2210%22 x2%3D%2221%22 y2%3D%2210%22%3E%3C%2Fline%3E%3C%2Fsvg%3E\");\n}\n\n.cart-edwEp {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Ccircle cx%3D%228%22 cy%3D%2221%22 r%3D%221%22%3E%3C%2Fcircle%3E  %3Ccircle cx%3D%2219%22 cy%3D%2221%22 r%3D%221%22%3E%3C%2Fcircle%3E  %3Cpath d%3D%22M2.05 2.05h2l2.66 12.42a2 2 0 0 0 2 1.58h9.78a2 2 0 0 0 1.95-1.57l1.65-7.43H5.12%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n\n.back-qhIiE {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Cpath d%3D%22m12 19-7-7 7-7%22%3E%3C%2Fpath%3E  %3Cpath d%3D%22M19 12H5%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n\n.inverted-0l0El .create-leaflet-jEhMa {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22%23fff%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Cpath d%3D%22M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4%22%3E%3C%2Fpath%3E  %3Cpolyline points%3D%227 10 12 15 17 10%22%3E%3C%2Fpolyline%3E  %3Cline x1%3D%2212%22 y1%3D%2215%22 x2%3D%2212%22 y2%3D%223%22%3E%3C%2Fline%3E%3C%2Fsvg%3E\");\n}\n\n.inverted-0l0El .share-1uEOU {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22%23fff%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Ccircle cx%3D%2218%22 cy%3D%225%22 r%3D%223%22%3E%3C%2Fcircle%3E  %3Ccircle cx%3D%226%22 cy%3D%2212%22 r%3D%223%22%3E%3C%2Fcircle%3E  %3Ccircle cx%3D%2218%22 cy%3D%2219%22 r%3D%223%22%3E%3C%2Fcircle%3E  %3Cline x1%3D%228.59%22 y1%3D%2213.51%22 x2%3D%2215.42%22 y2%3D%2217.49%22%3E%3C%2Fline%3E  %3Cline x1%3D%2215.41%22 y1%3D%226.51%22 x2%3D%228.59%22 y2%3D%2210.49%22%3E%3C%2Fline%3E%3C%2Fsvg%3E\");\n}\n\n.inverted-0l0El .save-EMvD0 {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22%23fff%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Cpath d%3D%22M19 21H5a2 2 0 0 1-2-2V5a2 2 0 0 1 2-2h11l5 5v11a2 2 0 0 1-2 2z%22%3E%3C%2Fpath%3E  %3Cpolyline points%3D%2217 21 17 13 7 13 7 21%22%3E%3C%2Fpolyline%3E  %3Cpolyline points%3D%227 3 7 8 15 8%22%3E%3C%2Fpolyline%3E%3C%2Fsvg%3E\");\n}\n\n.inverted-0l0El .calendar-zayqu {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22%23fff%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Crect x%3D%223%22 y%3D%224%22 width%3D%2218%22 height%3D%2218%22 rx%3D%222%22 ry%3D%222%22%3E%3C%2Frect%3E  %3Cline x1%3D%2216%22 y1%3D%222%22 x2%3D%2216%22 y2%3D%226%22%3E%3C%2Fline%3E  %3Cline x1%3D%228%22 y1%3D%222%22 x2%3D%228%22 y2%3D%226%22%3E%3C%2Fline%3E  %3Cline x1%3D%223%22 y1%3D%2210%22 x2%3D%2221%22 y2%3D%2210%22%3E%3C%2Fline%3E%3C%2Fsvg%3E\");\n}\n\n.inverted-0l0El .cart-edwEp {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22%23fff%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Ccircle cx%3D%228%22 cy%3D%2221%22 r%3D%221%22%3E%3C%2Fcircle%3E  %3Ccircle cx%3D%2219%22 cy%3D%2221%22 r%3D%221%22%3E%3C%2Fcircle%3E  %3Cpath d%3D%22M2.05 2.05h2l2.66 12.42a2 2 0 0 0 2 1.58h9.78a2 2 0 0 0 1.95-1.57l1.65-7.43H5.12%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n\n.inverted-0l0El .mail-VJEvY {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22%23fff%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Crect x%3D%222%22 y%3D%224%22 width%3D%2220%22 height%3D%2216%22 rx%3D%222%22%3E%3C%2Frect%3E  %3Cpath d%3D%22m22 7-8.97 5.7a1.94 1.94 0 0 1-2.06 0L2 7%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n\n.inverted-0l0El .copy-aOn72 {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22%23fff%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Crect x%3D%229%22 y%3D%229%22 width%3D%2213%22 height%3D%2213%22 rx%3D%222%22 ry%3D%222%22%3E%3C%2Frect%3E  %3Cpath d%3D%22M5 15H4a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h9a2 2 0 0 1 2 2v1%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n\n.inverted-0l0El .check-lsKWw {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22%23fff%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Cpath d%3D%22M22 11.08V12a10 10 0 1 1-5.93-9.14%22%3E%3C%2Fpath%3E  %3Cpolyline points%3D%2222 4 12 14.01 9 11.01%22%3E%3C%2Fpolyline%3E%3C%2Fsvg%3E\");\n}\n\n.message-GzlsK {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Cpath d%3D%22m22 2-7 20-4-9-9-4Z%22%3E%3C%2Fpath%3E  %3Cpath d%3D%22M22 2 11 13%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n\n.url-7RCQe {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Cpath d%3D%22M10 13a5 5 0 0 0 7.54.54l3-3a5 5 0 0 0-7.07-7.07l-1.72 1.71%22%3E%3C%2Fpath%3E  %3Cpath d%3D%22M14 11a5 5 0 0 0-7.54-.54l-3 3a5 5 0 0 0 7.07 7.07l1.71-1.71%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n\n.mail-VJEvY {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22currentColor%22 stroke-width%3D%222%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Crect x%3D%222%22 y%3D%224%22 width%3D%2220%22 height%3D%2216%22 rx%3D%222%22%3E%3C%2Frect%3E  %3Cpath d%3D%22m22 7-8.97 5.7a1.94 1.94 0 0 1-2.06 0L2 7%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n.facebook-xMM4M {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 viewBox%3D%220 0 24 24%22 fill%3D%22currentColor%22%3E%3Cpath d%3D%22M14 13.5H16.5L17.5 9.5H14V7.5C14 6.47062 14 5.5 16 5.5H17.5V2.1401C17.1743 2.09685 15.943 2 14.6429 2C11.9284 2 10 3.65686 10 6.69971V9.5H7V13.5H10V22H14V13.5Z%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n.linkedin-EWr8J {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 viewBox%3D%220 0 24 24%22 fill%3D%22currentColor%22%3E%3Cpath d%3D%22M6.94048 4.99993C6.94011 5.81424 6.44608 6.54702 5.69134 6.85273C4.9366 7.15845 4.07187 6.97605 3.5049 6.39155C2.93793 5.80704 2.78195 4.93715 3.1105 4.19207C3.43906 3.44699 4.18654 2.9755 5.00048 2.99993C6.08155 3.03238 6.94097 3.91837 6.94048 4.99993ZM7.00048 8.47993H3.00048V20.9999H7.00048V8.47993ZM13.3205 8.47993H9.34048V20.9999H13.2805V14.4299C13.2805 10.7699 18.0505 10.4299 18.0505 14.4299V20.9999H22.0005V13.0699C22.0005 6.89993 14.9405 7.12993 13.2805 10.1599L13.3205 8.47993Z%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n.twitter-QharN {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 viewBox%3D%220 0 24 24%22 fill%3D%22currentColor%22%3E%3Cpath d%3D%22M10.4883 14.651L15.25 21H22.25L14.3917 10.5223L20.9308 3H18.2808L13.1643 8.88578L8.75 3H1.75L9.26086 13.0145L2.31915 21H4.96917L10.4883 14.651ZM16.25 19L5.75 5H7.75L18.25 19H16.25Z%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E\");\n}\n\n.check-green-C5tRq {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 width%3D%2224%22 height%3D%2224%22 viewBox%3D%220 0 24 24%22 fill%3D%22none%22 stroke%3D%22%2322c55e%22 stroke-width%3D%223%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%3E  %3Cpolyline points%3D%2220 6 9 17 4 12%22 %2F%3E%3C%2Fsvg%3E\");\n}\n\n.reset-QL8tU {\n  background-image: url(\"data:image/svg+xml,%3Csvg width%3D%2225%22 height%3D%2221%22 viewBox%3D%220 0 25 21%22 fill%3D%22none%22 xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%3E%3Cg clip-path%3D%22url(%23clip0)%22%3E%3Cpath d%3D%22M4.3782 14.1701C3.9275 13.2399 3.70215 12.0771 3.70215 10.9143C3.70215 5.68177 7.75849 1.49573 12.8289 1.49573C14.181 1.49573 15.5331 1.84456 16.6599 2.42596%22 stroke%3D%22%236F6F6F%22 stroke-miterlimit%3D%2210%22 stroke-linecap%3D%22round%22%2F%3E%3Cpath d%3D%22M20.7159 6.14685C21.5047 7.5422 21.9554 9.17011 21.9554 10.9143C21.9554 16.1469 17.899 20.3329 12.8286 20.3329C10.9131 20.3329 9.1103 19.7515 7.64551 18.5887%22 stroke%3D%22%236F6F6F%22 stroke-miterlimit%3D%2210%22 stroke-linecap%3D%22round%22%2F%3E%3Cpath d%3D%22M19.1387 9.05387L20.4908 5.44922L23.9837 6.84457%22 stroke%3D%22%236F6F6F%22 stroke-miterlimit%3D%2210%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%2F%3E%3Cpath d%3D%22M6.0681 11.2632L4.60331 14.8678L1.11035 13.2399%22 stroke%3D%22%236F6F6F%22 stroke-miterlimit%3D%2210%22 stroke-linecap%3D%22round%22 stroke-linejoin%3D%22round%22%2F%3E%3C%2Fg%3E%3Cdefs%3E%3CclipPath id%3D%22clip0%22%3E%3Crect width%3D%2224%22 height%3D%2220%22 fill%3D%22white%22 transform%3D%22translate(0.546875 0.914307)%22%2F%3E%3C%2FclipPath%3E%3C%2Fdefs%3E%3C%2Fsvg%3E\");\n  transform: rotate(0);\n}\n\n.b-GGIBZ:hover .reset-QL8tU,\n.button-tJIp6:hover .reset-QL8tU,\n.reset-QL8tU:hover {\n  transform: rotate(180deg);\n}\n\n.print-Zo7U- {\n  background-image: url(\"data:image/svg+xml,%3Csvg fill%3D%22%236F6F6F%22 height%3D%22800px%22 width%3D%22800px%22 version%3D%221.1%22 id%3D%22Layer_1%22 xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 xmlns%3Axlink%3D%22http%3A%2F%2Fwww.w3.org%2F1999%2Fxlink%22 %09 viewBox%3D%220 0 64 64%22 enable-background%3D%22new 0 0 64 64%22 xml%3Aspace%3D%22preserve%22%3E%3Cg id%3D%22Printer%22%3E%09%3Cpath d%3D%22M57.7881012%2C14.03125H52.5v-8.0625c0-2.2091999-1.7909012-4-4-4h-33c-2.2091999%2C0-4%2C1.7908001-4%2C4v8.0625H6.2119002%09%09C2.7871001%2C14.03125%2C0%2C16.8183498%2C0%2C20.2431507V46.513649c0%2C3.4248009%2C2.7871001%2C6.2119026%2C6.2119002%2C6.2119026h2.3798995%09%09c0.5527%2C0%2C1-0.4472008%2C1-1c0-0.5527-0.4473-1-1-1H6.2119002C3.8896%2C50.7255516%2C2%2C48.8359489%2C2%2C46.513649V20.2431507%09%09c0-2.3223%2C1.8896-4.2119007%2C4.2119002-4.2119007h51.5762024C60.1102982%2C16.03125%2C62%2C17.9208508%2C62%2C20.2431507V46.513649%09%09c0%2C2.3223-1.8897018%2C4.2119026-4.2118988%2C4.2119026H56c-0.5527992%2C0-1%2C0.4473-1%2C1c0%2C0.5527992%2C0.4472008%2C1%2C1%2C1h1.7881012%09%09C61.2128983%2C52.7255516%2C64%2C49.9384499%2C64%2C46.513649V20.2431507C64%2C16.8183498%2C61.2128983%2C14.03125%2C57.7881012%2C14.03125z%09%09 M13.5%2C5.96875c0-1.1027999%2C0.8971996-2%2C2-2h33c1.1027985%2C0%2C2%2C0.8972001%2C2%2C2v8h-37V5.96875z%22%2F%3E%09%3Cpath d%3D%22M44%2C45.0322495H20c-0.5517998%2C0-0.9990005%2C0.4472008-0.9990005%2C0.9990005S19.4482002%2C47.0302505%2C20%2C47.0302505h24%09%09c0.5517006%2C0%2C0.9990005-0.4472008%2C0.9990005-0.9990005S44.5517006%2C45.0322495%2C44%2C45.0322495z%22%2F%3E%09%3Cpath d%3D%22M44%2C52.0322495H20c-0.5517998%2C0-0.9990005%2C0.4472008-0.9990005%2C0.9990005S19.4482002%2C54.0302505%2C20%2C54.0302505h24%09%09c0.5517006%2C0%2C0.9990005-0.4472008%2C0.9990005-0.9990005S44.5517006%2C52.0322495%2C44%2C52.0322495z%22%2F%3E%09%3Ccircle cx%3D%227.9590998%22 cy%3D%2221.8405495%22 r%3D%222%22%2F%3E%09%3Ccircle cx%3D%2214.2856998%22 cy%3D%2221.8405495%22 r%3D%222%22%2F%3E%09%3Ccircle cx%3D%2220.6121998%22 cy%3D%2221.8405495%22 r%3D%222%22%2F%3E%09%3Cpath d%3D%22M11%2C62.03125h42v-26H11V62.03125z M13.4036999%2C38.4349518h37.1925964v21.1925964H13.4036999V38.4349518z%22%2F%3E%3C%2Fg%3E%3C%2Fsvg%3E\");\n}\n\n.help-1diDf {\n  background-image: url(\"data:image/svg+xml,%3Csvg width%3D%2223%22 height%3D%2222%22 viewBox%3D%220 0 23 22%22 fill%3D%22none%22 xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%3E%3Cpath d%3D%22M11.2764 16.7161C11.0993 16.7161 10.9352 16.6588 10.7842 16.5442C10.6331 16.4296 10.5342 16.302 10.4873 16.1614C10.4404 16.0208 10.417 15.8853 10.417 15.7551C10.417 15.5624 10.4639 15.3879 10.5576 15.2317C10.6514 15.0754 10.7633 14.9635 10.8936 14.8958C11.029 14.828 11.1566 14.7942 11.2764 14.7942C11.4535 14.7942 11.6097 14.8437 11.7451 14.9426C11.8805 15.0416 11.9795 15.1692 12.042 15.3254C12.1097 15.4765 12.1436 15.6197 12.1436 15.7551C12.1436 15.9478 12.0967 16.1223 12.0029 16.2786C11.9144 16.4348 11.7998 16.5468 11.6592 16.6145C11.5238 16.6822 11.3962 16.7161 11.2764 16.7161ZM10.6904 13.5911V12.9192C10.6904 12.5806 10.7607 12.2551 10.9014 11.9426C11.042 11.6249 11.2008 11.3645 11.3779 11.1614C11.5602 10.953 11.7243 10.7708 11.8701 10.6145C12.2347 10.2187 12.4873 9.927 12.6279 9.7395C12.7738 9.552 12.8779 9.35148 12.9404 9.13794C13.0081 8.9244 13.042 8.70304 13.042 8.47388C13.042 7.94263 12.8936 7.53117 12.5967 7.2395C12.305 6.94263 11.8936 6.79419 11.3623 6.79419C11.0186 6.79419 10.693 6.87752 10.3857 7.04419C10.0785 7.20565 9.82585 7.39836 9.62793 7.62231C9.43522 7.84106 9.27116 8.0494 9.13574 8.24731L8.28418 7.63794C8.51335 7.25252 8.79199 6.91659 9.12012 6.63013C9.44824 6.33846 9.81022 6.12231 10.2061 5.98169C10.6019 5.84106 11.0029 5.77075 11.4092 5.77075C11.8779 5.77075 12.292 5.83586 12.6514 5.96606C13.016 6.09106 13.318 6.27336 13.5576 6.51294C13.8024 6.75252 13.9873 7.04419 14.1123 7.38794C14.2373 7.73169 14.2998 8.11971 14.2998 8.552C14.2998 8.89575 14.2399 9.21606 14.1201 9.51294C14.0003 9.80461 13.8467 10.0754 13.6592 10.3254C13.4769 10.5754 13.3076 10.7838 13.1514 10.9504C12.7503 11.3931 12.4587 11.7291 12.2764 11.9583C12.0993 12.1874 11.9873 12.3827 11.9404 12.5442C11.8936 12.7004 11.8701 12.8567 11.8701 13.0129V13.5911H10.6904Z%22 fill%3D%22%236F6F6F%22%2F%3E%3Ccircle cx%3D%2211.4799%22 cy%3D%2210.9999%22 r%3D%2210.2026%22 stroke%3D%22%236F6F6F%22%2F%3E%3C%2Fsvg%3E\");\n}\n\n.p-share-qaR6O {\n  background-image: url(\"data:image/svg+xml,%3Csvg fill%3D%22%236F6F6F%22 height%3D%22800px%22 width%3D%22800px%22 version%3D%221.1%22 id%3D%22Capa_1%22 xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 xmlns%3Axlink%3D%22http%3A%2F%2Fwww.w3.org%2F1999%2Fxlink%22 %09 viewBox%3D%220 0 481.6 481.6%22 xml%3Aspace%3D%22preserve%22%3E%3Cg%3E%09%3Cpath d%3D%22M381.6%2C309.4c-27.7%2C0-52.4%2C13.2-68.2%2C33.6l-132.3-73.9c3.1-8.9%2C4.8-18.5%2C4.8-28.4c0-10-1.7-19.5-4.9-28.5l132.2-73.8%09%09c15.7%2C20.5%2C40.5%2C33.8%2C68.3%2C33.8c47.4%2C0%2C86.1-38.6%2C86.1-86.1S429%2C0%2C381.5%2C0s-86.1%2C38.6-86.1%2C86.1c0%2C10%2C1.7%2C19.6%2C4.9%2C28.5%09%09l-132.1%2C73.8c-15.7-20.6-40.5-33.8-68.3-33.8c-47.4%2C0-86.1%2C38.6-86.1%2C86.1s38.7%2C86.1%2C86.2%2C86.1c27.8%2C0%2C52.6-13.3%2C68.4-33.9%09%09l132.2%2C73.9c-3.2%2C9-5%2C18.7-5%2C28.7c0%2C47.4%2C38.6%2C86.1%2C86.1%2C86.1s86.1-38.6%2C86.1-86.1S429.1%2C309.4%2C381.6%2C309.4z M381.6%2C27.1%09%09c32.6%2C0%2C59.1%2C26.5%2C59.1%2C59.1s-26.5%2C59.1-59.1%2C59.1s-59.1-26.5-59.1-59.1S349.1%2C27.1%2C381.6%2C27.1z M100%2C299.8%09%09c-32.6%2C0-59.1-26.5-59.1-59.1s26.5-59.1%2C59.1-59.1s59.1%2C26.5%2C59.1%2C59.1S132.5%2C299.8%2C100%2C299.8z M381.6%2C454.5%09%09c-32.6%2C0-59.1-26.5-59.1-59.1c0-32.6%2C26.5-59.1%2C59.1-59.1s59.1%2C26.5%2C59.1%2C59.1C440.7%2C428%2C414.2%2C454.5%2C381.6%2C454.5z%22%2F%3E%3C%2Fg%3E%3C%2Fsvg%3E\");\n}\n\n.p-save-U0OTD {\n  background-image: url(\"data:image/svg+xml,%3Csvg fill%3D%22%236F6F6F%22 height%3D%22800px%22 width%3D%22800px%22 version%3D%221.1%22 id%3D%22Capa_1%22 xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 xmlns%3Axlink%3D%22http%3A%2F%2Fwww.w3.org%2F1999%2Fxlink%22 %09 viewBox%3D%220 0 471.2 471.2%22 xml%3Aspace%3D%22preserve%22%3E%3Cg%3E%09%3Cg%3E%09%09%3Cpath d%3D%22M457.7%2C230.15c-7.5%2C0-13.5%2C6-13.5%2C13.5v122.8c0%2C33.4-27.2%2C60.5-60.5%2C60.5H87.5c-33.4%2C0-60.5-27.2-60.5-60.5v-124.8%09%09%09c0-7.5-6-13.5-13.5-13.5s-13.5%2C6-13.5%2C13.5v124.8c0%2C48.3%2C39.3%2C87.5%2C87.5%2C87.5h296.2c48.3%2C0%2C87.5-39.3%2C87.5-87.5v-122.8%09%09%09C471.2%2C236.25%2C465.2%2C230.15%2C457.7%2C230.15z%22%2F%3E%09%09%3Cpath d%3D%22M226.1%2C346.75c2.6%2C2.6%2C6.1%2C4%2C9.5%2C4s6.9-1.3%2C9.5-4l85.8-85.8c5.3-5.3%2C5.3-13.8%2C0-19.1c-5.3-5.3-13.8-5.3-19.1%2C0l-62.7%2C62.8%09%09%09V30.75c0-7.5-6-13.5-13.5-13.5s-13.5%2C6-13.5%2C13.5v273.9l-62.8-62.8c-5.3-5.3-13.8-5.3-19.1%2C0c-5.3%2C5.3-5.3%2C13.8%2C0%2C19.1%09%09%09L226.1%2C346.75z%22%2F%3E%09%3C%2Fg%3E%3C%2Fg%3E%3C%2Fsvg%3E\");\n}\n\n.l-calendar-jt18N {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 height%3D%2240%22 viewBox%3D%220 -960 960 960%22 width%3D%2240%22%3E%3Cpath d%3D%22M214.461-100q-31.302 0-53.548-22.246-22.246-22.247-22.246-53.549v-537.231q0-31.302 22.246-53.548 22.246-22.247 53.548-22.247h57.385v-80.41h55.539v80.41h308.307v-80.41h52.975v80.41h56.872q31.301 0 53.548 22.247 22.246 22.246 22.246 53.548v537.231q0 31.302-22.246 53.549Q776.84-100 745.539-100H214.461Zm0-51.179h531.078q9.23 0 16.923-7.693 7.692-7.692 7.692-16.923v-376H189.846v376q0 9.231 7.692 16.923 7.692 7.693 16.923 7.693Zm-24.615-451.795h580.308v-110.052q0-9.231-7.692-16.923-7.693-7.692-16.923-7.692H214.461q-9.231 0-16.923 7.692-7.692 7.692-7.692 16.923v110.052Zm0 0v-134.667 134.667Zm290.025 213.128q-13.589 0-24.115-10.655-10.525-10.654-10.525-24.243 0-13.589 10.654-24.115 10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.589-10.654 24.115-10.654 10.526-24.244 10.526Zm-160 0q-13.589 0-24.115-10.655-10.525-10.654-10.525-24.243 0-13.589 10.654-24.115 10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.589-10.654 24.115-10.654 10.526-24.244 10.526Zm320 0q-13.589 0-24.115-10.655-10.525-10.654-10.525-24.243 0-13.589 10.654-24.115 10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.589-10.654 24.115-10.654 10.526-24.244 10.526ZM479.871-236q-13.589 0-24.115-10.654-10.525-10.655-10.525-24.244t10.654-24.115q10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.59-10.654 24.115Q493.461-236 479.871-236Zm-160 0q-13.589 0-24.115-10.654-10.525-10.655-10.525-24.244t10.654-24.115q10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.59-10.654 24.115Q333.461-236 319.871-236Zm320 0q-13.589 0-24.115-10.654-10.525-10.655-10.525-24.244t10.654-24.115q10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.59-10.654 24.115Q653.461-236 639.871-236Z%22%2F%3E%3C%2Fsvg%3E\");\n}\n\n.b-GGIBZ:hover .l-calendar-jt18N,\n.button-tJIp6:hover .l-calendar-jt18N,\n.l-calendar-jt18N:hover {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 height%3D%2240%22 viewBox%3D%220 -960 960 960%22 width%3D%2240%22%3E%3Cpath d%3D%22M479.871-389.846q-13.589 0-24.115-10.655-10.525-10.654-10.525-24.243 0-13.589 10.654-24.115 10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.589-10.654 24.115-10.654 10.526-24.244 10.526Zm-160 0q-13.589 0-24.115-10.655-10.525-10.654-10.525-24.243 0-13.589 10.654-24.115 10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.589-10.654 24.115-10.654 10.526-24.244 10.526Zm320 0q-13.589 0-24.115-10.655-10.525-10.654-10.525-24.243 0-13.589 10.654-24.115 10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.589-10.654 24.115-10.654 10.526-24.244 10.526ZM479.871-236q-13.589 0-24.115-10.654-10.525-10.655-10.525-24.244t10.654-24.115q10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.59-10.654 24.115Q493.461-236 479.871-236Zm-160 0q-13.589 0-24.115-10.654-10.525-10.655-10.525-24.244t10.654-24.115q10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.59-10.654 24.115Q333.461-236 319.871-236Zm320 0q-13.589 0-24.115-10.654-10.525-10.655-10.525-24.244t10.654-24.115q10.654-10.526 24.244-10.526 13.589 0 24.115 10.655 10.525 10.654 10.525 24.243 0 13.59-10.654 24.115Q653.461-236 639.871-236Zm-425.41 136q-31.302 0-53.548-22.246-22.246-22.247-22.246-53.549v-537.231q0-31.302 22.246-53.548 22.246-22.247 53.548-22.247h57.385v-80.41h55.539v80.41h308.307v-80.41h52.975v80.41h56.872q31.301 0 53.548 22.247 22.246 22.246 22.246 53.548v537.231q0 31.302-22.246 53.549Q776.84-100 745.539-100H214.461Zm0-51.179h531.078q9.23 0 16.923-7.693 7.692-7.692 7.692-16.923v-376H189.846v376q0 9.231 7.692 16.923 7.692 7.693 16.923 7.693Z%22%2F%3E%3C%2Fsvg%3E\");\n}\n\n.heart-YJDRN {\n  background-image: url(\"data:image/svg+xml,%3C%3Fxml version%3D%221.0%22 encoding%3D%22UTF-8%22 standalone%3D%22no%22%3F%3E%3C!DOCTYPE svg PUBLIC %22-%2F%2FW3C%2F%2FDTD SVG 1.1%2F%2FEN%22 %22http%3A%2F%2Fwww.w3.org%2FGraphics%2FSVG%2F1.1%2FDTD%2Fsvg11.dtd%22%3E%3Csvg width%3D%22100%25%22  height%3D%22100%25%22 viewBox%3D%220 0 9 8%22 version%3D%221.1%22 xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 xmlns%3Axlink%3D%22http%3A%2F%2Fwww.w3.org%2F1999%2Fxlink%22 xml%3Aspace%3D%22preserve%22 style%3D%22fill-rule%3Aevenodd%3Bclip-rule%3Aevenodd%3Bstroke-linejoin%3Around%3Bstroke-miterlimit%3A1.41421%3B%22%3E%3Cpath  d%3D%22M4.104%2C7.2l-0.384%2C-0.24c-1.296%2C-0.768 -3.72%2C-2.208 -3.72%2C-4.56c0%2C-1.32 1.08%2C-2.4 2.4%2C-2.4c0.648%2C0 1.272%2C0.264 1.704%2C0.72c0.456%2C-0.456 1.056%2C-0.72 1.704%2C-0.72c1.32%2C0 2.4%2C1.08 2.4%2C2.4c0%2C2.376 -2.424%2C3.792 -3.72%2C4.56l-0.384%2C0.24Zm-1.704%2C-6.672c-1.032%2C0 -1.872%2C0.84 -1.872%2C1.872c0%2C2.064 2.256%2C3.384 3.456%2C4.104l0.12%2C0.072l0.12%2C-0.072c1.2%2C-0.72 3.456%2C-2.04 3.456%2C-4.104c0%2C-1.032 -0.84%2C-1.872 -1.872%2C-1.872c-0.576%2C0 -1.152%2C0.288 -1.488%2C0.744l-0.216%2C0.288l-0.216%2C-0.288c-0.36%2C-0.48 -0.912%2C-0.744 -1.488%2C-0.744Z%22 style%3D%22fill%3A%23black%3Bfill-rule%3Anonzero%3B%22%2F%3E%3C%2Fsvg%3E\");\n}\n\n.b-GGIBZ:hover .heart-YJDRN,\n.button-tJIp6:hover .heart-YJDRN,\n.heart-YJDRN:hover {\n  background-image: url(\"data:image/svg+xml,%3C%3Fxml version%3D%221.0%22 encoding%3D%22UTF-8%22 standalone%3D%22no%22%3F%3E%3C!DOCTYPE svg PUBLIC %22-%2F%2FW3C%2F%2FDTD SVG 1.1%2F%2FEN%22 %22http%3A%2F%2Fwww.w3.org%2FGraphics%2FSVG%2F1.1%2FDTD%2Fsvg11.dtd%22%3E%3Csvg width%3D%22100%25%22 height%3D%22100%25%22 viewBox%3D%220 0 9 8%22 version%3D%221.1%22 xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 xmlns%3Axlink%3D%22http%3A%2F%2Fwww.w3.org%2F1999%2Fxlink%22 xml%3Aspace%3D%22preserve%22 style%3D%22fill-rule%3Aevenodd%3Bclip-rule%3Aevenodd%3Bstroke-linejoin%3Around%3Bstroke-miterlimit%3A1.41421%3B%22%3E%3Cpath d%3D%22M4.104%2C7.2c1.231%2C-0.757 4.104%2C-2.296 4.104%2C-4.878c0%2C-1.279 -1.026%2C-2.322 -2.283%2C-2.322c-0.744%2C0 -1.411%2C0.365 -1.821%2C0.939c-0.41%2C-0.574 -1.077%2C-0.939 -1.821%2C-0.939c-1.257%2C0 -2.283%2C1.043 -2.283%2C2.322c-0.025%2C2.556 2.847%2C4.095 4.104%2C4.878Z%22 style%3D%22fill%3A%23000%3Bfill-rule%3Anonzero%3B%22%2F%3E%3C%2Fsvg%3E\");\n}\n\n.bookmark-ucaBU {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 height%3D%2240%22 viewBox%3D%220 -960 960 960%22 width%3D%2240%22%3E%3Cpath d%3D%22M220-154.667V-750q0-30.905 22.246-53.35 22.247-22.445 53.549-22.445h368.41q31.302 0 53.549 22.445Q740-780.905 740-750v595.333L480-265.744 220-154.667Zm51.179-77.948L480-321.824l208.821 89.209V-750q0-9.231-7.693-16.923-7.692-7.693-16.923-7.693h-368.41q-9.231 0-16.923 7.693-7.693 7.692-7.693 16.923v517.385Zm0-542.001h417.642-417.642Z%22%2F%3E%3C%2Fsvg%3E\");\n}\n\n.b-GGIBZ:hover .bookmark-ucaBU,\n.button-tJIp6:hover .bookmark-ucaBU,\n.bookmark-ucaBU:hover {\n  background-image: url(\"data:image/svg+xml,%3Csvg xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22 height%3D%2240%22 viewBox%3D%220 -960 960 960%22 width%3D%2240%22%3E%3Cpath d%3D%22M220-154.667V-750q0-30.905 22.189-53.35 22.188-22.445 53.606-22.445h368.41q31.418 0 53.606 22.445Q740-780.905 740-750v595.333L480-265.744 220-154.667Z%22%2F%3E%3C%2Fsvg%3E\");\n}\n.close-68IIu {\n  background-image: url(\"data:image/svg+xml,%3Csvg width%3D%2220%22 height%3D%2220%22 viewBox%3D%220 0 20 20%22 fill%3D%22none%22 xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%3E%3Cpath d%3D%22M0.5625 0.836182L18.5625 18.8362M18.5625 0.836182L0.5625 18.8362%22 stroke%3D%22%234A4A4A%22 stroke-linecap%3D%22round%22%2F%3E%3C%2Fsvg%3E\");\n}\n.requiredFieldsInfo-nkkEE {\n  font-size: 0.7em;\n  align-self: flex-start;\n}\n.nameErrorText-QN0MZ {\n  width: -moz-max-content;\n  width: max-content;\n}\n";
styleInject(css_248z$5);

var AcceptTerms = function AcceptTerms(_ref) {
  var _ref$privacyUrl = _ref.privacyUrl,
    privacyUrl = _ref$privacyUrl === void 0 ? 'https://www.diva-portal.de/datenschutz' : _ref$privacyUrl,
    _ref$agbUrl = _ref.agbUrl,
    agbUrl = _ref$agbUrl === void 0 ? 'https://www.diva-portal.de/agb' : _ref$agbUrl;
  var _useTranslation = useTranslation(),
    t = _useTranslation.t;
  var pUrl = /*#__PURE__*/jsx("a", {
    href: privacyUrl,
    target: "_blank",
    children: t('auth.privacypolicy')
  });
  var aUrl = /*#__PURE__*/jsx("a", {
    href: agbUrl,
    target: "_blank",
    children: t('auth.GeneralTerms')
  });
  return /*#__PURE__*/jsx(Trans, {
    i18nKey: "contactform.acceptprivacyandagb",
    components: {
      privacyUrl: pUrl,
      agbUrl: aUrl
    }
  });
};

var index$d = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': AcceptTerms
});

function ownKeys$7(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$7(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$7(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$7(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$8 = getLogger('Planner', 'WEB-PLANNER-REACT');
var CreateAppointment = function CreateAppointment(_ref) {
  var basket = _ref.basket,
    _ref$config = _ref.config,
    _ref$config$showDSGVO = _ref$config.showDSGVO,
    showDSGVO = _ref$config$showDSGVO === void 0 ? true : _ref$config$showDSGVO,
    emailConfig = _ref$config.emailConfig,
    showSubscription = _ref$config.showSubscription;
  var _useDivaCore = useDivaCore(),
    navigationConfiguration = _useDivaCore.state.navigationConfiguration,
    analyticsHandler = _useDivaCore.handler.analyticsHandler,
    sendEmail = _useDivaCore.actions.sendEmail;
  var _useTranslation = useTranslation(),
    i18n = _useTranslation.i18n,
    t = _useTranslation.t;
  var _useState = useState(true),
    _useState2 = _slicedToArray(_useState, 2),
    ableToSend = _useState2[0],
    setAbleToSend = _useState2[1];
  var _useState3 = useState(false),
    _useState4 = _slicedToArray(_useState3, 2),
    showWarningEmail = _useState4[0],
    setShowWarningEmail = _useState4[1];
  var _useState5 = useState(false),
    _useState6 = _slicedToArray(_useState5, 2),
    showWarningName = _useState6[0],
    setShowWarningName = _useState6[1];
  var _useState7 = useState(''),
    _useState8 = _slicedToArray(_useState7, 2),
    mail = _useState8[0],
    setMail = _useState8[1];
  var _useState9 = useState(''),
    _useState10 = _slicedToArray(_useState9, 2),
    name = _useState10[0],
    setName = _useState10[1];
  var _useState11 = useState(''),
    _useState12 = _slicedToArray(_useState11, 2),
    surName = _useState12[0],
    setSurName = _useState12[1];
  var _useState13 = useState(''),
    _useState14 = _slicedToArray(_useState13, 2),
    telNumber = _useState14[0],
    setTelNumber = _useState14[1];
  var _useState15 = useState(''),
    _useState16 = _slicedToArray(_useState15, 2),
    note = _useState16[0],
    setNote = _useState16[1];
  var _useState17 = useState(false),
    _useState18 = _slicedToArray(_useState17, 2),
    checkDSGVO = _useState18[0],
    setCheckDSGVO = _useState18[1];
  var _useState19 = useState(false),
    _useState20 = _slicedToArray(_useState19, 2),
    checkSubscriptionToNewsletter = _useState20[0],
    setCheckSubscriptionToNewsletter = _useState20[1];
  var _useState21 = useState(false),
    _useState22 = _slicedToArray(_useState21, 2),
    showWarningDSGVO = _useState22[0],
    setShowWarningDSGVO = _useState22[1];
  var sendMail = function sendMail() {
    var returnAfter = false;
    if (!name || !surName) {
      setAbleToSend(false);
      setShowWarningName(true);
      returnAfter = true;
    }
    if (!DivaUtils.validateEmail(mail)) {
      setAbleToSend(false);
      setShowWarningEmail(true);
      returnAfter = true;
    }
    if (showDSGVO && !checkDSGVO) {
      setAbleToSend(false);
      setShowWarningDSGVO(true);
      returnAfter = true;
    }
    if (returnAfter) return;
    LOG$8.businessEvent('onPlannerCreateAppointment', 'Planner', "Es wurde eine Terminanfrage gesendet", {
      hasAcceptedMarketing: checkSubscriptionToNewsletter
    });
    var utm = new UTMUtils(UtmSource.EMAIL, UtmMedium.EMAIL, UtmCampaign.PLANNER_CREATE_APPOINTMENT);
    var utmQueryString = utm.getQueryString();
    emailConfig === null || emailConfig === void 0 || emailConfig.map(function (c) {
      return sendEmail({
        showDialog: false,
        content: _objectSpread$7(_objectSpread$7({}, c), {}, {
          to: mail,
          divaNr: basket.DivaNr,
          type: 'basket',
          language: i18n.language,
          data: {
            utmQueryString: utmQueryString
          }
        })
      });
    });
    analyticsHandler.createLead({
      organizationId: basket.OrganizationId,
      divaNr: basket.DivaNr,
      email: mail,
      firstName: name,
      lastName: surName,
      phone: telNumber,
      message: note,
      pdfUrl: basket.PDFDocumentURL,
      //divaWebPlannerUrl: webplannerurl.toString(),
      marketingAccepted: checkSubscriptionToNewsletter,
      type: 'CREATE_APPOINTMENT'
    });
  };
  var onChangeMail = function onChangeMail(input) {
    var validMail = DivaUtils.validateEmail(input);
    setAbleToSend(validMail);
    if (validMail) setShowWarningEmail(false);
    setMail(input);
  };
  var onChangeDSGVO = function onChangeDSGVO(val) {
    setCheckDSGVO(val);
    if (val) setShowWarningDSGVO(false);
    if (!showWarningEmail) setAbleToSend(true);
  };
  var onChangeName = function onChangeName(input) {
    //check if name and surname are set
    if (input && surName) setShowWarningName(false);
    setName(input);
  };
  var onChangeSurName = function onChangeSurName(input) {
    //check if name and surname are set
    if (input && name) setShowWarningName(false);
    setSurName(input);
  };
  return /*#__PURE__*/jsxs("div", {
    className: subWrapper,
    children: [/*#__PURE__*/jsxs("div", {
      className: scrollWrapper,
      children: [/*#__PURE__*/jsx("div", {
        className: classnames(textWrapper, boldText),
        children: t('checkout.appointment.advanteges')
      }), /*#__PURE__*/jsxs("div", {
        className: textWrapper,
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon$1, checkGreen)
        }), t('checkout.appointment.advanteges_one')]
      }), /*#__PURE__*/jsxs("div", {
        className: textWrapper,
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon$1, checkGreen)
        }), t('checkout.appointment.advanteges_two')]
      }), /*#__PURE__*/jsxs("div", {
        className: textWrapper,
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon$1, checkGreen)
        }), t('checkout.appointment.advanteges_three')]
      }), /*#__PURE__*/jsxs("div", {
        className: inputWrapper2,
        children: [/*#__PURE__*/jsxs("div", {
          className: inputWrapper,
          children: [/*#__PURE__*/jsxs("div", {
            className: classnames(boldText),
            children: [t('checkout.save.firstname'), "*"]
          }), /*#__PURE__*/jsx("input", {
            className: input,
            type: "text",
            value: name,
            onChange: function onChange(e) {
              return onChangeName(e.target.value);
            }
          })]
        }), /*#__PURE__*/jsxs("div", {
          className: inputWrapper,
          children: [/*#__PURE__*/jsxs("div", {
            className: classnames(boldText),
            children: [t('checkout.surname'), "*"]
          }), /*#__PURE__*/jsx("input", {
            className: input,
            type: "text",
            value: surName,
            onChange: function onChange(e) {
              return onChangeSurName(e.target.value);
            }
          })]
        }), showWarningName && /*#__PURE__*/jsx("div", {
          className: classnames(errorText, nameErrorText),
          children: t('checkout.name_error')
        })]
      }), /*#__PURE__*/jsxs("div", {
        className: inputWrapper,
        children: [/*#__PURE__*/jsxs("div", {
          className: classnames(boldText),
          children: [t('checkout.save.email'), "*"]
        }), /*#__PURE__*/jsx("input", {
          className: input,
          type: "email",
          value: mail,
          onChange: function onChange(e) {
            return onChangeMail(e.target.value);
          }
        }), showWarningEmail && /*#__PURE__*/jsx("div", {
          className: errorText,
          children: t('checkout.share_mail.mail_error')
        })]
      }), /*#__PURE__*/jsxs("div", {
        className: inputWrapper,
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(boldText),
          children: t('telephoneSwitchBoard')
        }), /*#__PURE__*/jsx("input", {
          className: input,
          type: "text",
          value: telNumber,
          onChange: function onChange(e) {
            return setTelNumber(e.target.value);
          }
        })]
      }), /*#__PURE__*/jsxs("div", {
        className: inputWrapper,
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(boldText),
          children: t('checkout.appointment.note')
        }), /*#__PURE__*/jsx("textarea", {
          id: "message",
          placeholder: t('checkout.appointment.note_placehoder'),
          style: {
            height: 100,
            maxWidth: 350
          },
          onChange: function onChange(e) {
            return setNote(e.target.value);
          },
          value: note
        })]
      })]
    }), (!!showSubscription || !!showDSGVO) && /*#__PURE__*/jsxs("div", {
      className: inputWrapper,
      children: [!!showDSGVO && /*#__PURE__*/jsxs("div", {
        className: checkboxWrapper,
        children: [/*#__PURE__*/jsx("input", {
          type: "checkbox",
          className: checkbox,
          defaultChecked: checkDSGVO,
          onChange: function onChange(e) {
            return onChangeDSGVO(e.target.checked);
          }
        }), /*#__PURE__*/jsx("div", {
          className: classnames(text),
          children: /*#__PURE__*/jsx(AcceptTerms, {
            agbUrl: navigationConfiguration.tosUrl,
            privacyUrl: navigationConfiguration.privacyUrl
          })
        })]
      }), !!showSubscription && /*#__PURE__*/jsxs("div", {
        className: checkboxWrapper,
        children: [/*#__PURE__*/jsx("input", {
          type: "checkbox",
          className: checkbox,
          defaultChecked: checkSubscriptionToNewsletter,
          onChange: function onChange(e) {
            return setCheckSubscriptionToNewsletter(e.target.checked);
          }
        }), /*#__PURE__*/jsx("div", {
          className: classnames(text),
          children: t('checkout.appointment.subscription')
        })]
      }), showWarningDSGVO && /*#__PURE__*/jsx("div", {
        className: errorText,
        children: t('checkout.dsgvo_error')
      })]
    }), /*#__PURE__*/jsxs("div", {
      className: classnames(text, requiredFieldsInfo),
      children: ["* ", t('checkout.requiredfieldsinfo')]
    }), /*#__PURE__*/jsx("button", {
      className: classnames(button, inverted, !ableToSend ? disabled : undefined),
      onClick: function onClick() {
        return sendMail();
      },
      children: t('checkout.appointment.create')
    })]
  });
};

var index$c = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': CreateAppointment
});

var wrapper$1 = "wrapper-wqmkp";
var toggleSwitch = "toggle-switch-MfN-e";
var switchSelection = "switch-selection-bBWAz";
var switchLabel = "switch-label-KFwxM";
var css_248z$4 = ".wrapper-wqmkp {\n  height: 36px;\n  width: 100%;\n  position: relative;\n  display: flex;\n  justify-content: center;\n  align-items: center;\n  background-color: #f4f4f4;\n  border-radius: 6px;\n}\n.toggle-switch-MfN-e {\n  height: calc(100% - 6px);\n  width: calc(100% - 6px);\n  border-radius: 3px;\n  box-sizing: content-box;\n  box-sizing: initial;\n  position: relative;\n  display: flex;\n  justify-content: space-between;\n  align-items: center;\n}\n\n.switch-selection-bBWAz {\n  display: block;\n  position: absolute;\n  z-index: 1;\n  height: 30px;\n  background-color: #fff;\n  border-radius: 3px;\n  transition: left 0.25s ease-out;\n}\n\n.switch-label-KFwxM {\n  position: relative;\n  display: flex;\n  gap: 4px;\n  justify-content: center;\n  align-items: center;\n  z-index: 2;\n  line-height: 22px;\n  height: 20px;\n  width: calc(100% - 8px);\n  padding: 6px;\n  font-size: 14px;\n  color: rgba(0, 0, 0, 0.6);\n  text-align: center;\n  cursor: pointer;\n  -webkit-user-select: none;\n     -moz-user-select: none;\n          user-select: none;\n  transition: color 0.25s ease-out;\n}\n";
styleInject(css_248z$4);

var MultiToggleSwitch = function MultiToggleSwitch(_ref) {
  var values = _ref.values,
    _ref$state = _slicedToArray(_ref.state, 2),
    selected = _ref$state[0],
    setSelected = _ref$state[1];
  var widthPercentage = 100 / values.length;
  return /*#__PURE__*/jsx("div", {
    className: wrapper$1,
    children: /*#__PURE__*/jsxs("div", {
      className: toggleSwitch,
      children: [values.map(function (val, index) {
        return /*#__PURE__*/jsx("div", {
          style: {
            width: widthPercentage + '%'
          },
          children: /*#__PURE__*/jsx("label", {
            onClick: function onClick() {
              return setSelected(index);
            },
            className: switchLabel,
            children: val
          })
        }, index);
      }), /*#__PURE__*/jsx("div", {
        className: switchSelection,
        style: {
          left: selected / values.length * 100 + '%',
          width: widthPercentage + '%'
        }
      })]
    })
  });
};

var index$b = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': MultiToggleSwitch
});

var wrapper = "wrapper-Fhn4b";
var infoText = "info-text-NmbmI";
var bottomInfoText = "bottom-info-text-sUNyj";
var icon = "icon-tY7Kv";
var gridWrapper = "grid-wrapper-tN4Sf";
var wrappedText = "wrapped-text-55E8n";
var copyWrapper = "copy-wrapper-a6XV4";
var horizontalLine = "horizontal-line-pN5d1";
var css_248z$3 = ".wrapper-Fhn4b {\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  gap: 12px;\n  align-items: center;\n  width: 360px;\n  max-width: 100%;\n}\n.info-text-NmbmI {\n  color: #71717a;\n  font-size: 14px;\n}\n\n.bottom-info-text-sUNyj {\n  color: #000;\n  font-size: 14px;\n  text-align: center;\n  width: 100%;\n}\n\n.icon-tY7Kv {\n  width: 16px;\n  height: 16px;\n  background-position: center;\n  background-repeat: no-repeat;\n  background-size: 70%;\n}\n\n.grid-wrapper-tN4Sf {\n  display: grid;\n  grid-template-columns: repeat(2, 1fr);\n  grid-template-rows: repeat(2, 1fr);\n  grid-column-gap: 8px;\n  grid-row-gap: 8px;\n  width: 100%;\n}\n\n.wrapped-text-55E8n {\n  background-color: #f4f4f5;\n  padding: 12px;\n  width: calc(100% - 24px);\n  border-radius: 6px;\n  overflow: hidden;\n  line-break: anywhere;\n}\n\n.copy-wrapper-a6XV4 {\n  display: flex;\n  width: 100%;\n  gap: 8px;\n}\n\n.copy-wrapper-a6XV4 input {\n  width: 60%;\n}\n.copy-wrapper-a6XV4 button {\n  width: 40%;\n}\n\n.horizontal-line-pN5d1 {\n  border-bottom: 1px solid rgb(228, 228, 231);\n  width: 100%;\n}\n";
styleInject(css_248z$3);

var LOG$7 = getLogger('Planner', 'WEB-PLANNER-REACT');
var ShareViaUrl = function ShareViaUrl(_ref) {
  var url = _ref.url;
  var _useState = useState(false),
    _useState2 = _slicedToArray(_useState, 2),
    justCopied = _useState2[0],
    setJustCopied = _useState2[1];
  var _useState3 = useState(),
    _useState4 = _slicedToArray(_useState3, 2),
    time = _useState4[0],
    setTime = _useState4[1];
  var utmStandard = new UTMUtils(UtmSource.DIRECT, UtmMedium.REFERRAL, UtmCampaign.PLANNER_SHARE);
  var fullUrl = utmStandard.addUtmParams(url);
  var utmQr = new UTMUtils(UtmSource.DIRECT, UtmMedium.SOCIAL, UtmCampaign.PLANNER_QR_SHARE);
  var fullQrUrl = utmQr.addUtmParams(url);
  var onCopy = function onCopy() {
    var _navigator;
    LOG$7.businessEvent('onPlannerShareConfiguration', 'Planner', "Der Link zum Konfigurations-PDF wurde kopiert", {
      shareType: 'link',
      shareAction: 'copy_url'
    });
    setJustCopied(true);
    (_navigator = navigator) === null || _navigator === void 0 || _navigator.clipboard.writeText(fullUrl);
    if (time) clearTimeout(time);
    setTime(setTimeout(function () {
      return setJustCopied(false);
    }, 3000));
  };
  var onClickQRCode = function onClickQRCode() {
    var _window$open;
    LOG$7.businessEvent('onPlannerShareConfiguration', 'Planner', "Eine Konfiguration wurde \xFCber eine URL ge\xF6ffnet", {
      shareType: 'link',
      shareAction: 'qr_click'
    });
    (_window$open = window.open(fullQrUrl, '_blank')) === null || _window$open === void 0 || _window$open.focus();
  };
  var _useTranslation = useTranslation(),
    t = _useTranslation.t;
  return /*#__PURE__*/jsxs(Fragment, {
    children: [/*#__PURE__*/jsx("div", {
      className: infoText,
      children: t('checkout.share_url.share_via_QR')
    }), /*#__PURE__*/jsx(QRCodeSVG, {
      style: {
        cursor: 'pointer'
      },
      size: 200,
      value: fullQrUrl,
      onClick: onClickQRCode
    }), /*#__PURE__*/jsxs("div", {
      className: copyWrapper,
      children: [/*#__PURE__*/jsx("input", {
        className: input,
        type: "text",
        value: fullUrl
      }), /*#__PURE__*/jsx("button", {
        className: classnames(button, inverted, justCopied ? dark : undefined),
        onClick: onCopy,
        children: justCopied ? /*#__PURE__*/jsxs(Fragment, {
          children: [/*#__PURE__*/jsx("div", {
            className: classnames(icon$1, check)
          }), t('checkout.share_url.copied')]
        }) : /*#__PURE__*/jsxs(Fragment, {
          children: [/*#__PURE__*/jsx("div", {
            className: classnames(icon$1, copy)
          }), t('checkout.share_url.copy')]
        })
      })]
    }), /*#__PURE__*/jsx("div", {
      className: bottomInfoText,
      children: t('checkout.share_url.scann_QR')
    })]
  });
};

var index$a = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': ShareViaUrl
});

var WhatsappLogo = "data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20viewBox%3D%220%200%20175.216%20175.552%22%3E%3Cdefs%3E%3ClinearGradient%20id%3D%22b%22%20x1%3D%2285.915%22%20x2%3D%2286.535%22%20y1%3D%2232.567%22%20y2%3D%22137.092%22%20gradientUnits%3D%22userSpaceOnUse%22%3E%3Cstop%20offset%3D%220%22%20stop-color%3D%22%2357d163%22%2F%3E%3Cstop%20offset%3D%221%22%20stop-color%3D%22%2323b33a%22%2F%3E%3C%2FlinearGradient%3E%3Cfilter%20id%3D%22a%22%20width%3D%221.115%22%20height%3D%221.114%22%20x%3D%22-.057%22%20y%3D%22-.057%22%20color-interpolation-filters%3D%22sRGB%22%3E%3CfeGaussianBlur%20stdDeviation%3D%223.531%22%2F%3E%3C%2Ffilter%3E%3C%2Fdefs%3E%3Cpath%20fill%3D%22%23b3b3b3%22%20d%3D%22m54.532%20138.45%202.235%201.324c9.387%205.571%2020.15%208.518%2031.126%208.523h.023c33.707%200%2061.139-27.426%2061.153-61.135.006-16.335-6.349-31.696-17.895-43.251A60.75%2060.75%200%200%200%2087.94%2025.983c-33.733%200-61.166%2027.423-61.178%2061.13a60.98%2060.98%200%200%200%209.349%2032.535l1.455%202.312-6.179%2022.558zm-40.811%2023.544L24.16%20123.88c-6.438-11.154-9.825-23.808-9.821-36.772.017-40.556%2033.021-73.55%2073.578-73.55%2019.681.01%2038.154%207.669%2052.047%2021.572s21.537%2032.383%2021.53%2052.037c-.018%2040.553-33.027%2073.553-73.578%2073.553h-.032c-12.313-.005-24.412-3.094-35.159-8.954zm0%200%22%20filter%3D%22url%28%23a%29%22%2F%3E%3Cpath%20fill%3D%22%23fff%22%20d%3D%22m12.966%20161.238%2010.439-38.114a73.42%2073.42%200%200%201-9.821-36.772c.017-40.556%2033.021-73.55%2073.578-73.55%2019.681.01%2038.154%207.669%2052.047%2021.572s21.537%2032.383%2021.53%2052.037c-.018%2040.553-33.027%2073.553-73.578%2073.553h-.032c-12.313-.005-24.412-3.094-35.159-8.954z%22%2F%3E%3Cpath%20fill%3D%22url%28%23linearGradient1780%29%22%20d%3D%22M87.184%2025.227c-33.733%200-61.166%2027.423-61.178%2061.13a60.98%2060.98%200%200%200%209.349%2032.535l1.455%202.312-6.179%2022.559%2023.146-6.069%202.235%201.324c9.387%205.571%2020.15%208.518%2031.126%208.524h.023c33.707%200%2061.14-27.426%2061.153-61.135a60.75%2060.75%200%200%200-17.895-43.251%2060.75%2060.75%200%200%200-43.235-17.929z%22%2F%3E%3Cpath%20fill%3D%22url%28%23b%29%22%20d%3D%22M87.184%2025.227c-33.733%200-61.166%2027.423-61.178%2061.13a60.98%2060.98%200%200%200%209.349%2032.535l1.455%202.313-6.179%2022.558%2023.146-6.069%202.235%201.324c9.387%205.571%2020.15%208.517%2031.126%208.523h.023c33.707%200%2061.14-27.426%2061.153-61.135a60.75%2060.75%200%200%200-17.895-43.251%2060.75%2060.75%200%200%200-43.235-17.928z%22%2F%3E%3Cpath%20fill%3D%22%23fff%22%20fill-rule%3D%22evenodd%22%20d%3D%22M68.772%2055.603c-1.378-3.061-2.828-3.123-4.137-3.176l-3.524-.043c-1.226%200-3.218.46-4.902%202.3s-6.435%206.287-6.435%2015.332%206.588%2017.785%207.506%2019.013%2012.718%2020.381%2031.405%2027.75c15.529%206.124%2018.689%204.906%2022.061%204.6s10.877-4.447%2012.408-8.74%201.532-7.971%201.073-8.74-1.685-1.226-3.525-2.146-10.877-5.367-12.562-5.981-2.91-.919-4.137.921-4.746%205.979-5.819%207.206-2.144%201.381-3.984.462-7.76-2.861-14.784-9.124c-5.465-4.873-9.154-10.891-10.228-12.73s-.114-2.835.808-3.751c.825-.824%201.838-2.147%202.759-3.22s1.224-1.84%201.836-3.065.307-2.301-.153-3.22-4.032-10.011-5.666-13.647%22%2F%3E%3C%2Fsvg%3E";

var TelegramLogo = "data:image/svg+xml,%3Csvg%20id%3D%22Livello_1%22%20data-name%3D%22Livello%201%22%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20xmlns%3Axlink%3D%22http%3A%2F%2Fwww.w3.org%2F1999%2Fxlink%22%20viewBox%3D%220%200%20240%20240%22%3E%3Cdefs%3E%3ClinearGradient%20id%3D%22linear-gradient%22%20x1%3D%22120%22%20y1%3D%22240%22%20x2%3D%22120%22%20gradientUnits%3D%22userSpaceOnUse%22%3E%3Cstop%20offset%3D%220%22%20stop-color%3D%22%231d93d2%22%2F%3E%3Cstop%20offset%3D%221%22%20stop-color%3D%22%2338b0e3%22%2F%3E%3C%2FlinearGradient%3E%3C%2Fdefs%3E%3Ctitle%3ETelegram_logo%3C%2Ftitle%3E%3Ccircle%20cx%3D%22120%22%20cy%3D%22120%22%20r%3D%22120%22%20fill%3D%22url%28%23linear-gradient%29%22%2F%3E%3Cpath%20d%3D%22M81.229%2C128.772l14.237%2C39.406s1.78%2C3.687%2C3.686%2C3.687%2C30.255-29.492%2C30.255-29.492l31.525-60.89L81.737%2C118.6Z%22%20fill%3D%22%23c8daea%22%2F%3E%3Cpath%20d%3D%22M100.106%2C138.878l-2.733%2C29.046s-1.144%2C8.9%2C7.754%2C0%2C17.415-15.763%2C17.415-15.763%22%20fill%3D%22%23a9c6d8%22%2F%3E%3Cpath%20d%3D%22M81.486%2C130.178%2C52.2%2C120.636s-3.5-1.42-2.373-4.64c.232-.664.7-1.229%2C2.1-2.2%2C6.489-4.523%2C120.106-45.36%2C120.106-45.36s3.208-1.081%2C5.1-.362a2.766%2C2.766%2C0%2C0%2C1%2C1.885%2C2.055%2C9.357%2C9.357%2C0%2C0%2C1%2C.254%2C2.585c-.009.752-.1%2C1.449-.169%2C2.542-.692%2C11.165-21.4%2C94.493-21.4%2C94.493s-1.239%2C4.876-5.678%2C5.043A8.13%2C8.13%2C0%2C0%2C1%2C146.1%2C172.5c-8.711-7.493-38.819-27.727-45.472-32.177a1.27%2C1.27%2C0%2C0%2C1-.546-.9c-.093-.469.417-1.05.417-1.05s52.426-46.6%2C53.821-51.492c.108-.379-.3-.566-.848-.4-3.482%2C1.281-63.844%2C39.4-70.506%2C43.607A3.21%2C3.21%2C0%2C0%2C1%2C81.486%2C130.178Z%22%20fill%3D%22%23fff%22%2F%3E%3C%2Fsvg%3E";

function ownKeys$6(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$6(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$6(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$6(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$6 = getLogger('Planner', 'WEB-PLANNER-REACT');
var ShareViaSocialMedias = function ShareViaSocialMedias(_ref) {
  var url = _ref.url,
    _ref$config = _ref.config,
    _ref$config$title = _ref$config.title,
    ti = _ref$config$title === void 0 ? '' : _ref$config$title,
    _ref$config$text = _ref$config.text,
    te = _ref$config$text === void 0 ? '' : _ref$config$text;
  var _useTranslation = useTranslation(),
    t = _useTranslation.t;
  var title = t(ti);
  var text = t(te);
  var shareContent = {
    text: text,
    title: title,
    url: url
  };
  var _useState = useState('WHATSAPP'),
    _useState2 = _slicedToArray(_useState, 2),
    shareVia = _useState2[0],
    setShareVia = _useState2[1];
  var utmSource = shareVia == 'WHATSAPP' ? UtmSource.WHATSAPP : UtmSource.TELEGRAM;
  var utm = new UTMUtils(utmSource, UtmMedium.SOCIAL, UtmCampaign.PLANNER_SHARE);
  var fullUrl = utm.addUtmParams(url);
  return /*#__PURE__*/jsxs(Fragment, {
    children: [/*#__PURE__*/jsx("div", {
      className: infoText,
      children: t('checkout.share_social.share_whatsapp')
    }), /*#__PURE__*/jsx(QRCodeSVG, {
      size: 200,
      minVersion: 6,
      value: shareVia == 'WHATSAPP' ? 'https://wa.me/?text=' + encodeURIComponent("".concat(text, "\n").concat(fullUrl)) : 'https://t.me/share/url?url=' + encodeURIComponent(fullUrl) + '&text=' + encodeURIComponent(text),
      style: {
        cursor: 'pointer'
      },
      onClick: function onClick() {
        return shareSocial(shareVia == 'WHATSAPP' ? 'whatsapp' : 'telegram', shareContent, true);
      },
      imageSettings: {
        src: shareVia == 'WHATSAPP' ? WhatsappLogo : TelegramLogo,
        height: 40,
        width: 40,
        excavate: true
      }
    }), /*#__PURE__*/jsx("div", {
      className: bottomInfoText,
      children: t(shareVia == 'WHATSAPP' ? 'checkout.share_social.text_whatsapp' : 'checkout.share_social.text_telegram')
    }), /*#__PURE__*/jsx("button", {
      className: classnames(button, inverted),
      onClick: function onClick() {
        return shareSocial(shareVia == 'WHATSAPP' ? 'whatsapp' : 'telegram', shareContent);
      },
      children: t(shareVia == 'WHATSAPP' ? 'checkout.share_social.text_whatsapp2' : 'checkout.share_social.text_telegram2')
    }), /*#__PURE__*/jsx("div", {
      className: horizontalLine
    }), /*#__PURE__*/jsx("div", {
      className: infoText,
      children: t('checkout.share_social.other_socials')
    }), /*#__PURE__*/jsxs("div", {
      className: gridWrapper,
      children: [/*#__PURE__*/jsxs("button", {
        className: classnames(button, iconTextButton),
        onClick: function onClick() {
          return shareSocial('facebook', shareContent);
        },
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon$1, facebook)
        }), /*#__PURE__*/jsx("div", {
          children: "Facebook"
        })]
      }), /*#__PURE__*/jsxs("button", {
        className: classnames(button, iconTextButton),
        onClick: function onClick() {
          return shareSocial('linkedin', shareContent);
        },
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon$1, linkedin)
        }), /*#__PURE__*/jsx("div", {
          children: "LinkedIn"
        })]
      }), /*#__PURE__*/jsxs("button", {
        className: classnames(button, iconTextButton),
        onClick: function onClick() {
          return setShareVia(shareVia == 'WHATSAPP' ? 'TELEGRAM' : 'WHATSAPP');
        },
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon$1, message)
        }), /*#__PURE__*/jsx("div", {
          children: shareVia == 'WHATSAPP' ? 'Telegram' : 'Whatsapp'
        })]
      }), /*#__PURE__*/jsxs("button", {
        className: classnames(button, iconTextButton),
        onClick: function onClick() {
          return shareSocial('twitter', shareContent);
        },
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon$1, twitter)
        }), /*#__PURE__*/jsx("div", {
          children: "\uD835\uDD4F / Twitter"
        })]
      })]
    })]
  });
};
var shareSocial = function shareSocial(platform, content, qrCode) {
  var url = content.url,
    title = content.title,
    text = content.text,
    hashtags = content.hashtags,
    via = content.via;
  var shareUrl = '';
  LOG$6.businessEvent('onPlannerShareConfiguration', 'Planner', "Eine Koniguration wurde \xFCber einen Social Kanal geteilt", {
    shareType: 'social',
    shareAction: !qrCode ? 'button_click' : 'qr_click',
    sharePlatform: platform
  });
  var fullUrl = '';
  var utm = undefined;
  switch (platform) {
    case 'facebook':
      utm = new UTMUtils(UtmSource.FACEBOOK, UtmMedium.SOCIAL, UtmCampaign.PLANNER_SHARE);
      fullUrl = utm.addUtmParams(url);
      shareUrl = "https://www.facebook.com/sharer/sharer.php?u=".concat(encodeURIComponent(fullUrl), "&quote=").concat(encodeURIComponent(text));
      break;
    case 'twitter':
      utm = new UTMUtils(UtmSource.X, UtmMedium.SOCIAL, UtmCampaign.PLANNER_SHARE);
      fullUrl = utm.addUtmParams(url);
      shareUrl = "https://x.com/intent/tweet?url=".concat(encodeURIComponent(fullUrl), "&text=").concat(encodeURIComponent(text)).concat(hashtags ? "&hashtags=".concat(encodeURIComponent(hashtags)) : '').concat(via ? "&via=".concat(encodeURIComponent(via)) : '');
      break;
    case 'linkedin':
      utm = new UTMUtils(UtmSource.LINKEDIN, UtmMedium.SOCIAL, UtmCampaign.PLANNER_SHARE);
      fullUrl = utm.addUtmParams(url);
      shareUrl = "https://www.linkedin.com/shareArticle?mini=true&url=".concat(encodeURIComponent(fullUrl), "&title=").concat(encodeURIComponent(title), "&summary=").concat(encodeURIComponent(text));
      break;
    case 'whatsapp':
      utm = new UTMUtils(UtmSource.WHATSAPP, UtmMedium.SOCIAL, UtmCampaign.PLANNER_SHARE);
      fullUrl = utm.addUtmParams(url);
      // Use WhatsApp Web for desktop and fallback to WhatsApp mobile protocol
      shareUrl = "https://web.whatsapp.com/send?text=".concat(encodeURIComponent("".concat(text, "\n").concat(fullUrl)));
      break;
    case 'telegram':
      utm = new UTMUtils(UtmSource.TELEGRAM, UtmMedium.SOCIAL, UtmCampaign.PLANNER_SHARE);
      fullUrl = utm.addUtmParams(url);
      shareUrl = "https://t.me/share/url?url=".concat(encodeURIComponent(fullUrl), "&text=").concat(encodeURIComponent(text));
      break;
  }

  // For WhatsApp, we'll open in a new tab instead of a popup
  if (platform === 'whatsapp' || platform == 'telegram') {
    window.open(shareUrl, '_blank');
  } else {
    createShareWindow(shareUrl);
  }
};
var WINDOW_CONFIG = {
  width: 600,
  height: 400,
  location: 'no',
  toolbar: 'no',
  status: 'no',
  directories: 'no',
  menubar: 'no',
  scrollbars: 'yes',
  resizable: 'no',
  centerscreen: 'yes',
  chrome: 'yes'
};
var createShareWindow = function createShareWindow(url) {
  var left = window.screen.width / 2 - WINDOW_CONFIG.width / 2;
  var top = window.screen.height / 2 - WINDOW_CONFIG.height / 2;
  return window.open(url, 'share', Object.entries(_objectSpread$6(_objectSpread$6({}, WINDOW_CONFIG), {}, {
    left: left,
    top: top
  })).map(function (_ref2) {
    var _ref3 = _slicedToArray(_ref2, 2),
      key = _ref3[0],
      value = _ref3[1];
    return "".concat(key, "=").concat(value);
  }).join(','));
};

var index$9 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    shareSocial: shareSocial,
    'default': ShareViaSocialMedias
});

function ownKeys$5(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$5(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$5(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$5(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$5 = getLogger('Planner', 'WEB-PLANNER-REACT');
var ShareViaEmail = function ShareViaEmail(_ref) {
  var basket = _ref.basket,
    config = _ref.config;
    _ref.url;
  var _useDivaCore = useDivaCore(),
    sendEmail = _useDivaCore.actions.sendEmail;
  var _useTranslation = useTranslation(),
    i18n = _useTranslation.i18n,
    t = _useTranslation.t;
  var _useState = useState(true),
    _useState2 = _slicedToArray(_useState, 2),
    ableToSend = _useState2[0],
    setAbleToSend = _useState2[1];
  var _useState3 = useState(false),
    _useState4 = _slicedToArray(_useState3, 2),
    showWarning = _useState4[0],
    setShowWarning = _useState4[1];
  var _useState5 = useState(''),
    _useState6 = _slicedToArray(_useState5, 2),
    mail$1 = _useState6[0],
    setMail = _useState6[1];
  var utm = new UTMUtils(UtmSource.EMAIL, UtmMedium.EMAIL, UtmCampaign.PLANNER_SHARE);
  var utmQueryString = utm.getQueryString();
  var sendMail = function sendMail() {
    if (!DivaUtils$1.validateEmail(mail$1)) {
      setAbleToSend(false);
      setShowWarning(true);
      return;
    }
    LOG$5.businessEvent('onPlannerShareConfiguration', 'Planner', "Eine Konfiguration wurde als E-Mail geteilt", {
      shareType: 'email',
      shareAction: 'button_click'
    });
    config.map(function (c) {
      return sendEmail({
        showDialog: false,
        content: _objectSpread$5(_objectSpread$5({}, c), {}, {
          to: mail$1,
          divaNr: basket.DivaNr,
          type: 'basket',
          language: i18n.language,
          data: {
            utmQueryString: utmQueryString,
            websiteBaseUrl: window.location.origin + window.location.pathname
          }
        })
      });
    });
  };
  var _onChange = function onChange(input) {
    var validMail = DivaUtils$1.validateEmail(input);
    setAbleToSend(validMail);
    if (validMail) setShowWarning(false);
    setMail(input);
  };
  return /*#__PURE__*/jsxs(Fragment, {
    children: [/*#__PURE__*/jsx("div", {
      className: infoText,
      children: t('checkout.share_mail.easy_share')
    }), /*#__PURE__*/jsxs("div", {
      className: inputWrapper,
      children: [/*#__PURE__*/jsx("input", {
        className: input,
        type: "email",
        value: mail$1,
        onChange: function onChange(e) {
          return _onChange(e.target.value);
        },
        placeholder: t('checkout.share_mail.placeholder')
      }), showWarning && /*#__PURE__*/jsx("div", {
        className: errorText,
        children: t('checkout.share_mail.mail_error')
      })]
    }), /*#__PURE__*/jsxs("div", {
      className: wrappedText,
      children: [" ", t('checkout.share_mail.mail_text')]
    }), /*#__PURE__*/jsxs("button", {
      className: classnames(button, inverted, !ableToSend ? disabled : undefined),
      onClick: function onClick() {
        return sendMail();
      },
      children: [/*#__PURE__*/jsx("div", {
        className: classnames(icon$1, mail)
      }), t('checkout.share_mail.button_text')]
    })]
  });
};

var index$8 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': ShareViaEmail
});

var Share = function Share(_ref) {
  var config = _ref.config,
    basket = _ref.basket,
    url$1 = _ref.url;
  var _useState = useState([]),
    _useState2 = _slicedToArray(_useState, 2),
    possibleTypes = _useState2[0],
    setPossibleTypes = _useState2[1];
  var _useDivaCore = useDivaCore(),
    isMobile = _useDivaCore.state.dimensions.isMobile,
    _useDivaCore$actions = _useDivaCore.actions,
    closeModal = _useDivaCore$actions.closeModal,
    openModal = _useDivaCore$actions.openModal;
  var _useTranslation = useTranslation(),
    t = _useTranslation.t;
  useEffect(function () {
    var _config$emailConfig, _config$default;
    if (isMobile) {
      if (navigator.share) {
        openModal({
          accept: true,
          text: t('share.now_text'),
          title: t('share.now_title'),
          callback: function callback(a) {
            if (a) {
              var _config$socialMedia, _config$socialMedia2, _config$socialMedia3, _config$socialMedia4;
              navigator.share({
                title: (_config$socialMedia = config.socialMedia) !== null && _config$socialMedia !== void 0 && _config$socialMedia.title ? t((_config$socialMedia2 = config.socialMedia) === null || _config$socialMedia2 === void 0 ? void 0 : _config$socialMedia2.title) : undefined,
                text: (_config$socialMedia3 = config.socialMedia) !== null && _config$socialMedia3 !== void 0 && _config$socialMedia3.text ? t((_config$socialMedia4 = config.socialMedia) === null || _config$socialMedia4 === void 0 ? void 0 : _config$socialMedia4.text) : undefined,
                url: url$1
              });
            }
            closeModal();
          }
        });
      }
    }
    var newPossibleTypes = [];
    if (config.url) newPossibleTypes.push('SHARE_URL');
    if (config.socialMedia) newPossibleTypes.push('SHARE_WHATSAPP');
    if ((_config$emailConfig = config.emailConfig) !== null && _config$emailConfig !== void 0 && _config$emailConfig.length) newPossibleTypes.push('SHARE_EMAIL');
    setIndex(newPossibleTypes.indexOf((_config$default = config["default"]) !== null && _config$default !== void 0 ? _config$default : newPossibleTypes[0]));
    setPossibleTypes(newPossibleTypes);
  }, [config]);
  var _useState3 = useState(),
    _useState4 = _slicedToArray(_useState3, 2),
    index = _useState4[0],
    setIndex = _useState4[1];
  if (!possibleTypes.length || index === undefined) return /*#__PURE__*/jsx(Fragment, {});
  return /*#__PURE__*/jsxs("div", {
    className: wrapper,
    children: [/*#__PURE__*/jsx(MultiToggleSwitch, {
      state: [index, setIndex],
      values: [].concat(_toConsumableArray(config.url ? [/*#__PURE__*/jsxs(Fragment, {
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon, url)
        }), " ", t('checkout.share.title.url')]
      })] : []), _toConsumableArray(config.socialMedia ? [/*#__PURE__*/jsxs(Fragment, {
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon, message)
        }), " ", t('checkout.share.title.socials')]
      })] : []), _toConsumableArray(config.emailConfig ? [/*#__PURE__*/jsxs(Fragment, {
        children: [/*#__PURE__*/jsx("div", {
          className: classnames(icon, mail)
        }), " ", t('checkout.share.title.mail')]
      })] : []))
    }), possibleTypes[index] == 'SHARE_URL' && /*#__PURE__*/jsx(ShareViaUrl, {
      url: url$1
    }), possibleTypes[index] == 'SHARE_WHATSAPP' && /*#__PURE__*/jsx(ShareViaSocialMedias, {
      config: config.socialMedia,
      url: url$1
    }), possibleTypes[index] == 'SHARE_EMAIL' && /*#__PURE__*/jsx(ShareViaEmail, {
      config: config.emailConfig,
      basket: basket,
      url: url$1
    })]
  });
};

var index$7 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': Share
});

function ownKeys$4(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$4(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$4(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$4(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$4 = getLogger('Planner', 'WEB-PLANNER-REACT');
var Save = function Save(_ref) {
  var _ref$config = _ref.config,
    emailConfig = _ref$config.emailConfig,
    event = _ref$config.event,
    returnTo = _ref$config.returnTo,
    _ref$config$showDSGVO = _ref$config.showDSGVO,
    showDSGVO = _ref$config$showDSGVO === void 0 ? true : _ref$config$showDSGVO,
    showSubscription = _ref$config.showSubscription,
    basket = _ref.basket,
    viewHandler = _ref.viewHandler;
  var _useDivaCore = useDivaCore(),
    navigationConfiguration = _useDivaCore.state.navigationConfiguration,
    analyticsHandler = _useDivaCore.handler.analyticsHandler,
    sendEmail = _useDivaCore.actions.sendEmail;
  var _useTranslation = useTranslation(),
    i18n = _useTranslation.i18n,
    t = _useTranslation.t;
  var _useState = useState(true),
    _useState2 = _slicedToArray(_useState, 2),
    ableToSend = _useState2[0],
    setAbleToSend = _useState2[1];
  var _useState3 = useState(false),
    _useState4 = _slicedToArray(_useState3, 2),
    showWarningEmail = _useState4[0],
    setShowWarningEmail = _useState4[1];
  var _useState5 = useState(false),
    _useState6 = _slicedToArray(_useState5, 2),
    showWarningDSGVO = _useState6[0],
    setShowWarningDSGVO = _useState6[1];
  var _useState7 = useState(false),
    _useState8 = _slicedToArray(_useState7, 2),
    showWarningName = _useState8[0],
    setShowWarningName = _useState8[1];
  var _useState9 = useState(''),
    _useState10 = _slicedToArray(_useState9, 2),
    mail$1 = _useState10[0],
    setMail = _useState10[1];
  var _useState11 = useState(''),
    _useState12 = _slicedToArray(_useState11, 2),
    name = _useState12[0],
    setName = _useState12[1];
  var _useState13 = useState(false),
    _useState14 = _slicedToArray(_useState13, 2),
    checkDSGVO = _useState14[0],
    setCheckDSGVO = _useState14[1];
  var _useState15 = useState(false),
    _useState16 = _slicedToArray(_useState15, 2),
    checkSubscriptionToNewsletter = _useState16[0],
    setCheckSubscriptionToNewsletter = _useState16[1];
  var sendMail = /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      var _viewHandler$actions;
      var returnAfter, result, utm, utmQueryString;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            returnAfter = false;
            if (!name) {
              setAbleToSend(false);
              setShowWarningName(true);
              returnAfter = true;
            }
            if (!DivaUtils.validateEmail(mail$1)) {
              setAbleToSend(false);
              setShowWarningEmail(true);
              returnAfter = true;
            }
            if (showDSGVO && !checkDSGVO) {
              setAbleToSend(false);
              setShowWarningDSGVO(true);
              returnAfter = true;
            }
            if (!returnAfter) {
              _context.next = 6;
              break;
            }
            return _context.abrupt("return");
          case 6:
            LOG$4.businessEvent('onPlannerSaveEMail', 'Planner', "Eine Konfiguration wurde gespeichert und als Mail gesendet", {
              hasAcceptedMarketing: checkSubscriptionToNewsletter
            });
            (_viewHandler$actions = viewHandler.actions) === null || _viewHandler$actions === void 0 || _viewHandler$actions.closeModal();
            //save basket if needed
            _context.next = 11;
            return viewHandler.divaWebPlanner.getCurrentBasket(event, true, returnTo, true);
          case 11:
            result = _context.sent;
            if (result) {
              _context.next = 14;
              break;
            }
            return _context.abrupt("return");
          case 14:
            utm = new UTMUtils(UtmSource.EMAIL, UtmMedium.EMAIL, UtmCampaign.PLANNER_SAVE);
            utmQueryString = utm.getQueryString();
            emailConfig === null || emailConfig === void 0 || emailConfig.map(function (c) {
              return sendEmail({
                showDialog: false,
                content: _objectSpread$4(_objectSpread$4({}, c), {}, {
                  to: mail$1,
                  divaNr: basket.DivaNr,
                  type: 'basket',
                  language: i18n.language,
                  data: {
                    utmQueryString: utmQueryString,
                    websiteBaseUrl: window.location.origin + window.location.pathname
                  }
                })
              });
            });
            analyticsHandler.createLead({
              organizationId: basket.OrganizationId,
              divaNr: basket.DivaNr,
              email: mail$1,
              firstName: name,
              pdfUrl: basket.PDFDocumentURL,
              //divaWebPlannerUrl: webplannerurl.toString(), // Commented out because 'webplannerurl' is currently unused. Uncomment if needed for future functionality.
              marketingAccepted: checkSubscriptionToNewsletter,
              type: 'SAVED_PLANNING'
            });
          case 18:
          case "end":
            return _context.stop();
        }
      }, _callee);
    }));
    return function sendMail() {
      return _ref2.apply(this, arguments);
    };
  }();
  var onChangeMail = function onChangeMail(input) {
    var validMail = DivaUtils.validateEmail(input);
    setAbleToSend(validMail);
    if (validMail) setShowWarningEmail(false);
    setMail(input);
  };
  var onChangeDSGVO = function onChangeDSGVO(val) {
    setCheckDSGVO(val);
    if (val) setShowWarningDSGVO(false);
    if (!showWarningEmail) setAbleToSend(true);
  };
  var onChangeName = function onChangeName(input) {
    setName(input);
    if (input) setShowWarningName(false);
    var basket = viewHandler.divaWebPlanner.getBasket();
    if (!basket.Client) viewHandler.divaWebPlanner.getBasket().Client = {};
    basket.Client.firstName = input;
  };
  return /*#__PURE__*/jsxs("div", {
    className: subWrapper,
    children: [/*#__PURE__*/jsx("div", {
      className: classnames(textWrapper, boldText),
      children: t('checkout.save.advantages')
    }), /*#__PURE__*/jsxs("div", {
      className: textWrapper,
      children: [/*#__PURE__*/jsx("div", {
        className: classnames(icon$1, checkGreen)
      }), t('checkout.save.advantege1')]
    }), /*#__PURE__*/jsxs("div", {
      className: textWrapper,
      children: [/*#__PURE__*/jsx("div", {
        className: classnames(icon$1, checkGreen)
      }), t('checkout.save.advantege2')]
    }), /*#__PURE__*/jsxs("div", {
      className: inputWrapper,
      children: [/*#__PURE__*/jsxs("div", {
        className: classnames(boldText),
        children: [t('checkout.save.email'), "*"]
      }), /*#__PURE__*/jsx("input", {
        className: input,
        type: "email",
        value: mail$1,
        onChange: function onChange(e) {
          return onChangeMail(e.target.value);
        }
      }), showWarningEmail && /*#__PURE__*/jsx("div", {
        className: errorText,
        children: t('checkout.save.mail-error')
      })]
    }), /*#__PURE__*/jsxs("div", {
      className: inputWrapper,
      children: [/*#__PURE__*/jsxs("div", {
        className: classnames(boldText),
        children: [t('checkout.save.firstname'), "*"]
      }), /*#__PURE__*/jsx("input", {
        className: input,
        type: "text",
        value: name,
        onChange: function onChange(e) {
          return onChangeName(e.target.value);
        }
      }), showWarningName && /*#__PURE__*/jsx("div", {
        className: errorText,
        children: t('checkout.save.name_error')
      })]
    }), (!!showSubscription || !!showDSGVO) && /*#__PURE__*/jsxs("div", {
      className: inputWrapper,
      children: [!!showDSGVO && /*#__PURE__*/jsxs("div", {
        className: checkboxWrapper,
        children: [/*#__PURE__*/jsx("input", {
          type: "checkbox",
          className: checkbox,
          defaultChecked: checkDSGVO,
          onChange: function onChange(e) {
            return onChangeDSGVO(e.target.checked);
          }
        }), /*#__PURE__*/jsx("div", {
          className: classnames(text),
          children: /*#__PURE__*/jsx(AcceptTerms, {
            agbUrl: navigationConfiguration.tosUrl,
            privacyUrl: navigationConfiguration.privacyUrl
          })
        })]
      }), !!showSubscription && /*#__PURE__*/jsxs("div", {
        className: checkboxWrapper,
        children: [/*#__PURE__*/jsx("input", {
          type: "checkbox",
          className: checkbox,
          defaultChecked: checkSubscriptionToNewsletter,
          onChange: function onChange(e) {
            return setCheckSubscriptionToNewsletter(e.target.checked);
          }
        }), /*#__PURE__*/jsx("div", {
          className: classnames(text),
          children: t('checkout.appointment.subscription')
        })]
      }), showWarningDSGVO && /*#__PURE__*/jsx("div", {
        className: errorText,
        children: t('checkout.dsgvo_error')
      })]
    }), /*#__PURE__*/jsxs("div", {
      className: classnames(text, requiredFieldsInfo),
      children: ["* ", t('checkout.requiredfieldsinfo')]
    }), /*#__PURE__*/jsxs("button", {
      className: classnames(button, inverted, !ableToSend ? disabled : undefined),
      onClick: sendMail,
      children: [/*#__PURE__*/jsx("div", {
        className: classnames(icon$1, mail)
      }), t('checkout.save.save_now')]
    })]
  });
};

var index$6 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': Save
});

function _createForOfIteratorHelper(r, e) { var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (!t) { if (Array.isArray(r) || (t = _unsupportedIterableToArray(r)) || e && r && "number" == typeof r.length) { t && (r = t); var _n = 0, F = function F() {}; return { s: F, n: function n() { return _n >= r.length ? { done: !0 } : { done: !1, value: r[_n++] }; }, e: function e(r) { throw r; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var o, a = !0, u = !1; return { s: function s() { t = t.call(r); }, n: function n() { var r = t.next(); return a = r.done, r; }, e: function e(r) { u = !0, o = r; }, f: function f() { try { a || null == t["return"] || t["return"](); } finally { if (u) throw o; } } }; }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
function ownKeys$3(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$3(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$3(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$3(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }

/**
 *  Replaces the template strings in the content with the actual basket data
 *
 * @param content the content to parse

 * @param language the language to use
 */
function replaceTemplateStrings (content, data, language) {
  if (!data || !content) return content;
  if (typeof content == 'string') return content;
  var flatBasket = flatten(_objectSpread$3(_objectSpread$3({}, data.organization), data.basket));
  return recursiveReplaceTemplates(content, flatBasket, language);
}

/**
 * Replaces the template string in all fields of the content object with the actual basket data
 *
 * @param content the content to parse
 * @param basket the flattend version of the basket data
 */
function recursiveReplaceTemplates(content, basket, language) {
  for (var _i = 0, _Object$entries = Object.entries(content); _i < _Object$entries.length; _i++) {
    var _Object$entries$_i = _slicedToArray(_Object$entries[_i], 2),
      key = _Object$entries$_i[0],
      value = _Object$entries$_i[1];
    if (typeof content[key] === 'string' && key != 'link') {
      content[key] = replaceTemplateString(value, basket, language);
    } else if (_typeof(content[key]) === 'object') {
      //check if the object is a language dictionary, if so replace it with the value for the current language
      var keys = Object.keys(content[key]);
      if (keys.includes(language) || keys.includes('de')) {
        var _content$key$language;
        content[key] = replaceTemplateString((_content$key$language = content[key][language]) !== null && _content$key$language !== void 0 ? _content$key$language : content[key].de, basket, language);
      } else {
        content[key] = recursiveReplaceTemplates(content[key], basket, language);
      }
    }
  }
  return content;
}

/**
 * replaces the templates in on string with the actual basket data
 *
 * @param template the templatestring to parse
 * @param basket the flattend version of the basket data
 * @param locale the locale to use, if any
 */
function replaceTemplateString(template, basket, locale) {
  var matches = checkForTemplateStrings(template);
  var newTemplate = template;
  var _loop = function _loop() {
    var _basket$locale, _basket, _basket2;
    var match = _matches[_i2];
    //if a locale is provided, check if there is a locale version of the field (fallback is 'de'), if not use the default
    var basketField = match.replace('basket.', '').replace('organization.', '');
    var value;
    if (locale) value = (_basket$locale = (_basket = basket[basketField + '_locales']) === null || _basket === void 0 ? void 0 : _basket[locale]) !== null && _basket$locale !== void 0 ? _basket$locale : (_basket2 = basket[basketField + '_locales']) === null || _basket2 === void 0 ? void 0 : _basket2.de;
    if (!value) value = basket[basketField];
    if (!value) {
      //check if the match is a template string that should be resolved using resolveWindowVariable
      value = resolveWindowVariable(match);
    }
    if (!value) {
      //check if the match is a template string that should be resolved using templateMapping
      var mapping = templateMapping.find(function (m) {
        return m.key === '${' + match + '}';
      });
      if (mapping) {
        value = mapping.value;
      }
    }
    if (value != undefined) newTemplate = newTemplate.replace('${' + match + '}', value);
  };
  for (var _i2 = 0, _matches = matches; _i2 < _matches.length; _i2++) {
    _loop();
  }
  if (newTemplate != template) {
    //iterate recursively until template string does not change anymore
    return replaceTemplateString(newTemplate, basket, locale);
  }
  return newTemplate;
}

/**
 *  Extracts all substrings that are inside ${...}
 *
 * @param input the template to parse
 * @returns a list of values extracted form the template
 */
function checkForTemplateStrings(input) {
  // Regular expression to match ${...}
  var regex = /\$\{([^}]+)\}/g;
  var matches = [];
  var match;

  // Find all matches
  while ((match = regex.exec(input)) !== null) {
    // Extract the content inside ${...}
    matches.push(match[1]);
  }
  return matches;
}

// Function to resolve window variables
function resolveWindowVariable(match) {
  if (match.startsWith('window.')) {
    var windowVariablePath = match.replace('window.', '');
    var value = window;
    var pathParts = windowVariablePath.split('.');
    var _iterator = _createForOfIteratorHelper(pathParts),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var part = _step.value;
        if (value[part] !== undefined) {
          value = value[part];
        } else {
          return undefined;
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
    return value;
  }
  return undefined;
}
var templateMapping = [{
  key: '${data.websiteBaseUrl}',
  value: window.location.origin + window.location.pathname
}];

var Header = observer(function (_ref) {
  var _config$buttons$heade, _config$buttons;
  var index = _ref.index,
    viewHandler = _ref.viewHandler,
    goBack = _ref.goBack;
  var _useTranslation = useTranslation(),
    t = _useTranslation.t;
  var config = viewHandler.uiState.checkoutConfig;
  if (!config) return /*#__PURE__*/jsx(Fragment, {});
  var title = (_config$buttons$heade = (_config$buttons = config.buttons[index !== null && index !== void 0 ? index : -1]) === null || _config$buttons === void 0 ? void 0 : _config$buttons.headerTitle) !== null && _config$buttons$heade !== void 0 ? _config$buttons$heade : 'checkout.header_title_default';
  return /*#__PURE__*/jsxs("div", {
    className: headerWrapper,
    children: [index !== undefined && config.buttons.find(function (c) {
      return c.visibleInOverview !== false;
    }) && /*#__PURE__*/jsx("button", {
      className: classnames(button, iconButton),
      onClick: function onClick() {
        return goBack();
      },
      children: /*#__PURE__*/jsx("div", {
        className: classnames(icon$1, big, back)
      })
    }), /*#__PURE__*/jsx("div", {
      children: t(title)
    })]
  });
});

var index$5 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': Header
});

var Icon = function Icon(_ref) {
  var _ref$checkoutButton = _ref.checkoutButton,
    iconType = _ref$checkoutButton.icon,
    iconUrl = _ref$checkoutButton.iconUrl,
    isPrimary = _ref$checkoutButton.isPrimary,
    headerFunctionName = _ref$checkoutButton.headerFunctionName,
    size = _ref.size,
    showTitle = _ref.showTitle;
  var _useTranslation = useTranslation(),
    t = _useTranslation.t;
  return /*#__PURE__*/jsxs(Fragment, {
    children: [/*#__PURE__*/jsx("div", {
      className: classnames(icon$1, iconType ? getIconType(iconType) : undefined, isPrimary ? inverted : undefined),
      style: {
        backgroundImage: iconUrl ? 'url(' + iconUrl + ')' : undefined,
        width: size,
        height: size
      }
    }), !!showTitle && headerFunctionName !== undefined && /*#__PURE__*/jsx("div", {
      children: t(headerFunctionName)
    })]
  });
};
var getIconType = function getIconType(icon) {
  switch (icon) {
    case 'SHARE':
      return share;
    case 'CART':
      return cart;
    case 'LEAFTLET':
      return createLeaflet;
    case 'SAVE_MAIL':
      return mail;
    case 'APPOINTMENT':
      return calendar;
    case 'P_SHARE':
      return pShare;
    case 'P_SAVE':
      return pSave;
    case 'PRINT':
      return print;
    case 'HELP':
      return help;
    case 'L_CALENDAR':
      return lCalendar;
    case 'DO_RESET':
      return reset;
    case 'HEART':
      return heart;
    case 'BOOKMARK':
      return bookmark;
    case 'CLOSE':
      return close;
    default:
      return undefined;
  }
};

var index$4 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': Icon
});

function ownKeys$2(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$2(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$2(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$2(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$3 = getLogger('Planner', 'WEB-PLANNER-REACT');
var Checkout = observer(function (_ref) {
  var _basket;
  var inModal = _ref.inModal,
    index = _ref.index,
    viewHandler = _ref.viewHandler,
    basket = _ref.basket;
  var _useDivaCore = useDivaCore(),
    organization = _useDivaCore.state.organization;
  var _useTranslation = useTranslation(),
    t = _useTranslation.t,
    i18n = _useTranslation.i18n;
  if (!basket && viewHandler.divaWebPlanner.getBasket()) {
    basket = viewHandler.divaWebPlanner.getBasket();
  }
  var c = viewHandler.uiState.checkoutConfig;
  var _useState = useState(basket ? replaceTemplateStrings(c, {
      basket: (_basket = basket) === null || _basket === void 0 ? void 0 : _basket.getAsJson(),
      organization: organization
    }, i18n.language) : undefined),
    _useState2 = _slicedToArray(_useState, 2),
    config = _useState2[0],
    setConfig = _useState2[1];
  useEffect(function () {
    setConfig(replaceTemplateStrings(c, {
      organization: organization,
      basket: basket
    }, i18n.language));
  }, [basket]);
  var url = config === null || config === void 0 ? void 0 : config.url;
  var currentConfig = config === null || config === void 0 ? void 0 : config.buttons[index !== null && index !== void 0 ? index : -1];
  if (currentConfig && basket) switch (currentConfig.type) {
    case 'APPOINTMENT':
      return /*#__PURE__*/jsx(CreateAppointment, {
        config: currentConfig,
        basket: basket,
        url: url
      });
    case 'SHARE':
      return /*#__PURE__*/jsx(Share, {
        config: currentConfig,
        basket: basket,
        url: url
      });
    case 'SAVE':
      return /*#__PURE__*/jsx(Save, {
        config: currentConfig,
        basket: basket,
        url: url,
        viewHandler: viewHandler
      });
  }
  if (!c) return /*#__PURE__*/jsx(Fragment, {});
  return /*#__PURE__*/jsx("div", {
    className: classnames(wrapper$2, inModal ? inModalClass : undefined),
    children: c.buttons.map(function (c, index) {
      if (c.visibleInOverview === false) return /*#__PURE__*/jsx(Fragment, {});
      return /*#__PURE__*/jsxs("button", {
        className: classnames(button, c.isPrimary ? inverted : undefined),
        onClick: function onClick() {
          return _onClickCheckout(c, viewHandler, index, t);
        },
        children: [/*#__PURE__*/jsx(Icon, {
          checkoutButton: c
        }), c.label && /*#__PURE__*/jsx("div", {
          children: t(c.label)
        })]
      }, index);
    })
  });
});
var _onClickCheckout = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee(buttonConfig, viewHandler, index, t, dontCheckForModal) {
    var _viewHandler$actions2, _viewHandler$actions5;
    var returnTo, _viewHandler$actions, _buttonConfig$openMod, _viewHandler$actions3, _buttonConfig$event, result, savedBasket, _viewHandler$actions4;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          returnTo = buttonConfig.returnTo; //open modal first to ask if sure
          if (!(buttonConfig.openModal && !dontCheckForModal)) {
            _context.next = 4;
            break;
          }
          (_viewHandler$actions = viewHandler.actions) === null || _viewHandler$actions === void 0 || _viewHandler$actions.openModal(_objectSpread$2(_objectSpread$2({}, buttonConfig.openModal), {}, {
            text: t((_buttonConfig$openMod = buttonConfig.openModal.text) !== null && _buttonConfig$openMod !== void 0 ? _buttonConfig$openMod : ''),
            title: t(buttonConfig.openModal.title),
            callback: function callback(accept) {
              if (accept) _onClickCheckout(buttonConfig, viewHandler, index, t, true);
            }
          }));
          return _context.abrupt("return");
        case 4:
          LOG$3.businessEvent('onPlannerCheckoutAction', 'Planner', "Eine Checkout Action wurde geklickt", {
            checkoutType: buttonConfig.type,
            checkoutEvent: buttonConfig.event
          });
          (_viewHandler$actions2 = viewHandler.actions) === null || _viewHandler$actions2 === void 0 || _viewHandler$actions2.closeModal();

          //check if it is reset usecase
          if (!(buttonConfig.event == 'RESET')) {
            _context.next = 9;
            break;
          }
          viewHandler.productState.onClickReset();
          return _context.abrupt("return");
        case 9:
          if (!(buttonConfig.type == 'EVENT' && buttonConfig.onlyCallEvent)) {
            _context.next = 12;
            break;
          }
          (_viewHandler$actions3 = viewHandler.actions) === null || _viewHandler$actions3 === void 0 || _viewHandler$actions3.onEvent({
            type: (_buttonConfig$event = buttonConfig.event) !== null && _buttonConfig$event !== void 0 ? _buttonConfig$event : ''
          });
          return _context.abrupt("return");
        case 12:
          _context.next = 14;
          return viewHandler.divaWebPlanner.getCurrentBasket(buttonConfig.event, true, returnTo, true);
        case 14:
          result = _context.sent;
          if (result) {
            _context.next = 17;
            break;
          }
          return _context.abrupt("return");
        case 17:
          savedBasket = viewHandler.divaWebPlanner.getBasket(); //open PDF
          if (!(buttonConfig.returnTo == 'OPEN_PDF')) {
            _context.next = 21;
            break;
          }
          (_viewHandler$actions4 = viewHandler.actions) === null || _viewHandler$actions4 === void 0 || _viewHandler$actions4.openModal({
            text: t('modal.open_leaflet_pdf_now'),
            accept: t('modals.open_now'),
            callback: function callback() {
              window.open(savedBasket.PDFDocumentURL, '_blank');
            }
          });
          return _context.abrupt("return");
        case 21:
          if (!(buttonConfig.type == 'EVENT')) {
            _context.next = 23;
            break;
          }
          return _context.abrupt("return");
        case 23:
          (_viewHandler$actions5 = viewHandler.actions) === null || _viewHandler$actions5 === void 0 || _viewHandler$actions5.openModal({
            accept: false,
            rejectByIcon: true,
            title: /*#__PURE__*/jsx(Header, {
              viewHandler: viewHandler,
              index: index,
              goBack: function goBack() {
                return _goBack2(viewHandler, savedBasket);
              }
            }),
            content: /*#__PURE__*/jsx(Checkout, {
              index: index,
              basket: savedBasket,
              viewHandler: viewHandler,
              inModal: true
            }),
            dontRenderForm: true
          });
        case 24:
        case "end":
          return _context.stop();
      }
    }, _callee);
  }));
  return function onClickCheckout(_x, _x2, _x3, _x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();
var _goBack2 = function _goBack(viewHandler, basket) {
  var _viewHandler$actions6;
  (_viewHandler$actions6 = viewHandler.actions) === null || _viewHandler$actions6 === void 0 || _viewHandler$actions6.openModal({
    accept: false,
    rejectByIcon: true,
    title: /*#__PURE__*/jsx(Header, {
      viewHandler: viewHandler,
      index: undefined,
      goBack: function goBack() {
        return _goBack2(viewHandler, basket);
      }
    }),
    content: /*#__PURE__*/jsx(Checkout, {
      inModal: true,
      basket: basket,
      viewHandler: viewHandler
    }),
    dontRenderForm: true
  });
};

var index$3 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    onClickCheckout: _onClickCheckout,
    goBack: _goBack2,
    'default': Checkout
});

getLogger('Planner', 'ActionButtons');
var ActionButtonState = /*#__PURE__*/function () {
  function ActionButtonState(uiState, settingsState, configuratorStructureHandler, getState, updateConfiguratorState, updatePlannerMsgContainer, openModal) {
    var _this = this;
    _classCallCheck(this, ActionButtonState);
    _defineProperty(this, "divaWebPlanner", void 0);
    _defineProperty(this, "uiState", void 0);
    _defineProperty(this, "settingsState", void 0);
    _defineProperty(this, "configuratorStructureHandler", void 0);
    _defineProperty(this, "paletteViewHandler", void 0);
    _defineProperty(this, "getState", void 0);
    _defineProperty(this, "updateConfiguratorState", void 0);
    _defineProperty(this, "updatePlannerMsgContainer", void 0);
    _defineProperty(this, "openModal", void 0);
    //NavigationButtons
    _defineProperty(this, "isOnFirstStep", true);
    // weiter button hat 100% breite und zurückbutton wird ausgeblendet
    _defineProperty(this, "isOnLastStep", true);
    // zurück button hat 100% breite und weiterbutton wird ausgeblendet / im Mobil wird warenkorbbutton angezeigt
    _defineProperty(this, "isOnFirstStepDisabled", true);
    //in den Warenkorb
    _defineProperty(this, "checkoutButtonIsDisabled", false);
    _defineProperty(this, "onClickAddToBasket", /*#__PURE__*/function () {
      var _ref = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee(viewHandler, event, force, returnTo, dontGoToSummary) {
        var _this$divaWebPlanner$, _viewHandler$uiState$;
        var warning, _this$settingsState$i, _this$openModal, _this$uiState$i18nIns, confitStepWithWaring, warningText, variantsCount, _this$uiState$i18nIns2, _this$divaWebPlanner$2, _this$uiState$i18nIns3, articleAreDefined;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return _this.uiState.setActiveCatalogSection('3D/2D');
            case 2:
              warning = _this.configuratorStructureHandler.checkIfAnyConfigStepHasWarning();
              if (!(warning && !force)) {
                _context.next = 8;
                break;
              }
              confitStepWithWaring = warning.configStep;
              warningText = (_this$settingsState$i = _this.settingsState.itemPaletteArr.find(function (item) {
                return item[1].configStepKey === confitStepWithWaring.key;
              })) === null || _this$settingsState$i === void 0 || (_this$settingsState$i = _this$settingsState$i[1].info) === null || _this$settingsState$i === void 0 ? void 0 : _this$settingsState$i.text;
              (_this$openModal = _this.openModal) === null || _this$openModal === void 0 || _this$openModal.call(_this, {
                title: (_this$uiState$i18nIns = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns === void 0 ? void 0 : _this$uiState$i18nIns.t('modals.warning_at_option', {
                  option: warning.option.title
                }),
                text: warningText,
                accept: true,
                callback: function callback() {
                  _this.configuratorStructureHandler.navigateToConfigStepByConfigStepKey(confitStepWithWaring.key);
                  _this.updateConfiguratorState(true);
                }
              });
              return _context.abrupt("return", false);
            case 8:
              if (!((_this$divaWebPlanner$ = _this.divaWebPlanner.ReturnTo) !== null && _this$divaWebPlanner$ !== void 0 && _this$divaWebPlanner$.includes('FAVORITS') && _this.divaWebPlanner.favoritesDefined())) {
                _context.next = 15;
                break;
              }
              _context.next = 11;
              return _this.divaWebPlanner.getValidVariantsLength();
            case 11:
              variantsCount = _context.sent;
              if (!(variantsCount > _this.maxVariantCount)) {
                _context.next = 15;
                break;
              }
              _this.updatePlannerMsgContainer('CHECK_FAVORITES_TOGGLES', true, 'WARNING', (_this$uiState$i18nIns2 = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns2 === void 0 ? void 0 : _this$uiState$i18nIns2.t('configurator.to_many_variants', {
                max: _this.maxVariantCount
              }), true, 6000);
              return _context.abrupt("return", false);
            case 15:
              if (!(!dontGoToSummary && _this.getState() != 'SUMMARY' && _this.configuratorStructureHandler.allTabGroups['Summary'])) {
                _context.next = 21;
                break;
              }
              _this.configuratorStructureHandler.navigateToTabgroupByType('SUMMARY');
              _this.updateConfiguratorState(true);
              if (!((_this$divaWebPlanner$2 = _this.divaWebPlanner.ReturnTo) !== null && _this$divaWebPlanner$2 !== void 0 && _this$divaWebPlanner$2.includes('FAVORITS') && _this.divaWebPlanner.favoritesDefined())) {
                _context.next = 21;
                break;
              }
              //for favorites when navigation to summary dont save imediatly;
              _this.updatePlannerMsgContainer('CHECK_FAVORITES_TOGGLES', true, 'INFO', (_this$uiState$i18nIns3 = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns3 === void 0 ? void 0 : _this$uiState$i18nIns3.t('configurator.check_favorites_toggle'), true);
              return _context.abrupt("return", false);
            case 21:
              if (!(viewHandler && (_viewHandler$uiState$ = viewHandler.uiState.checkoutConfig) !== null && _viewHandler$uiState$ !== void 0 && _viewHandler$uiState$.openOnAddToBasket)) {
                _context.next = 24;
                break;
              }
              _goBack2(viewHandler);
              return _context.abrupt("return", false);
            case 24:
              articleAreDefined = _this.divaWebPlanner.checkIfAllOptionsAreDefined();
              if (!(articleAreDefined !== true && !force)) {
                _context.next = 29;
                break;
              }
              _context.next = 28;
              return _this.askToDefineAllOptions(articleAreDefined);
            case 28:
              return _context.abrupt("return", false);
            case 29:
              _context.next = 31;
              return _this.saveToBasket(event, returnTo);
            case 31:
              return _context.abrupt("return", _context.sent);
            case 32:
            case "end":
              return _context.stop();
          }
        }, _callee);
      }));
      return function (_x, _x2, _x3, _x4, _x5) {
        return _ref.apply(this, arguments);
      };
    }());
    _defineProperty(this, "onClickConfiguratorSettingsBack", function () {
      // zurück Klicken
      _this.configuratorStructureHandler.navigateBackwards();
      _this.updateConfiguratorState(true);
    });
    _defineProperty(this, "onClickConfiguratorSettingsForth", function () {
      var _this$configuratorStr;
      // weiter Klicken
      //falls währen der konfiguration nicht alle Configuratorsteps besucht wurden, wird zu denen gescrollt und noch nicht weiter gesprungen
      if (!((_this$configuratorStr = _this.configuratorStructureHandler) !== null && _this$configuratorStr !== void 0 && _this$configuratorStr.CurConfigStep)) return;
      var jumpToNext = true;
      _this.paletteViewHandler.visitActVisibleSteps();
      if (_this.getState() == 'OPTION') {
        if (!_this.paletteViewHandler.wereAllStepsVisited()) {
          // es wurde noch nicht alles besucht und wir scrollen zur ItemPalette -> jumpToNext
          jumpToNext = false;
          var index = _this.paletteViewHandler.getFirstUnvisitedStepIndex();
          _this.paletteViewHandler.scrollToConfigStep(index + 1);
        }
      }
      if (jumpToNext == true) {
        if (_this.configuratorStructureHandler.CurConfigStep.warning) {
          var _this$openModal2, _this$uiState$i18nIns4;
          (_this$openModal2 = _this.openModal) === null || _this$openModal2 === void 0 || _this$openModal2.call(_this, {
            title: (_this$uiState$i18nIns4 = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns4 === void 0 ? void 0 : _this$uiState$i18nIns4.t('modals.warning'),
            text: _this.configuratorStructureHandler.CurConfigStep.warning,
            accept: true
          });
        } else {
          _this.configuratorStructureHandler.navigateForwards();
          _this.paletteViewHandler.scrollToTopInItemPalette();
          _this.updateConfiguratorState(true);
        }
      }
    });
    makeObservable(this, {
      isOnFirstStep: observable,
      isOnLastStep: observable,
      isOnFirstStepDisabled: observable,
      initForUpdate: action,
      checkoutButtonIsDisabled: observable,
      onClickAddToBasket: action,
      saveToBasket: action
    });
    this.uiState = uiState;
    this.settingsState = settingsState;
    this.configuratorStructureHandler = configuratorStructureHandler;
    this.getState = getState;
    this.updateConfiguratorState = updateConfiguratorState;
    this.updatePlannerMsgContainer = updatePlannerMsgContainer;
    this.openModal = openModal;
  }
  return _createClass(ActionButtonState, [{
    key: "initFunctions",
    value: function initFunctions(divaWebPlanner, paletteViewHandler) {
      this.divaWebPlanner = divaWebPlanner;
      this.paletteViewHandler = paletteViewHandler;
    }
  }, {
    key: "initForUpdate",
    value: function initForUpdate() {
      this.isOnFirstStep = this.configuratorStructureHandler.isFirstStep; //nur "weiter Button" wird angeziegt da Zurück nicht geht
      this.isOnLastStep = this.configuratorStructureHandler.isLastStep && !this.isOnFirstStep;
      this.isOnFirstStepDisabled = this.configuratorStructureHandler.isLastStep && this.isOnFirstStep && this.configuratorStructureHandler.getTabgroupByKey('SUMMARY') == undefined;
    }
  }, {
    key: "askToDefineAllOptions",
    value: function () {
      var _askToDefineAllOptions = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee2(_ref2) {
        var _this2 = this;
        var articles, options, callBack, text, i, _this$uiState$i18nIns5, noDuplicates, _loop, _i, _this$uiState$i18nIns6;
        return _regeneratorRuntime.wrap(function _callee2$(_context3) {
          while (1) switch (_context3.prev = _context3.next) {
            case 0:
              articles = _ref2.articles, options = _ref2.options;
              callBack = function callBack(accepted) {
                if (accepted && !options.length) {
                  _this2.divaWebPlanner.defineArticlesOnOptions('defineAll');
                  _this2.onClickAddToBasket();
                } else if (options.length) {
                  _this2.configuratorStructureHandler.navigateToConfigStepByConfigStepKey(options[0].configStepKey);
                  _this2.updateConfiguratorState(true);
                  _this2.divaWebPlanner.updateConfiguratorStructureOfActSet();
                }
              };
              text = '';
              if (!options.length) {
                _context3.next = 7;
                break;
              }
              for (i = 0; i < options.length; i++) {
                text += ((_this$uiState$i18nIns5 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns5 === void 0 ? void 0 : _this$uiState$i18nIns5.t('configurator.NOT_YET_CONFIRMED_OPTION', {
                  option: options[i].title
                })) + '\n';
              }
              _context3.next = 15;
              break;
            case 7:
              noDuplicates = _toConsumableArray(new Set(articles));
              _loop = /*#__PURE__*/_regeneratorRuntime.mark(function _loop(_i) {
                var _this2$uiState$i18nIn;
                return _regeneratorRuntime.wrap(function _loop$(_context2) {
                  while (1) switch (_context2.prev = _context2.next) {
                    case 0:
                      text += noDuplicates[_i] + ' ' + ((_this2$uiState$i18nIn = _this2.uiState.i18nInstance) === null || _this2$uiState$i18nIn === void 0 ? void 0 : _this2$uiState$i18nIn.t('configurator.NOT_YET_CONFIRMED')) + (articles.filter(function (a) {
                        return noDuplicates[_i];
                      }).length > 1 ? ' x' + articles.filter(function (a) {
                        return noDuplicates[_i];
                      }).length : '');
                    case 1:
                    case "end":
                      return _context2.stop();
                  }
                }, _loop);
              });
              _i = 0;
            case 10:
              if (!(_i < noDuplicates.length)) {
                _context3.next = 15;
                break;
              }
              return _context3.delegateYield(_loop(_i), "t0", 12);
            case 12:
              _i++;
              _context3.next = 10;
              break;
            case 15:
              if (this.openModal) {
                this.openModal({
                  reject: !options.length,
                  accept: true,
                  title: (_this$uiState$i18nIns6 = this.uiState.i18nInstance) === null || _this$uiState$i18nIns6 === void 0 ? void 0 : _this$uiState$i18nIns6.t('configurator.NOT_DEFINED_OPTIONS_TITLE'),
                  text: text,
                  callback: callBack
                });
              } else {
                callBack(true);
              }
            case 16:
            case "end":
              return _context3.stop();
          }
        }, _callee2, this);
      }));
      function askToDefineAllOptions(_x6) {
        return _askToDefineAllOptions.apply(this, arguments);
      }
      return askToDefineAllOptions;
    }()
    /**
     * This function is called when we save the act Planning to basket
     */
  }, {
    key: "saveToBasket",
    value: (function () {
      var _saveToBasket = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee3(event, returnTo) {
        var result;
        return _regeneratorRuntime.wrap(function _callee3$(_context4) {
          while (1) switch (_context4.prev = _context4.next) {
            case 0:
              if (this.checkoutButtonIsDisabled) {
                _context4.next = 7;
                break;
              }
              this.checkoutButtonIsDisabled = true;
              _context4.next = 4;
              return this.divaWebPlanner.saveToBasketWrapper(event, undefined, returnTo);
            case 4:
              result = _context4.sent;
              this.checkoutButtonIsDisabled = false;
              return _context4.abrupt("return", result == 'canceled' ? false : true);
            case 7:
              return _context4.abrupt("return", false);
            case 8:
            case "end":
              return _context4.stop();
          }
        }, _callee3, this);
      }));
      function saveToBasket(_x7, _x8) {
        return _saveToBasket.apply(this, arguments);
      }
      return saveToBasket;
    }())
  }, {
    key: "maxVariantCount",
    get: function get() {
      var _this$uiState$ViewCon;
      return (_this$uiState$ViewCon = this.uiState.ViewConfig.maxVariantCount) !== null && _this$uiState$ViewCon !== void 0 ? _this$uiState$ViewCon : 1000;
    }
  }]);
}();

var container = "container-IPJX7";
var priceInfoClass = "price-info-class-SuRhS";
var priceClass = "price-class-u61JZ";
var css_248z$2 = ".container-IPJX7 {\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  height: 100%;\n}\n\n.price-info-class-SuRhS {\n  font-weight: 100;\n  font-size: 12px;\n}\n\n.price-class-u61JZ {\n  text-transform: uppercase;\n  font-weight: 300;\n  font-size: 16px;\n  letter-spacing: 0.1rem;\n}\n";
styleInject(css_248z$2);

var FrameworkPrice = function FrameworkPrice(priceInfo, price) {
  var _useDivaCore = useDivaCore(),
    hidePrices = _useDivaCore.state.hidePrices;
  if (hidePrices) return /*#__PURE__*/jsx(Fragment, {});
  return /*#__PURE__*/jsxs("div", {
    className: container,
    children: [/*#__PURE__*/jsx("span", {
      className: priceInfoClass,
      children: priceInfo
    }), /*#__PURE__*/jsx("span", {
      className: priceClass,
      children: price
    })]
  });
};

var index$2 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': FrameworkPrice
});

var wcFullScreenImage = "wcFullScreenImage-k7rmK";
var css_248z$1 = ".wcFullScreenImage-k7rmK {\n  width: 100%;\n  height: 100%;\n  background-size: contain;\n  background-repeat: no-repeat;\n  background-position: center center;\n}\n";
styleInject(css_248z$1);

var Fullscreen = function Fullscreen(_ref) {
  var image = _ref.image;
  return /*#__PURE__*/jsx("div", {
    className: wcFullScreenImage,
    style: {
      backgroundImage: image
    }
  });
};

var index$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': Fullscreen
});

var affiliateContainer = "affiliate-container-bOZvB";
var affiliateMain = "affiliate-main-ipOJf";
var affiliateMainMobile = "affiliate-main-mobile-VQW52";
var affiliateProduct = "affiliate-product-ich3t";
var affiliateProductDescription = "affiliate-product-description-ujnXk";
var affiliatePartnerList = "affiliate-partner-list-wXqwE";
var affiliatePartner = "affiliate-partner-Z5vLB";
var affiliatePartnerImage = "affiliate-partner-image-WSYog";
var affiliateParnterButton = "affiliate-parnter-button-myx7O";
var css_248z = ".affiliate-container-bOZvB {\n  max-width: 750px;\n}\n\n.affiliate-container-bOZvB > h2 {\n  margin-top: 0;\n}\n\n.affiliate-main-ipOJf {\n  display: flex;\n  margin-top: 20px;\n  flex-wrap: wrap;\n  gap: 50px;\n}\n.affiliate-main-mobile-VQW52 {\n  display: flex;\n  flex-direction: column;\n  flex-wrap: wrap;\n  margin-top: 10px;\n  gap: 10px;\n}\n\n.affiliate-product-ich3t {\n  display: flex;\n  flex-direction: column;\n}\n\n.affiliate-product-ich3t > img {\n  width: 276px;\n  max-width: 100%;\n}\n\n.affiliate-product-description-ujnXk {\n  line-height: 30px;\n  margin-top: 10px;\n  flex: 1 1 20px;\n}\n\n.affiliate-partner-list-wXqwE {\n  flex: 1 1 200px;\n  display: flex;\n  flex-wrap: wrap;\n  flex-direction: column;\n  gap: 20px;\n}\n\n.affiliate-partner-Z5vLB {\n  display: flex;\n  justify-content: space-between;\n  flex: 1 1 100px;\n  flex-wrap: wrap;\n  align-items: center;\n  justify-content: center;\n  gap: 20px;\n  position: relative;\n}\n\n.affiliate-partner-image-WSYog {\n  cursor: pointer;\n  flex: 1 1 120px;\n  background-color: #fff;\n  background-repeat: no-repeat;\n  background-position: center;\n  width: 204px;\n  max-width: 100%;\n  height: 98px;\n  border: 1px solid #aaa;\n}\n\n.affiliate-parnter-button-myx7O {\n  cursor: pointer;\n  height: 35px;\n  width: 100px;\n  background: var(--secondaryColor);\n  border-radius: 3px;\n  color: #fff;\n  text-align: center;\n  line-height: 35px;\n  flex: 1 1 25px;\n  padding: 0 5px;\n}\n";
styleInject(css_248z);

getLogger('Planner', 'Affiliate');
function Affiliate(_ref) {
  var _product$affiliate;
  var product = _ref.product;
  var _useTranslation = useTranslation(),
    t = _useTranslation.t;
  var _useDivaCore = useDivaCore(),
    isMobile = _useDivaCore.state.dimensions.isMobile;
  return /*#__PURE__*/jsxs("div", {
    className: affiliateContainer,
    children: [/*#__PURE__*/jsx("h2", {
      children: t('affiliatemodal.title')
    }), /*#__PURE__*/jsx("div", {
      children: t('affiliatemodal.text')
    }), /*#__PURE__*/jsxs("div", {
      className: isMobile ? affiliateMainMobile : affiliateMain,
      children: [/*#__PURE__*/jsxs("div", {
        className: affiliateProduct,
        children: [!isMobile && /*#__PURE__*/jsx("img", {
          src: product.image
        }), /*#__PURE__*/jsx("div", {
          className: affiliateProductDescription,
          children: product.shortDescription
        })]
      }), /*#__PURE__*/jsx("div", {
        className: affiliatePartnerList,
        children: (_product$affiliate = product.affiliate) === null || _product$affiliate === void 0 ? void 0 : _product$affiliate.map(function (partner) {
          return /*#__PURE__*/jsxs("div", {
            className: affiliatePartner,
            children: [/*#__PURE__*/jsx("button", {
              "data-test-id": "affiliate-button",
              className: affiliatePartnerImage,
              style: {
                backgroundImage: 'url(' + partner.logoUrl + ')'
              },
              onClick: function onClick() {
                window.open(partner.link, '_blank');
              }
            }), /*#__PURE__*/jsx("div", {
              className: affiliateParnterButton,
              onClick: function onClick() {
                window.open(partner.link, '_blank');
              },
              children: t('affiliatemodal.button')
            })]
          }, partner.id);
        })
      })]
    })]
  });
}

var index = /*#__PURE__*/Object.freeze({
    __proto__: null,
    'default': Affiliate
});

var LOG$2 = getLogger('Planner', 'BarrierHandler');
var BarrierHandler = /*#__PURE__*/function () {
  function BarrierHandler(uiState, actions) {
    _classCallCheck(this, BarrierHandler);
    this.uiState = uiState;
    this.actions = actions;
  }
  return _createClass(BarrierHandler, [{
    key: "openLeafeltBarrier",
    value: function openLeafeltBarrier(divaBasketHandler, config, product) {
      var _this$uiState$i18nIns,
        _this = this;
      var barrierText = (_this$uiState$i18nIns = this.uiState.i18nInstance) === null || _this$uiState$i18nIns === void 0 ? void 0 : _this$uiState$i18nIns.t(config !== null && config !== void 0 && config.title ? config === null || config === void 0 ? void 0 : config.title : config !== null && config !== void 0 && config.createOrder ? 'barrier.order.text' : 'barrier.leaflet.text');
      if (this.uiState.barrierTextClass) {
        barrierText = /*#__PURE__*/jsx("div", {
          className: this.uiState.barrierTextClass,
          children: barrierText
        });
      }
      return new Promise(function (resolve, reject) {
        var _this$actions, _config$additional, _this$actions3;
        if (!((_this$actions = _this.actions) !== null && _this$actions !== void 0 && _this$actions.openBarrier)) {
          reject('openBarrier is undefined.');
        }
        var actions = [];
        var addCreateBarrierAction = function addCreateBarrierAction() {
          if (!config || config !== null && config !== void 0 && config.create) {
            var _this$uiState$barrier, _this$uiState$i18nIns2;
            actions.push({
              loginType: _this.getLoginType(config, config === null || config === void 0 ? void 0 : config.create),
              className: (_this$uiState$barrier = _this.uiState.barrierActions) === null || _this$uiState$barrier === void 0 || (_this$uiState$barrier = _this$uiState$barrier[actions.length]) === null || _this$uiState$barrier === void 0 ? void 0 : _this$uiState$barrier.className,
              text: (_this$uiState$i18nIns2 = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns2 === void 0 ? void 0 : _this$uiState$i18nIns2.t('barrier.leaflet.button.create'),
              onClick: function onClick(email) {
                divaBasketHandler.actBasket.Status = 'LEAFLET';
                _this.setReportConfig(divaBasketHandler, config === null || config === void 0 ? void 0 : config.create);
                if (email) {
                  _this.setEmailConfig(divaBasketHandler, email);
                  resolve({
                    event: 'onSendEmail',
                    saveBasket: true
                  });
                } else {
                  resolve({
                    event: 'onAddToBasket',
                    saveBasket: true
                  });
                }
              }
            });
          }
        };
        if (!_this.uiState.barrierCreateActionOrder) addCreateBarrierAction();
        if (config !== null && config !== void 0 && config.createOrder) {
          var _this$uiState$barrier2, _this$uiState$i18nIns3;
          actions.push({
            loginType: _this.getLoginType(config, config === null || config === void 0 ? void 0 : config.createOrder),
            className: (_this$uiState$barrier2 = _this.uiState.barrierActions) === null || _this$uiState$barrier2 === void 0 || (_this$uiState$barrier2 = _this$uiState$barrier2[actions.length]) === null || _this$uiState$barrier2 === void 0 ? void 0 : _this$uiState$barrier2.className,
            text: (_this$uiState$i18nIns3 = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns3 === void 0 ? void 0 : _this$uiState$i18nIns3.t('barrier.leaflet.button.createOrder'),
            onClick: function onClick(email) {
              if (_this.getLoginType(config, config === null || config === void 0 ? void 0 : config.createOrder)) {
                divaBasketHandler.actBasket.Status = 'ORDER';
                _this.setReportConfig(divaBasketHandler, config === null || config === void 0 ? void 0 : config.createOrder);
                _this.setEmailConfig(divaBasketHandler, email);
                resolve({
                  event: 'onSendEmail',
                  saveBasket: true
                });
              }
            }
          });
        }
        if (!config || config.createAndContact) {
          var _this$uiState$i18nIns4;
          actions.push({
            loginType: _this.getLoginType(config, config === null || config === void 0 ? void 0 : config.createAndContact),
            text: (_this$uiState$i18nIns4 = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns4 === void 0 ? void 0 : _this$uiState$i18nIns4.t('barrier.leaflet.button.createandcontact'),
            onClick: function onClick(email) {
              divaBasketHandler.actBasket.Status = 'ORDER';
              _this.setReportConfig(divaBasketHandler, config === null || config === void 0 ? void 0 : config.createAndContact);
              if (_this.getLoginType(config, config === null || config === void 0 ? void 0 : config.createAndContact)) {
                _this.setEmailConfig(divaBasketHandler, email + ',ORGANIZATION');
                resolve({
                  event: 'onSendEmail',
                  saveBasket: true
                });
              } else {
                resolve({
                  event: 'onAddToBasket',
                  saveBasket: true
                });
              }
            }
          });
        }
        if (config !== null && config !== void 0 && config.affiliate && product) {
          var _this$uiState$barrier3, _this$uiState$i18nIns5;
          actions.push({
            loginType: 'NO_LOGIN',
            className: (_this$uiState$barrier3 = _this.uiState.barrierActions) === null || _this$uiState$barrier3 === void 0 || (_this$uiState$barrier3 = _this$uiState$barrier3[actions.length]) === null || _this$uiState$barrier3 === void 0 ? void 0 : _this$uiState$barrier3.className,
            text: (_this$uiState$i18nIns5 = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns5 === void 0 ? void 0 : _this$uiState$i18nIns5.t('barrier.leaflet.button.buyonline'),
            onClick: function onClick(email) {
              //set the email as client email to basket
              if (divaBasketHandler.actBasket.Client) {
                divaBasketHandler.actBasket.Client.email = email;
              }
              _this.openAffiliate(divaBasketHandler, product);
              resolve({
                event: undefined,
                saveBasket: true
              });
            }
          });
        }
        if (config !== null && config !== void 0 && config.openContactForm) {
          actions.push({
            loginType: _this.getLoginType(config, config === null || config === void 0 ? void 0 : config.createAndContact),
            text: 'planner.barrieraction.opencontactform',
            onClick: function () {
              var _onClick = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
                return _regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) switch (_context.prev = _context.next) {
                    case 0:
                      resolve({
                        event: 'onSendContactForm',
                        saveBasket: true
                      });
                    case 1:
                    case "end":
                      return _context.stop();
                  }
                }, _callee);
              }));
              function onClick() {
                return _onClick.apply(this, arguments);
              }
              return onClick;
            }()
          });
        }
        config === null || config === void 0 || (_config$additional = config.additional) === null || _config$additional === void 0 || _config$additional.forEach(function (a) {
          var _this$uiState$barrier4, _this$uiState$i18nIns6;
          if (_this.uiState.barrierCreateActionOrder == actions.length) addCreateBarrierAction();
          var idx = actions.length;
          actions.push({
            loginType: 'NO_LOGIN',
            className: (_this$uiState$barrier4 = _this.uiState.barrierActions) === null || _this$uiState$barrier4 === void 0 || (_this$uiState$barrier4 = _this$uiState$barrier4[idx]) === null || _this$uiState$barrier4 === void 0 ? void 0 : _this$uiState$barrier4.className,
            text: (_this$uiState$i18nIns6 = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns6 === void 0 ? void 0 : _this$uiState$i18nIns6.t(a.text),
            onClick: function onClick(email) {
              //set the email as client email to basket
              if (divaBasketHandler.actBasket.Client) {
                divaBasketHandler.actBasket.Client.email = email;
              }
              var baseA = a;
              var eventA = a;
              if (baseA.link) {
                var _this$actions2, _this$uiState$i18nIns7, _this$uiState$i18nIns8;
                (_this$actions2 = _this.actions) === null || _this$actions2 === void 0 || _this$actions2.openComponent({
                  url: baseA.localLink ? baseA.localLink[(_this$uiState$i18nIns7 = (_this$uiState$i18nIns8 = _this.uiState.i18nInstance) === null || _this$uiState$i18nIns8 === void 0 ? void 0 : _this$uiState$i18nIns8.language) !== null && _this$uiState$i18nIns7 !== void 0 ? _this$uiState$i18nIns7 : 'de'] : baseA.link,
                  openInNewTab: true
                });
                resolve({
                  event: baseA.resolve,
                  saveBasket: true
                });
              } else if (eventA.event) {
                resolve({
                  event: eventA.event,
                  saveBasket: eventA.saveBasket
                });
              }
            }
          });
        });
        if (_this.uiState.barrierCreateActionOrder !== undefined && _this.uiState.barrierCreateActionOrder >= actions.length) addCreateBarrierAction();
        (_this$actions3 = _this.actions) === null || _this$actions3 === void 0 || _this$actions3.openBarrier({
          text: barrierText,
          type: config === null || config === void 0 ? void 0 : config.type,
          onClose: function onClose() {
            resolve({
              event: undefined,
              saveBasket: true
            });
          },
          actions: actions
        });
      });
    }
  }, {
    key: "setReportConfig",
    value: function setReportConfig(divaBasketHandler, action) {
      if (action && typeof action !== 'boolean' && action.reportConfig) {
        divaBasketHandler.reportConfig = action === null || action === void 0 ? void 0 : action.reportConfig;
      } else {
        divaBasketHandler.reportConfig = undefined;
      }
    }
  }, {
    key: "getLoginType",
    value: function getLoginType(config, action) {
      var _config$type;
      if (config && typeof action !== 'boolean') {
        if (action !== null && action !== void 0 && action.login) return action.login;
      }
      return (_config$type = config === null || config === void 0 ? void 0 : config.type) !== null && _config$type !== void 0 ? _config$type : 'NO_LOGIN';
    }
  }, {
    key: "openAffiliate",
    value: function openAffiliate(divaBasketHandler, product) {
      var _this$actions4;
      divaBasketHandler.actBasket.SetImage;
      (_this$actions4 = this.actions) === null || _this$actions4 === void 0 || _this$actions4.openModal({
        content: /*#__PURE__*/React.createElement(Affiliate, {
          product: product
        }),
        rejectByIcon: true
      });
    }
  }, {
    key: "openContactForm",
    value: function openContactForm(divaBasketHandler) {
      var _this$actions5;
      (_this$actions5 = this.actions) === null || _this$actions5 === void 0 || _this$actions5.openModal({
        content: /*#__PURE__*/React.createElement(ContactFormComponent, {
          title: 'planner.barrieraction.contactform.title',
          submitButtonText: 'planner.barrieraction.contactform.buttontext',
          action: 'SEND_MAIL',
          basket: divaBasketHandler.actBasket
        }),
        doNotAcceptOnEnter: true,
        rejectByIcon: true,
        accept: false,
        width: '80%'
      });
    }
  }, {
    key: "setEmailConfig",
    value: function setEmailConfig(divaBasketHandler, clientEmail, salesEmail) {
      var _this2 = this;
      //will be executed after send process is finished
      var onFinshed = function onFinshed() {
        if (!divaBasketHandler.actBasket.PDFDocumentURL) {
          var _this2$actions, _this2$uiState$i18nIn, _this2$uiState$i18nIn2;
          LOG$2.errorTransaction('ON_PRINT_PLANNER', 'WC_0010', 'PDF url is missing in basket');
          (_this2$actions = _this2.actions) === null || _this2$actions === void 0 || _this2$actions.openModal({
            text: ((_this2$uiState$i18nIn = _this2.uiState.i18nInstance) === null || _this2$uiState$i18nIn === void 0 ? void 0 : _this2$uiState$i18nIn.t('modal.general_error')) + ' --- ' + DivaUtils.formatDate(new Date(), true, undefined, true) + ' --- ' + DivaUtils.getCurrentTraceId() ? ((_this2$uiState$i18nIn2 = _this2.uiState.i18nInstance) === null || _this2$uiState$i18nIn2 === void 0 ? void 0 : _this2$uiState$i18nIn2.t('modal.errorcode')) + ': ' + DivaUtils.getCurrentTraceId() : '',
            accept: true
          });
        } else {
          var _this2$actions2, _this2$uiState$i18nIn3, _this2$uiState$i18nIn4;
          LOG$2.successTransaction('ON_PRINT_PLANNER');
          (_this2$actions2 = _this2.actions) === null || _this2$actions2 === void 0 || _this2$actions2.openModal({
            text: (_this2$uiState$i18nIn3 = _this2.uiState.i18nInstance) === null || _this2$uiState$i18nIn3 === void 0 ? void 0 : _this2$uiState$i18nIn3.t(divaBasketHandler.actBasket.Status === 'ORDER' ? 'modal.open_order_pdf_now' : 'modal.open_leaflet_pdf_now'),
            accept: (_this2$uiState$i18nIn4 = _this2.uiState.i18nInstance) === null || _this2$uiState$i18nIn4 === void 0 ? void 0 : _this2$uiState$i18nIn4.t('modals.open_now'),
            reject: true,
            callback: function callback(accept) {
              if (accept) window.open(divaBasketHandler.actBasket.PDFDocumentURL, '_blank');
            }
          });
        }
      };
      LOG$2.startTransaction({
        event: 'onPrintPlanner',
        message: 'Ask Show PDF after sending mail.',
        eventId: 'ON_PRINT_PLANNER'
      });
      divaBasketHandler.actBasket.SalespersonEmail = salesEmail;

      //set the email as client email to basket
      if (divaBasketHandler.actBasket.Client) {
        divaBasketHandler.actBasket.Client.email = clientEmail;
      }
      //set the email config, so that email will be send
      divaBasketHandler.emailConfig = {
        v: '2',
        showDialog: false,
        to: clientEmail,
        callback: onFinshed
      };
    }
  }]);
}();

function ownKeys$1(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread$1(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys$1(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys$1(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG$1 = getLogger('Planner', 'PlannerInit');
var DivaViewHandler = /*#__PURE__*/function () {
  function DivaViewHandler(plannerConfig, isParentViewVisible) {
    var _this = this,
      _this$actions2,
      _this$actions3,
      _this$actions4;
    _classCallCheck(this, DivaViewHandler);
    _defineProperty(this, "actions", void 0);
    _defineProperty(this, "coreState", void 0);
    _defineProperty(this, "uiState", void 0);
    _defineProperty(this, "settingsState", void 0);
    _defineProperty(this, "productState", void 0);
    _defineProperty(this, "tabGroupState", void 0);
    _defineProperty(this, "actionButtonState", void 0);
    _defineProperty(this, "plannerHelper", void 0);
    _defineProperty(this, "divaWebPlanner", void 0);
    //nicht im constructor verwenden!
    _defineProperty(this, "configuratorStructureHandler", void 0);
    _defineProperty(this, "paletteViewHandler", void 0);
    _defineProperty(this, "viewPlannerHandler", void 0);
    //Planner classes
    _defineProperty(this, "viewIsReady", false);
    _defineProperty(this, "configState", void 0);
    _defineProperty(this, "barrierHandler", void 0);
    _defineProperty(this, "initializedWithErrors", false);
    _defineProperty(this, "isParentViewVisible", false);
    _defineProperty(this, "layoutRendered", false);
    _defineProperty(this, "rtvAnalyzeData", []);
    _defineProperty(this, "hqRenderScript", void 0);
    _defineProperty(this, "camInfo", void 0);
    /**
     * This function returns the acts state od the webConfigurator (can only be one of those 4)
     */
    _defineProperty(this, "getState", function () {
      if (_this.configuratorStructureHandler.CurConfigStep == undefined) return 'PLANNER';
      if (_this.configuratorStructureHandler.curOptionCodex.startsWith('SHELFARTICLE_')) return 'SHELFPLANNER'; // if curOption an Shelfplanner is return SHELFPLANNER because the Shelfplanner can be in an option tabgroup
      return _this.configuratorStructureHandler.curTabgroupType;
    });
    _defineProperty(this, "tabgroupTypeActive", function (type) {
      if (_this.getState() === type) return true;
      if (type === 'SHELFPLANNER' && _this.configuratorStructureHandler.actGroupedOptionsCodexes.find(function (o) {
        return o.startsWith('SHELFARTICLE_');
      })) return true;
      return false;
    });
    //---------------------------------------Registered Functions called from Logic----------------------------------------------------------------------------
    _defineProperty(this, "getCurrentAlternativRendering", function () {
      if (_this.productState.curAlternateRendering) {
        return [_this.productState.curAlternateRendering];
      }
      return _this.configuratorStructureHandler.curTabgroupAlternativeRendering;
    });
    /**
     * this function sets the act TabGroup and the first ConfigStep of that TabGroup by TabGroupType
     * @param type typ of TabGroup
     */
    _defineProperty(this, "navigateToTabGroup", function (type, autoNavIfNotFound, dontNavigate) {
      var successful = _this.configuratorStructureHandler.navigateToTabgroupByType(type);
      if (!successful && autoNavIfNotFound) {
        _this.configuratorStructureHandler.navigateToTabgroupByType(_this.configuratorStructureHandler.curTabgroupType);
        successful = true;
      }
      if (successful) _this.updateConfiguratorState(!dontNavigate);
      return successful;
    });
    /**
     * This functions is always calls when the Configuration Structure was updated (so maybe steps were added or removed) or the user Navigated
     * It relodes the TabMenu and the ItemPalettes
     * @param wasNaviagtion identifyer if it was Navigated
     */
    _defineProperty(this, "updateConfiguratorState", function (wasNaviagtion) {
      _this.settingsState.initForUpdate();
      _this.actionButtonState.initForUpdate();
      _this.paletteViewHandler.updateCurPaletteData(wasNaviagtion);
      _this.tabGroupState.configuratorSturucture;
      if (wasNaviagtion) {
        _this.divaWebPlanner.updatePlannerView('NAVIGATION');
        _this.logAnalytics(); //pfister specific
      }
    });
    _defineProperty(this, "getPlanningIsInCompleteOrEmpty", function () {
      return !_this.divaWebPlanner.getPlanningIsComplete(true) || _this.divaWebPlanner.getPlanningIsEmpty();
    });
    _defineProperty(this, "getIsInMobileView", function () {
      return _this.uiState.isInMobileView;
    });
    _defineProperty(this, "getOrbitInteractionAPI", function () {
      return _this.productState.orbitCubeElement;
    });
    _defineProperty(this, "onCloseConfigurator", function () {
      var _this$actions, _this$getI18nInstance, _this$getI18nInstance2;
      (_this$actions = _this.actions) === null || _this$actions === void 0 || _this$actions.openModal({
        title: (_this$getI18nInstance = _this.getI18nInstance()) === null || _this$getI18nInstance === void 0 ? void 0 : _this$getI18nInstance.t('modals.close'),
        accept: true,
        reject: true,
        text: (_this$getI18nInstance2 = _this.getI18nInstance()) === null || _this$getI18nInstance2 === void 0 ? void 0 : _this$getI18nInstance2.t('modals.are_you_sure_to_close'),
        callback: function callback(accepted) {
          if (accepted) {
            var _this$coreState;
            _this.divaWebPlanner.callEvent('onWebPlanerClose');
            ((_this$coreState = _this.coreState) === null || _this$coreState === void 0 ? void 0 : _this$coreState.isFullscreenComponent) && _this.divaWebPlanner.callEvent('onFullscreenClose');
            _this.logAnalytics('onClose', 'Click on Exit Button');
          }
        }
      });
    });
    window.viewHandler = this;
    this.configState = new ConfigState();
    this.configState.setConfigParams(plannerConfig);
    this.actions = plannerConfig.actions;
    this.coreState = plannerConfig.state;
    this.isParentViewVisible = isParentViewVisible;
    this.plannerHelper = new ViewHelper(this.getState, this.navigateToTabGroup, this.updateConfiguratorState);
    this.uiState = new UIState();
    this.settingsState = new SettingsState();
    this.configuratorStructureHandler = new ConfiguratorStructureHandler();
    this.productState = new ProductState(this.initialize.bind(this), this.logAnalytics.bind(this),
    //pfister specific
    this.getState, this.eventIsManagedByClient.bind(this), this.plannerHelper.forceViewToEnterPlannerMode.bind(this.plannerHelper), (_this$actions2 = this.actions) === null || _this$actions2 === void 0 ? void 0 : _this$actions2.toggleLoading);
    this.viewPlannerHandler = new ViewPlannerHandler(this.plannerHelper.plannerElementSelectionChanged, this.getState, this.productState.getPlannerRenderingSize.bind(this.productState), this.uiState.doShowConfigPalette, this.productState.setDiva3dImgSrc.bind(this.productState), this.plannerHelper.forceViewToEnterPlannerMode.bind(this.plannerHelper), this.getOrbitInteractionAPI, this.productState.onDoShowNav3DChange.bind(this.productState), this.productState.doShow2DGridInViewModel.bind(this.productState), this.productState.setPlannerRenderingIsStaticBG.bind(this.productState), this.getPosNrByBasketId.bind(this));
    this.tabGroupState = new TabGroupState(this.uiState, this.configuratorStructureHandler, this.viewPlannerHandler, this.productState, this.settingsState, this.updateConfiguratorState, (_this$actions3 = this.actions) === null || _this$actions3 === void 0 ? void 0 : _this$actions3.onEvent);
    this.actionButtonState = new ActionButtonState(this.uiState, this.settingsState, this.configuratorStructureHandler, this.getState, this.updateConfiguratorState, this.productState.updatePlannerMsgContainer, (_this$actions4 = this.actions) === null || _this$actions4 === void 0 ? void 0 : _this$actions4.openModal);
    this.barrierHandler = new BarrierHandler(this.uiState, this.actions);
    makeAutoObservable(this, {
      hqRenderScript: observable,
      setHQRenderScript: action,
      camInfo: observable,
      setCamInfo: action,
      rtvAnalyzeData: observable
    });
  }
  return _createClass(DivaViewHandler, [{
    key: "getPosNrByBasketId",
    value: function getPosNrByBasketId(basketId) {
      return this.divaWebPlanner.getPosNr(basketId);
    }
  }, {
    key: "initialize",
    value: function initialize(firstTime, plannerConfig) {
      var _this$divaWebPlanner;
      DivaUtils.startSpanOfApm('initialize Configurator');
      this.uiState.setIsLoading(true);
      if (this.configState == undefined) return;
      if (!firstTime) {
        if (plannerConfig) {
          if (plannerConfig !== true) {
            this.configState = plannerConfig;
          } else {
            var _this$divaWebPlanner$, _this$divaWebPlanner$2;
            if (Object.keys((_this$divaWebPlanner$ = (_this$divaWebPlanner$2 = this.divaWebPlanner.actSet()) === null || _this$divaWebPlanner$2 === void 0 ? void 0 : _this$divaWebPlanner$2.CatalogData.allCatalogPlannerArticles) !== null && _this$divaWebPlanner$ !== void 0 ? _this$divaWebPlanner$ : []).length > 0) {
              this.configState.articleCodexes = undefined;
            }
          }
          this.viewPlannerHandler.clearView();
          this.hqRenderScript = undefined;
          this.camInfo = undefined;
          this.divaWebPlanner.destroyWebPlanner();
        } else {
          this.uiState.setIsLoading(false);
          return;
        }
      }
      this.divaWebPlanner = new DivaWebPlanner(this.configState);
      this.tabGroupState.initFunctions(this.divaWebPlanner);
      this.paletteViewHandler = new PaletteViewHandler(this.settingsState, this.productState, this.uiState, this.configuratorStructureHandler, this.getState, this.updateConfiguratorState, this.viewPlannerHandler.getSelectedElementsArr, this.viewPlannerHandler.getSelectedDockingPoint, this.viewPlannerHandler.updateElementSelectionPlanner, this.divaWebPlanner.configSingleIsChanged, this.divaWebPlanner.addPlannerElement, this.divaWebPlanner.deletePlannerElement.bind(this.divaWebPlanner), this.divaWebPlanner.switchPlannerElement, this.divaWebPlanner.copyPlannerElement.bind(this.divaWebPlanner), this.divaWebPlanner.deleteAccessoire, this.divaWebPlanner.updateAccessoireQuantity, this.divaWebPlanner.doConfigure, this.divaWebPlanner.setActSetCurConfigStepKey.bind(this.divaWebPlanner), this.divaWebPlanner.setActSet, this.divaWebPlanner.doAllowSetPlanner.bind(this.divaWebPlanner), this.divaWebPlanner.handle3DVisualizationChanged, this.divaWebPlanner.loadCatalogPlannerArticles, this.divaWebPlanner.loadAccessoiresOfArticles, this.divaWebPlanner.loadConfigItemsOfOptions, this.divaWebPlanner.loadSummaryItems, this.divaWebPlanner.loadStaticRoomList, this.divaWebPlanner.loadShelfPlannerItems, this.divaWebPlanner.loadDoorPlannerItems, this.divaWebPlanner.selectShelfAccArticle.bind(this.divaWebPlanner), this.divaWebPlanner.getFreePlacesOnGrids.bind(this.divaWebPlanner), this.divaWebPlanner.addAccessorieOnShelfPlanner, this.divaWebPlanner.moveShelfItem, this.divaWebPlanner.moveDoors, this.divaWebPlanner.deleteAccessorieOnShelfPlanner, this.divaWebPlanner.getBasketIdsOfElementIdLogic, this.divaWebPlanner.loadPaletteInfos, this.divaWebPlanner.changeOnFavorites.bind(this.divaWebPlanner), this.divaWebPlanner.defineArticlesOnOptions.bind(this.divaWebPlanner), this.actionButtonState.onClickConfiguratorSettingsForth.bind(this.actionButtonState), this.getPlanningIsInCompleteOrEmpty, this.divaWebPlanner.checkNotificationRules.bind(this.divaWebPlanner), this.divaWebPlanner.checkBannerRules.bind(this.divaWebPlanner), this.actions);
      this.viewPlannerHandler.initFunctions(this.paletteViewHandler.deletePlannerElement.bind(this.paletteViewHandler), this.paletteViewHandler.copyPlannerElement.bind(this.paletteViewHandler));
      this.actionButtonState.initFunctions(this.divaWebPlanner, this.paletteViewHandler);
      this.uiState.initFunctions(this.divaWebPlanner.updatePlannerView.bind(this.divaWebPlanner), this.paletteViewHandler.visitActVisibleSteps.bind(this.paletteViewHandler), this.tabGroupState.updateCurTabGroupType.bind(this.tabGroupState), this.settingsState, this.divaWebPlanner.getReturnTo, this.configState.initProductFolded);
      (_this$divaWebPlanner = this.divaWebPlanner) === null || _this$divaWebPlanner === void 0 || _this$divaWebPlanner.clearCache();
      this.productState.initialize(this.divaWebPlanner, this.settingsState, this.viewPlannerHandler.setCurPlannerViewMode.bind(this.viewPlannerHandler), this.viewPlannerHandler.handlePlanner3DGridChanged.bind(this.viewPlannerHandler), this.viewPlannerHandler.initPlannerViewMode.bind(this.viewPlannerHandler), this.actionButtonState.onClickAddToBasket.bind(this.actionButtonState), this.paletteViewHandler.changeOnCheckoutConfig.bind(this.paletteViewHandler));
      this.uiState.reset();
      this.settingsState.reset();
      this.registerFunctions();
      this.initializeLogic();
      //TabgroupViewHandler erstellen dieser Kümmert sich umm alle Click events usw.

      this.plannerHelper.initialize(this.paletteViewHandler, this.configuratorStructureHandler, this.divaWebPlanner, this.productState, this.tabGroupState, this.settingsState);
      if (!firstTime) {
        this.uiState.onResize();
        DivaUtils.endSpanOfApm('initialize Configurator');
      }
      this.uiState.setConfiguratorDisabled(false);
      this.productState.plannerMessages = new Map();
    }
  }, {
    key: "initializeLogic",
    value: function () {
      var _initializeLogic = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
        var _this$actions5, _this$actions6;
        var _this$getI18nInstance3, _this$getI18nInstance4, _this$configState, _this$configState2;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              (_this$actions5 = this.actions) === null || _this$actions5 === void 0 || _this$actions5.toggleLoading('init_planner_logic', false, true);
              _context.prev = 1;
              _context.next = 4;
              return this.divaWebPlanner.initialize();
            case 4:
              this.initializedWithErrors = !_context.sent;
              LOG$1.successTransaction('ON_OPEN_PLANNER', {
                divaNr: this.divaWebPlanner.divaNr,
                productId: this.configState.productId,
                cat: this.divaWebPlanner.getCurCatalogCodex(),
                artNr: JSON.stringify(this.configState.articleCodexes),
                lang: (_this$getI18nInstance3 = this.getI18nInstance()) === null || _this$getI18nInstance3 === void 0 ? void 0 : _this$getI18nInstance3.language
              });
              _context.next = 13;
              break;
            case 8:
              _context.prev = 8;
              _context.t0 = _context["catch"](1);
              if (_context.t0.message.includes('does not exist in the Catalog:')) {
                if ((_this$configState = this.configState) !== null && _this$configState !== void 0 && _this$configState.articleCodexes && ((_this$configState2 = this.configState) === null || _this$configState2 === void 0 ? void 0 : _this$configState2.articleCodexes.length) > 0) {
                  this.uiState.setConfiguratorHidden(true);
                  alert('Something went wrong. Please try again or contact support');
                }
              }
              LOG$1.error(_context.t0);
              LOG$1.errorTransaction('ON_OPEN_PLANNER', 'WC_0007', _context.t0.message, {
                divaNr: this.divaWebPlanner.divaNr,
                productId: this.configState.productId,
                cat: this.divaWebPlanner.getCurCatalogCodex(),
                artNr: JSON.stringify(this.configState.articleCodexes),
                lang: (_this$getI18nInstance4 = this.getI18nInstance()) === null || _this$getI18nInstance4 === void 0 ? void 0 : _this$getI18nInstance4.language
              });
            case 13:
              (_this$actions6 = this.actions) === null || _this$actions6 === void 0 || _this$actions6.toggleLoading('init_planner_logic', false, false);
            case 14:
            case "end":
              return _context.stop();
          }
        }, _callee, this, [[1, 8]]);
      }));
      function initializeLogic() {
        return _initializeLogic.apply(this, arguments);
      }
      return initializeLogic;
    }()
  }, {
    key: "tabGroupAlreadyVisited",
    value: function tabGroupAlreadyVisited(tabGroup) {
      return this.configuratorStructureHandler.tabGroupAlreadyVisited(tabGroup);
    }
  }, {
    key: "configStepAlreadyVisited",
    value: function configStepAlreadyVisited(configStep) {
      return this.configuratorStructureHandler.configStepAlreadyVisited(configStep);
    }
  }, {
    key: "onBasketInitialized",
    value: function onBasketInitialized() {
      this.uiState.setIsLoading(false);
      this.uiState.buildResizeObserver();
    }
  }, {
    key: "initCSSVariables",
    value: function initCSSVariables() {
      var obj = {};
      var cssVariables = this.uiState.ViewConfig.cssVariables;
      if (cssVariables) {
        for (var key in cssVariables) {
          obj['--' + key] = cssVariables[key];
        }
        this.initCSSVariablesObj(obj, true);
      }
    }
  }, {
    key: "initCSSVariablesObj",
    value: function initCSSVariablesObj(cssVariables, ignoreKeys) {
      var _this$uiState$ViewCon;
      var root = document.documentElement;
      var keys = Object.keys((_this$uiState$ViewCon = this.uiState.ViewConfig.cssVariables) !== null && _this$uiState$ViewCon !== void 0 ? _this$uiState$ViewCon : {});
      if (cssVariables) {
        for (var key in cssVariables) {
          if (!ignoreKeys && keys.includes(key.substring(2))) continue;
          root.style.setProperty(key, cssVariables[key]);
        }
      }
    }
  }, {
    key: "logAnalytics",
    value: function logAnalytics(eventAction, message) {
      if (eventAction && message) {
        if (this.uiState.ViewConfig.logGAEnabled) {
          this.divaWebPlanner.logGA(message);
        }
      } else {
        switch (this.getState()) {
          case 'PLANNER':
            if (this.uiState.ViewConfig.logGAEnabled) {
              this.divaWebPlanner.logGA('Planen');
            }
            break;
          case 'ACC':
          case 'ACC_PLANNER':
          case 'OPTION':
            var _message = 'show Config (' + this.configuratorStructureHandler.getCurTabgroupCount() + '/' + this.configuratorStructureHandler.getTabgroupCount() + ')';
            if (this.uiState.ViewConfig.logGAEnabled) {
              this.divaWebPlanner.logGA(_message);
            }
            break;
          case 'SUMMARY':
            if (this.uiState.ViewConfig.logGAEnabled) {
              this.divaWebPlanner.logGA('Summary');
            }
            break;
        }
      }
    }
  }, {
    key: "addProduct",
    value: function () {
      var _addProduct = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee2(productId) {
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) switch (_context2.prev = _context2.next) {
            case 0:
              if (this.divaWebPlanner.getPlannerMode() == 'ProductList') {
                //first time click on productlist in an empty planner
                this.loadProduct(productId);
              }
            case 1:
            case "end":
              return _context2.stop();
          }
        }, _callee2, this);
      }));
      function addProduct(_x) {
        return _addProduct.apply(this, arguments);
      }
      return addProduct;
    }()
  }, {
    key: "loadProduct",
    value: function () {
      var _loadProduct = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee3(productId) {
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) switch (_context3.prev = _context3.next) {
            case 0:
              this.configState.productId = productId;
              this.initialize(false, this.configState);
            case 2:
            case "end":
              return _context3.stop();
          }
        }, _callee3, this);
      }));
      function loadProduct(_x2) {
        return _loadProduct.apply(this, arguments);
      }
      return loadProduct;
    }()
  }, {
    key: "addDivaNrToSet",
    value: function () {
      var _addDivaNrToSet = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee4(divaNr) {
        return _regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return this.divaWebPlanner.addSetToActPlanningByDivaNr(divaNr);
            case 2:
              this.updateConfiguratorState(true);
            case 3:
            case "end":
              return _context4.stop();
          }
        }, _callee4, this);
      }));
      function addDivaNrToSet(_x3) {
        return _addDivaNrToSet.apply(this, arguments);
      }
      return addDivaNrToSet;
    }()
  }, {
    key: "navigateToConfigStep",
    value:
    /**
     * this function sets the Act ConfigStep
     * @param configStepKey U-ID
     */
    function navigateToConfigStep(configStepKey, dontScroll, visitRest) {
      if (visitRest) this.configuratorStructureHandler.addAllPreviousTabGroupsAsVisited(configStepKey);
      this.configuratorStructureHandler.navigateToConfigStepByConfigStepKey(configStepKey);
      if (!dontScroll) {
        this.scrollToTabgroup(configStepKey);
      }
      this.updateConfiguratorState(true);
    }
  }, {
    key: "scrollToTabgroup",
    value: function scrollToTabgroup(configStepKey) {
      var _this$configuratorStr;
      var posNr = (_this$configuratorStr = this.configuratorStructureHandler.getConfigStepByKey(configStepKey)) === null || _this$configuratorStr === void 0 ? void 0 : _this$configuratorStr.configStep.posNr;
      if (posNr) this.paletteViewHandler.scrollToConfigStepByPosNr(posNr, true);
    }
  }, {
    key: "setWebConifguratorTitle",
    value:
    /**
     * This Function sets the Configurator Tilte -> for Examlps "CLASSICS SOFA"
     * @param webConifguratorTitle
     */
    function setWebConifguratorTitle(setName) {
      var newTitle;
      if (setName !== undefined) {
        if (this.divaWebPlanner.showDivaNr()) {
          newTitle = setName + ' - ' + this.divaWebPlanner.divaNr;
        } else {
          newTitle = setName;
        }
        this.productState.setWebConifguratorTitle(newTitle);
      }
    }

    /**
     * This function is called after the api Handler is initialized. It sets all api dependent settings
     */
  }, {
    key: "initApiDependencies",
    value: (function () {
      var _initApiDependencies = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee5() {
        var config, _this$actions7, _this$configState$cur;
        return _regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) switch (_context5.prev = _context5.next) {
            case 0:
              config = this.divaWebPlanner.getAllConfig();
              if (config) {
                this.uiState.initFromConfig(config);
                this.uiState.initTemplateConfig(this.divaWebPlanner.getTemplateConfig(), (_this$actions7 = this.actions) === null || _this$actions7 === void 0 ? void 0 : _this$actions7.userHasPermission('advancedconfigurator'));
                this.settingsState.setForceSettingsHeaderInScrollContainer(config.viewConfig.forceSettingsHeaderInScrollContainer);
                this.productState.initViewConfig(config.viewConfig.doShowFunctions, !!this.divaWebPlanner.userIsLoggedIn, (_this$configState$cur = this.configState.currentOrganization) !== null && _this$configState$cur !== void 0 && (_this$configState$cur = _this$configState$cur.settings) !== null && _this$configState$cur !== void 0 && _this$configState$cur.enableAR ? this.getViewConfig().arQrCodeConfig ? this.getViewConfig().arQrCodeConfig : {
                  doAutoShow: false
                } : undefined);
                this.tabGroupState.updateCurTabGroupType();
                this.initCSSVariables();
                //View is ready. Logic waits for this variable to start
                this.viewIsReady = true;
              }
            case 2:
            case "end":
              return _context5.stop();
          }
        }, _callee5, this);
      }));
      function initApiDependencies() {
        return _initApiDependencies.apply(this, arguments);
      }
      return initApiDependencies;
    }())
  }, {
    key: "updatePlannerConfig",
    value: function updatePlannerConfig(plannerConfig) {
      this.divaWebPlanner.updatePlannerConfig(plannerConfig);
      this.initApiDependencies();
    }
  }, {
    key: "getViewConfig",
    value: function getViewConfig() {
      return this.uiState.ViewConfig;
    }
  }, {
    key: "initPlannerView",
    value: function () {
      var _initPlannerView = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee6() {
        var _this$divaWebPlanner$3, _this$actions8;
        return _regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) switch (_context6.prev = _context6.next) {
            case 0:
              if (!this.uiState.ViewConfig) {
                _context6.next = 9;
                break;
              }
            case 1:
              if (this.viewIsReady) {
                _context6.next = 6;
                break;
              }
              _context6.next = 4;
              return DivaUtils.Sleep(20);
            case 4:
              _context6.next = 1;
              break;
            case 6:
              _context6.next = 8;
              return this.viewPlannerHandler.initPlannerView(this.uiState.ViewConfig, this.productState, this.divaWebPlanner.updatePlannerTrans, this.divaWebPlanner.updatePlannerView, (_this$divaWebPlanner$3 = this.divaWebPlanner.actSet()) === null || _this$divaWebPlanner$3 === void 0 ? void 0 : _this$divaWebPlanner$3.CatalogData.PlannerPaletteViewInfo);
            case 8:
              (_this$actions8 = this.actions) === null || _this$actions8 === void 0 || _this$actions8.toggleLoading('Planner_Loading', false, false);
            case 9:
            case "end":
              return _context6.stop();
          }
        }, _callee6, this);
      }));
      function initPlannerView() {
        return _initPlannerView.apply(this, arguments);
      }
      return initPlannerView;
    }()
    /**
     * Updates the new calculated pirce in View
     * ApiFormat contains the format that the api has configured - take this format if nothing is configured in the article
     * ArticlePriceFormat contains the format that is configured in the article. If this is set - take this format
     * If no format is set - take the default format
     * @param newPrice the new Price
     */
  }, {
    key: "updatePrice",
    value: function updatePrice(newTotalPrice, newCurSetPrice, bruttoPrice, articlePriceFormat, customDimensions) {
      var _this$getViewConfig;
      this.productState.setFormatedPrices(DivaUtils.formatPrice(newTotalPrice, _objectSpread$1(_objectSpread$1({}, DivaUtils.defaultPriceFormat), articlePriceFormat)), DivaUtils.formatPrice(newCurSetPrice, _objectSpread$1(_objectSpread$1({}, DivaUtils.defaultPriceFormat), articlePriceFormat)), bruttoPrice !== undefined && DivaUtils.formatPrice(bruttoPrice, _objectSpread$1(_objectSpread$1({}, DivaUtils.defaultPriceFormat), articlePriceFormat)), customDimensions);
      if ((_this$getViewConfig = this.getViewConfig()) !== null && _this$getViewConfig !== void 0 && _this$getViewConfig.renderPriceInFrameworkHeader && !this.uiState.hidePrices) {
        var _this$uiState$i18nIns, _this$actions9;
        var priceInfo = (_this$uiState$i18nIns = this.uiState.i18nInstance) === null || _this$uiState$i18nIns === void 0 ? void 0 : _this$uiState$i18nIns.t('configurator.PRICE_INFO_HEADER');
        var price = this.productState.formatedTotalPrice;
        (_this$actions9 = this.actions) === null || _this$actions9 === void 0 || _this$actions9.updateCustomHeaderElement(['DIVA_WEBPLANNER'], function () {
          return FrameworkPrice(priceInfo, price);
        });
      } else {
        var _this$actions10;
        (_this$actions10 = this.actions) === null || _this$actions10 === void 0 || _this$actions10.updateCustomHeaderElement(['DIVA_WEBPLANNER'], undefined);
      }
    }
  }, {
    key: "clearPrice",
    value: function clearPrice() {
      var _this$actions11;
      (_this$actions11 = this.actions) === null || _this$actions11 === void 0 || _this$actions11.updateCustomHeaderElement(['DIVA_WEBPLANNER'], undefined);
    }

    /**
     * the ConfiguratorStructure gets initialize
     * @param configuratorStructure the new ConfiguratorStructure
     */
  }, {
    key: "initConfiguratorStructure",
    value: function initConfiguratorStructure(configuratorStructure) {
      var _this$getViewConfig2;
      if (Object.keys(configuratorStructure).length === 0) throw new Error('No ConfiguratorStep was generated. Normally by default at least a plannerstep would be visible. One possible reason could be that the catalog has no planner articles due to having no svgs imported.');
      this.configuratorStructureHandler.initialize(this.tabGroupState, configuratorStructure, !!((_this$getViewConfig2 = this.getViewConfig()) !== null && _this$getViewConfig2 !== void 0 && _this$getViewConfig2.hideSingleOPVOnlyAlways), this.viewPlannerHandler.enterPlannerMode, this.uiState.navigateStepByStep);
    }

    /**
     * the ConfiguratorStructure gets updated and the View of it gets updated.
     * @param configuratorStructure the new ConfiguratorStructure
     */
  }, {
    key: "updateConfiguratorStructure",
    value: function updateConfiguratorStructure(configuratorStructure) {
      this.configuratorStructureHandler.updateConfiguratorStructure(configuratorStructure);
    }

    /**
     * This function is called when a new rendering for the fullscreen view is loaded. It loads the fullscreen view and inserts the rendering.
     * @param rendering
     */
  }, {
    key: "updateFullscreenRendering",
    value: function updateFullscreenRendering(rendering) {
      var image = "url('" + rendering.getImageForSrc() + "')";
      this.divaWebPlanner.callEvent('onWebPlanerZoomClick', image);
      if (!this.eventIsManagedByClient('onWebPlanerZoomClick')) {
        var _this$actions12, _this$actions13;
        (_this$actions12 = this.actions) === null || _this$actions12 === void 0 || _this$actions12.openFullscreen({
          content: Fullscreen({
            image: image
          })
        });
        (_this$actions13 = this.actions) === null || _this$actions13 === void 0 || _this$actions13.toggleLoading('Planner_Rendering', false, false);
      }
      this.logAnalytics('onFullScreen', 'Click on Zoom Button'); //pfister specific
    }
  }, {
    key: "updateCatalogPlannerArticles",
    value: /***************************PALETTE Updates********************************** */
    //all Of these just pass on to the PaletteHandler
    /**
     * Updates the Article-List(on PLannerStep) with articles that are comming for logic
     * @param plannerItems articles
     */
    function updateCatalogPlannerArticles(plannerItems, paletteId, isUpdate, type, dockFilter, actStepName, stepKey) {
      this.paletteViewHandler.updateCatalogPlannerArticles(plannerItems, paletteId, isUpdate, type, dockFilter, actStepName, stepKey);
    }

    /**
     * Updates the Accessoire-List(on AccessoireStep)  with accessoires from the actArticle
     * @param accessoires accessorie Items
     */
  }, {
    key: "updateAccessoiresPalette",
    value: function updateAccessoiresPalette(accessoires, accType) {
      this.paletteViewHandler.updateAccessoiresPalette(accessoires, accType);
    }

    /**
     * Updates all on one specific ConfigPalette (can be more than one!)
     * @param configItems selectable / Visible ConfigItems
     * @param actStepName Title of the ConfigPalette
     * @param paletteType Palette Template e.g: GAL for Gallery
     * @param filterType Filter Type e.g: RAL for Colorfilter
     * @param paletteId iD of the ConfigPalette
     */
  }, {
    key: "updateConfigItems",
    value: function updateConfigItems(configStepKey, configItems, selectedItems, actStepName, paletteType, filterType, paletteId, isUpdate, visible, stepInfo, isRenderedInAccessorieStep, isRenderedNotInSettingPalette, optionCodex, notTotallyMatichingOPVs, subTabGroup, pairWithOptionsToGroup, isMultiselect, groupsDefaultClosed, contentItemId) {
      this.paletteViewHandler.updateConfigItems(configStepKey, configItems, actStepName, paletteType, filterType, paletteId, isUpdate, visible, stepInfo, isRenderedInAccessorieStep, isRenderedNotInSettingPalette, optionCodex, notTotallyMatichingOPVs, subTabGroup, pairWithOptionsToGroup, selectedItems, isMultiselect, groupsDefaultClosed, contentItemId);
    }

    /**
     * Updates the summary Items
     * @param summaryItems Items in the Summary
     */
  }, {
    key: "updateSummaryItems",
    value: function updateSummaryItems(ret, articlePriceFormat, stepInfo) {
      this.paletteViewHandler.updateSummaryItems(ret, articlePriceFormat, stepInfo, this.uiState.checkoutConfig);
    }
  }, {
    key: "updateStaticRoomListItems",
    value: function updateStaticRoomListItems(roomListItems, dontUpdateView) {
      this.paletteViewHandler.updateStaticRoomListItems(roomListItems, dontUpdateView);
    }
  }, {
    key: "updateShelfPlannerItems",
    value: function updateShelfPlannerItems(paletteId, accessoires, grids, configStepKey, from) {
      this.paletteViewHandler.updateShelfPlannerItems(paletteId, accessoires, grids, configStepKey, from);
    }
  }, {
    key: "updateFreePlacesOnGrids",
    value: function updateFreePlacesOnGrids(data) {
      var _data$freeShelfGrids;
      this.settingsState.setGridsData((_data$freeShelfGrids = data.freeShelfGrids) !== null && _data$freeShelfGrids !== void 0 ? _data$freeShelfGrids : [], data.plannedShelfGrids);
    }
  }, {
    key: "updateDoorPlannerItems",
    value: function updateDoorPlannerItems(doorPlannerViewItems, configStepKey) {
      this.paletteViewHandler.updateDoorPlannerItems(doorPlannerViewItems, configStepKey);
    }
  }, {
    key: "updatePaletteInfos",
    value: function updatePaletteInfos(articleInfos, doShowConfigSingle, isConfigSingle) {
      var image = articleInfos[Number(Object.keys(articleInfos)[0])].image;
      if (image) this.productState.setArticleImage(image);
      this.paletteViewHandler.updatePaletteInfos(articleInfos, doShowConfigSingle, isConfigSingle);
    }
  }, {
    key: "getI18nInstance",
    value: function getI18nInstance() {
      var _this$uiState;
      return (_this$uiState = this.uiState) === null || _this$uiState === void 0 ? void 0 : _this$uiState.i18nInstance;
    }
  }, {
    key: "eventIsManagedByClient",
    value: function eventIsManagedByClient(event) {
      var _this$getViewConfig$c;
      return !!((_this$getViewConfig$c = this.getViewConfig().clientEvents) !== null && _this$getViewConfig$c !== void 0 && _this$getViewConfig$c.includes(event));
    }
  }, {
    key: "setHQRenderScript",
    value: function setHQRenderScript(hqRenderScript) {
      this.hqRenderScript = hqRenderScript;
    }
  }, {
    key: "setCamInfo",
    value: function setCamInfo(camInfo) {
      this.camInfo = camInfo;
    }
  }, {
    key: "openARObserverConnection",
    value: function openARObserverConnection(handleNoHqrs) {
      return this.divaWebPlanner.openARObserverConnection(handleNoHqrs);
    }
  }, {
    key: "registerFunctions",
    value: function registerFunctions() {
      //Register all functions that are called from the logic
      this.divaWebPlanner.setI18n(this.getI18nInstance.bind(this));
      this.divaWebPlanner.registerInitPlannerView(this.initPlannerView.bind(this));
      this.divaWebPlanner.registerInitConfiguratorMode(this.productState.initConfiguratorMode.bind(this.productState));
      this.divaWebPlanner.registerInitConfiguratorStructure(this.initConfiguratorStructure.bind(this));
      this.divaWebPlanner.registerUpdateConfiguratorStructure(this.updateConfiguratorStructure.bind(this));
      this.divaWebPlanner.registerNavigateToTabGroup(this.navigateToTabGroup.bind(this));
      this.divaWebPlanner.registerNavigateToConfigStep(this.navigateToConfigStep.bind(this));
      this.divaWebPlanner.registerUpdateCatalogPlannerArticles(this.updateCatalogPlannerArticles.bind(this));
      this.divaWebPlanner.registerUpdateAccessoiresPalette(this.updateAccessoiresPalette.bind(this));
      this.divaWebPlanner.registerSetWebConifguratorTitle(this.setWebConifguratorTitle.bind(this));
      this.divaWebPlanner.registerOnBasketInitialized(this.onBasketInitialized.bind(this));
      this.divaWebPlanner.registerUpdateConfigItems(this.updateConfigItems.bind(this));
      this.divaWebPlanner.registerUpdatePaletteInfos(this.updatePaletteInfos.bind(this));
      this.divaWebPlanner.registerUpdatePlannerElements(this.viewPlannerHandler.updatePlannerElements.bind(this.viewPlannerHandler));
      this.divaWebPlanner.registerUpdateElementSelectionConfig(this.viewPlannerHandler.updateElementSelectionConfig.bind(this.viewPlannerHandler));
      this.divaWebPlanner.registerUpdateElementSelectionPlanner(this.viewPlannerHandler.updateElementSelectionPlanner.bind(this.viewPlannerHandler));
      this.divaWebPlanner.registerGetCurPlannerViewMode(this.viewPlannerHandler.getCurPlannerViewMode.bind(this.viewPlannerHandler));
      this.divaWebPlanner.registerGetInitPlannerViewMode(this.viewPlannerHandler.getInitPlannerViewMode.bind(this.viewPlannerHandler));
      this.divaWebPlanner.registerUpdatePlanner3DImage(this.viewPlannerHandler.updatePlanner3DImage.bind(this.viewPlannerHandler));
      this.divaWebPlanner.registerSetRotationOrbitCube(this.viewPlannerHandler.setRotationOfOrbitCube.bind(this.viewPlannerHandler));
      this.divaWebPlanner.registerGetCanvas2DAsJPG(this.viewPlannerHandler.get2DPlanning2DAsJPG.bind(this));
      this.divaWebPlanner.registerUpdatePlannerMsgContainer(this.productState.updatePlannerMsgContainer.bind(this.productState));
      this.divaWebPlanner.registerUpdateFullScreenRendering(this.updateFullscreenRendering.bind(this));
      this.divaWebPlanner.registerUpdateSetImage(this.productState.updateSetImage.bind(this.productState));
      this.divaWebPlanner.registerGetPlanner3DGridEnabled(this.viewPlannerHandler.getPlanner3DGridEnabled.bind(this.viewPlannerHandler));
      this.divaWebPlanner.registerGetDoShow3DDimensionLines(this.productState.getDoShow3DDimensionLines.bind(this.productState));
      this.divaWebPlanner.registerGetCurrentAlternativRendering(this.getCurrentAlternativRendering.bind(this));
      this.divaWebPlanner.registerUpdatePlanner2DGrid(this.viewPlannerHandler.updatePlanner2DGrid.bind(this.viewPlannerHandler));
      this.divaWebPlanner.registerGetPlannerRenderingSize(this.productState.getPlannerRenderingSize.bind(this.productState));
      this.divaWebPlanner.registerUpdateSummaryItems(this.updateSummaryItems.bind(this));
      this.divaWebPlanner.registerUpdateStaticRoomListItems(this.updateStaticRoomListItems.bind(this));
      this.divaWebPlanner.registerUpdateShelfPlannerItems(this.updateShelfPlannerItems.bind(this));
      this.divaWebPlanner.registerUpdateShelfPlannerGrids(this.updateFreePlacesOnGrids.bind(this));
      this.divaWebPlanner.registerUpdateDoorPlannerItems(this.updateDoorPlannerItems.bind(this));
      this.divaWebPlanner.registerUpdatePrice(this.updatePrice.bind(this));
      this.divaWebPlanner.registerOpenLeafeltBarrier(this.barrierHandler.openLeafeltBarrier.bind(this.barrierHandler));
      this.divaWebPlanner.registerOnClickAddToBasket(this.actionButtonState.onClickAddToBasket.bind(this.actionButtonState));
      this.divaWebPlanner.registerUpdateViewButtonsDependigOnConfigStructure(this.actionButtonState.initForUpdate.bind(this.actionButtonState));
      this.divaWebPlanner.registerInitApiDependencies(this.initApiDependencies.bind(this));
      this.divaWebPlanner.registerSetHQRenderScript(this.setHQRenderScript.bind(this));
      this.divaWebPlanner.registerSetCamInfo(this.setCamInfo.bind(this));
      this.divaWebPlanner.registerOpenContactForm(this.barrierHandler.openContactForm.bind(this.barrierHandler));
    }
  }]);
}();

var _excluded = ["forceReload"];
var viewHandlerContext = /*#__PURE__*/createContext(null);
var ViewHandlerProvider = function ViewHandlerProvider(_ref) {
  var _plannerConfig$settin;
  var children = _ref.children,
    plannerConfig = _ref.plannerConfig,
    isVisible = _ref.isVisible;
  var _useTranslation = useTranslation(),
    i18n = _useTranslation.i18n,
    t = _useTranslation.t;
  var _useState = useState(false),
    _useState2 = _slicedToArray(_useState, 2),
    initialized = _useState2[0],
    setInitialized = _useState2[1];
  var _useState3 = useState(),
    _useState4 = _slicedToArray(_useState3, 2),
    addDivaNr = _useState4[0],
    setAddDivaNr = _useState4[1];
  var _useDivaCore = useDivaCore(),
    _useDivaCore$state = _useDivaCore.state,
    user = _useDivaCore$state.user,
    organization = _useDivaCore$state.organization,
    isFullscreenComponent = _useDivaCore$state.isFullscreenComponent,
    openModal = _useDivaCore.actions.openModal;
  if (!isVisible && isFullscreenComponent && (plannerConfig === null || plannerConfig === void 0 || (_plannerConfig$settin = plannerConfig.settings) === null || _plannerConfig$settin === void 0 ? void 0 : _plannerConfig$settin.applicationEnvironment) == 'WEB') {
    isVisible = true;
  }
  var viewHandler = useMemo(function () {
    if (plannerConfig.addDivaNr) {
      plannerConfig.divaNr = plannerConfig.addDivaNr;
      setAddDivaNr(plannerConfig.addDivaNr);
    }
    var newViewHandler = new DivaViewHandler(plannerConfig, isVisible);
    newViewHandler.initialize(true);
    if (plannerConfig.productId) {
      newViewHandler.configState.currentInitializedProductId = plannerConfig.productId;
    }
    return newViewHandler;
  }, []);
  var keyDownFunction = useCallback(function (e) {
    if (!viewHandler) return;
    if (e.key === 'Enter' && viewHandler.getViewConfig().navigateOnEnter) {
      if (!viewHandler.tabGroupState.currentStepIsVisible) {
        var firstStep = viewHandler.tabGroupState.firstVisibleConfigStep;
        if (firstStep) viewHandler.navigateToConfigStep(firstStep.key);
      } else viewHandler.actionButtonState.onClickConfiguratorSettingsForth();
    }
    if (viewHandler.getViewConfig().listenToStrgF && e.keyCode === 114 || viewHandler.getViewConfig().listenToStrgF && (e.ctrlKey || e.metaKey) && e.keyCode === 70) {
      //focus on search
      if (viewHandler.paletteViewHandler.focusSearch()) e.preventDefault();
    }
  }, []);
  useEffect(function () {
    document.addEventListener('keydown', keyDownFunction, false);
    return function () {
      document.removeEventListener('keydown', keyDownFunction, false);
    };
  }, []);
  useEffect(function () {
    initConfigurator();
  }, [plannerConfig.forceReload, plannerConfig.divaNr, plannerConfig.addDivaNr, plannerConfig.productId, plannerConfig.catalogCodex, plannerConfig.catalogGuid, JSON.stringify(plannerConfig.variantValues), JSON.stringify(plannerConfig.articleCodexes), JSON.stringify(plannerConfig.differentOptions), plannerConfig.setId, plannerConfig.lang.toUpperCase(), plannerConfig.returnTo, plannerConfig.settings, plannerConfig.loginMode, plannerConfig.resetArticle, organization]);
  useEffect(function () {
    triggerUpdateByParams();
  }, [plannerConfig.opt, plannerConfig.opv]);
  var initConfigurator = /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      var _viewHandler$divaWebP, _viewHandler$divaWebP2, _plannerConfig$varian, _plannerConfig$varian2, _plannerConfig$varian3, articles, article, options, _plannerConfig$action, _ref3, otherInternals;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            if (viewHandler.initializedWithErrors) {
              viewHandler.configState.setConfigParams(plannerConfig);
              viewHandler.initialize(false, viewHandler.configState);
              viewHandler.initializedWithErrors = false;
            }
            if (initialized) {
              _context.next = 5;
              break;
            }
            setInitialized(true);
            _context.next = 46;
            break;
          case 5:
            if (!(viewHandler.configState.divaNr != plannerConfig.divaNr && plannerConfig.addDivaNr == undefined || viewHandler.configState.customProductId != plannerConfig.customProductId || viewHandler.configState.catalogCodex != plannerConfig.catalogCodex || JSON.stringify(viewHandler.configState.articleCodexes) != JSON.stringify(plannerConfig.articleCodexes) || viewHandler.configState.setId != plannerConfig.setId || viewHandler.configState.catalogGuid != plannerConfig.catalogGuid || viewHandler.configState.lang.toUpperCase() != plannerConfig.lang.toUpperCase() || viewHandler.configState.initMode != plannerConfig.initMode || viewHandler.configState.loginMode != plannerConfig.loginMode || viewHandler.configState.organizationId != plannerConfig.organizationId || viewHandler.configState.returnTo != plannerConfig.returnTo || viewHandler.configState.currentInitializedProductId != plannerConfig.productId && plannerConfig.productId !== undefined && !((_viewHandler$divaWebP = viewHandler.divaWebPlanner) !== null && _viewHandler$divaWebP !== void 0 && _viewHandler$divaWebP.doAllowSetPlanner()) || plannerConfig.forceReload != undefined)) {
              _context.next = 11;
              break;
            }
            //something changed
            if (viewHandler.configState.currentInitializedProductId != plannerConfig.productId) {
              viewHandler.configState.currentInitializedProductId = plannerConfig.productId;
            }
            viewHandler.configState.setConfigParams(plannerConfig);
            viewHandler.initialize(false, viewHandler.configState);
            _context.next = 45;
            break;
          case 11:
            if (!(plannerConfig.addDivaNr && (addDivaNr != plannerConfig.addDivaNr || plannerConfig.forceReload != undefined))) {
              _context.next = 16;
              break;
            }
            // load / add divaNr
            setAddDivaNr(plannerConfig.addDivaNr);
            viewHandler.addDivaNrToSet(plannerConfig.addDivaNr);
            _context.next = 45;
            break;
          case 16:
            if (!(plannerConfig.productId && (viewHandler.configState.currentInitializedProductId != plannerConfig.productId || plannerConfig.forceReload != undefined))) {
              _context.next = 21;
              break;
            }
            // load / add product
            viewHandler.configState.currentInitializedProductId = plannerConfig.productId;
            viewHandler.addProduct(plannerConfig.productId);
            _context.next = 45;
            break;
          case 21:
            if (!plannerConfig.forceReload) {
              _context.next = 25;
              break;
            }
            if (plannerConfig.forceReload == 'check') {
              openModal({
                title: t('modals.reset'),
                accept: true,
                reject: true,
                text: t('modals.are_you_sure_to_reset'),
                callback: function callback(accepted) {
                  if (accepted) {
                    viewHandler.initialize(false, viewHandler.configState);
                  }
                }
              });
            } else {
              viewHandler.configState.setConfigParams(plannerConfig);
              viewHandler.initialize(false, viewHandler.configState);
            }
            _context.next = 45;
            break;
          case 25:
            if (!(JSON.stringify(viewHandler.configState.variantValues) != JSON.stringify(plannerConfig.variantValues) || JSON.stringify(viewHandler.configState.differentOptions) != JSON.stringify(plannerConfig.differentOptions) || plannerConfig.resetArticle)) {
              _context.next = 44;
              break;
            }
            // only variant values change. means only to update the config

            viewHandler.configState.setConfigParams(plannerConfig);
            articles = (_viewHandler$divaWebP2 = viewHandler.divaWebPlanner.getDivaArticles()) !== null && _viewHandler$divaWebP2 !== void 0 ? _viewHandler$divaWebP2 : {};
            article = articles[Object.keys(articles)[0]];
            if (article) {
              _context.next = 31;
              break;
            }
            return _context.abrupt("return");
          case 31:
            if (!plannerConfig.resetArticle) {
              _context.next = 34;
              break;
            }
            _context.next = 34;
            return viewHandler.divaWebPlanner.resetArticle(article.basketId);
          case 34:
            if (viewHandler.configState.differentOptions) viewHandler.divaWebPlanner.setDifferentOptions(viewHandler.configState.differentOptions);
            _context.next = 37;
            return viewHandler.divaWebPlanner.doConfigure((_plannerConfig$varian = plannerConfig.variantValues) !== null && _plannerConfig$varian !== void 0 ? _plannerConfig$varian : {}, viewHandler.settingsState.allBaksetIds, false, true);
          case 37:
            options = Object.keys((_plannerConfig$varian2 = plannerConfig.variantValues) !== null && _plannerConfig$varian2 !== void 0 ? _plannerConfig$varian2 : {});
            viewHandler.paletteViewHandler.visitOptions(options.filter(function (o) {
              return article.getArticleOptionState(o);
            }).map(function (o) {
              return article.getArticleOptionState(o).optionCodex;
            }).filter(function (o) {
              return !!o;
            }));
            _context.next = 41;
            return viewHandler.divaWebPlanner.doConfigure((_plannerConfig$varian3 = plannerConfig.variantValues) !== null && _plannerConfig$varian3 !== void 0 ? _plannerConfig$varian3 : {}, viewHandler.settingsState.allBaksetIds, false);
          case 41:
            viewHandler.paletteViewHandler.updateCurPaletteData(false);
            _context.next = 45;
            break;
          case 44:
            if (JSON.stringify(viewHandler.configState.settings) !== JSON.stringify(plannerConfig.settings)) {
              viewHandler.configState.setConfigParams(plannerConfig);
              viewHandler.updatePlannerConfig(viewHandler.configState);
            }
          case 45:
            if (plannerConfig.productId === undefined) viewHandler.configState.currentInitializedProductId = plannerConfig.productId;
          case 46:
            if (plannerConfig.forceReload != undefined && plannerConfig.actions) {
              // reset forceReload
              _ref3 = plannerConfig.internal || {}, _ref3.forceReload, otherInternals = _objectWithoutProperties(_ref3, _excluded);
              (_plannerConfig$action = plannerConfig.actions) === null || _plannerConfig$action === void 0 || _plannerConfig$action.openComponent({
                type: 'DIVA_WEBPLANNER',
                parameters: plannerConfig === null || plannerConfig === void 0 ? void 0 : plannerConfig.parameters,
                internal: otherInternals
              });
            }
          case 47:
          case "end":
            return _context.stop();
        }
      }, _callee);
    }));
    return function initConfigurator() {
      return _ref2.apply(this, arguments);
    };
  }();
  var triggerUpdateByParams = /*#__PURE__*/function () {
    var _ref4 = _asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
      var opt, configStep, opv, _configStep;
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) switch (_context2.prev = _context2.next) {
          case 0:
            if (initialized) {
              _context2.next = 2;
              break;
            }
            return _context2.abrupt("return");
          case 2:
            if (!plannerConfig.opt) {
              _context2.next = 8;
              break;
            }
            opt = plannerConfig.opt;
            configStep = viewHandler.configuratorStructureHandler.getFirstConfigStepByOptionCodex(opt);
            if (configStep) {
              viewHandler.navigateToConfigStep(configStep.key);
              viewHandler.paletteViewHandler.scrollToConfigStepByPosNr(configStep.posNr, true);
            }
            _context2.next = 16;
            break;
          case 8:
            if (!plannerConfig.opv) {
              _context2.next = 16;
              break;
            }
            opv = plannerConfig.opv;
            _configStep = viewHandler.configuratorStructureHandler.getFirstConfigStepByVisibleOptionValueCodex(opv);
            if (!_configStep) {
              _context2.next = 16;
              break;
            }
            viewHandler.navigateToConfigStep(_configStep.key, true);
            _context2.next = 15;
            return DivaUtils.Sleep(300);
          case 15:
            viewHandler.paletteViewHandler.scrollToOPV(_configStep.posNr, opv);
          case 16:
          case "end":
            return _context2.stop();
        }
      }, _callee2);
    }));
    return function triggerUpdateByParams() {
      return _ref4.apply(this, arguments);
    };
  }();
  useEffect(function () {
    viewHandler.uiState.i18nInstance = i18n;
  }, [i18n]);
  useEffect(function () {
    viewHandler.uiState.setUser(user);
  }, [user]);
  useEffect(function () {
    viewHandler.uiState.setOrganization(organization);
  }, [organization]);
  useEffect(function () {
    viewHandler.isParentViewVisible = isVisible;
    if (!isVisible) {
      viewHandler.productState.doShowRTV = false;
      viewHandler.settingsState.onClickContentItemId();
    }
  }, [isVisible]);
  return /*#__PURE__*/jsx(viewHandlerContext.Provider, {
    value: viewHandler,
    children: children
  });
};
var useViewHandler = function useViewHandler() {
  var viewHandler = React.useContext(viewHandlerContext);
  if (!viewHandler) {
    throw new Error('useViewHandler must be used within a ViewHandlerProvider.');
  }
  return viewHandler;
};
var usePlannerUIHandler = function usePlannerUIHandler() {
  var viewHandler = useViewHandler();
  if (!viewHandler) {
    throw new Error('usePlannerUIHandler must be used within a ViewHandlerProvider.');
  }
  return viewHandler.viewPlannerHandler.PlannerUIHandler;
};

function __variableDynamicImportRuntime0__(path) {
  switch (path) {
    case '../components/Layout/index.tsx': return Promise.resolve().then(function () { return index$e; });
    case '../components/Settings/index.tsx': return import('./index-ecc85fdb.js');
    case '../components/TabGroup/index.tsx': return import('./index-9bbd0638.js');
    case '../components/Action/Basket/index.tsx': return import('./index-0224a06e.js');
    case '../components/Action/BasketMobile/index.tsx': return import('./index-02175451.js');
    case '../components/Action/Navigation/index.tsx': return import('./index-30cc40bb.js');
    case '../components/GlobalElements/Affiliate/index.tsx': return Promise.resolve().then(function () { return index; });
    case '../components/GlobalElements/BarrierActions/index.tsx': return import('./index-f9b64766.js');
    case '../components/GlobalElements/Checkout/index.tsx': return Promise.resolve().then(function () { return index$3; });
    case '../components/GlobalElements/FrameworkPrice/index.tsx': return Promise.resolve().then(function () { return index$2; });
    case '../components/GlobalElements/Fullscreen/index.tsx': return Promise.resolve().then(function () { return index$1; });
    case '../components/Layout/classification/index.tsx': return import('./index-e60ae1da.js');
    case '../components/Layout/gold/index.tsx': return import('./index-c7e0197d.js');
    case '../components/Layout/ralph/index.tsx': return import('./index-7578a762.js');
    case '../components/Layout/serious/index.tsx': return import('./index-05be5f49.js');
    case '../components/Layout/seriousRight/index.tsx': return import('./index-c76be027.js');
    case '../components/Layout/standard/index.tsx': return import('./index-040b30fc.js');
    case '../components/Layout/wysiwyg/index.tsx': return import('./index-7a3b99b5.js');
    case '../components/Product/ContentItemWrapper/index.tsx': return import('./index-5840610f.js');
    case '../components/Product/Product/index.tsx': return import('./index-75079ab1.js');
    case '../components/Product/ProductCenter/index.tsx': return import('./index-f4b93084.js');
    case '../components/Product/ProductFooter/index.tsx': return import('./index-3b9e0a7d.js');
    case '../components/Product/ProductInfo/index.tsx': return import('./index-12f39bd1.js');
    case '../components/Product/ProductSettings/index.tsx': return import('./index-a9e7ae3a.js');
    case '../components/Settings/SettingsHeaderContainer/index.tsx': return import('./index-d5750f25.js');
    case '../components/Settings/SettingsScroller/index.tsx': return import('./index-6a14c4ba.js');
    case '../components/TabGroup/ProgressBar/index.tsx': return import('./index-8f60e411.js');
    case '../components/TabGroup/Search/index.tsx': return import('./index-c8380cdc.js');
    case '../components/TabGroup/TabGroupHorizontal/index.tsx': return import('./index-fcc87454.js');
    case '../components/TabGroup/TabGroupVertical/index.tsx': return import('./index-d7e7bc22.js');
    case '../components/Action/Basket/gold/index.tsx': return import('./index-7e6e3b96.js');
    case '../components/Action/Basket/ralph/index.tsx': return import('./index-95da9a36.js');
    case '../components/Action/Basket/serious/index.tsx': return import('./index-22688611.js');
    case '../components/Action/Basket/standard/index.tsx': return import('./index-d7af0a97.js');
    case '../components/Action/Navigation/blue/index.tsx': return import('./index-32169a91.js');
    case '../components/Action/Navigation/gold/index.tsx': return import('./index-c46d7d70.js');
    case '../components/Action/Navigation/ralph/index.tsx': return import('./index-85ea44be.js');
    case '../components/Action/Navigation/standard/index.tsx': return import('./index-d0f67e02.js');
    case '../components/GlobalElements/BarrierActions/blue/index.tsx': return import('./index-a6d56f01.js');
    case '../components/GlobalElements/BarrierActions/jungle/index.tsx': return import('./index-c7542aeb.js');
    case '../components/GlobalElements/BarrierActions/standard/index.tsx': return import('./index-459de627.js');
    case '../components/GlobalElements/Checkout/AGB/index.tsx': return Promise.resolve().then(function () { return index$d; });
    case '../components/GlobalElements/Checkout/CreateAppointment/index.tsx': return Promise.resolve().then(function () { return index$c; });
    case '../components/GlobalElements/Checkout/Header/index.tsx': return Promise.resolve().then(function () { return index$5; });
    case '../components/GlobalElements/Checkout/IconComponent/index.tsx': return Promise.resolve().then(function () { return index$4; });
    case '../components/GlobalElements/Checkout/Save/index.tsx': return Promise.resolve().then(function () { return index$6; });
    case '../components/GlobalElements/Checkout/Share/index.tsx': return Promise.resolve().then(function () { return index$7; });
    case '../components/GlobalElements/PopupWindows/Inputbox/index.tsx': return import('./index-784d4adc.js');
    case '../components/Layout/wysiwyg/Toolbar/index.tsx': return import('./index-937d3306.js');
    case '../components/Product/Product/catalog/index.tsx': return import('./index-d6f5d277.js');
    case '../components/Product/Product/serious/index.tsx': return import('./index-9168be77.js');
    case '../components/Product/Product/standard/index.tsx': return import('./index-502da53a.js');
    case '../components/Product/ProductCenter/MobileNavIcons/index.tsx': return import('./index-dffb0718.js');
    case '../components/Product/ProductCenter/Planner2DGird/index.tsx': return import('./index-2a3b670f.js');
    case '../components/Product/ProductCenter/Planner3D/index.tsx': return import('./index-8c24cd90.js');
    case '../components/Product/ProductCenter/Planner3DGrid/index.tsx': return import('./index-cfae8a69.js');
    case '../components/Product/ProductCenter/Planner3DImage/index.tsx': return import('./index-0b374555.js');
    case '../components/Product/ProductCenter/PlannerMsgContainer/index.tsx': return import('./index-26008f13.js');
    case '../components/Product/ProductCenter/PlannerRTV/index.tsx': return import('./index-ac1eb76e.js');
    case '../components/Product/ProductCenter/PlannerUI/index.tsx': return import('./index-f4979347.js');
    case '../components/Product/ProductCenter/PromotedOpvs/index.tsx': return import('./index-780b98e2.js');
    case '../components/Product/ProductCenter/RoomListSelection/index.tsx': return import('./index-27d2d386.js');
    case '../components/Product/ProductCenter/RotateInfo/index.tsx': return import('./index-4d2c6f51.js');
    case '../components/Product/ProductFooter/gold/index.tsx': return import('./index-adda72ca.js');
    case '../components/Product/ProductFooter/ralph/index.tsx': return import('./index-b6b8ae9e.js');
    case '../components/Product/ProductFooter/standard/index.tsx': return import('./index-6f67533a.js');
    case '../components/Product/ProductHeader/Functions/index.tsx': return import('./index-4498f013.js');
    case '../components/Product/ProductHeader/ProductHeader/index.tsx': return import('./index-e3eefbe6.js');
    case '../components/Settings/SettingsScroller/SettingsItem/index.tsx': return import('./index-9ac352ac.js').then(function (n) { return n.i; });
    case '../components/Settings/SettingsScroller/SubTabGroup/index.tsx': return import('./index-e0e38ace.js');
    case '../components/TabGroup/TabGroupHorizontal/item/index.tsx': return import('./index-54442cec.js');
    case '../components/TabGroup/TabGroupVertical/ItemVariants/index.tsx': return import('./index-9309f785.js');
    case '../components/Action/Basket/ralph/PriceSumm/index.tsx': return import('./index-7dd50f24.js');
    case '../components/GlobalElements/Checkout/Share/MultiToggleSwitch/index.tsx': return Promise.resolve().then(function () { return index$b; });
    case '../components/GlobalElements/Checkout/Share/ShareViaEmail/index.tsx': return Promise.resolve().then(function () { return index$8; });
    case '../components/GlobalElements/Checkout/Share/ShareViaSocialMedias/index.tsx': return Promise.resolve().then(function () { return index$9; });
    case '../components/GlobalElements/Checkout/Share/ShareViaUrl/index.tsx': return Promise.resolve().then(function () { return index$a; });
    case '../components/Layout/wysiwyg/Toolbar/ToolContainer/index.tsx': return import('./index-4cca5326.js');
    case '../components/Product/Product/catalog/Sidebar/index.tsx': return import('./index-65d88ad9.js');
    case '../components/Product/Product/serious/CheckoutForm/index.tsx': return import('./index-6303f649.js');
    case '../components/Product/ProductCenter/Planner3DImage/Logo/index.tsx': return import('./index-e863915f.js');
    case '../components/Product/ProductCenter/Planner3DImage/NavBar/index.tsx': return import('./index-9eae4b33.js');
    case '../components/Product/ProductCenter/Planner3DImage/OrbitInteraction/index.tsx': return import('./index-794f08b4.js');
    case '../components/Product/ProductCenter/PlannerMsgContainer/standard/index.tsx': return import('./index-3d1120ed.js');
    case '../components/Product/ProductCenter/PlannerUI/ItemsContainer/index.tsx': return import('./index-831b8abe.js');
    case '../components/Product/ProductCenter/PlannerUI/PlannerUIVariants/index.tsx': return import('./index-7f0c8231.js');
    case '../components/Product/ProductCenter/RoomListSelection/serious/index.tsx': return import('./index-0ffe9a6c.js');
    case '../components/Product/ProductCenter/RoomListSelection/standard/index.tsx': return import('./index-35c37d4f.js');
    case '../components/Product/ProductHeader/Functions/gold/index.tsx': return import('./index-e73936f8.js');
    case '../components/Product/ProductHeader/Functions/ralph/index.tsx': return import('./index-512a9a8a.js');
    case '../components/Product/ProductHeader/Functions/standard/index.tsx': return import('./index-b84555c0.js');
    case '../components/Product/ProductHeader/ProductHeader/blue/index.tsx': return import('./index-fc305c9d.js');
    case '../components/Product/ProductHeader/ProductHeader/gold/index.tsx': return import('./index-be461462.js');
    case '../components/Product/ProductHeader/ProductHeader/ralph/index.tsx': return import('./index-8c9dfa29.js');
    case '../components/Product/ProductHeader/ProductHeader/standard/index.tsx': return import('./index-6511abf0.js');
    case '../components/Settings/SettingsHeaderContainer/Filter/DockFilter/index.tsx': return import('./index-bccbc243.js');
    case '../components/Settings/SettingsHeaderContainer/Filter/RALFilter/index.tsx': return import('./index-18e04d12.js');
    case '../components/Settings/SettingsHeaderContainer/Filter/Searchbar/index.tsx': return import('./index-576885e5.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/Header/index.tsx': return import('./index-bec7758a.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/SettingsConfigSingle/index.tsx': return import('./index-f0b13a76.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/SettingsPosNrContainer/index.tsx': return import('./index-9e5c6d0b.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ComponentMap/index.tsx': return import('./index-63a72d8c.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemWrapper/index.tsx': return import('./index-3dac3051.js');
    case '../components/Settings/SettingsScroller/SettingsItem/PaletteGroup/index.tsx': return import('./index-dab96ea8.js');
    case '../components/Settings/SettingsScroller/SettingsItem/PaletteGroupDropdown/index.tsx': return import('./index-227ad099.js');
    case '../components/Settings/SettingsScroller/SettingsItem/StepInfo/index.tsx': return import('./index-ec30cce1.js');
    case '../components/Settings/SettingsScroller/SubTabGroup/SubTabGroupHeader/index.tsx': return import('./index-fc676393.js');
    case '../components/TabGroup/TabGroupHorizontal/item/gold/index.tsx': return import('./index-ac081343.js');
    case '../components/TabGroup/TabGroupHorizontal/item/ralph/index.tsx': return import('./index-75e4d43b.js');
    case '../components/TabGroup/TabGroupHorizontal/item/standard/index.tsx': return import('./index-db93605e.js');
    case '../components/TabGroup/TabGroupVertical/ItemVariants/classification/index.tsx': return import('./index-9687ef69.js');
    case '../components/TabGroup/TabGroupVertical/ItemVariants/free/index.tsx': return import('./index-cd4a0711.js');
    case '../components/TabGroup/TabGroupVertical/ItemVariants/mountain/index.tsx': return import('./index-31bd7b19.js');
    case '../components/TabGroup/TabGroupVertical/ItemVariants/serious/index.tsx': return import('./index-1aeb684e.js');
    case '../components/TabGroup/TabGroupVertical/ItemVariants/standard/index.tsx': return import('./index-72797c1c.js');
    case '../components/Product/Product/catalog/Components/ArticleOverview/index.tsx': return import('./index-76e01c42.js');
    case '../components/Product/Product/catalog/Components/ArticleSelector/index.tsx': return import('./index-e1eb89c3.js');
    case '../components/Product/Product/catalog/Components/CatalogMakeup/index.tsx': return import('./index-99f2670a.js');
    case '../components/Product/Product/catalog/Components/DecisionAnalyzer/DecisionVizualizer.tsx': return import('./DecisionVizualizer-5fd20957.js').then(function (n) { return n.a; });
    case '../components/Product/Product/catalog/Components/DecisionAnalyzer/index.tsx': return import('./index-abb51d3b.js');
    case '../components/Product/Product/catalog/Components/EkTextViewer/index.tsx': return import('./index-8a17202a.js');
    case '../components/Product/Product/catalog/Components/J3DEditor/index.tsx': return import('./index-fdff31e1.js');
    case '../components/Product/Product/catalog/Components/JsonAnalyser/index.tsx': return import('./index-3f4573b4.js');
    case '../components/Product/Product/catalog/Components/MaskEditor/index.tsx': return import('./index-c761c9ae.js');
    case '../components/Product/Product/catalog/Components/OptionTree/index.tsx': return import('./index-3149a590.js');
    case '../components/Product/Product/catalog/Components/PartList/index.tsx': return import('./index-6ec1e2d8.js');
    case '../components/Product/Product/catalog/Components/PriceAnalyzer/index.tsx': return import('./index-02358a12.js');
    case '../components/Product/Product/catalog/Components/PropertiesEditor/index.tsx': return import('./index-eeaeb0ef.js');
    case '../components/Product/Product/catalog/Components/RTVAnalyzer/index.tsx': return import('./index-e6d14062.js');
    case '../components/Product/Product/catalog/Components/SVGAnalyser/index.tsx': return import('./index-a5a35a65.js');
    case '../components/Product/ProductCenter/Planner3DImage/NavBar/gold/index.tsx': return import('./index-aa7fa69e.js');
    case '../components/Product/ProductCenter/Planner3DImage/NavBar/serious/index.tsx': return import('./index-c395b9a3.js');
    case '../components/Product/ProductCenter/Planner3DImage/NavBar/standard/index.tsx': return import('./index-22128600.js');
    case '../components/Product/ProductCenter/Planner3DImage/OrbitInteraction/serious/index.tsx': return import('./index-35e40be5.js');
    case '../components/Product/ProductCenter/Planner3DImage/OrbitInteraction/standard/index.tsx': return import('./index-c5b8e85a.js');
    case '../components/Product/ProductCenter/PlannerMsgContainer/standard/PlannerMsgItem/index.tsx': return import('./index-cd429f5c.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Move3DGrid/index.tsx': return import('./index-adbd3100.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Outline/index.tsx': return import('./index-b47706ce.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Outline3D/index.tsx': return import('./index-93bd143d.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/PosNr/index.tsx': return import('./index-221a1630.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Trash/index.tsx': return import('./index-7c069afc.js');
    case '../components/Product/ProductCenter/PlannerUI/ItemsContainer/ItemsDPAccContainer/index.tsx': return import('./index-3ba6d4ce.js').then(function (n) { return n.i; });
    case '../components/Product/ProductCenter/PlannerUI/PlannerUIVariants/gold/index.tsx': return import('./index-b7bf05d9.js');
    case '../components/Product/ProductCenter/PlannerUI/PlannerUIVariants/hidden/index.tsx': return import('./index-9d9294e2.js');
    case '../components/Product/ProductCenter/PlannerUI/PlannerUIVariants/standard/index.tsx': return import('./index-0e23cdd0.js');
    case '../components/Product/ProductHeader/Functions/standard/share/index.tsx': return import('./index-eb5fc3b8.js');
    case '../components/Settings/SettingsHeaderContainer/Filter/RALFilter/DropdownFilter/index.tsx': return import('./index-d0cb686e.js');
    case '../components/Settings/SettingsHeaderContainer/Filter/RALFilter/SlideFilter/index.tsx': return import('./index-c43b9bf6.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/Header/serious/index.tsx': return import('./index-d3d7f7bf.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/Header/standard/index.tsx': return import('./index-9dcf09fb.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/SettingsPosNrContainer/SettingsPosNr/index.tsx': return import('./index-f919fe39.js').then(function (n) { return n.i; });
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/SettingsPosNrContainer/SettingsPosNrToolTip/index.tsx': return import('./index-2bdd798a.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Accessoires/index.tsx': return import('./index-79b89ba2.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/DoorPlanner/index.tsx': return import('./index-ca62a83e.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/ProductList/index.tsx': return import('./index-093c7922.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Promoted/index.tsx': return import('./index-ea0c61b7.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/ShelfPlanner/index.tsx': return import('./index-dbf150a1.js');
    case '../components/Settings/SettingsScroller/SettingsItem/PaletteGroup/jungle/index.tsx': return import('./index-a1b2be60.js');
    case '../components/Settings/SettingsScroller/SettingsItem/PaletteGroup/standard/index.tsx': return import('./index-4029c325.js');
    case '../components/TabGroup/TabGroupVertical/ItemVariants/classification/Header/index.tsx': return import('./index-8723de1f.js');
    case '../components/TabGroup/TabGroupVertical/ItemVariants/classification/configstep/index.tsx': return import('./index-670caa1d.js');
    case '../components/Product/Product/catalog/Components/ArticleOverview/JsonEditor/index.tsx': return import('./index-3b53b8af.js');
    case '../components/Product/Product/catalog/Components/ArticleOverview/JsonViewer/index.tsx': return import('./index-bbed3b5e.js');
    case '../components/Product/Product/catalog/Components/ArticleOverview/SVGViewer/index.tsx': return import('./index-363d60d0.js');
    case '../components/Product/Product/catalog/Components/CatalogMakeup/LayoutSelection/index.tsx': return import('./index-5431bb3a.js');
    case '../components/Product/Product/catalog/Components/CatalogMakeup/OptionSelector/index.tsx': return import('./index-3aa270fc.js');
    case '../components/Product/Product/catalog/Components/OptionTree/Price/index.tsx': return import('./index-098552ff.js');
    case '../components/Product/Product/catalog/Components/OptionTree/ProduktDetails/index.tsx': return import('./index-85970525.js');
    case '../components/Product/Product/catalog/Components/OptionTree/TabGroupTitle/index.tsx': return import('./index-dc516763.js');
    case '../components/Product/Product/catalog/Components/PartList/PartListTable/index.tsx': return import('./index-148f1439.js');
    case '../components/Product/Product/catalog/Components/PropertiesEditor/Properties/ArticleProperties.tsx': return import('./ArticleProperties-f0aadcc2.js');
    case '../components/Product/Product/catalog/Components/PropertiesEditor/Properties/CatalogProperties.tsx': return import('./CatalogProperties-e8824fd5.js');
    case '../components/Product/Product/catalog/Components/PropertiesEditor/Properties/OptionProperties.tsx': return import('./OptionProperties-a5e2a157.js');
    case '../components/Product/Product/catalog/Components/PropertiesEditor/Properties/OptionValueProperties.tsx': return import('./OptionValueProperties-d068ecbe.js');
    case '../components/Product/Product/catalog/Components/PropertiesEditor/Table/index.tsx': return import('./index-679accfc.js').then(function (n) { return n.i; });
    case '../components/Product/ProductCenter/Planner3DImage/NavBar/standard/ArQrCodeIcon/index.tsx': return import('./index-46b86e6c.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Copy/BoxCopy/index.tsx': return import('./index-de8c491e.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Copy/ElementCopy/index.tsx': return import('./index-7435b475.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Dimension/DistanceToGround/index.tsx': return import('./index-a8b51921.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/DockingPoint/AVL/index.tsx': return import('./index-1c76201d.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/DockingPoint/AVR/index.tsx': return import('./index-48300bf2.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/DockingPoint/DPACC/index.tsx': return import('./index-c8bb81dc.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Rotation/Rotation3DGrid/index.tsx': return import('./index-b3e8cff0.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Rotation/RotationClockwise/index.tsx': return import('./index-951a070a.js');
    case '../components/Product/ProductCenter/PlannerUI/Items/Rotation/RotationCounterClockwise/index.tsx': return import('./index-ff286eac.js');
    case '../components/Settings/SettingsHeaderContainer/Filter/RALFilter/DropdownFilter/Arrow/index.tsx': return import('./index-98a94c2a.js');
    case '../components/Settings/SettingsHeaderContainer/Filter/RALFilter/SlideFilter/FilterItem/index.tsx': return import('./index-a9750c3d.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/Header/standard/InfoButton/index.tsx': return import('./index-12a3b76a.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/Header/standard/NumberInput/index.tsx': return import('./index-874e49f3.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/Header/standard/PDFButton/index.tsx': return import('./index-64216061.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/Header/standard/SelectedOpv/index.tsx': return import('./index-c62648be.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/Header/standard/SummaryVariantButton/index.tsx': return import('./index-f70a9fa5.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Accessoires/serious/index.tsx': return import('./index-f8418b56.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Accessoires/standard/index.tsx': return import('./index-2269a4ff.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Article/Default/index.tsx': return import('./index-cbdc56a8.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Article/PlannerSwitch/index.tsx': return import('./index-a3492836.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Article/ReplaceSwitch/index.tsx': return import('./index-b4b3a823.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/GenericComponents/InfoIcon/index.tsx': return import('./index-a00c316e.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/GenericComponents/MetaData/index.tsx': return import('./index-f7d87c9f.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/GenericComponents/PricePreview/index.tsx': return import('./index-b813b7d7.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/GenericComponents/Tooltip/index.tsx': return import('./index-a53f6c9e.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/HeaderAndOption/DropDownText/index.tsx': return import('./index-7f563f4a.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/HeaderAndOption/TextBox/index.tsx': return import('./index-03ec1878.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Gallery/index.tsx': return import('./index-60ade46e.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/GalleryText/index.tsx': return import('./index-53b89034.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/IsActive/index.tsx': return import('./index-243b2e2c.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/ProductList/Search/index.tsx': return import('./index-60476d4d.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/ShelfPlanner/Grid/index.tsx': return import('./index-64c155c8.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/ShelfPlanner/SelectedAcc/index.tsx': return import('./index-5ebc2f7a.js').then(function (n) { return n.j; });
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/ShelfPlanner/ShelfAccessoires/index.tsx': return import('./index-b4eb85dd.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/ShelfPlanner/ShelfAccessoiresOld/index.tsx': return import('./index-7954790a.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Summary/Articles/index.tsx': return import('./index-5317aaf3.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Summary/OptionTree/index.tsx': return import('./index-123dc27c.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Summary/PartList/index.tsx': return import('./index-c6c521c0.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Summary/TotalPrice/index.tsx': return import('./index-46bea554.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Summary/Variants/index.tsx': return import('./index-b87282a4.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Toggle/ToggleElement/index.tsx': return import('./index-d9e0b8ab.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Toggle/ToggleHeader/index.tsx': return import('./index-837461cb.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Toggle/ToggleName/index.tsx': return import('./index-118cdb5f.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Toggle/ToggleOption/index.tsx': return import('./index-5ed724e9.js');
    case '../components/Settings/SettingsScroller/SettingsItem/PaletteGroup/jungle/group/index.tsx': return import('./index-137a559f.js');
    case '../components/Settings/SettingsScroller/SettingsItem/PaletteGroup/standard/Group/index.tsx': return import('./index-41f29148.js');
    case '../components/Product/Product/catalog/Components/ArticleOverview/SVGViewer/SVGSourceViewer/index.tsx': return import('./index-4008c20a.js');
    case '../components/Product/Product/catalog/Components/OptionTree/TabGroupTitle/Options/index.tsx': return import('./index-fe6bc7ff.js');
    case '../components/Product/Product/catalog/Components/PartList/PartListTable/AsyncRender/index.tsx': return import('./index-2e0d8ffb.js');
    case '../components/Product/ProductCenter/Planner3DImage/NavBar/standard/ArQrCodeIcon/FullscreenAr/index.tsx': return import('./index-cd5a0b3f.js');
    case '../components/Product/ProductCenter/Planner3DImage/NavBar/standard/ArQrCodeIcon/Tooltip/index.tsx': return import('./index-39923a1d.js');
    case '../components/Settings/SettingsHeaderContainer/SettingsItemHeader/Header/standard/SelectedOpv/Item/index.tsx': return import('./index-5eda5a49.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/HeaderAndOption/DropDownText/serious/index.tsx': return import('./index-678a64b3.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/HeaderAndOption/DropDownText/standard/index.tsx': return import('./index-edbae3cc.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Default/index.tsx': return import('./index-68efdadc.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/NumberBox/index.tsx': return import('./index-27943660.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Range/index.tsx': return import('./index-1e6362c6.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Gallery/serious/index.tsx': return import('./index-5cec85d6.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Gallery/standard/index.tsx': return import('./index-ce181fa9.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/IsActive/standard/index.tsx': return import('./index-53dfc1ca.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/ShelfPlanner/Grid/DeleteButton/index.tsx': return import('./index-964b1dbd.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Summary/OptionTree/serious/index.tsx': return import('./index-c81507c8.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Summary/OptionTree/standard/index.tsx': return import('./index-e985d6da.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/HeaderAndOption/DropDownText/standard/MobileDropdown/index.tsx': return import('./index-e9ac55b9.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Default/jungle/index.tsx': return import('./index-a1380061.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Default/serious/index.tsx': return import('./index-18d827e4.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Default/standard/index.tsx': return import('./index-fc7f27e2.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Room/RoomItem/index.tsx': return import('./index-b18bffae.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Room/ViewPoint/index.tsx': return import('./index-ab3f39ef.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Room/RoomItem/round/index.tsx': return import('./index-45ec87df.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Room/RoomItem/serious/index.tsx': return import('./index-9fdf3cf5.js');
    case '../components/Settings/SettingsScroller/SettingsItem/ItemPalette/Options/Default/Room/RoomItem/standard/index.tsx': return import('./index-13d4b3d1.js');
    default: return new Promise(function(resolve, reject) {
      (typeof queueMicrotask === 'function' ? queueMicrotask : setTimeout)(
        reject.bind(null, new Error("Unknown variable dynamic import: " + path))
      );
    })
   }
 }
var DivaWebPlannerMain = observer(function () {
  var viewHandler = useViewHandler();
  var _useDivaCore = useDivaCore(),
    _useDivaCore$state = _useDivaCore.state,
    dimensions = _useDivaCore$state.dimensions,
    hidePrices = _useDivaCore$state.hidePrices;
  useEffect(function () {
    var _viewHandler$uiState;
    (_viewHandler$uiState = viewHandler.uiState) === null || _viewHandler$uiState === void 0 || _viewHandler$uiState.resizeCore(dimensions);
  }, [dimensions.isMobile, dimensions.height, dimensions.width, dimensions.isPortrait, dimensions.isTablet, viewHandler.uiState.ViewConfig]);
  useEffect(function () {
    var _viewHandler$uiState2;
    (_viewHandler$uiState2 = viewHandler.uiState) === null || _viewHandler$uiState2 === void 0 || _viewHandler$uiState2.setHidePrices(hidePrices);
  }, [hidePrices]);
  var dynamicMultiRenderConfig = undefined;
  if (viewHandler.uiState.templateConfig) {
    dynamicMultiRenderConfig = {
      templateConfig: viewHandler.uiState.templateConfig,
      importFactory: function importFactory(path) {
        return __variableDynamicImportRuntime0__("../components/".concat(path, ".tsx"));
      }
    };
  }
  if (!viewHandler.uiState.templateConfigKeys) return /*#__PURE__*/jsx(Fragment, {});
  return /*#__PURE__*/jsx(Suspense, {
    fallback: /*#__PURE__*/jsx(Fragment, {}),
    children: /*#__PURE__*/jsx(DynamicMultiRenderProvider, {
      value: dynamicMultiRenderConfig,
      children: /*#__PURE__*/jsx(Layout, {})
    })
  });
});

function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var LOG = getLogger('Planner', 'PlannerInit');
// settings -> Webconfig
// parameters -> url params
// internal -> basketData usw

var WebPlannerReact = function WebPlannerReact(_ref) {
  var _parameters$organizat;
  var parameters = _ref.parameters,
    settings = _ref.settings,
    internal = _ref.internal,
    isVisible = _ref.isVisible;
  var _useDivaCore = useDivaCore(),
    state = _useDivaCore.state,
    actions = _useDivaCore.actions,
    handler = _useDivaCore.handler;
  var organization = state.organization,
    organizationHierarchy = state.organizationHierarchy,
    user = state.user,
    apiConfig = state.apiConfig,
    loginMode = state.loginMode,
    jwt = state.jwt,
    applicationEnvironment = state.applicationEnvironment;
  var initObject = useInitObject(actions.subscribeEvents, actions.unSubscribeEvents, settings, parameters, internal);
  var _useTranslation = useTranslation(),
    i18n = _useTranslation.i18n;
  useEffect(function () {
    if (isVisible === true && initObject || isVisible === undefined) {
      LOG.startTransaction({
        event: 'onOpenModule',
        message: 'Open Planner',
        eventId: 'ON_OPEN_PLANNER',
        errorTime: 5000,
        warningTime: 3000,
        labels: {
          productId: initObject === null || initObject === void 0 ? void 0 : initObject.productId,
          cat: initObject === null || initObject === void 0 ? void 0 : initObject.catalogCodex,
          artNr: JSON.stringify(initObject === null || initObject === void 0 ? void 0 : initObject.articleCodexes),
          lang: i18n.language
        }
      });
    }
  }, [isVisible]);
  useEffect(function () {
    if (internal) internal.accountSessionGuid = undefined;
  }, [organization]);
  if (initObject == undefined || !organization) {
    //do not load new Planer on first time, before we know if it is even necessary
    //if currentInitObject is undefined it is the first time that we are rendering a planner
    return /*#__PURE__*/jsx(Fragment, {});
  }
  return /*#__PURE__*/jsx(ViewHandlerProvider, {
    isVisible: isVisible || false,
    plannerConfig: _objectSpread(_objectSpread({}, initObject), {}, {
      //use the organizationId as parameter only for the classification use case
      organizationId: (_parameters$organizat = parameters === null || parameters === void 0 ? void 0 : parameters.organizationId) !== null && _parameters$organizat !== void 0 ? _parameters$organizat : organization._id,
      currentOrganization: parameters !== null && parameters !== void 0 && parameters.organizationId ? {
        _id: parameters === null || parameters === void 0 ? void 0 : parameters.organizationId
      } : organization,
      organizationHierarchy: organizationHierarchy.map(function (org) {
        return org._id;
      }),
      currentUser: user,
      apiConfig: _objectSpread(_objectSpread({}, apiConfig), {}, {
        api2: _objectSpread(_objectSpread({}, apiConfig.api2), {}, {
          "default": parameters !== null && parameters !== void 0 && parameters.api2Url ? parameters.api2Url : apiConfig.api2["default"]
        })
      }),
      loginMode: loginMode,
      jwt: jwt,
      lang: i18n.language,
      returnTo: parameters === null || parameters === void 0 ? void 0 : parameters.returnTo,
      paramsSendViaAdapt: true,
      parameters: parameters,
      internal: internal,
      settings: settings,
      actions: _objectSpread(_objectSpread({}, actions), internal !== null && internal !== void 0 && internal.actions ? _objectSpread({}, internal.actions) : {}),
      state: state,
      handler: handler,
      applicationEnvironment: applicationEnvironment
    }),
    children: /*#__PURE__*/jsx(DivaWebPlannerMain, {})
  });
};
function useInitObject(eventSubscriber, eventUnsubscriber, settings, parameters, internal) {
  return useMemo(function () {
    var _parameters$accountGu, _internal$accountSess, _internal$no3DConnect, _parameters$divaNr, _parameters$DivaNR, _parameters$divanr, _parameters$addDivaNr, _parameters$startDiva, _parameters$initProdu;
    console.log('parameters.opt', parameters === null || parameters === void 0 ? void 0 : parameters.opt);
    return {
      eventSubscriber: eventSubscriber,
      eventUnsubscriber: eventUnsubscriber,
      basketData: internal === null || internal === void 0 ? void 0 : internal.basketData,
      setId: parameters === null || parameters === void 0 ? void 0 : parameters.setId,
      accountId: parameters === null || parameters === void 0 ? void 0 : parameters.accountId,
      accountGuid: (_parameters$accountGu = parameters === null || parameters === void 0 ? void 0 : parameters.accountGuid) !== null && _parameters$accountGu !== void 0 ? _parameters$accountGu : parameters === null || parameters === void 0 ? void 0 : parameters.accountId,
      accountSessionGuid: (_internal$accountSess = internal === null || internal === void 0 ? void 0 : internal.accountSessionGuid) !== null && _internal$accountSess !== void 0 ? _internal$accountSess : parameters === null || parameters === void 0 ? void 0 : parameters.accountSessionGuid,
      supplierColors: internal === null || internal === void 0 ? void 0 : internal.supplierColors,
      smSupplierNr: parameters === null || parameters === void 0 ? void 0 : parameters.smSupplierNr,
      catalogMask: internal === null || internal === void 0 ? void 0 : internal.catalogMask,
      no3DConnection: (_internal$no3DConnect = internal === null || internal === void 0 ? void 0 : internal.no3DConnection) !== null && _internal$no3DConnect !== void 0 ? _internal$no3DConnect : (parameters === null || parameters === void 0 ? void 0 : parameters.no3DConnection) == 'true',
      catalogCodex: parameters !== null && parameters !== void 0 && parameters.catalogCodex ? parameters.catalogCodex.toString() : undefined,
      catalogGuid: parameters === null || parameters === void 0 ? void 0 : parameters.catalogGuid,
      isIDMData: parameters === null || parameters === void 0 ? void 0 : parameters.isIDMData,
      packageId: parameters === null || parameters === void 0 ? void 0 : parameters.packageId,
      serieId: parameters === null || parameters === void 0 ? void 0 : parameters.serieId,
      articleCodexes: parameters !== null && parameters !== void 0 && parameters.articleCodexes ? parameters.articleCodexes.split(';') : undefined,
      variantValues: parameters !== null && parameters !== void 0 && parameters.variantValues ? JSON.parse(parameters.variantValues) : undefined,
      differentOptions: parameters !== null && parameters !== void 0 && parameters.differentOptions ? JSON.parse(parameters.differentOptions) : undefined,
      resetArticle: parameters !== null && parameters !== void 0 && parameters.resetArticle ? Number(parameters === null || parameters === void 0 ? void 0 : parameters.resetArticle) : undefined,
      forceReload: internal === null || internal === void 0 ? void 0 : internal.forceReload,
      divaNr: (parameters === null || parameters === void 0 || (_parameters$divaNr = parameters.divaNr) === null || _parameters$divaNr === void 0 ? void 0 : _parameters$divaNr.trim()) || (parameters === null || parameters === void 0 || (_parameters$DivaNR = parameters.DivaNR) === null || _parameters$DivaNR === void 0 ? void 0 : _parameters$DivaNR.trim()) || (parameters === null || parameters === void 0 || (_parameters$divanr = parameters.divanr) === null || _parameters$divanr === void 0 ? void 0 : _parameters$divanr.trim()),
      productId: parameters === null || parameters === void 0 ? void 0 : parameters.productId,
      customProductId: parameters === null || parameters === void 0 ? void 0 : parameters.customProductId,
      initMode: (parameters === null || parameters === void 0 ? void 0 : parameters.initMode) || (parameters === null || parameters === void 0 ? void 0 : parameters.InitMode) || settings.initMode,
      opt: (parameters === null || parameters === void 0 ? void 0 : parameters.opt) || (parameters === null || parameters === void 0 ? void 0 : parameters.OPT),
      opv: (parameters === null || parameters === void 0 ? void 0 : parameters.opv) || (parameters === null || parameters === void 0 ? void 0 : parameters.OPV),
      configuratorTitle: (parameters === null || parameters === void 0 ? void 0 : parameters.configuratorTitle) || (parameters === null || parameters === void 0 ? void 0 : parameters.ProductName) || (parameters === null || parameters === void 0 ? void 0 : parameters.productName),
      assignNewDivaNr: (parameters === null || parameters === void 0 ? void 0 : parameters.assignNewDivaNr) == 'true' || (parameters === null || parameters === void 0 ? void 0 : parameters.assignNewDivaNr) == '1' || (parameters === null || parameters === void 0 ? void 0 : parameters.assignNewDivaNr) == 'True',
      initEmptyPlanner: (parameters === null || parameters === void 0 ? void 0 : parameters.initEmptyPlanner) == 'true' || (parameters === null || parameters === void 0 ? void 0 : parameters.initEmptyPlanner) == '1' || (parameters === null || parameters === void 0 ? void 0 : parameters.initEmptyPlanner) == 'True',
      used_logged_in: (parameters === null || parameters === void 0 ? void 0 : parameters.used_logged_in) == 'true',
      addDivaNr: parameters === null || parameters === void 0 || (_parameters$addDivaNr = parameters.addDivaNr) === null || _parameters$addDivaNr === void 0 ? void 0 : _parameters$addDivaNr.trim(),
      startDivaNr: parameters === null || parameters === void 0 || (_parameters$startDiva = parameters.startDivaNr) === null || _parameters$startDiva === void 0 ? void 0 : _parameters$startDiva.trim(),
      initProductFolded: !!((_parameters$initProdu = parameters === null || parameters === void 0 ? void 0 : parameters.initProductFolded) !== null && _parameters$initProdu !== void 0 ? _parameters$initProdu : ''),
      settings: settings
    };
  }, [internal === null || internal === void 0 ? void 0 : internal.basketData, parameters === null || parameters === void 0 ? void 0 : parameters.setId, parameters === null || parameters === void 0 ? void 0 : parameters.accountId, parameters === null || parameters === void 0 ? void 0 : parameters.accountGuid, internal === null || internal === void 0 ? void 0 : internal.accountSessionGuid, parameters === null || parameters === void 0 ? void 0 : parameters.catalogCodex, parameters === null || parameters === void 0 ? void 0 : parameters.catalogGuid, parameters === null || parameters === void 0 ? void 0 : parameters.articleCodexes, parameters === null || parameters === void 0 ? void 0 : parameters.variantValues, parameters === null || parameters === void 0 ? void 0 : parameters.differentOptions, parameters === null || parameters === void 0 ? void 0 : parameters.resetArticle, parameters === null || parameters === void 0 ? void 0 : parameters.smSupplierNr, internal === null || internal === void 0 ? void 0 : internal.forceReload, parameters === null || parameters === void 0 ? void 0 : parameters.divaNr, parameters === null || parameters === void 0 ? void 0 : parameters.DivaNR, parameters === null || parameters === void 0 ? void 0 : parameters.divanr, parameters === null || parameters === void 0 ? void 0 : parameters.isIDMData, parameters === null || parameters === void 0 ? void 0 : parameters.packageId, parameters === null || parameters === void 0 ? void 0 : parameters.serieId, parameters === null || parameters === void 0 ? void 0 : parameters.productId, parameters === null || parameters === void 0 ? void 0 : parameters.customProductId, parameters === null || parameters === void 0 ? void 0 : parameters.initMode, parameters === null || parameters === void 0 ? void 0 : parameters.InitMode, parameters === null || parameters === void 0 ? void 0 : parameters.opt, parameters === null || parameters === void 0 ? void 0 : parameters.opv, parameters === null || parameters === void 0 ? void 0 : parameters.OPT, parameters === null || parameters === void 0 ? void 0 : parameters.configuratorTitle, parameters === null || parameters === void 0 ? void 0 : parameters.ProductName, parameters === null || parameters === void 0 ? void 0 : parameters.productName, parameters === null || parameters === void 0 ? void 0 : parameters.assignNewDivaNr, parameters === null || parameters === void 0 ? void 0 : parameters.initEmptyPlanner, parameters === null || parameters === void 0 ? void 0 : parameters.addDivaNr, parameters === null || parameters === void 0 ? void 0 : parameters.initProductFolded, eventSubscriber, eventUnsubscriber, settings]);
}
var componentConfig = {
  name: 'DIVA_WEBPLANNER',
  renderFunction: WebPlannerReact
};

export { Checkout as C, Icon as I, WebPlannerReact as W, _onClickCheckout as _, usePlannerUIHandler as a, b, componentConfig as c, styleInject as s, useViewHandler as u };
